Package: surveysd
Type: Package
Title: Survey Standard Error Estimation for Cumulated Estimates and
        their Differences in Complex Panel Designs
Version: 2.0.3
Authors@R: c(person("Johannes", "Gussenbauer", role = c("aut", "cre"),
                     email = "Johannes.Gussenbauer@statistik.gv.at"),
              person("Alexander", "Kowarik", role = "aut",
                     email = "Alexander.Kowarik@statistik.gv.at", comment=c(ORCID="0000-0001-8598-4130")),
              person("Eileen", "Vattheuer", role = "aut",
                     email = "Eileen.Vattheuer@statistik.gv.at"),
              person("Gregor", "de Cillia", role = "aut",
                     email = "Gregor.deCillia@statistik.gv.at"),
              person("Matthias", "Till", role = "ctb",
                     email = "Matthias.Till@statistik.gv.at"))
Maintainer: Johannes Gussenbauer <Johannes.Gussenbauer@statistik.gv.at>
Description: Calculate point estimates and their standard errors in complex household surveys using bootstrap replicates. Bootstrapping considers survey design with a rotating panel. A comprehensive description of the methodology can be found under <https://statistikat.github.io/surveysd/articles/methodology.html>.
Encoding: UTF-8
License: GPL (>= 2)
Imports: Rcpp (>= 0.12.12),data.table,ggplot2,laeken,methods
LinkingTo: Rcpp
URL: https://github.com/statistikat/surveysd
BugReports: https://github.com/statistikat/surveysd/issues
RoxygenNote: 7.3.3
Suggests: testthat, knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-06-10 12:47:32 UTC; gussen
Author: Johannes Gussenbauer [aut, cre],
  Alexander Kowarik [aut] (ORCID:
    <https://orcid.org/0000-0001-8598-4130>),
  Eileen Vattheuer [aut],
  Gregor de Cillia [aut],
  Matthias Till [ctb]
Repository: CRAN
Date/Publication: 2026-06-11 20:50:02 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-06-11 23:51:20 UTC; unix
Archs: surveysd.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,surveysd)
S3method(print,summary.ipf)
S3method(print,surveysd)
S3method(summary,ipf)
export(calc.stError)
export(combine_factors)
export(computeFrac)
export(computeLinear)
export(computeLinearG1)
export(computeLinearG1_old)
export(demo.eusilc)
export(draw.bootstrap)
export(generate.HHID)
export(get.selection)
export(ipf)
export(ipf_step)
export(ipf_step_f)
export(ipf_step_ref)
export(kishFactor)
export(recalib)
export(rescaled.bootstrap)
export(weightedRatio)
export(weightedSum)
import(Rcpp)
import(data.table)
import(ggplot2)
importFrom("graphics","plot")
importFrom("laeken","weightedMedian")
importFrom("methods","formalArgs")
importFrom("methods","is")
importFrom("stats","as.formula")
importFrom("stats","formula")
importFrom("stats","na.omit")
importFrom("stats","quantile")
importFrom("stats","sd")
importFrom("stats","var")
importFrom("stats","xtabs")
importFrom("utils","capture.output")
importFrom("utils","data")
importFrom("utils","find")
importFrom("utils","head")
importFrom("utils","tail")
useDynLib(surveysd)

# surveysd <img src="man/figures/logo.png" align="right" alt="" />

[![Travis build
status](https://img.shields.io/travis/statistikat/surveysd.svg?logo=travis)](https://app.travis-ci.com/statistikat/surveysd)
[![Coverage
Status](https://img.shields.io/coveralls/github/statistikat/surveysd.svg?colorB=red&logo=codecov)](https://coveralls.io/github/statistikat/surveysd?branch=master)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-maturing-blue.svg?logo=github)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![GitHub last
commit](https://img.shields.io/github/last-commit/statistikat/surveysd.svg?logo=github)](https://github.com/statistikat/surveysd/commits/master)
[![GitHub code size in
bytes](https://img.shields.io/github/languages/code-size/statistikat/surveysd.svg?logo=github)](https://github.com/statistikat/surveysd)
[![Downloads](http://cranlogs.r-pkg.org/badges/surveysd)](https://CRAN.R-project.org/package=surveysd)
[![CRAN](https://img.shields.io/cran/v/surveysd.svg?colorB=green&logo=R&logoColor=blue&label=CRAN)](https://CRAN.R-project.org/package=surveysd)
[![R-CMD-check](https://github.com/statistikat/surveysd/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/statistikat/surveysd/actions/workflows/R-CMD-check.yaml)

[![Mentioned in Awesome Official
Statistics](https://awesome.re/mentioned-badge.svg)](https://github.com/SNStatComp/awesome-official-statistics-software)

This is the development place for the R-package `surveysd`. The package
can be used to estimate the standard deviation of estimates in complex
surveys using bootstrap weights.

## Installation

``` r
# Install release version from CRAN
install.packages("surveysd")

# Install development version from GitHub
devtools::install_github("statistikat/surveysd")
```

## Concept

Bootstrapping has long been around and used widely to estimate
confidence intervals and standard errors of point estimates. This
package aims to combine all necessary steps for applying a calibrated
bootstrapping procedure with custom estimating functions.

## Workflow

A typical workflow with this package consists of three steps. To see
these concepts in practice, please refer to the [getting started
vignette](https://statistikat.github.io/surveysd/articles/surveysd.html).

- Calibrated weights can be generated with the function `ipf()` using an
  iterative proportional updating algorithm.
- Bootstrap samples are drawn with rescaled bootstrapping in the
  function `draw.bootstrap()`.
- These samples can then be calibrated with an iterative proportional
  updating algorithm using `recalib()`.
- Finally, estimation functions can be applied over all bootstrap
  replicates with `calc.stError()`.

## Further reading

More information can be found on the [github-pages
site](https://statistikat.github.io/surveysd/) for surveysd.

- The methodology is covered in the [methodology
  vignette](https://statistikat.github.io/surveysd/articles/methodology.html).
- A more comprehensive documentation of `calc.stError()` is available in
  the [error estimation
  vignette](https://statistikat.github.io/surveysd/articles/error_estimation.html).

## ----echo = FALSE-------------------------------------------------------------
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(eval = FALSE)

library(data.table)
setDTthreads(1)

## ----setup, include=FALSE-----------------------------------------------------
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)

library(data.table)
setDTthreads(1)

## -----------------------------------------------------------------------------
library(surveysd)

set.seed(1234)
eusilc <- demo.eusilc(prettyNames = TRUE)
dat_boot <- draw.bootstrap(eusilc, REP = 10, hid = "hid", weights = "pWeight",
                           strata = "region", period = "year")
dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region",
                          epsP = 1e-2, epsH = 2.5e-2, verbose = FALSE)
dat_boot_calib[, onePerson := nrow(.SD) == 1, by = .(year, hid)]

## print part of the dataset
dat_boot_calib[1:5, .(year, povertyRisk, eqIncome, onePerson, pWeight, w1, w2, w3, w4, w5)]

## -----------------------------------------------------------------------------
povertyRate <- calc.stError(dat_boot_calib, var = "povertyRisk", fun = weightedRatio)
totalIncome <- calc.stError(dat_boot_calib, var = "eqIncome", fun = weightedSum)

## -----------------------------------------------------------------------------
povertyRate$Estimates
totalIncome$Estimates

## -----------------------------------------------------------------------------
## define custom estimator
myWeightedSum <- function(x, w) {
  sum(x*w)
}

## check if results are equal to the one using `surveysd::weightedSum()`
totalIncome2 <- calc.stError(dat_boot_calib, var = "eqIncome", fun = myWeightedSum)
all.equal(totalIncome$Estimates, totalIncome2$Estimates)

## -----------------------------------------------------------------------------
## use add.arg-argument
fun <- function(x, w, b) {
  sum(x*w*b)
}
add.arg = list(b="onePerson")

err.est <- calc.stError(dat_boot_calib, var = "povertyRisk", fun = fun,
                        period.mean = 0, add.arg=add.arg)
err.est$Estimates

# compare with direct computation
compare.value <- dat_boot_calib[,fun(povertyRisk,pWeight,b=onePerson),
                                 by=c("year")]
all((compare.value$V1-err.est$Estimates$val_povertyRisk)==0)

## -----------------------------------------------------------------------------
# custom estimator to first derive poverty threshold 
# and then estimate a weighted ratio
povmd <- function(x, w) {
 md <- laeken::weightedMedian(x, w)*0.6
 pmd60 <- x < md
 # weighted ratio is directly estimated inside the function
 return(sum(w[pmd60])/sum(w)*100)
}

err.est <- calc.stError(
  dat_boot_calib, var = "povertyRisk", fun = weightedRatio,
  fun.adjust.var = povmd, adjust.var = "eqIncome")
err.est$Estimates


## -----------------------------------------------------------------------------
# using fun.adjust.var and adjust.var to estimate povmd60 indicator
# for each period and bootstrap weight before applying the weightedRatio
povmd2 <- function(x, w) {
 md <- laeken::weightedMedian(x, w)*0.6
 pmd60 <- x < md
 return(as.integer(pmd60))
}

# set adjust.var="eqIncome" so the income vector is used to estimate
# the povmd60 indicator for each bootstrap weight
# and the resulting indicators are passed to function weightedRatio
group <- "gender"
err.est <- calc.stError(
  dat_boot_calib, var = "povertyRisk", fun = weightedRatio, group = "gender",
  fun.adjust.var = povmd2, adjust.var = "eqIncome")
err.est$Estimates

## -----------------------------------------------------------------------------
multipleRates <- calc.stError(dat_boot_calib, var = c("povertyRisk", "onePerson"), fun = weightedRatio)
multipleRates$Estimates

## -----------------------------------------------------------------------------
dat2 <- subset(dat_boot_calib, year == 2010)
for (att  in c("period", "weights", "b.rep"))
  attr(dat2, att) <- attr(dat_boot_calib, att)

## -----------------------------------------------------------------------------
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, group = "region")
povertyRates$Estimates

## -----------------------------------------------------------------------------
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = c("gender", "region"))
povertyRates$Estimates


## -----------------------------------------------------------------------------
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = list(c("gender", "region")))
povertyRates$Estimates

## -----------------------------------------------------------------------------
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = list("gender", "region", c("gender", "region")))
povertyRates$Estimates

## -----------------------------------------------------------------------------
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = c("gender", "region"),
                             group.diff = TRUE)
povertyRates$Estimates

## -----------------------------------------------------------------------------
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = list(c("gender", "region")),
                             group.diff = TRUE)
povertyRates$Estimates[,.N,by=.(estimate_type)]

## -----------------------------------------------------------------------------
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             period.diff = c("2017 - 2016", "2016 - 2015", "2015 - 2014"))
povertyRates$Estimates

## -----------------------------------------------------------------------------
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             group = "gender",
                             period.diff = c("2017 - 2016", "2016 - 2015", "2015 - 2014"))
povertyRates$Estimates

## -----------------------------------------------------------------------------
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             period.mean = 3)
povertyRates$Estimates

## -----------------------------------------------------------------------------
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             period.mean = 3, period.diff = "2016 - 2015",
                             group = "gender")
povertyRates$Estimates

#' Generate new houshold ID for survey data with rotating panel design taking
#' into account split households
#'
#' Generating a new houshold ID for survey data using a houshold ID and a
#' personal ID.
#' For surveys with rotating panel design containing housholds, houshold members
#' can move from an existing household to a new one, that was not originally in
#' the sample. This leads to the creation of so called split households. Using a
#' peronal ID (that stays fixed over the whole survey), an indicator for
#' different time steps and a houshold ID, a new houshold ID is assigned to the
#' original and the split household.
#'
#' @param dat data table of data frame containing the survey data
#' @param period column name of \code{dat} containing an indicator for the
#'   rotations, e.g years, quarters, months, ect...
#' @param pid column name of \code{dat} containing the personal identifier. This
#'   needs to be fixed for an indiviual throught the whole survey
#' @param hid column name of \code{dat} containing the household id. This needs
#'   to for a household throught the whole survey
#'
#' @return the survey data \code{dat} as data.table object containing a new and
#'   an old household ID. The new household ID which considers the split
#'   households is now named \code{hid} and the original household ID has a
#'   trailing "_orig".
#' @export generate.HHID
#'
#' @examples
#' \dontrun{
#' library(surveysd)
#' library(laeken)
#' library(data.table)
#'
#' eusilc <- surveysd:::demo.eusilc(n=4)
#'
#' # create spit households
#' eusilc[,rb030split:=rb030]
#' year <- eusilc[,unique(year)]
#' year <- year[-1]
#' leaf_out <- c()
#' for(y in year) {
#'   split.person <- eusilc[year==(y-1)&!duplicated(db030)&!db030%in%leaf_out,
#'                          sample(rb030,20)]
#'   overwrite.person <- eusilc[year==(y)&!duplicated(db030)&!db030%in%leaf_out,
#'                              .(rb030=sample(rb030,20))]
#'   overwrite.person[,c("rb030split","year_curr"):=.(split.person,y)]
#'
#'   eusilc[overwrite.person,
#'          rb030split:=i.rb030split,on=.(rb030,year>=year_curr)]
#'   leaf_out <- c(
#'     leaf_out,
#'     eusilc[rb030%in%c(overwrite.person$rb030,overwrite.person$rb030split),
#'     unique(db030)])
#' }
#'
#' # pid which are in split households
#' eusilc[,.(uniqueN(db030)),by=list(rb030split)][V1>1]
#'
#' eusilc.new <- generate.HHID(eusilc, period = "year", pid = "rb030split",
#'                             hid = "db030")
#'
#' # no longer any split households in the data
#' eusilc.new[,.(uniqueN(db030)),by=list(rb030split)][V1>1]
#' }
#'


generate.HHID <- function(dat, period = "RB010", pid = "RB030", hid = "DB030") {

  ID_new <- ID_orig <- ALL_NEW <- ID_new_help <- NULL


  ID_new <- ID_orig <- ALL_NEW <- na.omit
  # check input
  if (!is.data.frame(dat) & !is.data.table(dat)) {
    stop("dat must be a data frame or data table")
  }
  if (!is.data.table(dat)) {
    dat <- data.table(dat)
  }
  dat <- copy(dat)
  c.names <- colnames(dat)

  #
  if (!is.character(period)) {
    stop("period must be a string")
  } else {
    if (length(period) > 1) {
      stop("period must have length 1")
    }
    if (!period %in% c.names) {
      stop(period, " is not a column of dat")
    }
    if (!is.numeric(dat[[period]]) & !is.integer(dat[[period]])) {
      stop(period, " must be an integer or numeric vector")
    }
  }

  if (!is.character(pid)) {
    stop("pid must be a string")
  }else{
    if (length(pid) > 1) {
      stop("pid must have length 1")
    }
    if (!pid %in% c.names) {
      stop(pid, " is not a column of dat")
    }
  }
  if (!is.character(hid)) {
    stop("hid must be a string")
  } else {
    if (length(hid) > 1) {
      stop("hid must have length 1")
    }
    if (!hid %in% c.names) {
      stop(hid, " is not a column of dat")
    }
  }

  # create lookup table starting from first period
  ID_lookup <- dt.eval("dat[", period, "==min(", period, "),.(", pid,
                       ",ID_orig=", hid, ")]")
  ID_lookup[, ID_new := .GRP, by = ID_orig]
  ID_lookup[, ID_orig := NULL]

  periods <- sort(dt.eval("dat[,unique(", period, ")]"))

  for (i in periods[-1]) {

    ID_lookup <- merge(ID_lookup, dt.eval(
      "dat[", period, "==", i, ",.(", pid, ",ID_orig=", hid, ")]"),
      by = pid, all = TRUE)
    ID_lookup[!is.na(ID_orig), ALL_NEW := all(is.na(ID_new)), by = ID_orig]
    ID_lookup[ALL_NEW == FALSE, ID_new := na.omit(ID_new)[1], by = ID_orig]
    ID_next <- ID_lookup[, max(ID_new, na.rm = TRUE)]
    ID_lookup[ALL_NEW == TRUE, ID_new := .GRP + ID_next, by = ID_orig]
    ID_lookup[, c("ID_orig", "ALL_NEW") := NULL]
  }
  dat <- merge(dat, ID_lookup, by = pid)
  # if ID not unique by hid and year
  # leave original grouping for this year
  # this happens if household splits up and people move to already existing
  #   households
  group_broke <- dt.eval("dat[,length(unique(ID_new)),by=list(", period, ",",
                         hid, ")][V1>1,.(", period, ",", hid, ")]")
  if (nrow(group_broke) > 0) {
    setkeyv(dat, c(period, hid))
    dt.eval("dat[group_broke,ID_new_help:=paste0(head(", hid,
            ",1),'_1'),by=list(ID_new)]")
    dat[is.na(ID_new_help), ID_new_help := as.character(ID_new)]
    dat[, ID_new := .GRP, by = ID_new_help]
    dat[, ID_new_help := NULL]
  }

  setnames(dat, hid, paste0(hid, "_orig"))
  setnames(dat, "ID_new", hid)
  return(dat)
}

#################################
# test generate.HHID()
#

context("generate.HHID()")
library(surveysd)
library(laeken)
library(data.table)

eusilc <- surveysd:::demo.eusilc()
eusilc <- eusilc[!db040 %in% c("Vienna", "Lower Austria", "Upper Austria")]


eusilc[, rb030split := rb030]
# create spit households
eusilc[, rb030split := rb030]
year <- eusilc[, unique(year)]
year <- year[-1]
leaf_out <- c()
for (y in year) {
  split.person <- eusilc[year == (y - 1) & !duplicated(db030) &
                           !db030 %in% leaf_out,
                         sample(rb030, 20)]
  overwrite.person <- eusilc[year == (y) & !duplicated(db030) &
                               !db030 %in% leaf_out,
                             .(rb030 = sample(rb030, 20))]
  overwrite.person[, c("rb030split", "year_curr") := .(split.person, y)]

  eusilc[overwrite.person, rb030split := i.rb030split,
         on = .(rb030, year >= year_curr)]
  leaf_out <- c(
    leaf_out,
    eusilc[rb030 %in% c(overwrite.person$rb030, overwrite.person$rb030split),
           unique(db030)])
}

# test input parameter
test_that("test para - data", {
  expect_error(
    generate.HHID(
      as.matrix(eusilc), period = "year", pid = "rb030", hid = "db030"),
    "dat must be a data.frame or data.table")
  expect_error(generate.HHID(
    eusilc, period = "year", pid = "rb030", hid = "db030"), NA)
})


test_that("test para - hid, pid and period", {
  expect_error(
    generate.HHID(
      eusilc, period = "years", pid = "rb030", hid = "db030"),
    "years is not a column of dat")
  expect_error(
    generate.HHID(
      eusilc, period = "year", pid = "rb030s", hid = "db030"),
    "rb030s is not a column of dat")
  expect_error(
    generate.HHID(
      eusilc, period = "year", pid = "rb030", hid = "db030s"),
    "db030s is not a column of dat")

  eusilc[, year.char := as.character(year)]
  expect_error(
    generate.HHID(
      eusilc, period = "year.char", pid = "rb030", hid = "db030"),
    "year.char must be an integer or numeric vector")
})

test_that("test return", {
  dat.HHID <- generate.HHID(eusilc, period = "year", pid = "rb030split",
                            hid = "db030")
  dat.HHID <- dat.HHID[, uniqueN(db030), by = rb030split][V1 > 1]
  expect_true(nrow(dat.HHID) == 0)
})

#' @title Draw bootstrap replicates
#'
#' @description Draw bootstrap replicates from survey data using either the rescaled
#'   bootstrap for stratified multistage sampling, presented by J. Preston
#'   (2009) or the Rao-Wu boostrap by J. N. K. Rao and C. F. J. Wu (1988) 
#'
#' @param dat either data frame or data table containing the survey sample
#' @param method for bootstrap replicates, either `"Preston"` or `"Rao-Wu"` 
#' @param REP integer indicating the number of bootstraps to be drawn
#' @param strata string specifying the column name in `dat` that is used for
#'   stratification. For multistage sampling multiple column names can be
#'   specified by `strata=c("strata1","strata2","strata3")` or 
#'   `strata=c("strata1>strata2>strata3")`. See Details for more
#'   information.
#' @param cluster string specifying the column name in `dat` that is used for
#'   clustering. For instance given a household sample the column containing
#'   the household ID should be supplied.
#'   For multistage sampling multiple column names can be specified
#'   by `cluster=c("cluster1","cluster2","cluster3")` or 
#'   `cluster=c("cluster1>cluster2>cluster3")`.
#'   See Details for more information.
#' @param fpc string specifying the column name in `dat` that contains the
#'   number of PSUs at the first stage. For multistage sampling the number of
#'   PSUs at each stage must be specified by `strata=c("fpc1","fpc2","fpc3")` 
#'   or `strata=c("fpc1>fpc2>fpc3")`.
#' @param single.PSU either "merge" or "mean" defining how single PSUs need to
#'   be dealt with. For `single.PSU="merge"` single PSUs at each stage are
#'   merged with the strata or cluster with the next least number of PSUs. If
#'   multiple of those exist one will be select via random draw. For
#'   `single.PSU="mean"` single PSUs will get the mean over all bootstrap
#'   replicates at the stage which did not contain single PSUs.
#' @param return.value either "data", "replicates" and/or "selection" 
#'   specifying the return value of the function. For "data" the survey data is 
#'   returned as class `data.table`, for "replicates" only the bootstrap replicates 
#'   are returned as `data.table`. For "selection" list of data.tables with 
#'   length of `length(strata)` is returned containing 1:`REP` 0-1 columns 
#'   indicating if a PSU was selected for each sampling stage.
#' @param run.input.checks logical, if TRUE the input will be checked before applying
#'   the bootstrap procedure
#' @param already.selected list of data.tables or `NULL` where each data.table contains 
#'   columns in `cluster`, `strata` and additionally 1:`REP` columns containing
#'   0-1 values which indicate if a PSU was selected for each bootstrap replicate.
#'   Each of the data.tables corresponds to one of the sampling stages. First entry
#'   in the list corresponds to the first sampling stage and so on.
#' @param seed integer specifying the seed for the random number generator.
#' 
#' @details For specifying multistage sampling designs the column names in
#' `strata`,`cluster` and `fpc` need to be seperated by ">".\cr
#' For multistage sampling the strings are read from left to right meaning that
#' the first vector entry or column name before the first ">" is taken as the column for
#' stratification/clustering/number of PSUs at the first and the last vector entry
#' or column after
#' the last ">" is taken as the column for stratification/clustering/number of
#' PSUs at the last stage.
#' If for some stages the sample was not stratified or clustered one must
#' specify this by "1" or "I", e.g. `strata=c("strata1","I","strata3")` or 
#' `strata=c("strata1>I>strata3")` if there was
#' no stratification at the second stage or 
#' `cluster=c("cluster1","cluster2","I")` respectively 
#' `cluster=c("cluster1>cluster2>I")`
#' if there were no clusters at the last stage.\cr
#' The number of PSUs at each stage is not calculated internally and must be
#' specified for any sampling design.
#' For single stage sampling using stratification this can usually be done by
#' adding over all sample weights of each PSU by each strata-code.\cr
#' Spaces in each of the strings will be removed, so if column names contain
#' spaces they should be renamed before calling this procedure!\cr 
#' If `already.selected` is supplied the sampling of bootstrap replicates 
#' considers if speficif PSUs have already been selected by a previous survey wave.
#' For a specific `strata` and `cluster` this could lead to more than `floor(n/2)` 
#' records selected. In that case records will be de-selected such that `floor(n/2)` records,
#' with `n` as the total number of records, are selected for each 
#' `strata` and `cluster`. This parameter ist mostly used by [draw.bootstrap] in
#' order to consider the rotation of the sampling units over time.
#'
#' @return returns the complete data set including the bootstrap replicates or
#'   just the bootstrap replicates, depending on `return.value="data"` or
#'   `return.value="replicates"` respectively.
#' @export rescaled.bootstrap
#'
#' @references 
#' Preston, J. (2009). Rescaled bootstrap for stratified multistage
#'   sampling. Survey Methodology. 35. 227-234.
#' Rao, J. N. K., and C. F. J. Wu. (1988). Resampling Inference with Complex Survey Data.
#'  Journal of the American Statistical Association 83 (401): 231–41.
#' @author Johannes Gussenbauer, Eileen Vattheuer, Statistics Austria
#'
#' @examples
#' 
#' library(surveysd)
#' library(data.table)
#' setDTthreads(1)
#' set.seed(1234)
#' eusilc <- demo.eusilc(n = 1,prettyNames = TRUE)
#' 
#' eusilc[,N.households := sum(pWeight[!duplicated(hid)]),
#'                                   by = .(region, year)]
#' eusilc.bootstrap <- rescaled.bootstrap(eusilc, REP = 10, strata = "region",
#'                                        cluster = "hid", fpc = "N.households")
#' 
#' eusilc[,new_strata := paste(region,hsize,sep="_")]
#' eusilc[,N.housholds := sum(pWeight[!duplicated(hid)]), by = new_strata]
#' eusilc.bootstrap <- rescaled.bootstrap(eusilc, REP = 10, strata = c("new_strata"),
#'                                        cluster = "hid", fpc = "N.households")
#'
#'


rescaled.bootstrap <- function(
    dat, 
    method = c("Preston", "Rao-Wu"), 
    REP = 1000, 
    strata = "DB050>1",   
    cluster = "DB060>DB030",   
    fpc = "N.cluster>N.households",
    single.PSU = c("merge", "mean"), 
    return.value = c("data", "replicates"),
    run.input.checks = TRUE, 
    already.selected = NULL, 
    seed = NULL) {

  
  InitialOrder <- N <- SINGLE_BOOT_FLAG <- SINGLE_BOOT_FLAG_FINAL <- f <-
    n_prev <- n_draw_prev <- sum_prev <- n_draw <- strata_i <- 
    V1 <- fpc_i <- . <- new.var_col <- new.var <- NULL  
  
  dat <- copy(dat)
  method <- method[1]
  # check run.input.checks
  if (!is.logical(run.input.checks)) {
    stop("run.input.checks can only be logical")
  }
  if (run.input.checks) {
    # check input data
    if (!is.data.table(dat) & !is.data.frame(dat)) {
      stop("dat needs to be a data frame or data table")
    } else {
      dat <- data.table(dat)
    }
    # check seed
    if(!is.null(seed)){
      check.input(seed, input.name = "seed", input.length=1, input.type="numeric")
    }
  }
  set.seed(seed) 
  
  # prepare input
  removeCols <- c()  
  if (is.null(cluster)) {
    cluster <- generateRandomName(20, colnames(dat))
    removeCols <- c(removeCols, cluster)
    dat[, c(cluster) := 1:.N]
  }
  
  if (is.null(strata)) {
    strata <- generateRandomName(20, colnames(dat))
    removeCols <- c(removeCols, strata)
    dat[, c(strata) := 1]
  }
  
  
  # check inputs strata, cluster, fpc
  check.input(strata, input.name = "strata", input.type="character")
  check.input(fpc, input.name = "fpc", input.type="character")
  check.input(cluster, input.name = "cluster", input.type="character")
  
  if(length(strata)==1 && strata %like% ">"){
    strata <- unlist(strsplit(strata, ">"))
    strata <- gsub("\\s", "", strata)
  } 
  
  if(length(cluster)==1 && cluster %like% ">"){
    cluster <- unlist(strsplit(cluster, ">"))
    cluster <- gsub("\\s", "", cluster)
  }
  if(length(fpc)==1 && fpc %like% ">"){
    fpc <- unlist(strsplit(fpc, ">"))
    fpc <- gsub("\\s", "", fpc)
  }
  
  single.PSU <- single.PSU[1]  
  
  # continue input checks
  if (run.input.checks) {
    
    # check REP
    check.input(REP, input.name = "REP", input.length=1, input.type="numeric",
                decimal.possible = FALSE)
    
    # check method
    method <- match.arg(method)
    
    # Warning for Rao-Wu method
    if (method == "Rao-Wu") {
      warning("The 'Rao-Wu' method should only be used when the first stage sampling is conducted without replacement. Ensure that your sampling design follows this requirement.")
    }
    
    # check design variables
    # check length of strata, cluster, fpc
    if(length(strata) != length(fpc) | length(strata) != length(cluster)){
      stop("strata, cluster, and fpc need to have the same number of arguments",
           " either separated with '>' or be character vectors of the same length")
    }
    
    # cluster can only be "1" or "I" in the final stage
    if(length(cluster)>1 && any(cluster[1:(length(cluster)-1)] %in% c("I","1"))){
      stop("Specifying 'I' or '1' for parameter 'cluster' 
           can only be set in the final sampling stage!")
    }
    
    check.values <- c(strata, cluster, fpc)
    check.values <- check.values[!check.values %in% c("1", "I")]
    check.values <- check.values[!check.values %in% colnames(dat)]
    if (length(check.values) > 0) {
      stop("dat does not contain the column(s): ", check.values)
    }
    
    # check if there are missings in fpc
    if (any(is.na(dat[, .SD, .SDcols = c(fpc)]))) {
      stop("missing values not allowed in fpc")
    }
    
    # check return.value
    if (any(!return.value %in% c("data", "replicates", "selection"))) {
      stop("return.value can only take the values 'data', 'replicates' or 'selection'")
    }

    # check single.PSU
    if (is.null(single.PSU) || !single.PSU %in% c("merge", "mean")) {
      warning("single.PSU was not set to either 'merge' or 'mean'!\n Bootstrap",
              " replicates for single PSUs cases will be missing!")
      single.PSU <- FALSE
    }
    
    # check for each stage that PSUs are not in mutiple strata
    for(i in seq_along(strata)){
      if(!strata[i]%in%c("1","I") & !cluster[i]%in%c("1","I")){
        countMultiple <- dat[,uniqueN(strata_i), by=c(cluster[i]),
                             env = list(strata_i = strata[i])]
        countMultiple <- countMultiple[V1>1]
        if(nrow(countMultiple)>0){
          stop("Some sampling units in ",cluster[i]," occur in multiple strata of ",strata[i])
        }
      }
    }
    
  }
  
  # check if variable f, N, n are in data.table
  overwrite.names <- c("f", "N", "n", "n_prev", "n_draw", "n_draw_prev") 
  overwrite.names <- overwrite.names[overwrite.names %in% colnames(dat)]
  if (length(overwrite.names) > 0) {
    overwrite.names.new <- paste0("ORIGINAL_", overwrite.names)
    setnames(dat, overwrite.names, overwrite.names.new)
    
    strata[strata %in% overwrite.names] <-
      overwrite.names.new[strata %in% overwrite.names]
    cluster[cluster %in% overwrite.names] <-
      overwrite.names.new[cluster %in% overwrite.names]
    fpc[fpc %in% overwrite.names] <-
      overwrite.names.new[fpc %in% overwrite.names]
  }
  
  # set index for data to return dat in correct order
  dat[, InitialOrder := .I] 
  
  # calculate bootstrap replicates
  stages <- length(strata) 
  
  n <- nrow(dat)
  
  # define values for final calculation
  n.calc <- matrix(0, nrow = n, ncol = stages) # n = sample size
  N.calc <- matrix(0, nrow = n, ncol = stages) #  N = total population
  n_draw.calc <- matrix(0, nrow = n, ncol = stages)  # n_draw = draws in different satges
  delta.calc <- array(0, dim = c(n, stages, REP)) 
  
  delta_selection <- list()
  
  for (i in 1:stages) {  
    
    # define by.val
    by.val <- strata[i]  # by.val determines the grouping for the current level
    if (i > 1) {
      by.val <- c(strata[1:(i - 1)], cluster[1:(i - 1)], strata[i]) 
    }
    by.val <- by.val[!by.val %in% c("1", "I")] # filter out placeholders
    
    # define cluster value
    clust.val <- cluster[i]
    if (clust.val %in% c("1", "I")) {
      clust.val <- paste0("ID_help_", i)  # creates helper ids if no real cluster is defined, sampling takes place at individual level (because a cluster definition is always required)
      dat[, c(clust.val) := .I, by = c(by.val)]
    }
    
    # check of fpc[i] is well defined
    check.fpc <- dat[,uniqueN(fpc_i), by=c(by.val), 
                     env = list(fpc_i = fpc[i])]  
    check.fpc <- check.fpc[V1>1]
    if (nrow(check.fpc) > 0) {
      stop("values in ", fpc[i], " do vary in some strata-cluster ",
           "combinations at sampling stage ", i)
    }
    
    singles <- dat[fpc_i>1, sum(!duplicated(clust.val)), by=c(by.val),   # finds groups in which only a single psu is present -> this is problematic for bootstrap (cannot be resampled with only one element)
                   env = list(fpc_i = fpc[i],
                              clust.val = clust.val)]
    singles <- singles[V1==1]
    if (nrow(singles) > 0) {
      higher.stages <- c(head(strata,i-1), head(cluster,i-1))
      by.val.tail <- tail(by.val,1)
      firstStage <- length(higher.stages) == 0
      if (firstStage) {
        higher.stages <- by.val
      }
      singles <- unique(subset(singles, select = higher.stages))
      
################ SINGLE BEGIN ################ ##############################################################################################
      
      if (method == "Preston") {
        if (single.PSU == "merge") { # strata with only one PSU are merged with the nearest smaller stratum to avoid issues in the bootstrap procedure.
          
          if ("data" %in% return.value) {
            by.val.orig <- paste0(by.val.tail, "_ORIGINALSINGLES")
            fpc_orig <- paste0(fpc[i], "_ORIGINALSINGLES")
            
            set(dat, j = by.val.orig, value = dat[[by.val.tail]])
            set(dat, j = fpc_orig, value = dat[[fpc[i]]])
          }
          
          setkeyv(dat, higher.stages)
          next.PSU <- dat[singles, .(N = sum(!duplicated(clust.val))), by = c(by.val),
                          on = c(higher.stages),
                          env = list(clust.val = clust.val)]
          
          if(nrow(next.PSU)==1){
            stop("Only 1 single PSU in first sampling stage! Cannot combine single PSU.\nPlease manually combine sampling STRATA to eliminate single PSU!")
          }
          
          new.var <- paste0(tail(by.val, 1), "_NEWVAR")
          
          next.PSU[,c(new.var) := next_minimum(N, by.val.tail), by = c(higher.stages),
                   env = list(by.val.tail = by.val.tail)]
          if(any(is.na(next.PSU[[new.var]]))){
            if(firstStage){
              next.PSU[is.na(new.var_col), c(new.var) := head(higher.stages, 1),      
                       env = list(new.var_col = new.var,
                                  higher.stages = higher.stages)]
            }else{
              next.PSU[is.na(new.var_col), c(new.var) := head(by.val.tail,1),      
                       env = list(new.var_col = new.var,
                                  by.val.tail = by.val.tail)]
            }
          }
          
          # select singel PSUs and PSUs to join with
          next.PSU <- next.PSU[N == 1 | new.var == by.val.tail,
                               env = list( new.var = new.var,
                                           by.val.tail = by.val.tail)]
          dat <- merge(dat, next.PSU[, mget(c(by.val, new.var))],
                       by = c(by.val), all.x = TRUE)
          
          # sum over margins
          fpc.i_ADD <- paste0(fpc[i], "_ADD")
          dat[!is.na(new.var), c(fpc.i_ADD) := 
                sum(fpc_i[!duplicated(by.val.tail)]), by = .(new.var),
              env = list(new.var = new.var,
                         fpc_i = fpc[i],
                         by.val.tail = by.val.tail)]
          
          # assign to new group
          dat[!is.na(new.var), c(by.val.tail) := new.var, 
              env = list(new.var = new.var)]
          dat[ ,c(fpc[i]) := fpc_i[is.na(new.var)][1], by=c(by.val),
               env = list(new.var = new.var)]
          dat[!is.na(new.var), c(fpc[i]) := fpc.i_ADD,
              env = list(new.var = new.var,
                         fpc.i_ADD = fpc.i_ADD)]
          
          dat[, c(new.var, paste0(fpc[i], "_ADD")) := NULL]
        } else if (single.PSU == "mean") {  # if single = mean: Marks the affected observations with a SINGLE_BOOT_FLAG in order to simply take the average of other replicates later in the bootstrap process.
          
          singles[, SINGLE_BOOT_FLAG := paste(higher.stages, .GRP, sep = "-"),  
                  by = c(higher.stages)]
          
          dat <- merge(dat, singles, by = c(higher.stages), all.x = TRUE)
          if (!"SINGLE_BOOT_FLAG_FINAL" %in% colnames(dat)) {
            dat[, SINGLE_BOOT_FLAG_FINAL := SINGLE_BOOT_FLAG]
          } else {
            dat[is.na(SINGLE_BOOT_FLAG_FINAL),
                SINGLE_BOOT_FLAG_FINAL := SINGLE_BOOT_FLAG]
          }
          dat[, SINGLE_BOOT_FLAG := NULL]
          
        } else {
          message("Single PSUs detected at the following stages:\n")
          dat.print <- dat[,sum(!duplicated(clust.val)), by=c(by.val),
                           env = list(clust.val = clust.val)]
          dat.print <- dat.print[V1==1,.SD,.SDcols=c(by.val)]
          print(dat.print)
        }
        
      } else if (method == "Rao-Wu") {
        if (single.PSU == "merge") {
          
          if ("data" %in% return.value) {
            by.val.orig <- paste0(by.val.tail, "_ORIGINALSINGLES")
            fpc_orig <- paste0(fpc[i], "_ORIGINALSINGLES")
            
            set(dat, j = by.val.orig, value = dat[[by.val.tail]])
            set(dat, j = fpc_orig, value = dat[[fpc[i]]])
          }
          
          setkeyv(dat, higher.stages)
          next.PSU <- dat[singles, .(N = sum(!duplicated(clust.val))), by = c(by.val),
                          on = c(higher.stages),
                          env = list(clust.val = clust.val)]
          
          if(nrow(next.PSU)==1){
            stop("Only 1 single PSU in first sampling stage! Cannot combine single PSU.\nPlease manually combine sampling STRATA to eliminate single PSU!")
          }
          
          new.var <- paste0(tail(by.val, 1), "_NEWVAR")
          
          next.PSU[,c(new.var) := next_minimum(N, by.val.tail), by = c(higher.stages),
                   env = list(by.val.tail = by.val.tail)]
          if(any(is.na(next.PSU[[new.var]]))){
            if(firstStage){
              next.PSU[is.na(new.var_col), c(new.var) := head(higher.stages, 1),
                       env = list(new.var_col = new.var,
                                  higher.stages = higher.stages)]
            }else{
              next.PSU[is.na(new.var_col), c(new.var) := head(by.val.tail,1),
                       env = list(new.var_col = new.var,
                                  by.val.tail = by.val.tail)]
            }
          }
          
          # select singel PSUs and PSUs to join with
          next.PSU <- next.PSU[N == 1 | new.var == by.val.tail,
                               env = list( new.var = new.var,
                                           by.val.tail = by.val.tail)]
          dat <- merge(dat, next.PSU[, mget(c(by.val, new.var))],
                       by = c(by.val), all.x = TRUE)
          
          # sum over margins
          fpc.i_ADD <- paste0(fpc[i], "_ADD")  
          dat[!is.na(new.var), c(fpc.i_ADD) := 
                sum(fpc_i[!duplicated(by.val.tail)]), by = c(new.var),
              env = list(new.var = new.var,
                         fpc_i = fpc[i],
                         by.val.tail = by.val.tail)]
          
          # assign to new group
          dat[!is.na(new.var), c(by.val.tail) := new.var, 
              env = list(new.var = new.var)]
          dat[ c(fpc[i]) := fpc_i[is.na(new.var)][1], by=c(by.val),
               env = list(new.var = new.var)]
          dat[!is.na(new.var), c(fpc[i]) := fpc.i_ADD,
              env = list(new.var = new.var,
                         fpc.i_ADD = fpc.i_ADD)]
          
          dat[, c(new.var, paste0(fpc[i], "_ADD")) := NULL]
        } 
        
        else if (single.PSU == "mean") {
          
          dat[!is.na(SINGLE_BOOT_FLAG_FINAL), c(bootRep) := lapply( 
            .SD,
            function(z) {  # Ensure that the single PSU is pulled at least once
              if (all(is.na(z))) {
                avg <- mean(z, na.rm = TRUE)
                
                if (is.na(avg)) avg <- 0  # If all values of a replica column are NA, a default value (e.g. 0) is used -> Rao-Wu method is designed to deal with uncertain data
                return(rep(avg, length(z)))  
              } else {
                return(z)
              }
            }
          ), by = SINGLE_BOOT_FLAG_FINAL, .SDcols = c(bootRep)]
        }
        else {
          message("Single PSUs detected at the following stages:\n")
          dat.print <- dat[,sum(!duplicated(clust.val)), by=c(by.val),
                           env = list(clust.val = clust.val)]
          dat.print <- dat.print[V1==1,.SD,.SDcols=c(by.val)]
          print(dat.print)
          
        }
        
      }
    }
################ SINGLE END ################ ##############################################################################################
    
    # get Stage
    if (i == 1) { 
      dati <- dat[,.(N=fpc_i[1], 
                     clust.val = unique(clust.val),
                     f = 1, 
                     n_prev = 1, n_draw_prev = 1, 
                     sum_prev = 1),  
                  by = c(by.val),
                  env = list(fpc_i = fpc[i],
                             clust.val = clust.val)]
      
    } else { # In level 2 or higher: The information from the previous draw is now transferred
      dati <- dat[,.(N=fpc_i[1], 
                     clust.val = unique(clust.val),
                     f = f[1], 
                     n_prev = n_prev[1], 
                     n_draw_prev = n_draw_prev, 
                     sum_prev = sum_prev[1]),
                  by = c(by.val),
                  env = list(fpc_i = fpc[i],
                             clust.val = clust.val)]
    }
    
    deltai <- paste0("delta_", i, "_", 1:REP) 
    dati[, n := .N, by = c(by.val)]  
    
    if (method == "Preston") {
      dati[,n_draw:=floor(n/2),by = c(by.val)]   
    } else {
      dati[, n_draw := n - 1, by = c(by.val)] 
    }
    
    
    if (nrow(dati[n_draw == 0]) > 0) {  
      stop("Resampling 0 PSUs should not be possible! Please report bug in ",
           "https://github.com/statistikat/surveysd")
    }
    
    # do bootstrap for i-th stage
    if (!is.null(already.selected)) { 
      # if already.selected was supplied, consider already selected units
      dati_selected <- already.selected[[i]]
      on_cols <- c(by.val, clust.val)[c(by.val, clust.val) %in% colnames(dati_selected)]
      dati[dati_selected, c(deltai) := mget(deltai), on = c(on_cols)]
      
      dati[, c(deltai) := lapply(.SD, function(delta, n, n_draw) {
        
        # Handle method-specific sampling
        if (method == "Rao-Wu") {  
          (draw.with.replacement(n[1], n_draw[1], delta = delta))
        } else if (method == "Preston") {
          (draw.without.replacement(n[1], n_draw[1], delta = delta))
        } else {
          stop("Invalid method specified: ", method)
        }
      }, n = n, n_draw = n_draw), by = c(by.val), .SDcols = c(deltai)]
      
      dati_check <- dati[, lapply(.SD, function(z, n_draw) {
        sum(z) == n_draw[1]
        
        if (length(z) == 0 || is.null(z)) {
          print("z ist leer oder NULL!")
          return(FALSE)  # Gebe FALSE zurück, um den Fehler zu vermeiden
        }
        
        # Prüfe die Bedingung
        result <- sum(z, na.rm = TRUE) == n_draw[1]
        # print(paste("Result:", result))
        
        return(as.logical(result))
        
        
      }, n_draw = n_draw), by = c(by.val), .SDcols = c(deltai)]
      
      if (any(!dati_check[, .SD, .SDcols = c(deltai)])) {
        stop("Wrong number of units selected! Please report bug in ",
             "https://github.com/statistikat/surveysd")
      }
      
    } else {
      # Do sampling based on the specified method
      if (method == "Preston") {
        
        # Sampling without replacement
        dati[, c(deltai) := as.data.table(
          replicate(REP, draw.without.replacement(n[1], n_draw[1]), simplify = FALSE)
        ), by = c(by.val)]
      } else if (method == "Rao-Wu") {

        # Sampling with replacement
        dati[, c(deltai) := as.data.table(
          replicate(REP, draw.with.replacement(n[1], n_draw[1]), simplify = FALSE)
        ), by = c(by.val)]
      } else {
        stop("Invalid method specified. Please use 'Preston' or 'Rao-Wu'.")
      }
    }
    
    # merge with data
    if(i>1){ 
      dat[,c("f","n_prev","n_draw_prev","sum_prev"):=NULL]
    }
    
    dat <- merge(dat,dati,by=c(by.val,clust.val)) 
    setorder(dat,InitialOrder) 
    
    # prepare output for return.value %in% "selection"
    keep_cols <- c(cluster, strata, deltai)
    keep_cols <- keep_cols[keep_cols %in% colnames(dat)]
    delta_selection <- c(delta_selection,
                         list(subset(dat, select = keep_cols)))
    
    # only matrices and arrays needed for final calculation
    n.calc[, i] <- dat[, n]
    N.calc[, i] <- dat[, N]
    n_draw.calc[, i] <- dat[, n_draw]
    delta.calc[, i, ] <- as.matrix(dat[, mget(deltai)])
    
    dat[, f := n / N * f] 
    dat[, n_prev := n * n_prev]
    dat[, n_draw_prev := n_draw * n_draw_prev]
    
    dat[, c("n", "N", deltai, "n_draw") := NULL] 
    
    rm(dati);gc() 
  }
  
  dat[,c("f","n_prev","n_draw_prev","sum_prev"):=NULL]
  
  names(delta_selection) <- paste0("SamplingStage",1:stages)
  
  bootRep <- paste0("bootRep", 1:REP) 
  
  dat[, c(bootRep) := as.data.table(calc.replicate( # call function calc.replicate -> calculate bootstrap replicate for each repetition
    method = method,
    n = n.calc, 
    N = N.calc, 
    n_draw = n_draw.calc, 
    delta = delta.calc))] 
  
  if (single.PSU == "mean") {
    dat[!is.na(SINGLE_BOOT_FLAG_FINAL), c(bootRep) := lapply(  
      .SD,
      function(z) {
        mean(z, na.rm = TRUE)
      }
    ), by = SINGLE_BOOT_FLAG_FINAL, .SDcols = c(bootRep)]
  }
  
  setkey(dat, InitialOrder)  
  if (length(removeCols) > 0) {
    dat[, c(removeCols) := NULL]
  }
  
  if ("data" %in% return.value) {
    # get original values for PSUs and fpc - if singles PSUs have been detected and merged
    if (single.PSU == "merge") {  
      c.names <- colnames(dat)
      c.names <- c.names[grepl("_ORIGINALSINGLES", c.names)]
      if (length(c.names) > 0) {
        drop.names <- gsub("_ORIGINALSINGLES", "", c.names)
        dat[, c(drop.names) := NULL]
        setnames(dat, c(c.names), drop.names)
      }
    }
    dat[, c("InitialOrder") := NULL]  

    if (length(overwrite.names) > 0) {
      setnames(dat, overwrite.names.new, overwrite.names)
    }
    
    output_bootstrap <- list(data = dat)
  } else if ("replicates" %in% return.value) {
    output_bootstrap <- list(replicates = dat[, mget(bootRep)]) 
  }
  
  if("selection" %in% return.value){
    if(length(return.value)>1){
      output_bootstrap <- c(output_bootstrap, list(selection = delta_selection))
    }else{
      output_bootstrap <- list(selection = delta_selection)
    }
  }
  
  if(length(output_bootstrap) == 1){
    output_bootstrap <- output_bootstrap[[1]]
  }
  return(output_bootstrap)
}



# Helper Functions 

select.nstar <- function(n, N, f, n_prev, n_draw_prev, lambda_prev,
                         sum_prev = NULL, new.method) {
  if (n == 1) {
    return(1L)
  }
  
  if (!is.null(sum_prev)) {
    n_draw <- (sum_prev) ^ 2 / ((1 - (n / N)) * n_prev * f +
                                  (sum_prev) ^ 2) * n
    n_draw <- floor(n_draw)
  } else {
    if (new.method) {
      n_draw <- (n * n_draw_prev) / (n_prev * f * (1 - n / N) + n_draw_prev)
      n_draw <- floor(n_draw)
    } else {
      n_draw <- floor(n / 2)
    }
  }
  
  return(n_draw) 
}

# Sampling WITHOUT replacement (Preston method)
draw.without.replacement <- function(n, n_draw, delta = NULL) {   
  
  # if no units have been selected prior
  if(is.null(delta)){
    delta <- rep(c(1.0, 0.0), c(n_draw, n - n_draw)) 
    if (length(delta) > 1) {
      delta <- sample(delta)
    }
  }else{
    # if units have already been selected
    n_selected <- sum(delta,na.rm=TRUE)  
    delta_new_unit <- is.na(delta) 
    
    # check if only 1 unit needs to be selected
    if(sum(delta_new_unit)==1){
      if(n_selected<n_draw){
        delta <- set.random2NA(delta,value2NA = 0)  
      }else{
        delta <- set.random2NA(delta,value2NA = 1) 
      }
      
      n_selected <- sum(delta,na.rm=TRUE)
      delta_new_unit <- is.na(delta)
    }
    
    if(n_draw<=n_selected){
      add1 <- as.numeric(sum(delta_new_unit)>1) 
      nChanges <- n_selected - n_draw + add1
      
      if(nChanges == sum(delta_new_unit==FALSE)){ 
        delta <- set.random2NA(delta, value2NA=1,
                               nChanges = n_selected - n_draw + add1) 
      }else{
        delta <- change.random.value(delta, changeVal=1,
                                     nChanges = n_selected - n_draw + add1)
      }
      delta_new_unit <- is.na(delta)
      n_selected <- sum(delta,na.rm=TRUE)
    }
    
    if((n_draw-n_selected) >= sum(delta_new_unit)){
      add1 <- as.numeric(sum(delta_new_unit)>1)
      nChanges <- n_draw-n_selected-sum(delta_new_unit) + add1
      delta <- change.random.value(delta, changeVal=0,
                                   nChanges = nChanges)
      n_selected <- sum(delta,na.rm=TRUE)
      delta_new_unit <- is.na(delta)
    }
    
    n_draw <- n_draw - n_selected
    delta_rest <- rep(c(1.0, 0.0), c(n_draw, sum(delta_new_unit)-n_draw))
    if (length(delta_rest) > 1) {
      delta_rest <- sample(delta_rest)
    }
    delta[delta_new_unit==TRUE] <- delta_rest
    
  }
  return(delta)
}


# Sampling WITH replacement (Rao-Wu method)
draw.with.replacement <- function(n, n_draw, delta = NULL) { 
  
  if (is.null(delta)) {
    delta <- rep(0, n)  # Set all units to 0 (not yet drawn)
    delta[is.na(delta)] <- 0
  }
  delta <- as.numeric(delta)
  
  if (is.na(n_draw) || n_draw > n) {
    stop("Error: Invalid n_draw value.")
  }
  
  n_selected <- sum(delta, na.rm = TRUE)
  
  # 1. draw additional units if less than n_draw are selected
  if (n_selected < n_draw) {
    n_needed <- n_draw - n_selected
    additional_draws <- sample(1:n, size = n_needed, replace = TRUE)
    
    for (i in additional_draws) {
      if (is.na(delta[i])) {
        delta[i] <- 1  
      } else {
        delta[i] <- delta[i] + 1  
      }
    }
  }
  
  # 2 Reduce excess units if more than n_draw are selected
  if (n_selected > n_draw) {
    over_selected <- n_selected - n_draw

    if (over_selected > 0) {
      indices <- which(delta > 0)
      if (length(indices) >= over_selected) {
        delta[indices[1:over_selected]] <- delta[indices[1:over_selected]] - 1
      } else {
        stop("Error: Too many units to remove from delta.")
      }
    }
  }
  
  # 3. handle NA values in delta, if available
  if (any(is.na(delta))) {
    delta[is.na(delta)] <- 0   
  }
  
  return(as.numeric(delta))
}


change.random.value <- function(delta,changeVal=0,nChanges=1){
  
  if(nChanges == 0){
    # dont change anything
    return(delta)
  }
  
  # change a 0 -> 1 or 1 -> 0
  changeVal2 <- fifelse(changeVal==0,1,0) 
  set2NA <- which(delta==changeVal & !is.na(delta))
  if(length(set2NA)>1){
    set2NA <- sample(set2NA,min(nChanges,length(set2NA)))
  }
  delta[set2NA] <- changeVal2
  return(delta)
}



set.random2NA <- function(delta,value2NA=0,nChanges=1){
  
  set2NA <- which(delta==value2NA & !is.na(delta))
  if(length(set2NA)>1){
    set2NA <- sample(set2NA,min(nChanges,length(set2NA)))
  }
  delta[set2NA] <- NA
  return(delta)
}


calc.replicate <- function(n, N, n_draw, delta , method = "Preston") {
  
  p <- ncol(n)  
  dimdelta <- dim(delta)
  
  for (i in 1:p) {
    if (method == "Preston") {
      if (i == 1) { # 
        lambda <- sqrt(n_draw[, 1] *
                         (1 - n[, 1] / N[, 1]) / (n[, 1] - n_draw[, 1]))
        rep_out <- 1 - lambda + lambda * n[, i] / n_draw[, i] * delta[, i, ] 
      } else if (i == 2) {
        lambda <- (1 - n[, i] / N[, i]) / (n[, i] - n_draw[, i])
        lambda <- sqrt((n[, i - 1] / N[, i - 1]) * n_draw[, i] * lambda)
        rep_out <- rep_out + lambda *                                       
          (sqrt(n[, i - 1] / n_draw[, i - 1]) * delta[, i - 1, ]) *
          (n[, i] / n_draw[, i] * delta[, i, ] - 1)
      } else {
        lambda <- (1 - n[, i] / N[, i]) / (n[, i] - n_draw[, i])
        lambda <- sqrt(rowProds(n[, 1:(i - 1)] / N[, 1:(i - 1)]) *
                         n_draw[, i] * lambda)
        prod_val <- matrix(0, ncol = dimdelta[3], nrow = dimdelta[1])
        for (r in 1:dimdelta[3]) {
          prod_val[, r] <- rowProds(sqrt(n[, 1:(i - 1)] / n_draw[, 1:(i - 1)]) *
                                      delta[, 1:(i - 1), r])
          
        }
        
        rep_out <- rep_out + lambda * prod_val * (n[, i] / n_draw[, i] *      
                                                    delta[, i, ] - 1)
      }
      
    } else if (method == "Rao-Wu") {
      
      n_h <- n[, i]           
      m_h <- n_h - 1        
      f_h <- n_h / N[, i]     
      
      if (any(f_h > 0.1)) {
        warning("Sampling Fraction is too big for method = 'Rao-Wu', choose method = 'Preston' instead")
      }
      
      w_hi <- N[, i] / n_h 
      
      # Scaling factor λ determines the variance correction
      lambda <- sqrt(m_h * (1 - f_h) / (n_h - 1))
      
      # r_hi_star: Create replicas by drawing (n_h - 1) units without putting them back
      r_hi_star <- delta[, i, ]
      w_hi_star <- (1 - lambda + lambda * (n_h / m_h) * r_hi_star) #* w_hi
      
      if (i == 1) {
        rep_out <- w_hi_star
      } else {
        rep_out <- rep_out * w_hi_star  
      }
      
    }
  }
  return(rep_out)
}


next_minimum <- function(N, by) {
  N_notOne <- N != 1
  by <- by[N_notOne][which.min(N[N_notOne])]
  if (length(by) > 1) {
    by <- sample(by, 1)
  }
  return(by)
}

#################################
# test rescaled.bootstrap()
#

context("rescaled.bootstrap()")
library(surveysd)
library(laeken)
library(data.table)

data("eusilc")
setDT(eusilc)
eusilc[, N.households := sum(db090[!duplicated(db030)]), by = db040]
eusilc[!duplicated(db030), N.households.error := sum(db090), by = db040]
eusilc[, N.households.all := sum(db090[!duplicated(db030)])]

# test input parameter
test_that("test para - data", {
  expect_error(
    rescaled.bootstrap(
      as.matrix(eusilc), REP = "a", strata = "db040", cluster = "db030",
      fpc = "N.households"),
    "dat needs to be a data frame or data table")
  expect_error(rescaled.bootstrap(
    eusilc, REP = 3, strata = "db040", cluster = "db030", fpc = "N.households"),
    NA)
})

test_that("test para - REP", {
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = "a", strata = "db040", cluster = "db030",
      fpc = "N.households"),
    "'REP' must be of type numeric")
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = c(1:3), strata = "db040", cluster = "db030",
      fpc = "N.households"),
    "'REP' must have length 1")
})

test_that("test para - strata, cluster and fpc", {
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = 2, strata = "db0400", cluster = "db030",
      fpc = "N.households"),
    "dat does not contain the column(s): db0400", fixed = TRUE)
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = 2, strata = "db040>db030", cluster = "db030",
      fpc = "N.households"),
    paste0("strata, cluster, and fpc need to have the same number ",
           "of arguments either separated with '>' or ", 
           "be character vectors of the same length"))
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = 2, strata = "db040", cluster = "10", fpc = "N.households"),
    "dat does not contain the column(s): 10", fixed = TRUE)
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = 2, strata = "db040", cluster = "db040>db030",
      fpc = "N.households"),
    paste0("strata, cluster, and fpc need to have the same number ",
           "of arguments either separated with '>' or ", 
           "be character vectors of the same length"))
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = 2, strata = "db040>db030", cluster = "db040>db030",
      fpc = "N.households>N.something"),
    "dat does not contain the column(s): N.something", fixed = TRUE)
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = 2, strata = "I", cluster = "db030", fpc = "N.households"),
    paste0("values in N.households do vary in some strata-cluster ",
           "combinations at sampling stage 1"))
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = 2, strata = "1", cluster = "db030", fpc = "N.households"),
    paste0("values in N.households do vary in some strata-cluster ",
           "combinations at sampling stage 1"))
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = 2, strata = "db040", cluster = "db030",
      fpc = "N.households.error"),
    "missing values not allowed in fpc")

  expect_error(rescaled.bootstrap(
    eusilc, REP = 2, strata = "db040", cluster = "I", fpc = "N.households"), NA)
  expect_error(rescaled.bootstrap(
    eusilc, REP = 2, strata = "db040", cluster = "1", fpc = "N.households"), NA)
  expect_error(rescaled.bootstrap(
    eusilc, REP = 2, strata = "1", cluster = "db030", fpc = "N.households.all"),
    NA)
  expect_error(rescaled.bootstrap(
    eusilc, REP = 2, strata = "I", cluster = "db030", fpc = "N.households.all"),
    NA)
})


test_that("test para - run.input.checks", {
  expect_error(rescaled.bootstrap(
    eusilc, REP = 2, strata = "db040", cluster = "db030", fpc = "N.households",
    run.input.checks = FALSE), NA)
  expect_error(
    rescaled.bootstrap(
      eusilc, REP = 2, strata = "db040", cluster = "db030", fpc =
        "N.households", run.input.checks = "FALSE"),
    "run.input.checks can only be logical")
})


test_that("test para - single.PSU", {
  expect_warning(
    rescaled.bootstrap(
      eusilc, REP = 2, strata = "db040", cluster = "db030", fpc =
        "N.households", single.PSU = "something"),
    paste0("single.PSU was not set to either 'merge' or 'mean'!\n ",
           "Bootstrap replicates for single PSUs cases will be missing!"),
    fixed = TRUE)
})

test_that("test return", {
  dat.boot <- rescaled.bootstrap(eusilc, REP = 2, strata = "db040", cluster =
                                   "db030", fpc = "N.households")
  expect_true(ncol(dat.boot) == (2 + ncol(eusilc)))
  dat.unique <- unique(dat.boot[, mget(c("db030", paste0("bootRep", 1:2)))])
  expect_true(nrow(dat.unique[, .N, by = db030][N > 2]) == 0)
  expect_false(any(unlist(dat.boot[, lapply(
    .SD,
    function(z) {
      any(is.infinite(z))
    }
  ), .SDcols = c(paste0("bootRep", 1:2))])))
  expect_false(any(is.na(dat.boot[, .SD, .SDcols = c(paste0("bootRep", 1:2))])))
  expect_true(all(dat.boot[, .SD, .SDcols = c(paste0("bootRep", 1:2))] > 0))

  dat.boot <- rescaled.bootstrap(
    eusilc, REP = 2, strata = "db040", cluster = "db030", fpc = "N.households",
    return.value = "replicates")
  expect_true(ncol(dat.boot) == 2)
})

#' @title Print function for surveysd objects
#'
#' @description
#' Prints the results of a call to [calc.stError]. Shows used variables and
#' function, number of point estiamtes
#' as well as properties of the results.
#'
#' @param x an object of class `'surveysd'`
#' @param ... additonal parameters
#'
#' @export
print.surveysd <- function(x, ...) {
  
  var <- NULL
  
  # get parameter
  col.val <- grepl("^val_*.", colnames(x[["Estimates"]]))
  col.val <- colnames(x[["Estimates"]])[col.val]
  # get number of estimates ~ variables in number of groups using function fun
  #  from package pack
  n.estimates <- nrow(x[["Estimates"]]) * length(col.val)
  # get number of subgroup which fall below size
  n.periods <- unique(x[["Estimates"]][[x[["param"]][["period"]]]])
  n.periods <- length(n.periods[!grepl("-|_", n.periods)])

  if (!is.null(unlist(x[["param"]][["group"]]))) {
    n.groups <- nrow(unique(
      x[["Estimates"]][, .N, by = c(unique(unlist(x[["param"]][["group"]])))]
    ))
  } else {
    n.groups <- NULL
  }

  # get number of missing values in the output due to subgroups with no
  #   observations or subgroups with all observations equal NA
  n.NAs <- sum(is.na(x[["Estimates"]][, mget(col.val)]))

  # print number of estimates calculated as well as functino and variables used
  cat("Calculated point estimates for variable(s)\n\n",
      paste(x[["param"]][["var"]], sep = ","), "\n\n")

  if (!is.null(n.groups)) {
    cat("Results hold", n.estimates, "point estimates for", n.periods,
        "periods in", n.groups, "subgroups\n")
  } else {
    cat("Results hold", n.estimates, "point estimates for", n.periods,
        "periods\n")
  }
  cat("\n")
  if (nrow(x[["smallGroups"]]) != 0) {
    if (nrow(x[["smallGroups"]]) > 10) {
      cat(nrow(x[["smallGroups"]]), "subgroups contained less than",
          x[["param"]][["size.limit"]], "observations\n")
    } else {
      cat("Subgroups with less than", x[["param"]][["size.limit"]],
          "observations\n")
      print(x[["smallGroups"]])
    }
    cat("\n")
  }


  # print number of missing values for point estimates
  if (n.NAs > 0) {
    cat("Point estimates are NAs in", n.NAs, "cases due to either\n no ",
        "observations or only NAs for the variable(s) in the corresponding",
        " subgroups.\n")
    cat("\n")
  }


  # get number of point estimates where sd exceeds cv.limit
  stEtoohigh <- as.matrix(subset(x[["cvHigh"]], select = var))
  if (sum(stEtoohigh, na.rm = TRUE) > 0) {
    cat("Estimated standard error exceeds", x[["param"]][["cv.limit"]],
        "% of the the point estimate in", sum(stEtoohigh, na.rm = TRUE),
        "cases (approx.",round(mean(stEtoohigh, na.rm = TRUE)*100,2),"%)\n")
    cat("\n")
  }
}

#' Print method for IPF calibration summary
#'
#' Provides a concise summary of an IPF (Iterative Proportional Fitting) calibration summary object. 
#' It extracts the calibration weight from `all_formulas`, computes the Kish factor for the weights, 
#' and prints the first 10 rows of any `calib_results_` tables. 
#' Useful for a quick overview of calibration results. Additional details can be explored with `str()` or `names()`.
#'
#' @param x An object of class \code{summary.ipf}, as returned by \code{\link{summary.ipf}}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return The input object \code{x}, invisibly (for chaining).
#' @method print summary.ipf
#' @export
#' @importFrom surveysd kishFactor


print.summary.ipf <- function(x, ...) {
  cat("IPF Calibration Summary (Short Version)\n")
  cat("=======================================\n\n")
  
  # --- 1. Automatically extract CalibWeight from all_formulas ---
  if (!is.null(x$all_formulas)) {
    calibWeightName <- trimws(strsplit(x$all_formulas[1], "~")[[1]][1])
  } else {
    calibWeightName <- NULL
  }
  
  # --- 2. Calculate Kish factor ---
  weight_candidates <- names(x[["weighted data"]])
  if (!is.null(calibWeightName) && calibWeightName %in% weight_candidates) {
    w <- x[["weighted data"]][[calibWeightName]]
    kish <- kishFactor(w, na.rm = TRUE)
    cat(sprintf("Kish Factor (based on '%s'): %.4f\n\n", calibWeightName, kish))
  } else {
    cat("Kish Factor: No valid weight column found\n\n")
  }
  
  # --- 3. Display calib_results_ tables (first 10 rows) ---
  calib_names <- grep("^calib_results_", names(x), value = TRUE)
  if (length(calib_names) > 0) {
    cat("Calibration Results (first 10 rows per table):\n")
    for (nm in calib_names) {
      cat("\n---", nm, "---\n")
      print(utils::head(x[[nm]], 10))
    }
  } else {
    cat("No calib_results_ tables found.\n")
  }
  
  cat("\n(See more details with str() or names())\n")
  invisible(x)
}

#' Numerical weighting functions
#'
#' Customize weight-updating within factor levels in case of numerical
#' calibration. The functions described here serve as inputs for [ipf].
#'
#' `computeFrac` provides the "standard" IPU updating scheme given as
#'
#' \deqn{f = target/curValue}
#'
#' which means that each weight inside the level will be multtiplied by the same
#' factor when doing the actual update step (`w := f*w`). `computeLinear` on the
#' other hand calculates `f` as
#'
#' \ifelse{html}{
#'   \out{<div style="text-align: center;"> f<sub>i</sub> = a  &middot; x<sub>i</sub> + b </div>}
#' }{\deqn{f_i = ax_i+b}}
#'
#' where `a` and `b` are chosen, so f satisfies the following two equations.
#'
#' \ifelse{html}{
#'   \out{<div style="text-align: center;">&sum; f<sub>i</sub> w<sub>i</sub>
#'        x<sub>i</sub> = target</div>}
#' }{\deqn{\sum f_i * w_i * x_i = target}}
#' \ifelse{html}{
#'   \out{<div style="text-align: center;">&sum; f<sub>i</sub>
#'        w<sub>i</sub> = &sum; w<sub>i</sub></div>}
#' }{\deqn{\sum f_i * w_i = \sum w_i}}
#'
# \eqn{\sum}\out{f<sub>i</sub> w<sub>i</sub> x<sub>i</sub>} = `target`
#' `computeLinearG1` calculates `f` in the same way as `computeLinear`, but if
#'`f_i*w_i<1` `f_i` will be set to `1/w_i`.
#'
#' @md
#' @param curValue Current summed up value. Same as `sum(x*w)`
#' @param target Target value. An element of `conP` in [ipf]
#' @param x Vector of numeric values to be calibrated against
#' @param w Vector of weights
#' @param boundLinear The output `f` will satisfy
#'   `1/boundLinear <= f <= boundLinear`. See `bound` in [ipf]
#'
#' @return A weight multiplier `f`
#'
#' @aliases numericalWeighting
#' @export computeFrac
computeFrac <- function(curValue, target, x, w) {
  target / curValue
}

#'
#' Kish Factor
#'
#' Compute the design effect due to unequal weighting.
#'
#' @name kishFactor
#' @param w a numeric vector with weights
#' @param na.rm a logical value indicating whether NA values should be stripped before the computation proceeds.
#' @return The function will return the the kish factor
#' @author Alexander Kowarik
#' @export kishFactor
#' @details The factor is computed acording to 'Weighting for Unequal P_i', Leslie Kish, Journal of Official Statistics, Vol. 8. No. 2, 1992
#' \deqn{ deff = \sqrt n \sum_j w_j^2 / (\sum_j w_j)^2}

#'
#' @examples
#' kishFactor(rep(1,10))
#' kishFactor(rlnorm(10))
kishFactor <- function(w, na.rm = FALSE) {
  if (!is.numeric(w)) {
    stop("The input must be a numeric vector")
  }
  if(na.rm){
    w <- na.omit(w)
  }
  n <- length(w)
  sqrt(n * sum(w ^ 2) / sum(w) ^ 2)
}

#' @title Internal function: Generate summary output for an IPF calibration
#'
#' @description
#' This function is used internally by \code{summary.ipf} to generate detailed
#' tables for calibration diagnostics.
#'
#' @param ipf_result An object of class \code{ipf} containing the calibration results.
#' @param av A list of auxiliary information used in calibration, including
#' constraints (\code{conP}, \code{conH}) and tolerance values (\code{epsP}, \code{epsH}).
#'
#' @return A named list of data.tables containing calibration diagnostics for
#' each constraint.
#'
#' @keywords internal
#' @noRd

ipf_summary_calibres <- function(ipf_result, av) {
  
  # 1. INITIAL SETUP
  . <- calibWeightName <- wg <- N <- CalibMargin <- PopMargin <- epsP <- epsH <- maxFac <- NULL
  
  if (any(names(av) == "conP")) {
    formula_name <- as.character(av$formP[[1]])[2]
    if (formula_name %in% names(ipf_result)) calibWeightName <- formula_name
  }
  
  if (is.null(calibWeightName) && any(names(av) == "conH")) {
    formula_name <- as.character(av$formH[[1]])[2]
    if (formula_name %in% names(ipf_result)) calibWeightName <- formula_name
  }
  
  if (is.null(calibWeightName)) {
    if ("gew2" %in% names(ipf_result)) {
      calibWeightName <- "gew2"
    } else if ("gew1" %in% names(ipf_result)) {
      calibWeightName <- "gew1"
    } else {
      stop("Could not determine the name of the calibrated weight column.")
    }
  }
  
  has_eps_values <- !is.null(av$epsP) && !is.null(av$epsH)
  if (!has_eps_values) cat("Warning: Epsilon values not found. Skipping maxFac calculations.\n")
  
  results_list <- list()
  
  # 2. PROCESSING PERSON CONSTRAINTS (conP)
  conP_list <- av$conP
  
  for (i in seq_along(conP_list)) {
    
    constraint_name <- names(conP_list)[i]
    if (is.null(constraint_name) || nchar(constraint_name) == 0) constraint_name <- paste0("conP", i)
    
    group_vars <- names(attr(conP_list[[i]], "dimnames"))
    
    tmp_calib_data <- copy(ipf_result)
    tmp_calib_data[, (group_vars) := lapply(.SD, as.character), .SDcols = group_vars]
    
    calib_results <- tmp_calib_data[
      , .(
        N = .N,
        CalibMargin = sum(get(calibWeightName))
      ),
      by = group_vars
    ]
    
    original_dt <- as.data.table(conP_list[[i]])
    setnames(original_dt, names(original_dt), c(group_vars, "PopMargin"))
    original_dt[, (group_vars) := lapply(.SD, as.character), .SDcols = group_vars]
    
    merged_results <- merge(calib_results, original_dt, by = group_vars, all.x = TRUE)
    merged_results[is.na(N), N := 0]
    merged_results[is.na(CalibMargin), CalibMargin := 0]
    
    # --- GEÄNDERT: EPSP Handling ---
    if (!is.null(av$epsP)) {
      epsP_obj <- if (is.list(av$epsP)) av$epsP[[i]] else av$epsP
      if (length(epsP_obj) == 1) {
        merged_results[, epsP := as.numeric(epsP_obj)]
      } else {
        # EPSP-Matrix wird "aufgelöst" in long-Format
        epsP_dt_long <- as.data.table(as.table(as.array(epsP_obj)))
        # Merge nur nach tatsächlich vorhandenen Variablen → verhindert NA
        merge_vars <- intersect(names(epsP_dt_long), group_vars)
        setnames(epsP_dt_long, c(merge_vars, "epsP"))
        epsP_dt_long[, (merge_vars) := lapply(.SD, as.character), .SDcols = merge_vars]
        merged_results <- merge(merged_results, epsP_dt_long, by = merge_vars, all.x = TRUE)
      }
    } else {
      merged_results[, epsP := NA_real_]
    }
    
    merged_results[, maxFac := abs(1 - CalibMargin / PopMargin)]
    merged_results <- merged_results[order(is.na(PopMargin), N)]
    results_list[[constraint_name]] <- merged_results[N > 0]
    
  }
  
  # 3. PROCESSING HOUSEHOLD CONSTRAINTS (conH)
  conH_list <- av$conH
  
  for (i in seq_along(conH_list)) {
    
    constraint_name <- names(conH_list)[i]
    if (is.null(constraint_name) || nchar(constraint_name) == 0) constraint_name <- paste0("conH", i)
    
    group_vars <- names(attr(conH_list[[i]], "dimnames"))
    
    tmp_calib_data <- copy(ipf_result)
    tmp_calib_data[, (group_vars) := lapply(.SD, as.character), .SDcols = group_vars]
    
    hid <- attr(ipf_result, "hid") 
    if (!is.null(hid) && hid %in% names(tmp_calib_data)) {
      tmp_calib_data[, `:=`(wg, .N), by = hid]
    } else {
      tmp_calib_data[, `:=`(wg, 1L)]
    }
    
    calib_results <- tmp_calib_data[
      , .(
        N = .N,
        CalibMargin = sum(get(calibWeightName) / as.numeric(wg))
      ),
      by = group_vars
    ]
    
    original_dt <- as.data.table(conH_list[[i]])
    setnames(original_dt, names(original_dt), c(group_vars, "PopMargin"))
    original_dt[, (group_vars) := lapply(.SD, as.character), .SDcols = group_vars]
    
    merged_results <- merge(calib_results, original_dt, by = group_vars, all.x = TRUE)
    merged_results[is.na(N), N := 0]
    merged_results[is.na(CalibMargin), CalibMargin := 0]
    
    # --- GEÄNDERT: EPSH Handling ---
    # Ziel: auch "teilweise aufgelöste" EPSH-Matrizen korrekt zuordnen
    if (!is.null(av$epsH)) {
      epsH_obj <- if (is.list(av$epsH)) av$epsH[[i]] else av$epsH
      if (length(epsH_obj) == 1) {
        merged_results[, epsH := as.numeric(epsH_obj)]
      } else {
        epsH_dt_long <- as.data.table(as.table(as.array(epsH_obj)))
        merge_vars <- intersect(names(epsH_dt_long), group_vars)
        setnames(epsH_dt_long, c(merge_vars, "epsH"))
        epsH_dt_long[, (merge_vars) := lapply(.SD, as.character), .SDcols = merge_vars]
        merged_results <- merge(merged_results, epsH_dt_long, by = merge_vars, all.x = TRUE)
      }
    } else {
      merged_results[, epsH := NA_real_]
    }
    
    merged_results[, maxFac := abs(1 - CalibMargin / PopMargin)]
    merged_results <- merged_results[order(is.na(PopMargin), N)]
    results_list[[constraint_name]] <- merged_results[N > 0]
  }
  
  return(results_list)
}




#' @title Generate Summary Output for IPF Calibration
#'
#' @description Generates a detailed summary of an Iterative Proportional Fitting (IPF) calibration, providing a complete tool for evaluating the calibration's success and the validity of the resulting weights.
#'
#' The output is a list of data.tables for a comprehensive evaluation, including:
#'
#' **Calibration Results:**
#'   - `calib_results_conP_*` and `calib_results_conH_*`: Key diagnostic tables that compare calibrated margins to population targets and assess the goodness of fit via metrics like `maxFac`.
#'
#' **Data and Diagnostics:**
#'   - `weighted data`: An excerpt of the final dataset with the calculated calibration weights.
#'   - `distribution of the weights`: A statistical overview of the weight distribution (min, max, CV).
#'
#' **Detailed Margin Comparisons:**
#'   - `conP_*`, `conH_*`, `*_adjusted`, `*_original`, `*_rel_diff_*`: Tables that compare original sample margins, calibrated margins, and population targets, along with their relative differences.
#'
#' @param object object of class ipf
#' @param ... additional arguments
#'
#' @return a list of the following outputs
#' @export
#'
#' @examples
#'
#' \dontrun{
#' # load data
#' eusilc <- demo.eusilc(n = 1, prettyNames = TRUE)
#'
#' # personal constraints
#' conP1 <- xtabs(pWeight ~ age, data = eusilc)
#' conP2 <- xtabs(pWeight ~ gender + region, data = eusilc)
#' conP3 <- xtabs(pWeight*eqIncome ~ gender, data = eusilc)
#'
#' # household constraints
#' conH1 <- xtabs(pWeight ~ hsize + region, data = eusilc)
#'
#' # simple usage ------------------------------------------
#'
#' calibweights1 <- ipf(
#'  eusilc,
#'  conP = list(conP1, conP2, eqIncome = conP3),
#'  bound = NULL,
#'  verbose = TRUE
#' )
#' output <- summary(calibweights1)
#' # the output can easily be exported to an Excel file, e.g. with
#' # library(openxlsx)
#' # write.xlsx(output, "SummaryIPF.xlsx")
#' }

summary.ipf <- function(object, ...){
  terms <- variable <- NULL
  dots <- list(...)
  extraExport <- dots$extraExport
  av <- attributes(object)
  w <- av$baseweight
  hid <- av$hid
  
  
  ## ---- FormP/FormH - Variables etc. ---- ##
  vars <- c()
  formulas <- c()
  if(any(names(av) == "conP")){
    calibWeightName <- as.character(av$formP[1][[1]])[2]
    varsP_help <- c()
    for(i in seq_along(av$formP)){
      varsP_help <- c(varsP_help,labels(terms(av$formP[[i]])))
    }
    vars <- c(vars,unique(varsP_help))
    
    #
    formPBase <- lapply(av$formP,function(x){
      x <- as.character(x)
      as.formula(paste(w,"~",x[3]))
    })
    #
    formulas <- c(formulas,as.character(av$formP))
    
  }
  
  if(any(names(av) == "conH")){
    calibWeightName <- as.character(av$formH[1][[1]])[2]
    varsH_help <- c()
    for(i in seq_along(av$formH)){
      varsH_help <- c(varsH_help,labels(terms(av$formH[[i]])))
    }
    vars <- c(vars,unique(varsH_help))
    
    #
    formHBase <- lapply(av$formH,function(x){
      x <- as.character(x)
      as.formula(paste(w,"~",x[3]))
    })
    #
    formulas <- c(formulas,as.character(av$formH))
    
  }
  vars <- as.list(unique(vars))
  
  dist_gew <- list()
  if(!is.null(w)){
    for(i in vars){
      part_i <- object[,list(cv=sd(get(w))/mean(get(w)),
                             min=min(get(w)),
                             max=max(get(w)),
                             quotient=min(get(w))/max(get(w))),keyby=c(i)]
      setnames(part_i,i,paste0("value",1:length(i)))
      part_i[,variable:=paste(i,collapse=" x ")]
      setcolorder(part_i,c("variable",paste0("value",1:length(i)),"cv","min","max","quotient"))
      dist_gew <- c(dist_gew,list(part_i))
    }
    dist_gew <- rbindlist(dist_gew,use.names=TRUE,fill=TRUE)
    setcolorder(dist_gew,c("variable",paste0("value",1:length(i)),"cv","min","max","quotient"))
  }
  
  
  cols <- c(which(colnames(object) %in% c(hid,vars,w,calibWeightName,extraExport)))
  
  # EXTENSION: The existing output is created here.
  original_output <- vector(mode = "list", length = 3)
  names(original_output)[1] <- "weighted data"
  original_output[[1]] <- subset(object,select=cols)
  
  formulaOutput <- data.table(Verteilungen=rep(NA,length(formulas)))
  formulaOutput$Verteilungen[seq_along(formulas)] <- formulas
  
  names(original_output)[2] <- "margin tables"
  original_output[[2]] <- formulaOutput
  
  names(original_output)[3] <- "distribution of the weights"
  original_output[[3]] <- dist_gew
  
  #### ---- ####
  if (any(names(av) == "conP")) {
    for (i in seq_along(av$conP)) {
      output4 <- list(
        "conP_%i" = as.data.table(av$conP[[i]]),
        "conP_%i_adjusted" = as.data.table(av$conP_adj[[i]]),
        "conP_%i_original" = as.data.table(xtabs(formPBase[[i]], data = object)),
        "conP_%i_rel_diff_original" = as.data.table(round(
          100*(av$conP[[i]] - xtabs(formPBase[[i]], data = object)) / av$conP[[i]], 2)),
        "conP_%i_rel_diff_calib" = as.data.table(round(
          100*(av$conP[[i]] - av$conP_adj[[i]]) / av$conP[[i]], 2))
      )
      names(output4) <- sprintf(names(output4), i)
      original_output <- append(original_output, output4)
    }
  }
  
  
  if(any(names(av) == "conH")){
    output5 <- vector(mode = "list", length = length(av$conH)*5)
    for(i in seq_along(av$conH)){
      j <- ((i-1)*5)+1
      names(output5)[(j):(j+4)] <- c(paste0("conH_",i),paste0("conH_",i,"_adjusted"),paste0("conH_",i,"_original"),
                                     paste0("conH_",i,"_rel_diff_original"),paste0("conH_",i,"_rel_diff_calib"))
      output5[[j]] <- as.data.table(av$conH[[i]]) #conH_i
      output5[[j+1]] <- as.data.table(av$conH_adj[[i]]) #conH_i_adjusted
      output5[[j+2]] <- as.data.table(xtabs(formHBase[[i]],data=object)) #conH_i_original
      output5[[j+3]] <- as.data.table(round(100*(av$conH[[i]]-xtabs(formHBase[[i]],data=object))/av$conH[[i]],2)) #conH_i_rel_diff_original
      output5[[j+4]] <- as.data.table(round(100*(av$conH[[i]]-av$conH_adj[[i]])/av$conH[[i]],2)) #conH_i__rel_diff_calib
      
    }
    original_output <- append(original_output,output5)
  }
  
  # --- Start of new table integration ---
  # Calls the generate_ipf_summary function to create the initial tables.
  custom_summary <- ipf_summary_calibres(object, av)
  
  # Renames the list appropriately to distinguish it from the other tables.
  names(custom_summary) <- paste0("calib_results_", names(custom_summary))
  
  # Initializes the final output with the new tables.
  output <- custom_summary
  # --- End of integration ---
  
  # Appends the original tables to the custom tables.
  output <- append(output, original_output)
  
  # ---- NEW: Append all formulas as the last element ----
  output[["all_formulas"]] <- formulas
  
  class(output) <- "summary.ipf"
  
  return(output)
}

#' @rdname ipf_step
#' @name ipf_step
#' @md
#'
#' @param dat a `data.frame` containing the factor variables to be combined.
#'
#' @export
#' 

combine_factors <- function(dat, targets) {

  x <- as.data.frame(targets)
  x$ID_ipu <- seq_len(nrow(x))
  x <- merge(dat, x, by = names(dimnames(targets)), sort = FALSE, all.x = TRUE)
  factor(x$ID_ipu, levels = seq_along(targets))
}

getMeanFun <- function(meanHH) {
  if (isTRUE(meanHH))
    meanHH <- "arithmetic"
  if (identical(meanHH, FALSE))
    meanHH <- "none"
  meanfun <- switch(meanHH,
                    arithmetic = arithmetic_mean,
                    geometric = geometric_mean,
                    none = function(x, w) {
                      x
                    }
  )
  if (is.null(meanfun))
    stop("invalid value for meanHH")
  meanfun
}
boundsFak <- function(g1, g0, f, bound = 4, minMaxTrim=NULL) {
  # Berechnet die neuen Gewichte (innerhalb 4, .25 Veraenderungsraten)
  g1 <- g1 * f
  if(!is.null(bound)){
    TF <- which((g1 / g0) > bound)
    TF[is.na(TF)] <- FALSE
    g1[TF] <- bound * g0[TF]
    TF <- which((g1 / g0) < (1 / bound))
    TF[is.na(TF)] <- FALSE
    g1[TF] <- (1 / bound) * g0[TF]
  }
  if(!is.null(minMaxTrim)){
    g1[g1<minMaxTrim[1]] <- minMaxTrim[1]
    g1[g1>minMaxTrim[2]] <- minMaxTrim[2]
  }
  return(g1)
}
boundsFakHH <- function(g1, g0, eps, orig, p, bound = 4, minMaxTrim=NULL) {
  # Berechnet die neuen Gewichte fuer Unter- und Obergrenze (innerhalb 4,
  #   .25 Veraenderungsraten)
  u <- orig * (1 - eps)
  o <- orig * (1 + eps)

  pbo <- which(p > o)
  psu <- which(p < u)
  g1[pbo] <- g1[pbo] * o[pbo] / p[pbo]
  g1[psu] <- g1[psu] * u[psu] / p[psu]
  if(!is.null(bound)){
    TF <- which((g1 / g0) > bound)
    g1[TF] <- bound * g0[TF]
    TF <- which((g1 / g0) < (1 / bound))
    g1[TF] <- (1 / bound) * g0[TF]
  }
  if(!is.null(minMaxTrim)){
    g1[g1<minMaxTrim[1]] <- minMaxTrim[1]
    g1[g1>minMaxTrim[2]] <- minMaxTrim[2]
  }
  return(g1)
}

check_eps <- function(con, eps, type){
  l <- length(con)
  if(is.list(eps)){
    if(length(eps)!=l){
      stop(paste("Provided",type,"eps argument does not fit",type,"constraints."))
    }
    for(i in 1:length(eps)){
      if(is.array(eps[[i]])){
        if(!identical(dim(eps[[i]]),dim(con[[i]]))){
          stop(paste("Provided",type,"eps argument",i,"does not fit in dimension to ",type,"constraints",i,"."))
        }
      }
    }
  }else if(length(eps)>1){
    stop("Individual eps arguments for each constraints must be defined as list.")
  }
}

check_population_totals <- function(con, dat, type = "personal") {
  # do not apply this check for numerical calibration
  if (is.null(names(con))) {
    ind <- seq_along(con)
  } else {
    ind <- which(names(con) == "")
  }

  # do not apply this check for constraints that only cover the population
  #   partially
  ind <- ind[vapply(
    ind,
    function(i) {
      constraint <- con[[i]]
      for (variable in names(dimnames(constraint))) {
        if (!(variable %in% names(dat))){
          stop("variable ", variable, " appears in a constraint but not ",
               "in the dataset")
        }
        vals_dat <- sort(unique(as.character(dat[[variable]])))
        vals_con <- sort(dimnames(constraint)[[variable]])
        if (any(!vals_dat%in%vals_con)) {
          message(type, " constraint ", i, " only covers a subset of the ",
                  "population")
          return(FALSE)
        }
        if(any(!vals_con%in%vals_dat)){
          message(type, " constraint ", i, " covers more combinations of ",
                  paste(names(dimnames(constraint)),collapse=" and "),
                  " then appear in dat")
          return(FALSE)
        }
      }
      return(TRUE)
    },
    TRUE)]

  if (length(ind) == 0)
    return(NULL)

  pop_totals <- vapply(
    ind,
    function(index) {
      sum(con[[index]])
    },
    0)
  rel_errors <- abs(pop_totals - pop_totals[1]) / pop_totals[1]

  # use a 1% tolerance. Maybe it would be better to make the tolerance
  #   dependent on conH?
  if (any(rel_errors > 1e-2))
    stop("population totals for different constraints do not match")
}

calibP <- function(i, dat, error, valueP, pColNames, bound, verbose, calIter,
                   numericalWeighting, numericalWeightingVar, w,
                   cw, minMaxTrim, print_every_n) {
  selectGroupNotConverged <- epsPcur <- maxFac <- OriginalSortingVariable <- V1 <-
    epsvalue <- fVariableForCalibrationIPF <- NULL
  temporary_hvar <- value <-
    wValue <- representativeHouseholdForCalibration <- NULL
  variableKeepingTheBaseWeight <- w
  variableKeepingTheCalibWeight <- cw
  combined_factors <- dat[[paste0("combined_factors_", i)]]
  setnames(dat, valueP[i], "value")
  setnames(dat, paste0("epsP_", i), "epsPcur")
  tmp <- data.table(x = factor(levels(combined_factors)))
  setnames(tmp, "x", paste0("combined_factors_", i))
  con_current <- dat[tmp, on = paste0("combined_factors_", i),
                     mult = "first", value]

  if (!is.null(numericalWeightingVar)) {
    ## numerical variable to be calibrated
    ## use name of conP list element to define numerical variable
    set(dat, j = "fVariableForCalibrationIPF",
        value = ipf_step_f(dat[[variableKeepingTheCalibWeight]] *
                             dat[[numericalWeightingVar]],
                           combined_factors, con_current))
    set(dat, j = "wValue", value = dat[["value"]] /
          dat[["fVariableForCalibrationIPF"]])

    # try to divide the weight between units with larger/smaller values in the
    #   numerical variable linear
    set(dat, j="fVariableForCalibrationIPF", value=1)
    dat[!is.na(value) & value!=0, fVariableForCalibrationIPF := numericalWeighting(
      head(wValue, 1), head(value, 1), get(numericalWeightingVar),
      get(variableKeepingTheCalibWeight)),
      by=c(paste0("combined_factors_", i))]
  
    # adjust weights to hit target 
    # weights might shift due  to applying boundary limits in numerialWeighting()
    
    dat[!is.na(value) & value!=0, fVariableForCalibrationIPF := fVariableForCalibrationIPF*value/sum(fVariableForCalibrationIPF*get(numericalWeightingVar)*get(variableKeepingTheCalibWeight)), by=c(paste0("combined_factors_", i))]
    
    # # do this only if wValue is bigger than target
    # # dat[!is.na(value) & value!=0, value_too_large := value < sum(fVariableForCalibrationIPF*get(numericalWeightingVar)*get(variableKeepingTheCalibWeight)), by=c(paste0("combined_factors_", i))]
    # # dat[!is.na(value) & value!=0 & value_too_large == TRUE, fVariableForCalibrationIPF := fVariableForCalibrationIPF*value/sum(fVariableForCalibrationIPF*get(numericalWeightingVar)*get(variableKeepingTheCalibWeight)), by=c(paste0("combined_factors_", i))]
    
    # result after applying factor
    # dat[!is.na(value) & value!=0, wValue:=sum(fVariableForCalibrationIPF*get(variableKeepingTheCalibWeight)*get(numericalWeightingVar)), by = c(paste0("combined_factors_", i))]
  } else {
    # categorical variable to be calibrated
    set(dat, j = "fVariableForCalibrationIPF", value = ipf_step_f(
      dat[[variableKeepingTheCalibWeight]], combined_factors, con_current))
    set(dat, j = "wValue", value = dat[["value"]] /
          dat[["fVariableForCalibrationIPF"]])
  }
  
  
  dat[, selectGroupNotConverged := (abs(wValue-value)/value)>epsPcur]
  dat[is.na(selectGroupNotConverged),selectGroupNotConverged:=FALSE]
  
  if (dat[,any(selectGroupNotConverged)]) {
    ## sicherheitshalber abs(epsPcur)? Aber es wird schon niemand negative eps
    ##   Werte uebergeben??
    if (verbose && (calIter %% 10 == 0| calIter %% print_every_n == 0)) {
      message(calIter, ":Not yet converged for P-Constraint", i, "\n")
      if (calIter %% print_every_n == 0) {

        tmp <- dat[
          selectGroupNotConverged == TRUE,
          list(
            maxFac = max(abs(1 / fVariableForCalibrationIPF - 1)), .N,
            epsP = head(epsPcur, 1),
            CalibMargin = {
              if (!is.null(numericalWeightingVar)) {
                sum(get(variableKeepingTheCalibWeight) *
                      get(numericalWeightingVar))
              }else{
                sum(get(variableKeepingTheCalibWeight))
              }
            },
            PopMargin = head(value, 1)),
          by = eval(pColNames[[i]])]
       
        print(tmp[order(maxFac, decreasing = TRUE), ])
        message("-----------------------------------------\n")
      }
    }
    
    # variableKeepingTheCalibWeight_helper <- paste0(variableKeepingTheCalibWeight,"_helper")
    # dat[,c(variableKeepingTheCalibWeight_helper):=get(variableKeepingTheCalibWeight)]
    
    if (!is.null(bound) || !is.null(minMaxTrim)) {
      dat[!is.na(fVariableForCalibrationIPF),
          c(variableKeepingTheCalibWeight) :=
            boundsFak(
              get(variableKeepingTheCalibWeight),
              get(variableKeepingTheBaseWeight), fVariableForCalibrationIPF,
              bound = bound, minMaxTrim = minMaxTrim)]
      #,by=eval(pColNames[[i]])]
    } else {
      dat[!is.na(fVariableForCalibrationIPF),
          c(variableKeepingTheCalibWeight) := fVariableForCalibrationIPF *
            get(variableKeepingTheCalibWeight),
          by = eval(paste0("combined_factors_", i))]
    }
    
    # # accept new solution only if it gets closer to target -> computeLinearG1 can diverge in some cases
    # dat[,accept_new:=abs(sum(get(numericalWeightingVar)*get(variableKeepingTheCalibWeight_helper))-value)<
    #       abs(sum(get(numericalWeightingVar)*get(variableKeepingTheCalibWeight))-value),by=eval(paste0("combined_factors_", i))]
    # # dont accept new solution if groups which have already converged and where solution will get worse
    # dat[!(accept_new==FALSE & selectGroupNotConverged==TRUE),c(variableKeepingTheCalibWeight):=get(variableKeepingTheCalibWeight_helper)]
    # dat[,c(variableKeepingTheCalibWeight_helper,"accept_new"):=NULL]
    dat[,c("fVariableForCalibrationIPF"):=NULL]
    error <- TRUE
  }
  dat[,c("selectGroupNotConverged"):=NULL]
  setnames(dat, "value", valueP[i])
  setnames(dat, "epsPcur", paste0("epsP_", i))
  
  # print(dat[combined_factors_1==144,.(kz,calibWeight,fVariableForCalibrationIPF,calibWeight*fVariableForCalibrationIPF)])
  
  # return(list(error=error,dat=dat))
  return(error)
}

calibH <- function(i, dat, error, valueH, hColNames, bound, verbose, calIter,
                   looseH, numericalWeighting, numericalWeightingVar,
                   w, cw, minMaxTrim, print_every_n) {
  variableKeepingTheBaseWeight <- w
  variableKeepingTheCalibWeight <- cw
  selectGroupNotConverged <- epsHcur <- OriginalSortingVariable <- V1 <-
    epsvalue <- fVariableForCalibrationIPF <- NULL
  maxFac <- temporary_hvar <-
    value <- wValue <- representativeHouseholdForCalibration <- NULL

  setnames(dat, valueH[i], "value")
  setnames(dat, paste0("epsH_", i), "epsHcur")

  combined_factors <- dat[[paste0("combined_factors_h_", i)]]
  tmp <- data.table(x = factor(levels(combined_factors)))
  setnames(tmp, "x", paste0("combined_factors_h_", i))
  paste0("combined_factors_h_", i)
  con_current <- dat[tmp, on = paste0("combined_factors_h_", i),
                     mult = "first", value]
  if (!is.null(numericalWeightingVar)) {
    ## numerical variable to be calibrated
    ## use name of conH list element to define numerical variable
    set(dat, j = "fVariableForCalibrationIPF", value = ipf_step_f(
      dat[[variableKeepingTheCalibWeight]] *
        dat[["representativeHouseholdForCalibration"]] *
      dat[[numericalWeightingVar]], combined_factors, con_current))
    set(dat, j = "wValue", value = dat[["value"]] /
          dat[["fVariableForCalibrationIPF"]])

    # try to divide the weight between units with larger/smaller value in the
    #   numerical variable linear
    set(dat, j="fVariableForCalibrationIPF", value=1)
    dat[!is.na(value) & value!=0, fVariableForCalibrationIPF := numericalWeighting(
      head(wValue, 1), head(value, 1), get(numericalWeightingVar),
      get(variableKeepingTheCalibWeight)),
      by = eval(paste0("combined_factors_h_", i))]
  } else {
    # categorical variable to be calibrated
    set(dat, j = "fVariableForCalibrationIPF", value = ipf_step_f(
      dat[[variableKeepingTheCalibWeight]] *
        dat[["representativeHouseholdForCalibration"]],
      combined_factors, con_current))
  }

  set(dat, j = "wValue", value = dat[["value"]] /
        dat[["fVariableForCalibrationIPF"]])

  dat[, selectGroupNotConverged := (abs(wValue-value)/value)>epsHcur]
  dat[is.na(selectGroupNotConverged),selectGroupNotConverged:=FALSE]
  
  if (dat[,any(selectGroupNotConverged)]) {
    if (verbose && (calIter %% 10 == 0| calIter %% print_every_n == 0)) {
      message(calIter, ":Not yet converged for H-Constraint", i, "\n")
      if (calIter %% print_every_n == 0) {
        tmp <- dat[selectGroupNotConverged == TRUE,
          list(maxFac = max(abs(1 / fVariableForCalibrationIPF - 1)), .N,
               epsH = head(epsHcur, 1),
               sumCalibWeight = sum(get(variableKeepingTheCalibWeight) *
                                      representativeHouseholdForCalibration),
               PopMargin = head(value, 1)),
          by = eval(hColNames[[i]])]
        print(tmp[order(maxFac, decreasing = TRUE), ])

        message("-----------------------------------------\n")
      }
    }
    if (!is.null(bound) || !is.null(minMaxTrim)) {
      if (!looseH) {
        set(dat, j = variableKeepingTheCalibWeight, value = boundsFak(
          g1 = dat[[variableKeepingTheCalibWeight]],
          g0 = dat[[variableKeepingTheBaseWeight]],
          f = dat[["fVariableForCalibrationIPF"]],
          bound = bound, minMaxTrim = minMaxTrim))
      }else{
        set(dat, j = variableKeepingTheCalibWeight, value = boundsFakHH(
          g1 = dat[[variableKeepingTheCalibWeight]],
          g0 = dat[[variableKeepingTheBaseWeight]],
          eps = dat[["epsHcur"]], orig = dat[["value"]],
          p = dat[["wValue"]], bound = bound, minMaxTrim = minMaxTrim)
        )
      }
    } else {
      dat[, c(variableKeepingTheCalibWeight) := fVariableForCalibrationIPF *
            get(variableKeepingTheCalibWeight),
          by = eval(paste0("combined_factors_h_", i))]
    }
    dat[,c("fVariableForCalibrationIPF"):=NULL]
    error <- TRUE
  }

  dat[,c("selectGroupNotConverged"):=NULL]
  setnames(dat, "value", valueH[i])
  setnames(dat, "epsHcur", paste0("epsH_", i))
  return(error)
}

## recreate the formula argument to xtabs based on conP, conH
getFormulas <- function(con, w) {
  formOut <- NULL
  for (i in seq_along(con)) {
    lhs <- names(con)[i]
    if (is.null(lhs) || lhs == "") {
      lhs <- w
    } else {
      lhs <- paste(lhs, "*", w)
    }
    rhs <- paste(names(dimnames(con[[i]])), collapse = "+")
    formOut[[i]] <- formula(paste(lhs, "~", rhs), env = .GlobalEnv)
  }
  formOut
}

## enrich dat_original with the calibrated weights and assign attributes

# addWeightsAndAttributes <- function(dat, conP, conH, epsP, epsH, dat_original,
#                                     maxIter, calIter, returnNA, cw, bw,verbose, looseH, hidVar = NULL) {
# 
#   variableKeepingTheCalibWeight <- cw
#   representativeHouseholdForCalibration <- OriginalSortingVariable <-
#     outTable <- copy(dat_original)
# 
#   formP <- getFormulas(conP, w = variableKeepingTheCalibWeight)
#   formH <- getFormulas(conH, w = variableKeepingTheCalibWeight)
# 
#   # general information
#   setattr(outTable, "iterations", min(maxIter, calIter))
# 
#   # input constraints
#   setattr(outTable, "conP", conP)
#   setattr(outTable, "conH", conH)
# 
#   # adjusted constraints (conP, conH according to the calibrated weights)
#   conP_adj <- lapply(formP, xtabs, dat)
#   conH_adj <- lapply(
#     formH, xtabs, dat[representativeHouseholdForCalibration == 1])
#   setattr(outTable, "conP_adj", conP_adj)
#   setattr(outTable, "conH_adj", conH_adj)
# 
#   # tolerances
#   setattr(outTable, "epsP", epsP)
#   setattr(outTable, "epsH", epsH)
# 
#   setkey(dat, OriginalSortingVariable)
# 
# 
#   # convergence
#   conP_converged <- sapply(seq_along(conP), function(i) {
#     epsP_current <- switch(is.list(epsP) + 1, epsP, epsP[[i]])
#     conP_zero <- conP[[i]]==0
#     cond1 <- abs(conP[[i]] - conP_adj[[i]]) <= epsP_current * conP[[i]]
#     all(conP_zero==TRUE | cond1==TRUE)
#   })
#   if(looseH){
# 
#     inc <- function(x){
#       10^(floor(log10(x))-1) * 9.99
#     }
# 
#     conH_converged <- sapply(seq_along(conH), function(i) {
#       epsH_current <- switch(is.list(epsH) + 1, epsH, epsH[[i]])
#       if(verbose){
#         message(paste("For looseH=TRUE epsH+",inc(epsH_current),"is allowed as tolerance for the convergence."))
#       }
#       conH_zero <- conH[[i]] ==0
#       cond1 <- abs(conH[[i]] - conH_adj[[i]])/conH[[i]] <= (epsH_current + inc(epsH_current))
#       all(conH_zero==TRUE | cond1==TRUE)
#     })
#   }else{
#     conH_converged <- sapply(seq_along(conH), function(i) {
#       epsH_current <- switch(is.list(epsH) + 1, epsH, epsH[[i]])
#       conH_zero <- conH[[i]] ==0
#       cond1 <- abs(conH[[i]] - conH_adj[[i]]) <= epsH_current * conH[[i]]
#       all(conH_zero==TRUE | cond1==TRUE)
#     })
#   }
#   converged <- all(conP_converged) && all(conH_converged)
#   setattr(outTable, "converged", converged)
#   if (verbose) {
#     if (converged)
#       message("Convergence reached")
#     else
#       message("No convergence reached")
#   }
# 
#   # add calibrated weights. Use setkey to make sure the indexes match
#   setkey(dat, OriginalSortingVariable)
# 
#   if (!converged & returnNA) {
#     outTable[, c(variableKeepingTheCalibWeight) := NA]
#   } else {
#     outTable[, c(variableKeepingTheCalibWeight) :=
#                dat[[variableKeepingTheCalibWeight]]]
#   }
# 
#   # formulas
#   setattr(outTable, "formP", formP)
#   setattr(outTable, "formH", formH)
#   setattr(outTable, "baseweight", bw)
#   setattr(outTable, "hid", hidVar)
#   # for the summary
#   class(outTable) <- c("ipf",class(outTable))
# 
#   invisible(outTable)
# }


# # # # # NEW addWeightsAndAttributes function # # # # # 
# 
addWeightsAndAttributes <- function(dat, conP, conH, epsP, epsH, dat_original,
                                    maxIter, calIter, returnNA, cw, bw,verbose, looseH, hidVar = NULL) {
  
  variableKeepingTheCalibWeight <- cw
  representativeHouseholdForCalibration <- OriginalSortingVariable <-
    outTable <- copy(dat_original)
  
  formP <- getFormulas(conP, w = variableKeepingTheCalibWeight) 
  # [[1]] calibWeight ~ amonat + abl + bsex + alter
  # [[2]] calibWeight ~ amonat + bstaathr
  # [[3]] calibWeight ~ amonat + abl + bsex + verwerb1
  formH <- getFormulas(conH, w = variableKeepingTheCalibWeight) #calibWeight ~ amonat + abl + wg
  
  # general information
  setattr(outTable, "iterations", min(maxIter, calIter))
  
  # input constraints
  setattr(outTable, "conP", conP)
  setattr(outTable, "conH", conH)
  
  # adjusted constraints (conP, conH according to the calibrated weights)
  conP_adj <- lapply(formP, xtabs, dat) # für monat 5 und 6 = 0
  conH_adj <- lapply(
    formH, xtabs, dat[representativeHouseholdForCalibration == 1])
  setattr(outTable, "conP_adj", conP_adj)
  setattr(outTable, "conH_adj", conH_adj)
  
  # tolerances
  setattr(outTable, "epsP", epsP)
  setattr(outTable, "epsH", epsH)
  
  setkey(dat, OriginalSortingVariable)
  
  
  # convergence
  
  # if (is.finite(hrParameters$monat)) {
  #   m <- (hrParameters$monat - 1) %% 3 + 1
  # } else {
  #   m <- (hrParameters$monate - 1) %% 3 + 1
  # }
  
  # new logic for m
  # Step 1: Find the indices of the 'amonat' dimension in the constraints
  amonat_dim_indices_P <- sapply(conP, function(x) which(names(dimnames(x)) == "amonat"))
  amonat_dim_indices_H <- sapply(conH, function(x) which(names(dimnames(x)) == "amonat"))
  
  # Step 2: Extract the relevant months based on the adjusted values
  # First for conP
  m_list_P <- mapply(function(adj_table, amonat_index) {
    if (length(amonat_index) > 0) {
      # find indices of months with at least one non-zero value
      relevant_months <- which(apply(adj_table, amonat_index, function(y) any(y != 0)))
      return(relevant_months)
    } else {
      return(NULL)
    }
  }, conP_adj, amonat_dim_indices_P, SIMPLIFY = FALSE)
  
  # Then for conH
  m_list_H <- mapply(function(adj_table, amonat_index) {
    if (length(amonat_index) > 0) {
      relevant_months <- which(apply(adj_table, amonat_index, function(y) any(y != 0)))
      return(relevant_months)
    } else {
      return(NULL)
    }
  }, conH_adj, amonat_dim_indices_H, SIMPLIFY = FALSE)
  
  # Step 3: Combine all relevant month indices and remove duplicates
  m <- unique(c(unlist(m_list_P), unlist(m_list_H)))
  
  # Fallback: If no 'amonat' dimension exists or all values are 0, use index 1
  if (is.null(m) || length(m) == 0) {
    m <- 1
  }
  
  
  conP_converged <- sapply(seq_along(conP), function(i) {
    epsP_current <- switch(is.list(epsP) + 1, epsP, epsP[[i]])
    conP_zero <- conP[[i]] == 0
    cond1 <- abs(conP[[i]] - conP_adj[[i]]) <= epsP_current * conP[[i]]
    
    # Index of "amonat"
    amonat_dim_index <- which(names(dimnames(conP[[i]])) == "amonat")
    
    # check if "amonat" is present
    if (length(amonat_dim_index) > 0) {
      
      # which month in the quarter?
      # m <- match(hrParameters$monat, dimnames(conP[[i]])[[amonat_dim_index]])
      
      # build subset: only the relevant month(s)
      num_dims <- length(dim(conP[[i]]))
      indices <- c(rep(list(TRUE), num_dims))
      indices[[amonat_dim_index]] <- m
      
      subset_conP_zero <- do.call("[", c(list(conP_zero), indices))
      subset_cond1 <- do.call("[", c(list(cond1), indices))
      
      return(all(subset_conP_zero == TRUE | subset_cond1 == TRUE))
    } else {
      # Fallback solution for quarterly data
      return(all(conP_zero == TRUE | cond1 == TRUE))
    }
  })
  
  if(looseH){
    
    inc <- function(x){
      10^(floor(log10(x))-1) * 9.99
    }
    
    conH_converged <- sapply(seq_along(conH), function(i) {
      epsH_current <- switch(is.list(epsH) + 1, epsH, epsH[[i]])
      if(verbose){
        message(paste("For looseH=TRUE epsH+",inc(epsH_current),"is allowed as tolerance for the convergence (-> ", epsH_current + inc(epsH_current), ")"))
      }
      conH_zero <- conH[[i]] ==0
      cond1 <- abs(conH[[i]] - conH_adj[[i]])/conH[[i]] <= (epsH_current + inc(epsH_current))
      
      # Index der Dimension "amonat"
      num_dims <- length(dim(conH[[i]]))
      amonat_dim_index <- which(names(dimnames(conH[[i]])) == "amonat")
      
      if (length(amonat_dim_index) > 0) {
        # Berechne korrekten Indexpositionen 
        # m <- match(hrParameters$monat, dimnames(conH[[i]])[[amonat_dim_index]])
        # Erstelle eine Liste von Index-Vektoren für dynamisches Subsetting
        indices <- c(rep(list(TRUE), num_dims))
        indices[[amonat_dim_index]] <- m
        
        subset_conH_zero <- do.call("[", c(list(conH_zero), indices))
        subset_cond1 <- do.call("[", c(list(cond1), indices))
        
        result <- all(subset_conH_zero == TRUE | subset_cond1 == TRUE)
      } else {
        # Fallback-Lösung für Constraints ohne amonat-Dimension
        result <- all(conH_zero == TRUE | cond1 == TRUE)
      }
      
      message(paste("all(conH_zero==TRUE | cond1==TRUE): ", result))
      return(result)
    })
  }else{
    conH_converged <- sapply(seq_along(conH), function(i) {
      epsH_current <- switch(is.list(epsH) + 1, epsH, epsH[[i]])
      conH_zero <- conH[[i]] ==0
      cond1 <- abs(conH[[i]] - conH_adj[[i]]) <= epsH_current * conH[[i]]
      
      # NEUE, ROBUSTERERE LOGIK FÜR H-CONSTRAINTS
      num_dims <- length(dim(conH[[i]]))
      # Ermittle den Index der Dimension namens "amonat"
      amonat_dim_index <- which(names(dimnames(conH[[i]])) == "amonat")
      
      if (length(amonat_dim_index) > 0) {
        # Berechne die korrekten Indexpositionen mit match()
        # m <- match(hrParameters$monat, dimnames(conH[[i]])[[amonat_dim_index]])
        # Liste von Index-Vektoren
        indices <- c(rep(list(TRUE), num_dims))
        indices[[amonat_dim_index]] <- m
        
        subset_conH_zero <- do.call("[", c(list(conH_zero), indices))
        subset_cond1 <- do.call("[", c(list(cond1), indices))
        
        return(all(subset_conH_zero == TRUE | subset_cond1 == TRUE))
      } else {
        # Fallback-Lösung für quartal
        return(all(conH_zero == TRUE | cond1 == TRUE))
      }
    })
  }
  
  
  converged <- all(conP_converged) && all(conH_converged)
  setattr(outTable, "converged", converged)
  if (verbose) {
    if (converged)
      message("Convergence reached")
    else
      message("No convergence reached")
  }
  
  # add calibrated weights. Use setkey to make sure the indexes match
  setkey(dat, OriginalSortingVariable)
  
  if (!converged & returnNA) {
    outTable[, c(variableKeepingTheCalibWeight) := NA]
  } else {
    outTable[, c(variableKeepingTheCalibWeight) :=
               dat[[variableKeepingTheCalibWeight]]]
  }
  
  # formulas
  setattr(outTable, "formP", formP)
  setattr(outTable, "formH", formH)
  setattr(outTable, "baseweight", bw)
  setattr(outTable, "hid", hidVar)
  # for the summary
  class(outTable) <- c("ipf",class(outTable))
  
  invisible(outTable)
}




#' Iterative Proportional Fitting
#'
#' Adjust sampling weights to given totals based on household-level and/or
#' individual level constraints.
#'
#' This function implements the weighting procedure described
#' here: \doi{10.17713/ajs.v45i3.120}.
#' Usage examples can be found in the corresponding vignette
#' (`vignette("ipf")`).
#'
#' `conP` and `conH` are contingency tables, which can be created with `xtabs`.
#' The `dimnames` of those tables should match the names and levels of the
#' corresponding columns in `dat`.
#'
#' `maxIter`, `epsP` and `epsH` are the stopping criteria. `epsP` and `epsH`
#' describe relative tolerances in the sense that
#' \deqn{1-epsP < \frac{w_{i+1}}{w_i} < 1+epsP}{1-epsP < w(i+1)/w(i) < 1+epsP}
#' will be used as convergence criterium. Here i is the iteration step and wi is
#' the weight of a specific person at step i.
#'
#' The algorithm
#' performs best if all varables occuring in the constraints (`conP` and `conH`)
#' as well as the household variable are coded as `factor`-columns in `dat`.
#' Otherwise, conversions will be necessary which can be monitored with the
#' `conversion_messages` argument. Setting `check_hh_vars` to `FALSE` can also
#' incease the performance of the scheme.
#'
#' @name ipf
#' @md
#' @aliases ipf
#' @param dat a `data.table` containing household ids (optionally), base
#'   weights (optionally), household and/or personal level variables (numerical
#'   or categorical) that should be fitted.
#' @param hid name of the column containing the household-ids within `dat` or
#'   NULL if such a variable does not exist.
#' @param w name if the column containing the base weights within `dat` or NULL
#'   if such a variable does not exist. In the latter case, every observation
#'   in `dat` is assigned a starting weight of 1.
#' @param conP list or (partly) named list defining the constraints on person
#'   level.  The list elements are contingency tables in array representation
#'   with dimnames corresponding to the names of the relevant calibration
#'   variables in `dat`. If a numerical variable is to be calibrated, the
#'   respective list element has to be named with the name of that numerical
#'   variable. Otherwise the list element shoud NOT be named.
#' @param conH list or (partly) named list defining the constraints on
#'   household level.  The list elements are contingency tables in array
#'   representation with dimnames corresponding to the names of the relevant
#'   calibration variables in `dat`. If a numerical variable is to be
#'   calibrated, the respective list element has to be named with the name of
#'   that numerical variable. Otherwise the list element shoud NOT be named.
#' @param epsP numeric value or list (of numeric values and/or arrays)
#'   specifying the convergence limit(s) for `conP`. The list can contain
#'   numeric values and/or arrays which must appear in the same order as the
#'   corresponding constraints in `conP`. Also, an array must have the same
#'   dimensions and dimnames as the corresponding constraint in `conP`.
#' @param epsH numeric value or list (of numeric values and/or arrays)
#'   specifying the convergence limit(s) for `conH`. The list can contain
#'   numeric values and/or arrays which must appear in the same order as the
#'   corresponding constraints in `conH`. Also, an array must have the same
#'   dimensions and dimnames as the corresponding constraint in `conH`.
#' @param verbose if TRUE, some progress information will be printed.
#' @param bound numeric value specifying the multiplier for determining the
#'   weight trimming boundary if the change of the base weights should be
#'   restricted, i.e. if the weights should stay between 1/`bound`*`w`
#'   and `bound`*\code{w}.
#' @param minMaxTrim numeric vector of length2, first element a minimum value
#'   for weights to be trimmed to, second element a maximum value for weights to
#'   be trimmed to.
#' @param maxIter numeric value specifying the maximum number of iterations
#' that should be performed.
#' @param meanHH if TRUE, every person in a household is assigned the mean of
#'   the person weights corresponding to the household. If `"geometric"`, the
#'   geometric mean is used rather than the arithmetic mean.
#' @param allPthenH if TRUE, all the person level calibration steps are
#'   performed before the houshold level calibration steps (and `meanHH`, if
#'   specified). If FALSE, the houshold level calibration steps (and `meanHH`,
#'   if specified) are performed after everey person level calibration step.
#'   This can lead to better convergence properties in certain cases but also
#'   means that the total number of calibration steps is increased.
#' @param returnNA if TRUE, the calibrated weight will be set to NA in case of
#'   no convergence.
#' @param looseH if FALSE, the actual constraints `conH` are used for
#'   calibrating all the hh weights. If TRUE, only the weights for which the
#'   lower and upper thresholds defined by `conH` and `epsH` are exceeded are
#'   calibrated. They are however not calibrated against the actual constraints
#'   `conH` but against these lower and upper thresholds, i.e.
#'   `conH`-`conH`*`epsH` and `conH`+`conH`*\code{epsH}.
#' @param numericalWeighting See [numericalWeighting]
#' @param check_hh_vars If `TRUE` check for non-unique values inside of a
#'   household for variables in household constraints
#' @param conversion_messages show a message, if inputs need to be reformatted.
#'   This can be useful for speed optimizations if ipf is called several times
#'   with similar inputs (for example bootstrapping)
#' @param nameCalibWeight character defining the name of the variable for the
#'   newly generated calibrated weight.
#' @param print_every_n number of interation steps after which a summary table
#'   is printed. The summary table shows all constraints which are not yet
#'   reached according to `epsP` and `epsH`
#' @return The function will return the input data `dat` with the calibrated
#'   weights `calibWeight` as an additional column as well as attributes. If no
#'   convergence has been reached in `maxIter` steps, and `returnNA` is `TRUE`
#'   (the default), the column `calibWeights` will only consist of `NA`s. The
#'   attributes of the table are attributes derived from the `data.table` class
#'   as well as the following.
#' \tabular{ll}{
#'   `converged` \tab Did the algorithm converge in `maxIter` steps? \cr
#'   `iterations` \tab The number of iterations performed. \cr
#'   `conP`, `conH`, `epsP`, `epsH` \tab See Arguments. \cr
#'   `conP_adj`, `conH_adj` \tab Adjusted versions of `conP` and `conH` \cr
#'   `formP`, `formH` \tab Formulas that were used to calculate `conP_adj` and
#'   `conH_adj` based on the output table.
#' }
#' @export ipf
#' @author Alexander Kowarik, Gregor de Cillia
#' @examples
#' \dontrun{
#'
#' # load data
#' eusilc <- demo.eusilc(n = 1, prettyNames = TRUE)
#'
#' # personal constraints
#' conP1 <- xtabs(pWeight ~ age, data = eusilc)
#' conP2 <- xtabs(pWeight ~ gender + region, data = eusilc)
#' conP3 <- xtabs(pWeight*eqIncome ~ gender, data = eusilc)
#'
#' # household constraints
#' conH1 <- xtabs(pWeight ~ hsize + region, data = eusilc[!duplicated(hid)])
#'
#' # simple usage ------------------------------------------
#'
#' calibweights1 <- ipf(
#'   eusilc,
#'   conP = list(conP1, conP2, eqIncome = conP3),
#'   bound = NULL,
#'   verbose = TRUE
#' )
#'
#' # compare personal weight with the calibweigth
#' calibweights1[, .(hid, pWeight, calibWeight)]
#'
#' # advanced usage ----------------------------------------
#'
#' # use an array of tolerances
#' epsH1 <- conH1
#' epsH1[1:4, ] <- 0.005
#' epsH1[5, ] <- 0.2
#'
#' # create an initial weight for the calibration
#' eusilc[, regSamp := .N, by = region]
#' eusilc[, regPop := sum(pWeight), by = region]
#' eusilc[, baseWeight := regPop/regSamp]
#'
# calibrate
#' calibweights2 <- ipf(
#'   eusilc,
#'   conP = list(conP1, conP2),
#'   conH = list(conH1),
#'   epsP = 1e-6,
#'   epsH = list(epsH1),
#'   bound = 4,
#'   w = "baseWeight",
#'   verbose = TRUE
#' )
#'
#' # show an adjusted version of conP and the original
#' attr(calibweights2, "conP_adj")
#' attr(calibweights2, "conP")
#' }
ipf <- function(
  dat, hid = NULL, conP = NULL, conH = NULL, epsP = 1e-6, epsH = 1e-2,
  verbose = FALSE, w = NULL, bound = 4, maxIter = 200, meanHH = TRUE,
  allPthenH = TRUE, returnNA = TRUE, looseH = FALSE, numericalWeighting =
    computeLinear, check_hh_vars = TRUE, conversion_messages = FALSE,
  nameCalibWeight = "calibWeight", minMaxTrim = NULL, print_every_n = 100) {

  check_population_totals(conP, dat, "personal")
  check_population_totals(conH, dat, "household")
  check_eps(conP, epsP, type = "personal")
  check_eps(conH, epsH, type = "household")
  variableKeepingTheBaseWeight <- w
  variableKeepingTheCalibWeight <- nameCalibWeight
  if ("variableKeepingTheBaseWeight" %in% names(dat))
    stop("The provided dataset must not have a column called",
         " 'variableKeepingTheBaseWeight'")
  if(!is.null(minMaxTrim)){
    if(length(minMaxTrim)!=2)
      stop("minMaxTrim must have exactly 2 elements, a minimum and a maximum.")
    if(!is.numeric(minMaxTrim)){
      stop("minMaxTrim must be a numeric vector of length two.")
    }
    if(minMaxTrim[2]<minMaxTrim[1]){
      stop("minMaxTrim must have a minimum as a first element and a maximum as a second element.
           But in the input the second element is smaller than the first.")
    }
  }
  OriginalSortingVariable <- V1 <- epsvalue <-
    f <- temporary_hvar <-
    value <- wValue <- representativeHouseholdForCalibration <- ..hid <- NULL
  dat_original <- dat
  dat <- copy(dat)
  ## originalsorting is fucked up without this
  dat[, OriginalSortingVariable := .I]
  meanfun <- getMeanFun(meanHH)

  # dat sollte ein data.table sein
  # w ein Name eines Basisgewichts oder NULL
  valueP <- paste0("valueP", seq_along(conP))
  ###fixed target value, should not be changed in iterations
  valueH <- paste0("valueH", seq_along(conH))
  ###Housekeeping of the varNames used
  usedVarNames <- c(valueP, valueH, "value",
                    "representativeHouseholdForCalibration", "wValue")

  if (any(names(dat) %in% usedVarNames)) {
    renameVars <- names(dat)[names(dat) %in% usedVarNames]
    setnames(dat, renameVars, paste0(renameVars, "_safekeeping"))
  }
  ### Treatment of HID, creating 0,1 var for being the first hh member
  #delVars <- c()
  if (is.null(hid)) {
    #delVars <- c("hid")
    hid <- "hid"
    dat[, hid := as.factor(seq_len(nrow(dat)))]
    dat[, representativeHouseholdForCalibration := 1]
  } else {

    if(!hid%in%colnames(dat)){
      stop("dat does not contain column ",hid)
    }

    if(any(is.na(dat[[hid]]))){
      stop("hid contains missing values")
    }

    if (!is.factor(dat[[hid]]))
      data.table::set(dat, NULL, hid, as.factor(dat[[hid]]))
    dat[, representativeHouseholdForCalibration :=
          as.numeric(!duplicated(get(..hid)))]
  }

  ## Names of the calibration variables for Person and household dimension
  pColNames <- lapply(conP, function(x) names(dimnames(x)))
  hColNames <- lapply(conH, function(x) names(dimnames(x)))

  for (i in seq_along(conP)) {
    current_colnames <- pColNames[[i]]

    for (colname in current_colnames) {
      if (!inherits(dat[[colname]], "factor")) {
        if (conversion_messages)
          message("converting column ", colname, " to factor")
        set(
          dat, j = colname,
          value = factor(dat[[colname]],
                         levels = dimnames(conP[[i]])[[colname]])
        )
      }
      else if (!identical(levels(dat[[colname]]),
                          dimnames(conP[[i]])[[colname]])) {
        if (conversion_messages)
          message("correct levels of column ", colname)
        set(
          dat, j = colname, value = factor(
            dat[[colname]], levels = dimnames(conP[[i]])[[colname]])
        )
      }
    }
    combined_factors <- combine_factors(dat, conP[[i]])
    set(dat, j = paste0("combined_factors_", i), value = combined_factors)
    set(dat, j = paste0("valueP", i),
        value = as.vector(conP[[i]][combined_factors]))
  }
  for (i in seq_along(conH)) {
    colnames <- hColNames[[i]]

    ## make sure the columns mentioned in the contingency table are in fact
    ##   factors
    for (colname in colnames) {
      if (!inherits(dat[[colname]], "factor")) {
        if (conversion_messages)
          message("converting column ", colname, " to factor")
        set(
          dat, j = colname, value = factor(
            dat[[colname]], levels = dimnames(conH[[i]])[[colname]])
        )
      }
      else if (!identical(levels(dat[[colname]]),
                          dimnames(conH[[i]])[[colname]])) {
        if (conversion_messages)
          message("correct levels of column ", colname)
        set(
          dat, j = colname, value = factor(
            dat[[colname]], levels = dimnames(conH[[i]])[[colname]])
        )
      }
    }

    combined_factors <- combine_factors(dat, conH[[i]])

    set(dat, j = paste0("combined_factors_h_", i), value = combined_factors)
    set(dat, j = paste0("valueH", i),
        value = as.vector(conH[[i]][combined_factors]))
  }

  if (is.null(variableKeepingTheBaseWeight)) {
    if (!is.null(bound) && is.null(w))
      stop("Bounds are only reasonable if base weights are provided")
    set(dat, j = variableKeepingTheCalibWeight, value = 1)
  } else {
    set(dat, j = variableKeepingTheCalibWeight,
        value = dat[[variableKeepingTheBaseWeight]])
  }

  if (check_hh_vars) {
    ## Check for non-unqiue values inside of a household for variabels used
    ##   in Household constraints
    for (hh in hColNames) {
      for (h in hh) {
        setnames(dat, h, "temporary_hvar")
        if (dat[, length(unique(temporary_hvar)),
                by = c(hid)][, any(V1 != 1)]) {
          stop(paste(h, "has different values inside a household"))
        }
        setnames(dat, "temporary_hvar", h)
      }
    }
  }

  if (is.list(epsP)) {
    for (i in seq_along(epsP)) {
      if (is.array(epsP[[i]])) {
        combined_factors <- dat[[paste0("combined_factors_", i)]]
        set(dat, j = paste0("epsP_", i),
            value = as.vector(epsP[[i]][combined_factors]))
      } else {
        set(dat, j = paste0("epsP_", i), value = epsP[[i]])
      }
    }
  } else {
    for (i in seq_along(conP)) {
      set(dat, j = paste0("epsP_", i), value = epsP)
    }
  }
  if (is.list(epsH)) {
    for (i in seq_along(epsH)) {
      if (is.array(epsH[[i]])) {
        combined_factors <- dat[[paste0("combined_factors_h_", i)]]
        set(dat, j = paste0("epsH_", i),
            value = as.vector(epsH[[i]][combined_factors]))
      } else {
        set(dat, j = paste0("epsH_", i), value = epsH[[i]])
      }
    }
  } else {
    for (i in seq_along(conH)) {
      set(dat, j = paste0("epsH_", i), value = epsH)
    }
  }
  ###Calib
  error <- TRUE
  calIter <- 1
  while (error && calIter <= maxIter) {
    error <- FALSE

    if (allPthenH) {
      ### Person calib
      
      for (i in seq_along(conP)) {
        
        numericalWeightingTmp <- NULL
        if (isTRUE(names(conP)[i] != "")) {
          numericalWeightingTmp <- names(conP)[i]
        }
        error <- calibP(
          i = i, dat = dat, error = error, valueP = valueP,
          pColNames = pColNames, bound = bound, verbose = verbose,
          calIter = calIter, numericalWeighting = numericalWeighting,
          numericalWeightingVar = numericalWeightingTmp,
          w = variableKeepingTheBaseWeight,
          cw = variableKeepingTheCalibWeight, minMaxTrim = minMaxTrim,
          print_every_n = print_every_n)

      }
      
      # ## replace person weight with household average
      set(dat, j = variableKeepingTheCalibWeight,
          value = meanfun(dat[[variableKeepingTheCalibWeight]], dat[[hid]]))
      
      ### Household calib
      for (i in seq_along(conH)) {
        numericalWeightingTmp <- NULL
        if (isTRUE(names(conH)[i] != "")) {
          numericalWeightingTmp <- names(conH)[i]
        }
        error <- calibH(
          i = i, dat = dat, error = error, valueH = valueH,
          hColNames = hColNames, bound = bound, verbose = verbose,
          calIter = calIter, looseH = looseH,
          numericalWeighting = numericalWeighting,
          numericalWeightingVar = numericalWeightingTmp,
          w = variableKeepingTheBaseWeight,
          cw = variableKeepingTheCalibWeight, minMaxTrim = minMaxTrim,
          print_every_n = print_every_n)
        
      }
    } else {
      ### Person calib
      for (i in seq_along(conP)) {
        numericalWeightingTmp <- NULL
        if (isTRUE(names(conP)[i] != "")) {
          numericalWeightingTmp <- names(conP)[i]
        }
        error <- calibP(
          i = i, dat = dat, error = error, valueP = valueP,
          pColNames = pColNames, bound = bound, verbose = verbose,
          calIter = calIter, numericalWeighting = numericalWeighting,
          numericalWeightingVar = numericalWeightingTmp,
          w = variableKeepingTheBaseWeight,
          cw = variableKeepingTheCalibWeight, minMaxTrim = minMaxTrim,
          print_every_n = print_every_n)

        ## replace person weight with household average
        set(dat, j = variableKeepingTheCalibWeight,
            value = meanfun(dat[[variableKeepingTheCalibWeight]], dat[[hid]]))

        ### Household calib
        for (i in seq_along(conH)) {
          numericalWeightingTmp <- NULL
          if (isTRUE(names(conH)[i] != "")) {
            numericalWeightingTmp <- numericalWeighting
          }
          error <- calibH(
            i = i, dat = dat, error = error, valueH = valueH,
            hColNames = hColNames, bound = bound, verbose = verbose,
            calIter = calIter, numericalWeighting = numericalWeighting,
            numericalWeightingVar = numericalWeightingTmp, looseH = looseH,
            w = variableKeepingTheBaseWeight,
            cw = variableKeepingTheCalibWeight, minMaxTrim = minMaxTrim,
            print_every_n = print_every_n)
        }
      }
    }

    if (verbose && !error) {
      message("Iteration stopped after ", calIter, " steps")
    } else if (maxIter == calIter) {
      warning("Not converged in ", maxIter, " steps")
    }
    calIter <- calIter + 1
  }
  # Remove Help Variables
  fVariableForCalibrationIPF <- NULL
  # dat[, fVariableForCalibrationIPF := NULL]
  addWeightsAndAttributes(dat, conP, conH, epsP, epsH, dat_original, maxIter,
                          calIter, returnNA, variableKeepingTheCalibWeight,variableKeepingTheBaseWeight,
                          verbose, looseH, hid)
}

context("ipf")
library(surveysd)
library(data.table)
eusilc <- demo.eusilc(n = 1, prettyNames = TRUE)

# treat households as a factor variable
eusilc[, hid := as.factor(hid)]

## example for base weights assuming a simple random sample of households
## stratified per region
eusilc[, regSamp := .N, by = region]
eusilc[, regPop := sum(pWeight), by = region]
eusilc[, baseWeight := regPop / regSamp]
eusilc[, pIncome := eqIncome / .N, by = hid]

## constraints on person level
# age
conP1 <- xtabs(pWeight ~ age, data = eusilc)
# gender by region
conP2 <- xtabs(pWeight ~ gender + region, data = eusilc)
# personal net income by gender
conP3 <- xtabs(pWeight * pIncome ~ gender, data = eusilc)

## constraints on household level
conH1 <- xtabs(pWeight ~ hsize + region, data = eusilc,
               subset = !duplicated(hid))
## constraints on household level netIncome
conH2 <- xtabs(pWeight * eqIncome ~ region, data = eusilc,
               subset = !duplicated(hid))
# array of convergence limits for conH1
epsH1 <- conH1
epsH1[1:4, ] <- 0.005
epsH1[5, ] <- 0.2

test_that("ipf with a numerical variable works as expected - computeLinear", {
  # without array epsP1
  calibweights1 <- ipf(
    eusilc, hid = "hid",
    conP = list(conP1, conP2, pIncome = conP3),
    conH = list(conH1),
    epsP = list(1e-06, 1e-06, 1e-03),
    epsH = 0.01,
    bound = NULL, verbose = FALSE,  maxIter = 200,
    numericalWeighting = computeLinear)
  conP3_adj <- xtabs(calibWeight * pIncome ~ gender, data = calibweights1)
  expect_true(abs(sum(conP3_adj) - sum(conP3)) / sum(conP3) < .01)
  expect_true(all(abs(conP3_adj - conP3) / conP3 < .01))

  err <- max(c(
    max(abs(xtabs(calibWeight ~ age, data = calibweights1) -
              conP1) / conP1),
    max(abs(xtabs(calibWeight ~ gender + region, data = calibweights1) -
              conP2) / conP2),
    max(abs(xtabs(calibWeight ~ hsize + region, data = calibweights1,
                  subset = !duplicated(hid)) - conH1) / conH1)))
  expect_true(err < .01)
})

test_that("ipf with a numerical variable works as expected - computeLinearG1", {
  # without array epsP1
  calibweights1 <- ipf(
    eusilc, hid = "hid",
    conP = list(conP1, conP2, pIncome = conP3),
    conH = list(conH1),
    epsP = list(1e-06, 1e-06, 1e-03),
    epsH = 0.01,
    bound = NULL, verbose = FALSE, maxIter = 200,
    numericalWeighting = computeLinearG1)
  conP3_adj <- xtabs(calibWeight * pIncome ~ gender, data = calibweights1)
  expect_true(abs(sum(conP3_adj) - sum(conP3)) / sum(conP3) < .01)
  expect_true(all(abs(conP3_adj - conP3) / conP3 < .01))

  err <- max(c(
    max(abs(xtabs(calibWeight ~ age, data = calibweights1) -
              conP1) / conP1),
    max(abs(xtabs(calibWeight ~ gender + region, data = calibweights1) -
              conP2) / conP2),
    max(abs(xtabs(calibWeight ~ hsize + region, data = calibweights1,
                  subset = !duplicated(hid)) - conH1) / conH1)))
  expect_true(err < .01)
})


test_that("ipf with a numerical variable in households  as expected", {
  # without array epsP1
  calibweights1 <- ipf(
    eusilc, hid = "hid",
    conP = list(conP1, conP2),
    conH = list(conH1, eqIncome = conH2),
    epsP = list(1e-06, 1e-06),
    epsH = list(0.01, 0.01),
    bound = NULL, verbose = FALSE,  maxIter = 50,
    numericalWeighting = computeFrac)
  conP3_adj <- xtabs(calibWeight * pIncome ~ gender, data = calibweights1)
  expect_true(abs(sum(conP3_adj) - sum(conP3)) / sum(conP3) < .01)
  expect_true(all(abs(conP3_adj - conP3) / conP3 < .01))

  err <- max(c(
    max(abs(xtabs(calibWeight ~ age, data = calibweights1) -
              conP1) / conP1),
    max(abs(xtabs(calibWeight ~ gender + region, data = calibweights1) -
              conP2) / conP2),
    max(abs(xtabs(calibWeight ~ hsize + region, data = calibweights1,
                  subset = !duplicated(hid)) - conH1) / conH1)))
  expect_true(err < .01)
})

test_that("ipf works as expected", {
  # with array epsP1, base weights and bound
  calibweights2 <- ipf(
    eusilc, hid = "hid", conP = list(conP1, conP2), conH = list(conH1),
    epsP = 1e-06, epsH = list(epsH1), w = "baseWeight", bound = 4,
    verbose = FALSE, maxIter = 200)
  err <- max(c(
    max(abs(xtabs(calibWeight ~ age, data = calibweights2) - conP1) /
          conP1),
    max(
      abs(xtabs(calibWeight ~ gender + region, data = calibweights2) - conP2) /
        conP2),
    max(abs(xtabs(calibWeight ~ hsize + region, data = calibweights2,
                  subset = !duplicated(hid)) - conH1) / conH1)))
  expect_true(err < .01)
})

test_that("ipf works as expected calibWeight renamed", {
  # with array epsP1, base weights and bound
  calibweights2 <- ipf(
    eusilc, hid = "hid", conP = list(conP1, conP2), conH = list(conH1),
    epsP = 1e-06, epsH = list(epsH1), w = "baseWeight", bound = 4,
    verbose = FALSE, maxIter = 200, nameCalibWeight = "calibWeightNew")
  err <- max(c(
    max(abs(xtabs(calibWeightNew ~ age, data = calibweights2) - conP1) /
          conP1),
    max(abs(
      xtabs(calibWeightNew ~ gender + region, data = calibweights2) - conP2) /
          conP2),
    max(abs(xtabs(calibWeightNew ~ hsize + region, data = calibweights2,
                  subset = !duplicated(hid)) - conH1) / conH1)))
  expect_true(err < .01)
})

test_that("ipf stops  as expected when dimensionality of eps does not fit", {
  # with array epsP1, base weights and bound
  expect_error(ipf(
    eusilc, hid = "hid", conP = list(conP1, conP2), conH = list(conH1,conH1,conH1),
    epsP = c(1e-06), epsH = list(epsH1,1e-06), w = "baseWeight", bound = 4,
    verbose = FALSE, maxIter = 200),
    regexp = "Provided household eps argument does not fit household constraints.")

  expect_error(ipf(
    eusilc, hid = "hid", conP = list(conP1, conP2), conH = list(conH1),
    epsP = c(1e-06), epsH = list(epsH1[,1:8]), w = "baseWeight", bound = 4,
    verbose = FALSE, maxIter = 200),
    regexp = "Provided household eps argument 1 does not fit in dimension to  household constraints 1 .")

    expect_error(ipf(
    eusilc, hid = "hid", conP = list(conP1, conP2), conH = list(conH1,conH1,conH1),
    epsP = c(1e-06,22), epsH = list(epsH1,1e-06,1e-6), w = "baseWeight", bound = 4,
    verbose = FALSE, maxIter = 200),
    regexp = "Individual eps arguments for each constraints must be defined as list.")

})


test_that("ipf works as expected with minMaxTrim only P", {
  # with array epsP1, base weights and bound
  calibweights2 <- ipf(
    eusilc, hid = "hid", conP = list(conP1, conP2),
    epsP = 1e-06, w = "baseWeight", bound = 4,
    verbose = FALSE, maxIter = 200, minMaxTrim = c(450,700))
  err <- max(c(
    max(abs(xtabs(calibWeight ~ age, data = calibweights2) - conP1) /
          conP1),
    max(
      abs(xtabs(calibWeight ~ gender + region, data = calibweights2) - conP2) /
        conP2)))
  expect_true(err < .01)
  expect_true(all(calibweights2$calibWeight<=700)&&all(calibweights2$calibWeight>=450))
})

test_that("ipf works as expected with minMaxTrim P and HH", {
  # with array epsP1, base weights and bound
  calibweights2 <- ipf(
    eusilc, hid = "hid", conP = list(conP1, conP2), conH = list(conH1),
    epsP = 1e-06, epsH = list(epsH1), w = "baseWeight", bound = 4,
    verbose = FALSE, maxIter = 200, minMaxTrim = c(340,870))
  err <- max(c(
    max(abs(xtabs(calibWeight ~ age, data = calibweights2) - conP1) /
          conP1),
    max(
      abs(xtabs(calibWeight ~ gender + region, data = calibweights2) - conP2) /
        conP2),
    max(abs(xtabs(calibWeight ~ hsize + region, data = calibweights2,
                  subset = !duplicated(hid)) - conH1) / conH1)))
  expect_true(err < .01)
  expect_true(all(calibweights2$calibWeight<=870)&&all(calibweights2$calibWeight>=340))
})

test_that("summary.ipf", {
  # with array epsP1, base weights and bound
  calibweights2 <- ipf(
    eusilc, hid = "hid", conP = list(conP1, conP2), conH = list(conH1),
    epsP = 1e-06, epsH = list(epsH1), w = "baseWeight", bound = 4,
    verbose = FALSE, maxIter = 200, minMaxTrim = c(340,870))
  out <- summary(calibweights2)
  # expect_length(out, 22)
  expect_length(out$`distribution of the weights`,6)
})


test_that("summary.ipf without base weight", {
  # with array epsP1, base weights and bound
  calibweights2 <- ipf(
    eusilc, hid = "hid", conP = list(conP1, conP2), conH = list(conH1),
    epsP = 1e-06, epsH = list(epsH1), bound = NULL)
  out <- summary(calibweights2)
  # expect_length(out, 22)
  expect_length(out$`distribution of the weights`,0)
})

#' @title Draw bootstrap replicates
#'
#' @description Draw bootstrap replicates from survey data with rotating panel
#'   design. Survey information, like ID, sample weights, strata and population
#'   totals per strata, should be specified to ensure meaningfull survey
#'   bootstraping.
#'
#' @param dat either data.frame or data.table containing the survey data with
#'   rotating panel design.
#' @param method for bootstrap replicates, either `"Preston"` or `"Rao-Wu"` 
#' @param REP integer indicating the number of bootstrap replicates.
#' @param hid character specifying the name of the column in `dat` containing
#'   the household id. If `NULL` (the default), the household structure is not
#'   regarded.
#' @param weights character specifying the name of the column in `dat`
#'   containing the sample weights.
#' @param period character specifying the name of the column in `dat` containing
#'   the sample periods. If `NULL` (the default), it is assumed that all
#'   observations belong to the same period.
#' @param strata character vector specifying the name(s) of the column in `dat`
#'   by which the population was stratified. If `strata` is a vector
#'   stratification will be assumed as the combination of column names contained
#'   in `strata`. Setting in addition `cluster` not NULL stratification will be
#'   assumed on multiple stages, where each additional entry in `strata`
#'   specifies the stratification variable for the next lower stage. see Details
#'   for more information.
#' @param cluster character vector specifying cluster in the data. If not
#'   already specified in `cluster` household ID is taken es the lowest level
#'   cluster.
#' @param totals character specifying the name of the column in `dat` containing
#'   the the totals per strata and/or cluster. Is ONLY optional if `cluster` is
#'   `NULL` or equal `hid` and `strata` contains one columnname! Then the
#'   households per strata will be calcualted using the `weights` argument. If
#'   clusters and strata for multiple stages are specified `totals` needs to be
#'   a vector of `length(strata)` specifying the column on `dat` that contain
#'   the total number of PSUs at each stage. `totals` is interpreted from left
#'   the right, meaning that the first argument corresponds to the number of
#'   PSUs at the first and the last argument to the number of PSUs at the last
#'   stage.
#' @param single.PSU either "merge" or "mean" defining how single PSUs need to
#'   be dealt with. For `single.PSU="merge"` single PSUs at each stage are
#'   merged with the strata or cluster with the next least number of PSUs. If
#'   multiple of those exist one will be select via random draw. For
#'   `single.PSU="mean"` single PSUs will get the mean over all bootstrap
#'   replicates at the stage which did not contain single PSUs.
#' @param boot.names character indicating the leading string of the column names
#'   for each bootstrap replica. If NULL defaults to "w".
#' @param split logical, if TRUE split households are considered using `pid`,
#'   for more information see Details.
#' @param pid column in `dat` specifying the personal identifier. This
#'   identifier needs to be unique for each person throught the whole data set.
#'   Can be `NULL`.
#' @param seed integer specifying the seed for the random number generator.
#' @param already.selected list of data.tables indicating if record was drawn
#' in previous `period`. `length(already.selected)` must be equal to 
#' the number of sampling stages specified. See [get.selection()] for an example. 
#' 
#' @return the survey data with the number of REP bootstrap replicates added as
#'   columns.
#'
#' @details `draw.bootstrap` takes `dat` and draws `REP` bootstrap replicates
#' from it.
#' `dat` must be household data where household members correspond to multiple
#' rows with the same household
#' identifier. For most practical applications, the following columns should be
#' available in the dataset
#' and passed via the corresponding parameters:
#'
#' * Column indicating the sample period (parameter `period`).
#' * Column indicating the household ID (parameter `hid`)
#' * Column containing the household sample weights (parameter `weights`);
#' * Columns by which population was stratified during the sampling process
#'   (parameter: `strata`).
#'
#' As methods either the either the rescaled bootstrap for stratified 
#' multistage sampling, presented by J. Preston (2009) (`method = "Preston"`) 
#' or the Rao-Wu boostrap by J. N. K. Rao and C. F. J. Wu (1988) (`method = "Rao-Wu"`)
#' are supported.
#' \cr
#' For single stage sampling design a column the argument `totals` is optional,
#' meaning that a column of the number of PSUs at the first stage does not need
#' to be supplied. The number of PSUs is calculated and added to `dat` using
#' `strata` and `weights`. By setting `cluster` to NULL single stage sampling
#' design is always assumed and
#' if `strata` contains of multiple column names the combination of all those
#' column names will be used for stratification.
#' \cr
#' In the case of multi stage sampling design the argument `totals` needs to be
#' specified and needs to have the same number of arguments as `strata`.
#' \cr
#' If `cluster` is `NULL` or does not contain `hid` at the last stage, `hid`
#' will automatically be used as the final cluster. If, besides `hid`,
#' clustering in additional stages is specified the number of column names in
#' `strata` and `cluster` (including `hid`) must be the same. If for any stage
#' there was no clustering or stratification one can set "1" or "I" for this
#' stage.
#' \cr
#' For example `strata=c("REGION","I"),cluster=c("MUNICIPALITY","HID")` would
#' speficy a 2 stage sampling design where at the first stage the municipalities
#' where drawn stratified by regions
#' and at the 2nd stage housholds are drawn in each municipality without
#' stratification.
#' \cr
#' Bootstrap replicates are drawn for each survey period consecutively (`period`) using the
#' function [rescaled.bootstrap].
#' Bootstrap replicates are drawn consistently in the way that in each `period` and
#' sampling stage always \eqn{\lfloor n/2 \rfloor} clusters are selected in each strata.
#' \cr
#' This ensures that the bootstrap replicates follow the same logic as the
#' sampled households, making the bootstrap replicates more comparable to the
#' actual sample units.
#' \cr
#' If `split` ist set to `TRUE` and `pid` is specified, the bootstrap replicates
#' are carried forward using the personal identifiers instead of the household
#' identifier.
#' This takes into account the issue of a houshold splitting up.
#' Any person in this new split household will get the same bootstrap replicate
#' as the person that has come from an other household in the survey.
#' People who enter already existing households will also get the same bootstrap
#' replicate as the other households members had in the previous periods.
#' \cr
#' `already.selected` can be specified in order to construct bootstrap replicates
#' considering that already drawn bootstrap replicates from a previous `period`
#' exist for the same survey. See [get.selection()] for more explanations and examples.
#'
#' @return Returns a data.table containing the original data as well as the
#'   number of `REP` columns containing the bootstrap replicates for each
#'   repetition.\cr
#'   The columns of the bootstrap replicates are by default labeled "w*Number*"
#'   where *Number* goes from 1 to `REP`. If the column names of the bootstrap
#'   replicates should start with a different character or string the parameter
#'   `boot.names` can be used.
#'
#' @seealso [`data.table`][data.table::data.table] for more information on
#'   data.table objects.
#'
#' @references 
#' Preston, J. (2009). Rescaled bootstrap for stratified multistage
#'   sampling. Survey Methodology. 35. 227-234.
#' Rao, J. N. K., and C. F. J. Wu. (1988). Resampling Inference with Complex Survey Data.
#'  Journal of the American Statistical Association 83 (401): 231–41.
#' @author Johannes Gussenbauer, Alexander Kowarik, Statistics Austria
#'
#' @examples
#' 
#' library(surveysd)
#' library(data.table)
#' setDTthreads(1)
#' set.seed(1234)
#' eusilc <- demo.eusilc(n = 2, prettyNames = TRUE)
#'
#' ## draw replicates without stratification or clustering
#' dat_boot <- draw.bootstrap(eusilc, REP = 1, weights = "pWeight",
#'                            period = "year")
#'
#' ## use stratification w.r.t. region and clustering w.r.t. households
#' dat_boot <- draw.bootstrap(
#'   eusilc, REP = 1, hid = "hid", weights = "pWeight",
#'   strata = "region", period = "year")
#'
#' ## stratification by multiple variables
#' dat_boot <- draw.bootstrap(
#'   eusilc, REP = 1, hid = "hid", weights = "pWeight",
#'   strata = c("region", "hsize"), period = "year")
#'
#'
#' # create split households
#' eusilc[, pidsplit := pid]
#' year <- eusilc[, unique(year)]
#' year <- year[-1]
#' leaf_out <- c()
#' for(y in year) {
#' split.person <- eusilc[
#'   year == (y-1) & !duplicated(hid) & !(hid %in% leaf_out),
#'   sample(pid, 20)
#' ]
#' overwrite.person <- eusilc[
#'   (year == (y)) & !duplicated(hid) & !(hid %in% leaf_out),
#'   .(pid = sample(pid, 20))
#' ]
#' overwrite.person[, c("pidsplit", "year_curr") := .(split.person, y)]
#' 
#' eusilc[overwrite.person, pidsplit := i.pidsplit,
#'        on = .(pid, year >= year_curr)]
#' 
#' # assign new PID to avoid duplicates in pidsplit
#' eusilc[overwrite.person, change_pid := TRUE, 
#'        on = .(pid = pidsplit, year >= year_curr)]
#' eusilc[change_pid == TRUE, pidsplit := as.integer(paste0(hid, 99))]
#' eusilc[, change_pid := NULL]
#' 
#' leaf_out <- c(leaf_out,
#'               eusilc[pid %in% c(overwrite.person$pid,
#'                                 overwrite.person$pidsplit),
#'                      unique(hid)])
#' }
#'
#' dat_boot <- draw.bootstrap(
#'   eusilc, REP = 1, hid = "hid", weights = "pWeight",
#'   strata = c("region", "hsize"), period = "year", split = TRUE,
#'   pid = "pidsplit")
#' # split households were considered e.g. household and
#' # split household were both selected or not selected
#' dat_boot[, data.table::uniqueN(w1), by = pidsplit][V1 > 1]
#' 
#'
#' @export draw.bootstrap
#'



draw.bootstrap <- function(
    dat, method = "Preston", REP = 1000, hid = NULL, weights, period = NULL, strata = NULL,
    cluster = NULL, totals = NULL, single.PSU = c("merge", "mean"), boot.names =
      NULL, split = FALSE, pid = NULL, seed = NULL, already.selected = NULL) {
  
  occurence_first_period <- V1 <- strata_i <- . <-
    hid_survey_segment_help <- N <- keep <- draw_randomly <- hid_orig <- NULL
  
  if (method == "Rao-Wu") {
    warning("The 'Rao-Wu' method should only be used when the first stage sampling is conducted without replacement. Ensure that your sampling design follows this requirement.")
  }
  
  # INPUT CHECKING
  if (is.data.frame(dat)) {
    dat <- as.data.table(dat)
  }else if (!is.data.table(dat)) {
    stop("dat must be a data.frame or data.table")
  }
  dat <- copy(dat)

  c.names <- colnames(dat)

  removeCols <- c()

  # check seed
  if(!is.null(seed)){
    check.input(seed, input.name = "seed", input.length=1, input.type="numeric")
    set.seed(seed)
  }

  # check REP
  check.input(REP, input.name = "REP", input.length=1, input.type="numeric",
              decimal.possible = FALSE)

  # check hid
  hidNULL <- is.null(hid)
  if (hidNULL) {
    hid <- generateRandomName(20, colnames(dat))
    dat[, c(hid) := 1:.N]
    removeCols <- c(removeCols, hid)
  }
  check.input(hid, input.name = "hid", input.length=1, input.type="character",
              c.names = c.names)

  # check pid
  pidNULL <- is.null(pid)
  if (pidNULL) {
    pid <- generateRandomName(20, colnames(dat))
    dat[, c(pid) := paste(hid,1:.N, sep="."), by=c(hid)]
    removeCols <- c(removeCols, pid)
  }
  check.input(pid, input.name = "pid", input.length=1, input.type="character",
              c.names = c.names)

  # check weights
  check.input(weights, input.name = "weights", input.length=1, input.type="character",
              c.names = c.names, dat = dat, dat.column.type = "numeric")


  # check period
  periodNULL <- is.null(period)
  if (periodNULL) {
    period <- generateRandomName(20, colnames(dat))
    dat[, c(period) := 1]
    removeCols <- c(removeCols, period)
  }

  check.input(period, input.name = "period", input.length=1, input.type="character",
              c.names = c.names, dat = dat, dat.column.type = "numeric")

  # check design
  strataNULL <- is.null(strata)
  if (strataNULL) {
    strata <- generateRandomName(20, colnames(dat))
    dat[, (strata) := 1]
    removeCols <- c(removeCols, strata)
  }

  clusterNULL <- FALSE
  if (is.null(cluster)) {
    cluster <- hid
    clusterNULL <- TRUE
  } else {
    if (length(cluster) == 1) {
      if (cluster %in% c("1", "I")) {
        cluster <- hid
      }
    }
    if (!hid %in% cluster & !pid %in% cluster) {
      if(pidNULL==FALSE){
        cluster <- c(cluster, pid)
      }else{
        cluster <- c(cluster, hid)
      }
    }
  }

  if (!all(strata[!strata %in% c("1", "I")] %in% c.names)) {
    stop("Not all elements in strata are column names in dat")
  }
  if (any(!cluster[!cluster %in% c("1", "I")] %in% c.names)) {
    stop("Not all names in cluster are column names in dat")
  }
  if (any(!totals %in% c.names)) {
    stop("Not all names in totals are column names in dat")
  }

  # check for missing values
  spec.variables <- c(hid, weights, period, strata, cluster, totals, pid)
  spec.variables <- spec.variables[!spec.variables %in% c("1", "I")]
  dat.na <- dat[, mget(spec.variables)]
  dat.na <- sapply(dat.na, function(z) {
    any(is.na(z))
  })
  if (any(dat.na)) {
    stop("Missing values found in column(s): ",
         paste(names(dat.na[dat.na == TRUE]), collapse = ", "))
  }

  if (length(cluster) > 1) {
    if (length(cluster) != length(strata)) {
      stop("strata and cluster need to have the same number of stages!\n ",
           "Please use either '1' or 'I' if there was no clustering or ",
           "stratification in one of the stages.")
    }
  }else{

    if (length(strata) > 1) {
      if (any(c("1", "I") %in% strata)) {
        stop("When defining multiple strata variables for single stage",
             " sampling design\n none of them can be '1' or 'I'.")
      }
      strata_var_help <- generateRandomName(20, colnames(dat))
      dat[, c(strata_var_help) := do.call(paste, c(.SD, sep = "-")), .SDcols=c(strata)]
      strata <- strata_var_help
      removeCols <- c(removeCols, strata)
    }
  }

  # check single.PSUs
  single.PSU <- single.PSU[1]
  if (is.null(single.PSU) || !single.PSU %in% c("merge", "mean")) {
    warning("single.PSU was not set to either 'merge' or 'mean'!\n Bootstrap",
            " replicates for single PSUs cases will be missing!")
    single.PSU <- FALSE
  }
  
  # check boot.names
  if (!is.null(boot.names)) {
    if (!grepl("^[[:alpha:]]", boot.names)) {
      stop("boot.names must start with an alphabetic character")
    }
  }
  
  # check split and pid
  if (is.null(split)) {
    stop("split needs to be logical")
  }
  if (!is.logical(split)) {
    stop("split needs to be logical")
  }
  if (split) {
    if (!is.character(pid) | pidNULL == TRUE) {
      stop("when split is TRUE pid must be specified")
    } 
    # check if pid is unique in each household and period
    unique.pid <- dat[,uniqueN(pid)==.N, by=c(period, hid),
                      env = list(pid = pid)]
    unique.pid <- unique.pid[V1==FALSE]
    if (nrow(unique.pid) > 0) {
      stop("pid is not unique in each household for each period")
    }
  }
  
  if (is.null(totals)) {
    if (length(cluster) == 1) {
      
      totals <- generateRandomName(20, existingNames = colnames(dat))
      fpc.strata <- strata[!strata %in% c("I", "1")] # nolint
      dat[,c(totals) := sum(weights[!duplicated(hid)]),
          by = c(fpc.strata, period),
          env = list(weights = weights,
                     hid = hid)]
      removeCols <- c(removeCols, totals)
    } else {
      
      stop("For multistage sampling the number of PSUs at each level needs to ",
           "be specified!")
    }
    
  } else {
    
    if (length(totals) != length(strata)) {
      stop("totals must be specified for each stage")
    }
    if (any(!totals %in% c.names)) {
      stop("Not all elements in totals are column names in dat")
    }
    if (!any(unlist(dat[, lapply(.SD, is.numeric), .SDcols = c(totals)]))) {
      stop("Not all elements in totals are numeric columns in dat")
    }
  }
  
  # check for each stage that PSUs are not in mutiple strata
  for(i in seq_along(strata)){
    if(!strata[i]%in%c("1","I") & !cluster[i]%in%c("1","I")){
      countMultiple <- dat[,uniqueN(strata_i), by=c(cluster[i], period),
                           env = list(strata_i = strata[i])]
      countMultiple <- countMultiple[V1>1]
      if(nrow(countMultiple)>0){
        stop("Some sampling units in ",cluster[i]," occur in multiple strata of ",strata[i])
      }
    }
  }
  
  # define sample design
  strata_design <- paste(strata, collapse = ">")
  cluster_design <- paste(cluster, collapse = ">")
  totals_design <- paste(totals, collapse = ">")
  
  if (is.null(boot.names)) {
    w.names <- paste0("w", 1:REP)
  } else{
    w.names <- paste0(boot.names, 1:REP)
  }
  
  # calculate bootstrap replicates
  periods <- sort(unique(dat[[period]]))
  dat_selection_prev <- NULL
  if(!is.null(already.selected)){
    dat_selection_prev <- already.selected
  }
  for(p in periods){

    dat_boot <- rescaled.bootstrap(dat[.(p),on=c(period)],
                                   method = method, 
                                   REP = REP, strata = strata_design, cluster = cluster_design,
                                   fpc = totals_design, single.PSU = single.PSU, return.value = c("replicates","selection"),
                                   already.selected = dat_selection_prev,
                                   run.input.checks = FALSE)
    dat[period == p, c(w.names) := dat_boot$replicates, env = list(period = period)]
    dat_selection_prev <- dat_boot$selection
    rm(dat_boot)
  }
  
  # respect split households
  if (split) {
    dat <- generate.HHID(dat, period = period, pid = pid, hid = hid)
    
    help_survey_grouping <- function(x){
      x_holes <- which(diff(x)>1)
      x_groups <- diff(c(0,x_holes,length(x)))
      x_groups <- rep(1:length(x_groups),times=x_groups)
      return(x_groups)
    }
    dat[,hid_survey_segment_help := help_survey_grouping(period), by=c(hid),
        env = list(period = period)]
    dat[,occurence_first_period := min(period), by=c(hid,"hid_survey_segment_help"),
        env = list(period = period)]
    # dat[, hid_survey_segment_help := help_survey_grouping(get(period)), by = hid]
    # dat[, occurence_first_period := min(get(period)), 
    #     by = .(hid, hid_survey_segment_help)]
    
    dat.first.occurence <- unique(dat[period==occurence_first_period, .SD,
                                      .SDcols = c(hid,"hid_survey_segment_help", w.names),
                                      env = list(period = period)])
    dupl.HID <- dat.first.occurence[,.N, by= c(hid,"hid_survey_segment_help")][N>1]
    if(nrow(dupl.HID) > 0){
      # duplicated HID
      # 2 or more HID started in same survey year ~ period and merged together
      # choose 1 row randomly
      dat.first.occurence[, keep := TRUE]
      hid_set <- dupl.HID[[hid]]
      dat.first.occurence[dupl.HID, draw_randomly := TRUE, on = c(hid,"hid_survey_segment_help")]
      dat.first.occurence[draw_randomly == TRUE, keep := sample(c(rep(TRUE,1),rep(FALSE,.N-1))), by=.(hid), env = list(hid = hid)]
      dat.first.occurence <- dat.first.occurence[keep == TRUE]
      dat.first.occurence[, keep := NULL]
    }
    
    # select_cols <- c(hid, "hid_survey_segment_help", w.names)
    # dat.first.occurence <- unique(
    #   dat[get(period) == occurence_first_period, 
    #       ..select_cols] 
    # )
    
    dat[, c(w.names) := NULL]
    dat <- merge(dat, dat.first.occurence, by = c(hid, "hid_survey_segment_help"), all.x = TRUE)
    
    # check if any missing values occur
    if(any(is.na(dat[[w.names[1]]]))){
      stop("Missing values generated while dealing with split households.\nPlease report bug in ",
           "https://github.com/statistikat/surveysd")
    }
    if(nrow(dat[, .N, by = c(pid, period)][N > 1]) > 1){
      stop("Duplicated ", pid," generated while dealing with split households.\nPlease report bug in ",
           "https://github.com/statistikat/surveysd")
    }
    
    dat[, c("occurence_first_period","hid_survey_segment_help") := NULL]
    dat[, hid := hid_orig, env = list(hid = hid,
                                      hid_orig = paste0(hid,"_orig"))]
    # dat[, (hid) := paste0(get(hid), "_orig")]
    dat[, c(paste0(hid, "_orig")) := NULL]
  }
  
  
  if (length(removeCols) > 0) {
    dat[, c(removeCols) := NULL]
  }

  if (periodNULL) {
    period <- NULL
  }
  if (hidNULL) {
    hid <- NULL
  }
  if (clusterNULL){
    cluster <- NULL
  }
  if (strataNULL){
    strata <- NULL
  }
  if (pidNULL){
    pid <- NULL
  }
  
  setattr(dat, "weights", weights)
  setattr(dat, "period", period)
  setattr(dat, "b.rep", w.names)
  setattr(dat, "hid", hid)
  setattr(dat, "pid", pid)
  setattr(dat, "cluster", cluster)
  setattr(dat, "strata", strata)
  setattr(dat, "totals", totals)
  
  
  return(dat)
}

#################################
# test calc.stError()
#
#Sys.unsetenv("R_TESTS")

context("calc.stError()")
library(surveysd)
library(laeken)
library(data.table)

eusilc <- surveysd:::demo.eusilc()
eusilc <- eusilc[!db040 %in% c("Vienna", "Lower Austria", "Upper Austria")]

eusilc <- draw.bootstrap(eusilc, REP = 2, hid = "db030", weights = "db090",
                         period = "year", strata = "db040")
eusilc <- recalib(eusilc, conP.var = c("rb090", "age"),
                  conH.var = c("db040", "hsize"))

# test input parameter
test_that("test para - data", {
  expect_error(calc.stError(as.matrix(eusilc),
                            var = "povmd60", group = c("rb090", "db040")),
               "dat must be a data.frame or data.table")
  expect_error(
    calc.stError(eusilc, var = "povmd60", group = c("rb090", "db040")),
    NA
  )
})

test_that("test para - weights, b.weights, year and group", {
  expect_error(
    calc.stError(
      eusilc, b.weights = "a", var = "povmd60", group = c("rb090", "db040")),
    "column(s) 'a' specified in 'b.weights' not found in dat", fixed = TRUE
  )
  expect_error(
    calc.stError(eusilc, b.weights = 1:2, var = "povmd60",
                 group = c("rb090", "db040")),
    "'b.weights' must be of type character"
  )

  expect_error(
    calc.stError(
      eusilc, weights = "db090s", var = "povmd60", group = c("rb090", "db040")),
    "column(s) 'db090s' specified in 'weights' not found in dat", fixed = TRUE)
  expect_error(
    calc.stError(
      eusilc, period = "years", var = "povmd60", group = c("rb090", "db040")),
    "column(s) 'years' specified in 'years' not found in dat", fixed = TRUE)

  expect_error(
    calc.stError(
      eusilc, var = "povmd60", group = c("rb090s", "db040")),
    "Not all elements on group are column names in dat")

  eusilc.est <- calc.stError(
    eusilc, var = "povmd60", group = list("rb090", "db040", c("rb090", "db040"),
                                          c("hsize", "age")))
  ngroups <- eusilc[, uniqueN(rb090) + uniqueN(db040) +
                      uniqueN(paste(rb090, db040)) +
                      uniqueN(paste(hsize, age))]
  expect_true(
    nrow(
      unique(eusilc.est$Estimates, by = c("rb090", "db040", "hsize", "age"))
    ) == ngroups + 1)

  expect_error(calc.stError(
    eusilc, weights = "db090", b.weights = paste0("w", 1:2), period = "year",
    var = "povmd60", group = list("rb090", "db040", c("rb090", "db040"),
                                  c("hsize", "age"))), NA)
})


test_that("test para - period.diff, period.mean", {

  expect_error(calc.stError(
    eusilc, var = "povmd60", group = c("rb090", "db040")), NA)
  expect_warning(
    calc.stError(
      eusilc, var = "povmd60", group = c("rb090", "db040"), period.mean = 4),
    "period.mean must be odd - average over periods will not be calculated")

  diff.warning <- capture_warnings(calc.stError(
    eusilc, var = "povmd60", group = c("rb090", "db040"),
    period.diff = "2015-2008"
  ))
  expect_true(diff.warning[1] == paste0(
    "Removing 2015-2008 from period.diff -",
    " period(s) not present in column year\n"))
  expect_true(diff.warning[2] == "No differences will be calculated\n")

  expect_warning(
    calc.stError(
      eusilc, var = "povmd60", group = c("rb090", "db040"),
      period.diff = c("2015-2008", "2016-2011")),
    paste0("Removing 2015-2008 from period.diff - period(s) not",
           " present in column year"),
    fixed = TRUE)

  expect_warning(
    calc.stError(
      eusilc, var = "povmd60", group = c("rb090", "db040"),
      period.diff = c("2015-2010", "2016-2011"), period.mean = 3),
    "Cannot calculate differences between periods 2015 and 2010 over 3 periods."
  )

  expect_error(calc.stError(
    eusilc, var = "povmd60", group = c("rb090", "db040"),
    period.diff = c("2015-2011", "2016-2012"), period.mean = 3), NA)
})

test_that("test para - bias, size.limit, cv.limit, p", {

  expect_error(
    calc.stError(eusilc, var = "povmd60", group = c("rb090", "db040"),
                 bias = "FALSE"),
    "'bias' must be of type logical")
  eusilc.bias <- calc.stError(
    eusilc, weights = "db090", b.weights = paste0("w", 1:2), period = "year",
    var = "povmd60", group = c("rb090", "db040"), bias = TRUE)
  expect_true("mean_povmd60" %in% colnames(eusilc.bias$Estimates))

  expect_error(
    calc.stError(
      eusilc, var = "povmd60", group = c("rb090", "db040"), size.limit = "10"),
    "'size.limit' must be of type numeric")
  expect_error(
    calc.stError(
      eusilc, var = "povmd60", group = c("rb090", "db040"), size.limit = 1:2),
    "'size.limit' must have length 1")
  expect_error(calc.stError(
    eusilc, var = "povmd60", group = c("rb090", "db040"), size.limit = 50), NA)

  expect_error(
    calc.stError(
      eusilc, var = "povmd60", group = c("rb090", "db040"), cv.limit = 1:10),
    "'cv.limit' must have length 1")
  expect_error(
    calc.stError(
      eusilc, var = "povmd60", group = c("rb090", "db040"), cv.limit = "1"),
    "'cv.limit' must be of type numeric")
  expect_error(calc.stError(
    eusilc, var = "povmd60", group = c("rb090", "db040"), cv.limit = 20), NA)


  expect_error(
    calc.stError(
      eusilc, var = "povmd60", group = c("rb090", "db040"), p = ".5"),
    "'p' must be of type numeric")
  expect_error(
    calc.stError(
      eusilc, var = "povmd60", group = c("rb090", "db040"), p = c(.1, .7, 1.2)),
    "Values in p must be between 0 and 1")
  expect_error(calc.stError(
    eusilc, var = "povmd60", group = c("rb090", "db040"), p = c(.1, .7, .9)),
    NA)
})

test_that("test return", {
  eusilc.est <- calc.stError(
    eusilc, var = "povmd60", group = c("rb090", "db040"))
  eusilc.comp <- rbindlist(
    list(
      eusilc[, .(V1 = weightedRatio(povmd60, db090), N_true = sum(db090)),
             by = year],
      eusilc[, .(V1 = weightedRatio(povmd60, db090), N_true = sum(db090)),
             by = list(year, rb090)],
      eusilc[, .(V1 = weightedRatio(povmd60, db090), N_true = sum(db090)),
             by = list(year, db040)]),
    use.names = TRUE, fill = TRUE
  )
  eusilc.comp <- merge(eusilc.comp, eusilc.est$Estimates[, .(year, rb090, db040,
                                                             N, val_povmd60)])
  expect_true(nrow(eusilc.comp[V1 != val_povmd60]) == 0)
  expect_true(nrow(eusilc.comp[N_true != N]) == 0)
})

#################################
# test recalib()
#

context("recalib()")
library(surveysd)
library(laeken)
library(data.table)

eusilc <- surveysd:::demo.eusilc(n = 4)
eusilc <- draw.bootstrap(eusilc, REP = 2, hid = "db030", weights = "db090",
                         period = "year", strata = "db040")

# test input parameter
test_that("test para - data", {
  expect_error(
    recalib(as.matrix(eusilc), conP.var = "rb090", conH.var = "db040"),
    "dat must be a data.frame or data.table")
  expect_error(recalib(
    eusilc, conP.var = "rb090", conH.var = "db040"), NA)
  expect_error(recalib(
    eusilc, conP.var = c("rb090", "age"), conH.var = c("db040", "hsize")), NA)
})

test_that("test para - REP", {
  expect_error(
    recalib(eusilc, b.rep = "a", conP.var = "rb090", conH.var = "db040"),
    "column(s) 'a' specified in 'b.rep' not found in dat", fixed = TRUE)
  expect_error(
    recalib(eusilc, b.rep = 1:2, conP.var = "rb090", conH.var = "db040"),
    "'b.rep' must be of type character")
})

test_that("test para - hid, weights and period", {
  expect_error(
    recalib(eusilc, hid = "db030s", conP.var = "rb090", conH.var = "db040"),
    "column(s) 'db030s' specified in 'hid' not found in dat", fixed = TRUE)
  expect_error(
    recalib(eusilc, weights = "db090s", conP.var = "rb090", conH.var = "db040"),
    "column(s) 'db090s' specified in 'weights' not found in dat", fixed = TRUE)
  expect_error(
    recalib(eusilc, period = "years", conP.var = "rb090", conH.var = "db040"),
    "column(s) 'years' specified in 'years' not found in dat", fixed = TRUE)
})

test_that("test para - conP.var conH.var", {

  # check single values
  expect_error(
    recalib(eusilc, conP.var = "rb090s", conH.var = "db040"),
    "Not all elements in conP.var are column names in dat")
  expect_error(
    recalib(eusilc, conP.var = "rb090", conH.var = "db040s"),
    "Not all elements in conH.var are column names in dat")

  # check multiple values through list
  expect_error(
    recalib(eusilc, conP.var = list("rb090s", "age"), conH.var = "db040"),
    "Not all elements in conP.var are column names in dat")
  expect_error(
    recalib(eusilc, conP.var = list("rb090"),
            conH.var = list("db040s", "hsize")),
    "Not all elements in conH.var are column names in dat")

  expect_error(recalib(
    eusilc, conP.var = NULL, conH.var = "db040"), NA)
  expect_error(recalib(
    eusilc, conP.var = "rb090", conH.var = NULL), NA)
  expect_error(recalib(
      eusilc, conP.var = NULL, conH.var = NULL), NA)
  expect_error(recalib(
    eusilc, conP.var = list("rb090", "age"), conH.var = "db040"), NA)
  expect_error(recalib(
    eusilc, conP.var = list("rb090", c("age", "rb090")), conH.var = "db040"),
    NA)
  expect_error(recalib(
    eusilc, conP.var = list("rb090"), conH.var = list("db040", "hsize")), NA)
})


test_that("test para - conP conH", {

  conP1 <- xtabs(db090 ~ age + year, data = eusilc)
  conP2 <- xtabs(db090 ~ rb090 + year, data = eusilc)
  # conP2 <- xtabs(db090 ~ rb090 + db040 + year, data = eusilc)

  conH1 <- xtabs(db090 ~ hsize + year,
                 data = eusilc[!duplicated(paste(db030, year))])
  conH2 <- xtabs(db090 ~ db040 + year,
                 data = eusilc[!duplicated(paste(db030, year))])

  expect_error(
    recalib(eusilc, conP.var = "rb090", conH.var = NULL, conP = list(conP2)),
    paste("contingency table for rb090 was supplied through parameter",
          "conP AND conP.var"))
  expect_error(
    recalib(eusilc, conP.var = NULL, conH.var = "db040", conH = list(conH2)),
    paste("contingency table for db040 was supplied through parameter",
          "conH AND conH.var"))

  expect_error(recalib(
    eusilc, conP.var = NULL, conH.var = "db040", conH = list(conH1)), NA)
  expect_error(recalib(
    eusilc, conP.var = "rb090", conH.var = NULL, conP = list(conP1)), NA)
  expect_error(recalib(
    eusilc, conP.var = NULL, conH.var = NULL, conP = list(conP1),
    conH = list(conH1)), NA)
})

test_that("test return", {
  dat.calib <- recalib(
    eusilc, hid = "db030", weights = "db090", b.rep = paste0("w", 1:2), period =
      "year", conP.var = c("rb090", "age"), conH.var = c("db040", "hsize"))
  rb090.compare <- dat.calib[, lapply(.SD, sum), by = list(year, rb090),
                             .SDcols = c("db090", paste0("w", 1:2))]
  expect_true(all(!is.na(rb090.compare[, .SD, .SDcols = paste0("w", 1:2)])))
  rb090.compare <- rb090.compare[, lapply(
    .SD,
    function(z) {
      abs(db090 - z) / db090
    }
    ), .SDcols = paste0("w", 1:2)]
  expect_true(all(rb090.compare < formals(recalib)$epsP))

  db040.compare <- dat.calib[, lapply(
    .SD,
    function(z) {
      sum(z[!duplicated(db030)])
    }
    ), by = list(year, db040), .SDcols = c("db090", paste0("w", 1:2))]
  db040.compare <- db040.compare[, lapply(
    .SD,
    function(z) {
      abs(db090 - z) / db090
    }
  ), .SDcols = paste0("w", 1:2)]
  expect_true(all(db040.compare < formals(recalib)$epsH))
})

#################################
# test plot.surveysd()
#

context("plot.surveysd()")
library(surveysd)
library(laeken)
library(data.table)
library(ggplot2)

eusilc <- surveysd:::demo.eusilc()
eusilc <- eusilc[!db040 %in% c("Vienna", "Lower Austria", "Upper Austria")]

eusilc <- draw.bootstrap(eusilc, REP = 2, hid = "db030", weights = "db090",
                         period = "year", strata = "db040")
eusilc <- recalib(eusilc, conP.var = c("rb090", "age"),
                  conH.var = c("db040", "hsize"))
res <- calc.stError(eusilc, var = "povmd60", group =
                      list("rb090", "db040", c("rb090", "db040"), "hsize"))

# test input parameter
test_that("test para - variable and type", {

  expect_error(plot(res), NA)

  expect_error(plot(res, variable = "povmd60s"),
               "No results for povmd60s present in the data!")
  expect_error(plot(res, variable = "povmd60"), NA)

  expect_error(plot(res, variable = "povmd60", type = "summarys"),
               "Parameter type can only take values 'summary' or 'grouping'!")
  expect_error(plot(res, variable = "povmd60", type = "summary"), NA)

  expect_error(plot(res, variable = "povmd60", type = "grouping"),
               "Paramter 'groups' cannot be NULL if type='grouping'!")
  expect_error(plot(res, variable = "povmd60", type = "grouping",
                    groups = "rb090"), NA)
  expect_error(
    plot(res, variable = "povmd60", type = "grouping", groups = "rb090s"),
    "Variables in 'groups' must contain variables from x$params$group!",
    fixed = TRUE)

})

test_that("test para - groups and sd.type", {
  expect_error(plot(res, variable = "povmd60", type = "grouping", groups =
                      "rb090", sd.type = "dots"),
               "Parameter 'sd.type' can only take values 'ribbon' or 'dot'!")
  expect_error(plot(res, variable = "povmd60", type = "grouping", groups =
                      "rb090", sd.type = "ribbon"), NA)

  expect_error(plot(res, variable = "povmd60", type = "grouping", groups =
                      c("db040", "rb090"), sd.type = "ribbon"), NA)
  expect_error(plot(res, variable = "povmd60", type = "grouping", groups =
                      c("db040", "rb090"), sd.type = "dot"), NA)
  expect_error(
    plot(res, variable = "povmd60", type = "grouping", groups =
           c("db040", "hsize"), sd.type = "ribbon"),
    "No results for the combination of db040 and hsize present in the data")
})

res <- calc.stError(
  eusilc, weights = "db090", b.weights = paste0("w", 1:2), period = "year",
  var = "povmd60", group = list("rb090", "db040", c("rb090", "db040"), "hsize"),
  period.mean = NULL)

test_that("test para - without period.mean", {
  expect_error(plot(res, variable = "povmd60", type = "grouping", groups =
                      "rb090", sd.type = "dots"),
               "Parameter 'sd.type' can only take values 'ribbon' or 'dot'!")
  expect_error(plot(res, variable = "povmd60", type = "grouping", groups =
                      "rb090", sd.type = "ribbon"), NA)

  expect_error(plot(res, variable = "povmd60", type = "grouping", groups =
                      c("db040", "rb090"), sd.type = "ribbon"), NA)
  expect_error(plot(res, variable = "povmd60", type = "grouping", groups =
                      c("db040", "rb090"), sd.type = "dot"), NA)
  expect_error(
    plot(res, variable = "povmd60", type = "grouping", groups =
           c("db040", "hsize"), sd.type = "ribbon"),
    "No results for the combination of db040 and hsize present in the data")
})

#################################
# test draw.bootstrap()
#

context("draw.bootstrap()")
library(surveysd)
library(laeken)
library(data.table)

eusilc <- surveysd:::demo.eusilc(n = 3)
eusilc[, N.households := sum(db090[!duplicated(db030)]), by = .(year, db040)]
eusilc[!duplicated(db030), N.households.error := sum(db090),
       by = .(year, db040)]
eusilc[, N.households.all := sum(db090[!duplicated(db030)]), by = .(year)]

# test input parameter
test_that("test para - data", {
  expect_error(
    draw.bootstrap(
      as.matrix(eusilc), REP = 2, hid = "db030", weights = "db090", period =
        "year", strata = "db040"),
    "dat must be a data.frame or data.table")
  expect_error(draw.bootstrap(
    eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
    strata = "db040"), NA)
})

test_that("test para - REP", {
  expect_error(
    draw.bootstrap(
      eusilc, REP = "a", hid = "db030", weights = "db090", period = "year",
      strata = "db040"),
    "'REP' must be of type numeric")
  expect_error(
    draw.bootstrap(
      eusilc, REP = 1:2, hid = "db030", weights = "db090", period = "year",
      strata = "db040"),
    "'REP' must have length 1")
})

test_that("test para - hid, weights and period", {
  expect_error(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030s", weights = "db090", period = "year",
      strata = "db040"),
    "column(s) 'db030s' specified in 'hid' not found in dat", fixed = TRUE)
  expect_error(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090s", period = "year",
      strata = "db040"),
    "column(s) 'db090s' specified in 'weights' not found in dat", fixed = TRUE)
  expect_error(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090", period = "years",
      strata = "db040"),
    "column(s) 'years' specified in 'period' not found in dat", fixed = TRUE)

  eusilc[, year.char := as.character(year)]
  expect_error(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090", period = "year.char",
      strata = "db040"),
    "column(s) 'year.char' in parameter 'period' must correspond to numeric column(s) in dat", fixed = TRUE)
})

test_that("test para - strata, cluster and totals", {
  expect_error(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
      strata = "db0400"),
    "Not all elements in strata are column names in dat")
  expect_error(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
      strata = "db040", cluster = "a"),
    "Not all names in cluster are column names in dat")
  expect_error(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
      strata = c("db040", "rb090"), cluster = c("db040", "db030"),
      totals = c("N.households")),
    "totals must be specified for each stage")
  expect_error(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
      strata = c("db040"), cluster = c("db040", "db030")),
    paste0("strata and cluster need to have the same number of stages!\n ",
           "Please use either '1' or 'I' if there was no clustering or ",
           "stratification in one of the stages."),
    fixed = TRUE)
  expect_error(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
      strata = c("db040"), cluster = c("db030"), totals = c("N.something")),
    "Not all names in totals are column names in dat")
  expect_error(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
      strata = c("db040"), cluster = c("db030"),
      totals = c("N.households.error")),
    "Missing values found in column(s): N.households.error", fixed = TRUE)

  expect_error(draw.bootstrap(
    eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
    strata = c("db040"), cluster = c("1")), NA)
  expect_error(draw.bootstrap(
    eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
    strata = c("db040"), cluster = c("I")), NA)
  expect_error(draw.bootstrap(
    eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
    strata = NULL), NA)
  expect_error(draw.bootstrap(
    eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
    strata = "I"), NA)
  expect_error(draw.bootstrap(
    eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
    strata = "1"), NA)
})


# these are some longer tests
if (Sys.getenv("SURVEYSD_ADDITIONAL_TEST") == "TRUE") {
  test_that("test para - bootnames, split and pid", {

    expect_error(
      draw.bootstrap(
        eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
        strata = "db040", split = "FALSE"),
      "split needs to be logical")
    expect_error(
      draw.bootstrap(
        eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
        strata = "db040", split = TRUE),
      "when split is TRUE pid must be specified")

    eusilc[, rb030error := rb030[1], by = list(year, db030)]
    expect_error(
      draw.bootstrap(
        eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
        strata = "db040", split = TRUE, pid = "rb030error"),
      "pid is not unique in each household for each period")

    # create split households
    eusilc[, rb030split := rb030]
    year <- eusilc[, unique(year)]
    year <- year[-1]
    leaf_out <- c()
    for (y in year) {
      split.person <- eusilc[
        year == (y - 1) & !duplicated(db030) & !db030 %in% leaf_out,
        sample(rb030, 20)]
      overwrite.person <- eusilc[
        year == (y) & !duplicated(db030) & !db030 %in% leaf_out,
        .(rb030 = sample(rb030, 20))]
      overwrite.person[, c("rb030split", "year_curr") := .(split.person, y)]

      eusilc[overwrite.person, rb030split := i.rb030split,
             on = .(rb030, year >= year_curr)]
      leaf_out <- c(leaf_out, eusilc[rb030 %in% c(overwrite.person$rb030,
                                                  overwrite.person$rb030split),
                                     unique(db030)])
    }
    eusilc.split <- draw.bootstrap(
      eusilc, REP = 10, hid = "db030", weights = "db090", period = "year",
      strata = "db040", split = TRUE, pid = "rb030split")
    eusilc.split <- eusilc.split[, lapply(.SD, uniqueN), by = rb030split,
                                 .SDcols = paste0("w", 1:10)]
    expect_true(all(eusilc.split[, .SD, .SDcols = paste0("w", 1:10)] == 1))

    expect_error(
      draw.bootstrap(
        eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
        strata = "db040", boot.names = "1"),
      "boot.names must start with an alphabetic character")
    expect_error(draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
      strata = "db040", boot.names = "weight"), NA)
  })
}


test_that("test para - single.PSU", {
  expect_warning(
    draw.bootstrap(
      eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
      strata = "db040", single.PSU = "something"),
    paste0("single.PSU was not set to either 'merge' or 'mean'!\n Bootstrap ",
           "replicates for single PSUs cases will be missing!"),
    fixed = TRUE)
})

test_that("test return", {
  dat.boot <- draw.bootstrap(
    eusilc, REP = 2, hid = "db030", weights = "db090", period = "year",
    strata = "db040")
  expect_true(ncol(dat.boot) == (2 + ncol(eusilc)))
  dat.unique <- unique(dat.boot[, mget(c("db030", paste0("w", 1:2)))])
  expect_true(nrow(dat.unique[, .N, by = db030][N > 2]) == 0)
  
  # check if any value is infinite
  expect_false(any(unlist(
    dat.boot[, lapply(
      .SD,
      function(z) {
        any(is.infinite(z))
      }),
      .SDcols = c(paste0("w", 1:2))
      ])))
  
  # check if sum of weights equals
  # numer of sampling units per strata
  tab_sums <- dat.boot[, lapply(
    .SD,
    function(w,db030) {
      s1 <- sum(w[!duplicated(db030)])
      s2 <- uniqueN(db030)
      abs(s1-s2)>1e-10
    },db030=db030),by=.(year,db040),
    .SDcols = c(paste0("w", 1:2))
  ]
  expect_false(any(unlist(tab_sums[,.SD,.SDcols=paste0("w", 1:2)])))
  
  expect_false(any(is.na(dat.boot[, .SD, .SDcols = c(paste0("w", 1:2))])))
  expect_true(all(dat.boot[, .SD, .SDcols = c(paste0("w", 1:2))] > 0))
})

#' @title Calibrate weights
#'
#' @description
#' Calibrate weights for bootstrap replicates by using iterative proportional
#' updating to match population totals on various household and personal levels.
#'
#'
#' @details
#' `recalib` takes survey data (`dat`) containing the bootstrap replicates
#' generated by [draw.bootstrap] and calibrates weights for each bootstrap
#' replication according to population totals for person- or household-specific
#' variables. \cr
#' `dat` must be household data where household members correspond to multiple
#' rows with the same household identifier. The data should at least containt
#' the following columns:
#'
#' * Column indicating the sample period;
#' * Column indicating the household ID;
#' * Column containing the household sample weights;
#' * Columns which contain the bootstrap replicates (see output of
#'   [draw.bootstrap]);
#' * Columns indicating person- or household-specific variables for which sample
#'   weight should be adjusted.
#'
#' For each period and each variable in `conP.var` and/or `conH.var` contingency
#' tables are estimated to get margin totals on personal- and/or
#' household-specific variables in the population.\cr
#' Afterwards the bootstrap replicates are multiplied with the original sample
#' weight and the resulting product ist then adjusted using [ipf()] to match the
#' previously calcualted contingency tables. In this process the columns of the
#' bootstrap replicates are overwritten by the calibrated weights.\cr
#'
#' @param dat either data.frame or data.table containing the sample survey for
#'   various periods.
#' @param hid character specifying the name of the column in `dat` containing
#'   the household ID.
#' @param weights character specifying the name of the column in `dat`
#'   containing the sample weights.
#' @param b.rep character specifying the names of the columns in `dat`
#'   containing bootstrap weights which should be recalibratet
#' @param period character specifying the name of the column in `dat` containing
#'   the sample period.
#' @param conP.var character vector containig person-specific variables to which
#'   weights should be calibrated or a list of such character vectors.
#'   Contingency tables for the population are calculated per `period` using
#'   `weights`. If a vector is supplied contingency tables will be calculated
#'   for each vector entry. If a list is supplied contingency tables will be
#'   calculated for each list entry. See examples for more details.
#' @param conH.var character vector containig household-specific variables to
#'   which weights should be calibrated or a list of such character vectors.
#'   Contingency tables for the population are calculated per `period` using
#'   `weights`. If a vector is supplied contingency tables will be calculated
#'   for each vector entry. If a list is supplied contingency tables will be
#'   calculated for each list entry. See examples for more details.
#' @param conP list or (partly) named list defining the constraints on person
#'   level.  The list elements are contingency tables in array representation
#'   with dimnames corresponding to the names of the relevant calibration
#'   variables in `dat`. If a numerical variable is to be calibrated, the
#'   respective list element has to be named with the name of that numerical
#'   variable. Otherwise the list element shoud NOT be named.
#' @param conH list or (partly) named list defining the constraints on
#'   household level.  The list elements are contingency tables in array
#'   representation with dimnames corresponding to the names of the relevant
#'   calibration variables in `dat`. If a numerical variable is to be
#'   calibrated, the respective list element has to be named with the name of
#'   that numerical variable. Otherwise the list element shoud NOT be named.

#' @param epsP numeric value specifying the convergence limit for `conP.var`
#'   or `conP`, see [ipf()].
#' @param epsH numeric value specifying the convergence limit for `conH.var`
#'   or `conH`, see [ipf()].
#' @param ... additional arguments passed on to function [ipf()] from this
#'   package.
#'
#' @return Returns a data.table containing the survey data as well as the
#'   calibrated weights for the bootstrap replicates. The original bootstrap
#'   replicates are overwritten by the calibrated weights. If calibration of a
#'   bootstrap replicate does not converge the bootsrap weight is not returned
#'   and numeration of the returned bootstrap weights is reduced by one.
#'
#'
#' @seealso [ipf()] for more information on iterative
#'   proportional fitting.
#'
#' @author Johannes Gussenbauer, Alexander Kowarik, Statistics Austria
#'
#' @examples
#' 
#' library(surveysd)
#' library(data.table)
#' setDTthreads(1)
#' set.seed(1234)
#' eusilc <- demo.eusilc(n = 3, prettyNames = TRUE)
#'
#' dat_boot <- draw.bootstrap(eusilc, REP = 1, hid = "hid",
#'                            weights = "pWeight",
#'                            strata = "region", period = "year")
#'
#' # calibrate weight for bootstrap replicates
#' dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region",
#'                           verbose = TRUE)
#'
#'
#' # calibrate on other variables
#' dat_boot_calib <- recalib(dat_boot, conP.var = c("gender", "age"),
#'                           conH.var = c("region", "hsize"), verbose = TRUE)
#'
#' # supply contingency tables directly
#' conP1 <- xtabs(pWeight ~ age + year, data = eusilc)
#' conP2 <- xtabs(pWeight ~ gender + year, data = eusilc)
#' conH1 <- xtabs(pWeight ~ region + year,
#'                data = eusilc[!duplicated(paste(hid,year))])
#' conH2 <- xtabs(pWeight ~ hsize + year,
#'                data = eusilc[!duplicated(paste(hid,year))])
#' 
#' conP <- list(conP1,conP2)
#' conH <- list(conH1,conH2)
#' dat_boot_calib <- recalib(dat_boot, conP.var = NULL,
#'                           conH.var = NULL, conP = conP,
#'                           conH = conH, verbose = TRUE)
#' 
#'
#' # calibrate on gender x age
#' dat_boot_calib <- recalib(dat_boot, conP.var = list(c("gender", "age")),
#'                           conH.var = NULL, verbose = TRUE)
#' 
#' # identical
#' conP1 <- xtabs(pWeight ~ age + gender + year, data = eusilc)
#' conP <- list(conP1)
#' dat_boot_calib <- recalib(dat_boot, conP.var = NULL,
#'                           conH.var = NULL, conP = conP,
#'                           conH = NULL, verbose = TRUE)
#'                                                      
#' @export recalib
#'

recalib <- function(
    dat, hid = attr(dat, "hid"), weights = attr(dat, "weights"), b.rep =
      attr(dat, "b.rep"), period = attr(dat, "period"), conP.var = NULL,
    conH.var = NULL, conP = NULL, conH = NULL, epsP = 1e-2, epsH = 2e-2, ...) {
  
  hidfactor <- calibWeight <- FirstPersonInHousehold_ <- verbose <-
    bound <- maxiter <- meanHH <- check_hh_vars <- allPthenH <-
    returnNA <- numericalWeighting <- conversion_messages <- maxIter <- 
    V1 <- NULL
  
  removeCols <- c()
  
  ##########################################################
  # define default values for ipf
  ipfDefaults <- formals(ipf)
  ipfDefaults <- ipfDefaults[!names(ipfDefaults) %in% names(formals(recalib))]
  ipfDefaults$numericalWeighting <- eval(parse(text = ipfDefaults$numericalWeighting))
  ellipsis <- list(...)
  # set these to FALSE by default
  ellipsis[["check_hh_vars"]] <- getEllipsis2("check_hh_vars",
                                              FALSE, ellipsis)
  ellipsis[["conversion_messages"]] <- getEllipsis2("conversion_messages",
                                                    FALSE, ellipsis)
  ellipsis[["verbose"]] <- getEllipsis2("verbose", TRUE, ellipsis)
  ellipsis <- lapply(names(ipfDefaults), function(z) {
    getEllipsis2(z, ipfDefaults[[z]], ellipsis)
  })
  names(ellipsis) <- names(ipfDefaults)
  
  ellipsisNames <- names(ellipsis)
  ellipsisContent <- paste0("ellipsis[['", ellipsisNames, "']]")
  eval(parse(text = paste(
    ellipsisNames,
    ellipsisContent, sep = "<-"
  )))
  
  ##########################################################
  # INPUT CHECKING
  if (is.data.frame(dat)) {
    dat <- as.data.table(dat)
  } else if (!is.data.table(dat)) {
    stop("dat must be a data.frame or data.table")
  }
  dat <- copy(dat)
  
  c.names <- colnames(dat)
  
  # check hid
  hidNULL <- is.null(hid)
  if (hidNULL) {
    hid <- generateRandomName(20, colnames(dat))
    dat[, c(hid) := 1:.N]
    removeCols <- c(removeCols, hid)
  }
  
  check.input(hid, input.name = "hid", input.length=1, input.type="character",
              c.names = c.names)
  
  # check weights
  check.input(weights, input.name = "weights", input.length=1, input.type="character",
              c.names = c.names, dat = dat, dat.column.type = "numeric")

  # check b.rep
  check.input(b.rep, "b.rep", input.type = "character", 
              c.names = c.names, dat = dat, dat.column.type = "numeric")
  
  if (any(!grepl("^[[:alpha:]]", b.rep)))
    stop("Column names of bootstrap replicates must start with alphabetic ",
         "character")
  
  # check conP.var
  if (!all(unique(unlist(conP.var)) %in% c.names)) {
    stop("Not all elements in conP.var are column names in dat")
  }
  
  # check conH.var
  if (!all(unique(unlist(conH.var)) %in% c.names)) {
    stop("Not all elements in conH.var are column names in dat")
  }
  
  # check period
  removeCols <- c()
  periodNULL <- is.null(period)
  if (periodNULL) {
    period <- generateRandomName(20, colnames(dat))
    dat[, c(period) := 1]
    removeCols <- c(removeCols, period)
  }
  
  check.input(period, period, input.length = 1, input.type = "character",
              c.names = c.names)
  
  
  ##########################################################
  
  # check conP and conH
  conPnames <- lapply(conP, function(z) {
    z <- names(dimnames(z))
    z[z != period]
  })
  conHnamesNumeric <- conPnamesNumeric <- NULL
  if(!is.null(names(conP))){
    conPnamesNumeric <- unique(names(conP))
    conPnamesNumeric <- conPnamesNumeric[conPnamesNumeric!=""]
  }
  conHnames <- lapply(conH, function(z) {
    z <- names(dimnames(z))
    z[z != period]
  })
  if(!is.null(names(conH))){
    conHnamesNumeric <- unique(names(conH))
    conHnamesNumeric <- conHnamesNumeric[conHnamesNumeric!=""]
  }
  
  if (!all(unlist(conPnames) %in% c.names)) {
    stop("Not all dimnames in conP are column names in dat")
  }
  if (!all(unlist(conHnames) %in% c.names)) {
    stop("Not all dimnames in conH are column names in dat")
  }
  
  
  if (!is.null(conH.var) | !is.null(conP.var) |
      !is.null(conP) | !is.null(conH)) {
    var.miss <- unlist(
      dat[, lapply(
        .SD,
        function(z) {
          sum(is.na(z))
        }
      ), .SDcols = c(unique(unlist(c(conH.var, conP.var,
                                     conPnames, conHnames))))])
    var.miss <- var.miss[var.miss > 0]
    if (length(var.miss) > 0) {
      stop("Missing values detected in column(s)", names(var.miss))
    }
  } else {
    message("recalib: conP.var, conH.var, conP and conH are all missing. ",
            "Only calibrating for the population totals")
  }
  
  
  
  # recode household and person variables to factor
  # improves runtime for ipf
  #
  vars <- c(period, unique(unlist(c(conP.var, conH.var, conPnames, conHnames))))
  vars.class <- unlist(lapply(dat[, mget(vars)], function(z) {
    z.class <- class(z)
    if (z.class[1] %in% c("labelled", "haven_labelled")) {
      z.class <- "factor"
    }
    return(z.class)
  }))
  # convert to factor
  for (i in seq_along(vars)) {
    if (vars.class[[vars[i]]] != "factor") {
      dt.eval("dat[,", vars[i], ":=as.factor(", vars[i], ")]")
    }
  }
  
  
  # calculate contingency tables
  for (p in seq_along(conP.var)) {
    existTab <- sapply(conPnames, setequal, y = conP.var[[p]])
    if (all(!existTab)) {
      formTab <- paste(weights, "~",
                       paste(c(period, conP.var[[p]]), collapse = "+"))
      conP <- c(conP, list(xtabs(formTab, data = dat)))
    }else{
      stop("contingency table for ", paste(conP.var[p], collapse = ", "),
           " was supplied through parameter conP AND conP.var")
    }
  }
  
  dat[, FirstPersonInHousehold_ := c(1L, rep(0, .N - 1)), by = c(hid, period)]
  for (h in seq_along(conH.var)) {
    existTab <- sapply(conHnames, setequal, y = conH.var[[h]])
    if (all(!existTab)) {
      formTab <- paste(weights, "~", paste(c(period, conH.var[[h]]),
                                           collapse = "+"))
      conH <- c(
        conH, list(xtabs(formTab, data = dat[FirstPersonInHousehold_ == 1])))
    } else {
      stop("contingency table for ", paste(conH.var[[h]], collapse = ", "),
           " was supplied through parameter conH AND conH.var")
    }
  }
  # define new Index
  dat[, hidfactor := .GRP, by = c(hid, period)]
  dat[, hidfactor := factor(hidfactor)]
  
  # calibrate weights to conP and conH
  select.var <- unique(c("hidfactor", weights, period,
                         unlist(c(conP.var, conH.var, conPnames, conHnames,
                                  conPnamesNumeric, conHnamesNumeric))))
  calib.fail <- c()
  
  for (g in b.rep) {
    dat[, g := g * weights, env = list(g = g, weights = weights)]
    
    # check if margins for bootstrap weights are always positive
    check.conP <- lapply(conP, function(z) {
      check.z <- dat[,sum(g), by=c(names(dimnames(z))), env = list(g = g)][V1 == 0]
      nrow(check.z) > 0
    })
    check.conH <- lapply(conH, function(z) {
      check.z <- dat[,sum(g), by=c(names(dimnames(z))), env = list(g = g)][V1 == 0]
      nrow(check.z) > 0
    })
    if (!is.null(conP) | !is.null(conH)) {
      if (any(unlist(c(check.conH, check.conP)))) {
        calib.fail <- c(calib.fail, g)
        set(dat, j = g, value = NA_real_)
      } else {
        set(dat, j = g, value = ipf(
          dat = copy(dat[, mget(c(g, select.var))]), conP = conP,
          conH = conH, verbose = verbose, epsP = epsP, epsH = epsH,
          w = g, bound = bound, maxIter = maxIter, meanHH = meanHH,
          hid = "hidfactor", check_hh_vars = check_hh_vars,
          allPthenH = allPthenH, returnNA = returnNA,
          numericalWeighting = numericalWeighting,
          conversion_messages = conversion_messages
        )[, calibWeight])
        if (dat[, any(is.na(get(g)))]) {
          calib.fail <- c(calib.fail, g)
        }
      }
    }
  }
  
  
  # paste warnings if calibration failed in some instances
  if (length(calib.fail) > 0) {
    
    dat[, c(calib.fail) := NULL]
    b.rep <- b.rep[!b.rep %in% calib.fail]
    
    if (length(b.rep) == 0) {
      cat("Calibration failed for all bootstrap replicates\n")
      cat("Returning no bootstrap weights\n")
    } else {
      cat("Calibration failed for bootstrap replicates", calib.fail, "\n")
      cat("Corresponding bootstrap replicates will be discarded\n")
      lead.char <- sub("[[:digit:]].*", "", b.rep[1])
      b.rep_new <- paste0(lead.char, seq_along(b.rep))
      setnames(dat, b.rep, b.rep_new)
      cat("Returning", length(b.rep), "calibrated bootstrap weights\n")
      b.rep <- b.rep_new
    }
  }
  
  
  dat[, c("hidfactor", "FirstPersonInHousehold_", removeCols) := NULL]
  
  # recode vars back to either integer of character
  vars.class <- vars.class[!vars %in% removeCols]
  vars <- vars[!vars %in% removeCols]
  for (i in seq_along(vars.class)) {
    if (vars.class[i] %in% c("integer", "numeric")) {
      dat[,vars := as.numeric(as.character(vars)), env = list(vars = vars[i])]
    } else if (vars.class[i] == "character") {
      dat[,vars := as.character(vars), env = list(vars = vars[i])]
    }
  }
  
  if (periodNULL) {
    period <- NULL
  }
  
  setattr(dat, "weights", weights)
  setattr(dat, "period", period)
  setattr(dat, "b.rep", b.rep)
  
  return(dat)
}

#' Generate multiple years of EU-SILC data
#'
#' Create a dummy dataset to be used for demonstrating the functionalities of
#' the `surveysd` package based on [laeken::eusilc]. Please refer to the
#' documentation page of the original data for details about the variables.
#'
#' @param n Number of years to generate. Should be at least 1
#' @param prettyNames Create easy-to-read names for certain variables.
#'   Recommended for demonstration purposes. Otherwise, use the original codes
#'   documented in [laeken::eusilc].
#'
#' @details
#' If `prettyNames` is `TRUE`, the following variables will be available in an
#' easy-to-read manner.
#'
#' * `hid` Household id. Consistent with respect to the reference period
#'   (`year`)
#' * `hsize` Size of the household. derived from `hid` and `period`
#' * `region` Federal state of austria where the household is located
#' * `pid` Personal id. Consistent with respect to the reference period (`year`)
#' * `age` Age-class of the respondent
#' * `gender` A persons gender (`"male"`, `"Female"`)
#' * `ecoStat` Ecnomic status
#'   (`"part time"`, `"full time"`, `"unemployed"`, ...)
#' * `citizenship` Citizenship (`"AT"`, `"EU"`, `"other"`)
#' * `pWeight` Personal sample weight inside the reference period
#' * `year`. Simulated reference period
#' * `povertyRisk`. Logical variable determining whether a respondent is at risk
#'   of poverty
#'
#' @examples
#' demo.eusilc(n = 1, prettyNames = TRUE)[, c(1:8, 26, 28:30)]
#' @export
demo.eusilc <- function(n = 8, prettyNames = FALSE) {

  db030 <- rb030 <- povmd60 <- eqincome <- db090 <- eqIncome <- age <- hsize <-
    . <- povertyRisk <- ecoStat <- NULL

  data("eusilc", package = "laeken", envir = environment())
  setDT(eusilc)
  # generate yearly data for y years
  # 25% drop out from 1 year to the other
  eusilc[, year := 2010]
  eusilc.i <- copy(eusilc)
  nsamp <- round(eusilc[, uniqueN(db030)] * .25)
  hhincome <- eusilc[!duplicated(db030)][["eqIncome"]]
  nextIDs <- (1:nsamp) + eusilc[, max(db030)]
  if (n > 1)
    for (i in 1:(n - 1)) {
      eusilc.i[db030 %in% sample(unique(eusilc.i$db030), nsamp),
               c("db030", "eqIncome") := .(nextIDs[.GRP], sample(hhincome, 1L)),
               by = db030]
      eusilc.i[, year := year + 1]
      eusilc <- rbind(eusilc, eusilc.i)
      nextIDs <- (1:nsamp) + eusilc[, max(db030)]
    }

  eusilc[, rb030 := as.integer(paste0(db030, "0", 1:.N)),
         by = list(year, db030)]
  eusilc[, povmd60 := as.numeric(
    eqIncome < .6 * laeken::weightedMedian(eqIncome, w = db090)
  ), by = year]
  eusilc[, age := cut(age, c(-Inf, 16, 25, 45, 65, Inf))]
  eusilc[, hsize := factor(pmin(hsize,5))]

  if (prettyNames) {
    data.table::setnames(eusilc, "db030", "hid")
    data.table::setnames(eusilc, "db040", "region")
    data.table::setnames(eusilc, "rb030", "pid")
    data.table::setnames(eusilc, "rb090", "gender")
    data.table::setnames(eusilc, "pb220a", "citizenship")
    data.table::setnames(eusilc, "pl030", "ecoStat")
    eusilc[, ecoStat := factor(ecoStat, labels = c(
      "full time", "part time", "unemployed",
      "education", "retired", "disabled", "domestic"
    ))]
    data.table::setnames(eusilc, "rb050", "pWeight")
    data.table::setnames(eusilc, "povmd60", "povertyRisk")
    eusilc[, povertyRisk := as.logical(povertyRisk)]
  }

  return(eusilc)
}

##############################################################
# Helper-Functions
#
#' @import ggplot2
#' @import data.table Rcpp
#' @importFrom "graphics" "plot"
#' @importFrom "stats" "as.formula" "na.omit" "quantile" "sd" "xtabs" "formula" "var"
#' @importFrom "utils" "data" "find" "tail" "head" "capture.output"
#' @importFrom "laeken" "weightedMedian"
#' @importFrom "methods" "formalArgs" "is"
#' @useDynLib surveysd

rowProds <- function(x, na.rm = FALSE) {
  n <- nrow(x)
  y <- double(length = n)
  for (ii in seq_len(n)) {
    y[ii] <- prod(x[ii, , drop = TRUE], na.rm = na.rm)
  }
  y
}


dt.eval <- function(..., env = parent.frame()) {

  expressions <- paste0(...)
  if (length(expressions) > 1) {
    return(lapply(
      expressions,
      function(z) {
        eval(parse(text = z), envir = env)
      }
    ))
  } else {
    return(eval(parse(text = paste0(...)), envir = env))
  }
}

dt.eval2 <- function(...) {
  return(eval(parse(text = paste0(...)), envir = parent.frame()))
}

getEllipsis <- function(element, default, ell) {
  ifelse(is.null(ell[[element]]), default, ell[[element]])
}
getEllipsis2 <- function(element, default, ell) {

  if (element %in% names(ell)) {
    return(ell[[element]])
  }else{
    return(default)
  }
}

# helpfunction to create contingency tables
makeCalibTable <- function(dat, weights, period, vars) {
  # make contingency table
  formTab <- paste(weights, "~", paste(c(period, vars), collapse = "+"))
  varsTab <- xtabs(formTab, data = dat)
  return(list(varsTab))
}

paste_ <- function(a, b) {
  paste(a, b, sep = ".")
}
paste_c <- function(a, b) {
  paste(a, b, sep = ",")
}
paste_addarg <- function(a, b) {

  a <- tstrsplit(a, ",")

  return(paste(a[[1]], b, paste(a[2:length(a)], collapse = ","), sep = ","))
}

# helpfunctions for point estimates
weightedRatioNat <- function(x, w, N) {
  weightedRatio(x, w) / N * 100
}
weightedRatioR <- function(x, w) {
  sum(w[x == 1], na.rm = TRUE) / sum(w[!is.na(x)], na.rm = TRUE) * 100
}
weightedSumR <- function(x, w) {
  sum(as.numeric(x) * w, na.rm = TRUE)
}

povmd <- function(x, w) {
  md <- laeken::weightedMedian(x, w) * 0.6
  pmd60 <- x < md
  return(as.integer(pmd60))
}

# helpfunction for quantile calcultion with missings
quantileNA <- function(x, probs, p.names, np = length(probs)) {

  if (any(is.na(x))) {
    out <- rep(NA_real_, np)
  } else {
    out <- quantile(x, probs = probs)
  }
  names(out) <- p.names
  return(out)
}

randomInsert <- function(x, y, n = 20) {
  if (length(x) < 20 | length(y) < 20) {
    stop("n must be smaller than length(x) and length(y)")
  }

  x.indices <- sample(length(x), n)
  y.values <- sample(y, n)
  x[x.indices] <- y.values
  return(x)
}

generateRandomName <- function(nchar = 20, existingNames) {

  newName <- paste(sample(c(letters, LETTERS), nchar), collapse = "")
  while (newName %in% existingNames) {
    newName <- paste(sample(c(letters, LETTERS), nchar), collapse = "")
  }


  return(newName)
}

# input checking function
check.input <- function(input, input.name, input.length=NULL, 
                        input.type = NULL, decimal.possible = NULL, 
                        c.names = NULL, dat = NULL, dat.column.type = NULL){
  
  input.name <- paste0("'",input.name,"'")
  
  if(!is.null(input.length)){
    if(length(input) != 1){
      stop(paste(input.name,"must have length",input.length))
    }
  }
  
  if(!is.null(input.type)){
    check.input.type <- sapply(input.type, function(z){is(input,z)})
    if(all(check.input.type==FALSE)){
      stop(paste(input.name,"must be of type",paste(input.type,collapse=" or ")))
    }
  }
  
  if(!is.null(decimal.possible)){
    if(input %%1 !=0 & decimal.possible==FALSE){
      stop(paste(input.name,"cannot have a decimal part"))
    }
  }
  
  if(!is.null(c.names)){
    if(any(!input %in% c.names)){
      input.miss <- input[!input %in% c.names]
      input.miss <- head(input.miss,5)
      input.miss <- paste(input.miss, collapse = "', '")
      input.miss <- paste0("'",input.miss,"'")
      
      if(sum(!input %in% c.names)>5){
        input.miss <- paste0(input.miss,", ...")
      }
      
      stop(paste("column(s)",input.miss,"specified in",input.name,"not found in dat"))
    }
  }
  
  if(!is.null(dat) & !is.null(dat.column.type)){
    check.input.type <- sapply(input, function(z){
      check.z <- sapply(dat.column.type,function(y,dat.col){
        is(dat.col,y)
      }, dat.col = dat[[z]])
      return(any(check.z))
      })
    if(any(!check.input.type)){
      check.fail <- names(check.input.type)[check.input.type == FALSE]
      check.fail <- head(check.fail,5)
      check.fail <- paste(check.fail, collapse = "', '")
      check.fail <- paste0("'",check.fail,"'")
      if(sum(!check.input.type)>5){
        check.fail <- paste0(check.fail,", ...")
      }
      stop(paste("column(s)",check.fail,"in parameter",input.name,"must correspond to",paste(dat.column.type, collapse = " OR "),"column(s) in dat"))
    }
  }
  
  return(NULL)
}

#################################
# test calc.stError()
#
#Sys.unsetenv("R_TESTS")

context("calc.stError() long")

if (Sys.getenv("SURVEYSD_ADDITIONAL_TEST") == "TRUE") {
  library(surveysd)
  library(laeken)
  library(data.table)
  eusilc <- surveysd:::demo.eusilc()

  eusilc <- draw.bootstrap(eusilc, REP = 2, hid = "db030", weights = "db090",
                           period = "year", strata = "db040")
  eusilc <- recalib(
    eusilc, hid = "db030", weights = "db090", b.rep = paste0("w", 1:2),
    period = "year", conP.var = c("rb090", "age"),
    conH.var = c("db040", "hsize"))

  # these are some longer tests

  test_that("test para -  var and fun", {

    # source_test_helpers("helper_myfun.R")

    expect_error(
      calc.stError(
        eusilc, var = "povmd60s", group = c("rb090", "db040")),
      "column(s) 'povmd60s' specified in 'var' not found in dat", fixed = TRUE)

    eusilc[sample(.N, 100), povmd60NA := NA]
    expect_error(calc.stError(
      eusilc, var = "povmd60NA", group = c("rb090", "db040")), NA)

    expect_error(
      calc.stError(
        eusilc, var = "povmd60", fun = myfun.undefined, group =
          c("rb090", "db040")),
      "object 'myfun.undefined' not found")

    myfun <- function(x, w) {
      return(sum(w * x))
    }
    myfun.char <- function(x, w) {
      return(as.character(sum(w * x)))
    }
    myfun.mulval <- function(x, w) {
      return(w * x)
    }

    expect_error(calc.stError(
      eusilc, var = "povmd60NA", fun = myfun, group = c("rb090", "db040")), NA)
    expect_error(calc.stError(
      eusilc, var = "povmd60", fun = myfun, group = c("rb090", "db040")), NA)

    expect_error(
      calc.stError(
        eusilc, var = "povmd60", fun = myfun.char, group =
          c("rb090", "db040")),
      "Function in fun does not return integer or numeric value")

    expect_error(
      calc.stError(eusilc, var = "povmd60", fun = myfun.mulval,
                   group = c("rb090", "db040")),
      paste("Function in fun does return more than one value. Only functions",
            "which return a single value are allowed."))

    help_gini <- function(x, w) {
      laeken::gini(x, w)$value
    }
    expect_error(calc.stError(
      eusilc, var = "eqIncome", fun = help_gini, group = c("rb090", "db040")),
      NA)

  })

  test_that("test para -  implemented functions", {

    comp <- rep(FALSE, 1000)
    for (i in 1:1000) {
      x <- sample(c(1, 0, NA_real_), 200, prob = c(.45, .45, .1),
                  replace = TRUE)
      w <- sample(30:400, 200, replace = TRUE)

      r_res <- surveysd:::weightedRatioR(x, w)
      c_res <- weightedRatio(x, w)

      comp[i] <- abs(r_res - c_res) < 1e-10
    }
    expect_true(all(comp))
    comp <- rep(FALSE, 1000)
    for (i in 1:1000) {
      x <- sample(c(1, 0, NA_real_), 200, prob = c(.45, .45, .1),
                  replace = TRUE)
      w <- sample(30:400, 200, replace = TRUE)

      r_res <- surveysd:::weightedSumR(x, w)
      c_res <- weightedSum(x, w)

      comp[i] <- abs(r_res - c_res) < 1e-10
    }
    expect_true(all(comp))
  })

  test_that("test para - fun.adjust.var and adjust.var", {

    group <- list("rb090", "db040", c("rb090", "db040"))
    povmd <- function(x, w) {
      md <- laeken::weightedMedian(x, w) * 0.6
      pmd60 <- x < md
      return(as.integer(pmd60))
    }

    expect_error(calc.stError(
      eusilc, weights = "rb050",
      var = "povmd60", fun = weightedRatio,
      group = group, fun.adjust.var = povmd, adjust.var = "eqIncome"), NA)

    myfun.char <- function(x, w) {
      return(as.character(sum(w * x)))
    }
    expect_error(
      calc.stError(
        eusilc, weights = "rb050", var = "povmd60", fun = weightedRatio, group =
          group, fun.adjust.var = myfun.char, adjust.var = "eqIncome"),
      "Function in fun.adjust.var does not return integer or numeric value")

    expect_error(
      calc.stError(
        eusilc, weights = "rb050", var = "povmd60", fun = weightedRatio, group =
          group, fun.adjust.var = povmd, adjust.var = 1),
      "'adjust.var' must be of type character")
    expect_error(
      calc.stError(
        eusilc, weights = "rb050", var = "povmd60", fun = weightedRatio, group =
          group, fun.adjust.var = povmd, adjust.var = "1"),
      "column(s) '1' specified in 'adjust.var' not found in dat", fixed = TRUE)
    expect_error(
      calc.stError(
        eusilc, weights = "rb050", var = "povmd60", fun = weightedRatio, group =
          group, fun.adjust.var = povmd, adjust.var = c("eqIncome", "1")),
      "If adjust.var is not NULL, adjust.var and var must have the same length!")


    # compare fun.adjust.var with results not using fun.adjust.var
    err.est <- calc.stError(
      eusilc, weights = "rb050", var = "povmd60", fun = weightedRatio, group =
        group, fun.adjust.var = povmd, adjust.var = "eqIncome")
    povmd2 <- function(x, w) {
      md <- laeken::weightedMedian(x, w) * 0.6
      pmd60 <- x < md
      # weighted ratio is directly estimated inside my function
      return(sum(w[pmd60]) / sum(w) * 100)
    }

    err.est.different <- calc.stError(
      eusilc, weights = "rb050", var = "eqIncome", fun = povmd2, group = group,
      fun.adjust.var = NULL, adjust.var = NULL)

    expect_true(all.equal(
      err.est.different$Estimates[is.na(rb090) & is.na(db040),
                                  .(val_eqIncome, stE_eqIncome)],
      err.est$Estimates[is.na(rb090) & is.na(db040),
                        .(val_povmd60, stE_povmd60)],
      check.attributes = FALSE))
    expect_false(isTRUE(all.equal(
      err.est.different$Estimates[!(is.na(rb090) & is.na(db040)),
                                  .(val_eqIncome, stE_eqIncome)],
      err.est$Estimates[!(is.na(rb090) & is.na(db040)),
                        .(val_povmd60, stE_povmd60)],
      check.attributes = FALSE)))

  })

  test_that("test para -  add.arg", {

    fun <- function(x, w, b, a) {
      sum(x * w * b)
    }
    add.arg <- list(b = "onePerson", c = "randNumber")

    eusilc[, onePerson := .N == 1, by = .(db030, year)]
    eusilc[, randNumber := rnorm(.N)]

    add.arg <- list(b = "onePerson", c = "randNumber")
    expect_error(
      calc.stError(
        eusilc,  var = "eqIncome", fun = fun, group = c("rb090", "db040"),
        add.arg = add.arg),
      "c not argument(s) of supplied function.", fixed = TRUE)

    add.arg <- list(b = "onePerson", a = "abcde")
    expect_error(
      calc.stError(
        eusilc, var = "eqIncome", fun = fun, group = c("rb090", "db040"),
        add.arg = add.arg),
      "abcde not in column names of dat.")

    add.arg <- list(b = "onePerson", a = "randNumber")
    res_1 <- calc.stError(
      eusilc,
      var = "eqIncome", fun = fun, group = c("rb090", "db040"),
      add.arg = add.arg)

    fun <- function(x, w, b, a) {
      sum(x * w * b * a)
    }

    res_2 <- calc.stError(
      eusilc,
      var = "eqIncome", fun = fun, group = list(c("rb090", "db040")),
      add.arg = add.arg)
    res_2 <- res_2$Estimates[
      !is.na(rb090) & nchar(year) == 4][, .(year = as.numeric(year),
                                            rb090, db040, val_eqIncome)]
    res_direct <- eusilc[, fun(eqIncome, db090, onePerson, randNumber),
                         by = .(year, rb090, db040)]
    res <- merge(res_2, res_direct, by = c("year", "rb090", "db040"))
    expect_true(nrow(res[val_eqIncome != V1]) == 0)
  })
}

#' @title Calcualte point estimates and their standard errors using bootstrap
#'   weights.
#'
#' @description
#' Calculate point estimates as well as standard errors of variables in surveys.
#' Standard errors are estimated using bootstrap weights (see [draw.bootstrap]
#' and [recalib]). In addition the standard error of an estimate can be
#' calcualted using the survey data for 3 or more consecutive periods, which
#' results in a reduction of the standard error.
#'
#' @param dat either data.frame or data.table containing the survey data.
#'   Surveys can be a panel survey or rotating panel survey, but does not need
#'   to be. For rotating panel survey bootstrap weights can be created using
#'   [draw.bootstrap] and [recalib].
#' @param weights character specifying the name of the column in `dat`
#'   containing the original sample weights. Used to calculate point estimates.
#' @param b.weights character vector specifying the names of the columns in
#'   `dat` containing bootstrap weights. Used to calculate standard errors.
#' @param period character specifying the name of the column in `dat`
#'   containing the sample periods.
#' @param var character vector containing variable names in `dat` on which `fun`
#'   shall be applied for each sample period. If `var = NULL` the results will 
#'   reflect the sum of `weights`.
#' @param fun function which will be applied on `var` for each sample period.
#'   Predefined functions are [weightedRatio], [weightedSum], but can also take
#'   any other function which returns a double or integer and uses weights as
#'   its second argument.
#' @param relative.share boolean, if `TRUE` point estimates resulting from `fun` will be
#'   divided by the point estimate at population level per `period`.
#' @param group character vectors or list of character vectors containig
#'   variables in `dat`. For each list entry `dat` will be split in subgroups
#'   according to the containing variables as well as `period`. The
#'   pointestimates are then estimated for each subgroup seperately. If
#'   `group=NULL` the data will split into sample periods by default.
#' @param group.diff boolen, if `TRUE` differences and the standard error
#'   between groups defined in `group` are calculated. See details for more explanations.
#' @param fun.adjust.var can be either `NULL` or a function. This argument can
#'   be used to apply a function for each `period` and bootstrap weight to the
#'   data. The resulting estimates will be passed down to `fun`. See details for
#'   more explanations.
#' @param adjust.var can be either `NULL` or a character specifying the first
#'   argument in `fun.adjust.var`.
#' @param period.diff character vectors, defining periods for which the
#'   differences in the point estimate as well it's standard error is
#'   calculated. Each entry must have the form of `"period1 - period2"`. Can be
#'   NULL
#' @param period.mean odd integer, defining the range of periods over which the
#'   sample mean of point estimates is additionally calcualted.
#' @param bias boolean, if `TRUE` the sample mean over the point estimates of
#'   the bootstrap weights is returned.
#' @param size.limit integer defining a lower bound on the number of
#'   observations on `dat` in each group defined by `period` and the entries in
#'   `group`. Warnings are returned if the number of observations in a subgroup
#'   falls below `size.limit`. In addition the concerned groups are available in
#'   the function output.
#' @param cv.limit non-negativ value defining a upper bound for the standard
#'   error in relation to the point estimate. If this relation exceed
#'   `cv.limit`, for a point estimate, they are flagged and available in the
#'   function output.
#' @param p numeric vector containing values between 0 and 1. Defines which
#'   quantiles for the distribution of `var` are additionally estimated.
#' @param add.arg additional arguments which will be passed to fun. Can be
#'   either a named list or vector. The names of the object correspond to the
#'   function arguments and the values to column names in dat, see also
#'   examples.
#' @param national DEPRECATED use `relative.share` instead! boolean, if TRUE point estimates resulting from fun will be
#'   divided by the point estimate at the national level.
#'
#' @details `calc.stError` takes survey data (`dat`) and returns point estimates
#' as well as their standard Errors defined by `fun` and `var` for each sample
#' period in `dat`. `dat` must be household data where household members
#' correspond to multiple rows with the same household identifier. The data
#' should at least contain the following columns:
#'
#'   * Column indicating the sample period;
#'   * Column indicating the household ID;
#'   * Column containing the household sample weights;
#'   * Columns which contain the bootstrap weights (see output of [recalib]);
#'   * Columns listed in `var` as well as in `group`
#'
#' For each variable in `var` as well as sample period the function `fun` is
#' applied using the original as well as the bootstrap sample weights.\cr
#' The point estimate is then selected as the result of `fun` when using the
#' original sample weights and it's standard error is estimated with the result
#' of `fun` using the bootstrap sample weights. \cr
#' \cr
#' `fun` can be any function which returns a double or integer and uses sample
#' weights as it's second argument. The predifined options are `weightedRatio`
#' and `weightedSum`.\cr
#' \cr
#' For the option `weightedRatio` a weighted ratio (in \%) of `var` is
#' calculated for `var` equal to 1, e.g
#' `sum(weight[var==1])/sum(weight[!is.na(var)])*100`.\cr
#' Additionally using the option `national=TRUE` the weighted ratio (in \%) is
#' divided by the weighted ratio at the national level for each `period`.
#' \cr
#' If `group` is not `NULL` but a vector of variables from `dat` then `fun` is
#' applied on each subset of `dat` defined by all combinations of values in
#' `group`.\cr
#' For instance if `group = "sex"` with "sex" having the values "Male" and
#' "Female" in `dat` the point estimate and standard error is calculated on the
#' subsets of `dat` with only "Male" or "Female" value for "sex". This is done
#' for each value of `period`. For variables in `group` which have `NA`s in
#' `dat` the rows containing the missings will be discarded. \cr
#' When `group` is a list of character vectors, subsets of `dat` and the
#' following estimation of the point estimate, including the estimate for the
#' standard error, are calculated for each list entry.\cr
#' \cr
#' If `group.diff = TRUE` difference between groups definded by `group` are calculated.
#' Differences are only calculated within each variables of `group`, 
#' e.g `group = c("gender", "region")` will calcualate estimates of each group and
#' also differences within `"gender"` and `"region"` seperately.
#' If grouping is done with multiple variables e.g `group = list(c("gender","region")`)
#' (~ grouping by `"gender"` x `"region"`) differences are calculated 
#' only between groups where one of the grouping variables is different.
#' For instance the difference between `gender = "female" & region = "Vienna"` and 
#' `gender = "male" & region = "Vienna"` OR `gender = "female" & region = "Vienna"` and 
#' `gender = "female" & region = "Salzburg"` will be calculated.
#' The difference between `gender = "female" & region = "Vienna"` and 
#' `gender = "male" & region = "Salzburg"` will not be calculated. The order of difference
#' is determined by order of value (alpha-numerical order) or 
#' if grouping contains factor variables the factor levels determin the order.
#' \cr
#' The optional parameters `fun.adjust.var` and `adjust.var` can be used if the
#' values in `var` are dependent on the `weights`. As is for instance the case
#' for the poverty thershhold calculated from EU-SILC.
#' In such a case an additional function can be supplied using `fun.adjust.var`
#' as well as its first argument `adjust.var`, which needs to be part of the
#' data set `dat`. Then, before applying `fun` on variable `var`
#' for all `period` and groups, the function `fun.adjust.var` is applied to
#' `adjust.var` using each of the bootstrap weights seperately (NOTE: weight is
#' used as the second argument of `fun.adjust.var`).
#' Thus creating i=1,...,`length(b.weights)` additional variables.
#' For applying `fun` on `var` the estimates for the bootstrap replicate will
#' now use each of the corresponding new additional variables. So instead of
#' \deqn{fun(var,weights,...),fun(var,b.weights[1],...),
#' fun(var,b.weights[2],...),...}
#' the function `fun` will be applied in the way
#' \deqn{fun(var,weights,...),fun(var.1,b.weights[1],...),fun(var.2,
#' b.weights[2],...),...}
#'
#' where `var.1`, `var.2`, `...` correspond to the estimates resulting from
#' `fun.adjust.var` and `adjust.var`.
#' NOTE: This procedure is especially usefull if the `var` is dependent on
#' `weights` and `fun` is applied on subgroups of the data set. Then it is not
#' possible to capture this procedure with `fun` and `var`, see examples for a
#' more hands on explanation.
#' \cr
#' When defining `period.diff` the difference of point estimates between periods
#' as well their standard errors are calculated.\cr
#' The entries in `period.diff` must have the form of `"period1 - period2"`
#' which means that the results of the point estimates for `period2` will be
#' substracted from the results of the point estimates for `period1`.\cr
#' \cr
#' Specifying `period.mean` leads to an improvement in standard error by
#' averaging the results for the point estimates, using the bootstrap weights,
#' over `period.mean` periods.
#' Setting, for instance, `period.mean = 3` the results in averaging these
#' results over each consecutive set of 3 periods.\cr
#' Estimating the standard error over these averages gives an improved estimate
#' of the standard error for the central period, which was used for
#' averaging.\cr
#' The averaging of the results is also applied in differences of point
#' estimates. For instance defining `period.diff = "2015-2009"` and
#' `period.mean = 3`
#' the differences in point estimates of 2015 and 2009, 2016 and 2010 as well as
#' 2014 and 2008 are calcualated and finally the average over these 3
#' differences is calculated.
#' The periods set in `period.diff` are always used as the middle periods around
#' which the mean over `period.mean` years is build.
#' \cr
#' Setting `bias` to `TRUE` returns the calculation of a mean over the results
#' from the bootstrap replicates. In  the output the corresponding columns is
#' labeled *_mean* at the end.\cr
#' \cr
#' If `fun` needs more arguments they can be supplied in `add.arg`. This can
#' either be a named list or vector.\cr
#' \cr
#' The parameter `size.limit` indicates a lower bound of the sample size for
#' subsets in `dat` created by `group`. If the sample size of a subset falls
#' below `size.limit` a warning will be displayed.\cr
#' In addition all subsets for which this is the case can be selected from the
#' output of `calc.stError` with `$smallGroups`.\cr
#' With the parameter `cv.limit` one can set an upper bound on the coefficient
#' of variantion. Estimates which exceed this bound are flagged with `TRUE` and
#' are available in the function output with `$cvHigh`.
#' `cv.limit` must be a positive integer and is treated internally as \%, e.g.
#' for `cv.limit=1` the estimate will be flagged if the coefficient of
#' variantion exceeds 1\%.\cr
#' \cr
#' When specifying `period.mean`, the decrease in standard error for choosing
#' this method is internally calcualted and a rough estimate for an implied
#' increase in sample size is available in the output with `$stEDecrease`.
#' The rough estimate for the increase in sample size uses the fact that for a
#' sample of size \eqn{n} the sample estimate for the standard error of most
#' point estimates converges with a factor \eqn{1/\sqrt{n}} against the true
#' standard error \eqn{\sigma}.
#'
#'
#' @return Returns a list containing:
#'
#' * `Estimates`: data.table containing period differences and/or k period
#'   averages for estimates of
#'   `fun` applied to `var` as well as the corresponding standard errors, which
#'   are calculated using the bootstrap weights. In addition the sample size,
#'   `n`, and poplutaion size for each group is added to the output.
#' * `smallGroups`: data.table containing groups for which the number of
#'   observation falls below `size.limit`.
#' * `cvHigh`: data.table containing a boolean variable which indicates for each
#'   estimate if the estimated standard error exceeds `cv.limit`.
#' * `stEDecrease`: data.table indicating for each estimate the theoretical
#'   increase in sample size which is gained when averaging over k periods. Only
#'   returned if `period.mean` is not `NULL`.
#'
#' @seealso [draw.bootstrap] \cr
#' [recalib]
#' @keywords survey manip
#' @author Johannes Gussenbauer, Alexander Kowarik, Statistics Austria
#'
#' @examples
#' # Import data and calibrate
#'
#' library(surveysd)
#' library(data.table)
#' setDTthreads(1)
#' set.seed(1234)
#' eusilc <- demo.eusilc(n = 4,prettyNames = TRUE)
#' dat_boot <- draw.bootstrap(eusilc, REP = 3, hid = "hid", weights = "pWeight",
#'                            strata = "region", period = "year")
#' dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region")
#'
#' # estimate weightedRatio for povertyRisk per period
#'
#' err.est <- calc.stError(dat_boot_calib, var = "povertyRisk",
#'                         fun = weightedRatio)
#' err.est$Estimates
#'
#' # calculate weightedRatio for povertyRisk and fraction of one-person
#' # households per period
#'
#' dat_boot_calib[, onePerson := .N == 1, by = .(year, hid)]
#' err.est <- calc.stError(dat_boot_calib, var = c("povertyRisk", "onePerson"),
#'                         fun = weightedRatio)
#' err.est$Estimates
#'
#'
#' # estimate weightedRatio for povertyRisk per period and gender and
#' # period x region x gender 
#' 
#' group <- list("gender", c("gender", "region"))
#' err.est <- calc.stError(dat_boot_calib, var = "povertyRisk",
#'                         fun = weightedRatio, group = group)
#' err.est$Estimates
#'
#' # use average over 3 periods for standard error estimation
#' # and calculate estimate for difference of
#' # period 2011 and 2012 inclulding standard errors
#' period.diff <- c("2012-2011")
#' err.est <- calc.stError(
#'   dat_boot_calib, var = "povertyRisk", fun = weightedRatio,
#'   period.diff = period.diff,  # <- take difference of periods 2012 and 2011
#'   period.mean = 3)  # <- average over 3 periods
#' err.est$Estimates
#'
#' # for more examples see https://statistikat.github.io/surveysd/articles/error_estimation.html
#'
#' @export calc.stError
#'


# wrapper-function to apply fun to var using weights (~weights, b.weights)
# and calculating standard devation (using the bootstrap replicates) per period
# and for k-period rolling means
calc.stError <- function(
    dat, weights = attr(dat, "weights"), b.weights = attr(dat, "b.rep"),
    period = attr(dat, "period"), var = NULL, fun = weightedRatio, 
    relative.share = FALSE,
    group = NULL, group.diff = FALSE, fun.adjust.var = NULL, adjust.var = NULL,
    period.diff = NULL, period.mean = NULL, bias = FALSE, 
    size.limit = 20, cv.limit = 10, p = NULL,
    add.arg = NULL, national = FALSE) {
  
  stE_high <- stE <- val <- as.formula <- est_type <- n_inc <-
    stE_roll <- n <- size <- est <- estimate_type <- NULL
  
  ##########################################################
  # INPUT CHECKING
  if (is.data.frame(dat)) {
    dat <- as.data.table(dat)
  }else if (!is.data.table(dat)) {
    stop("dat must be a data.frame or data.table")
  }
  
  c.names <- colnames(dat)
  
  # check weights
  check.input(weights, "weights", input.length = 1, input.type = "character", 
              c.names = c.names, dat = dat, dat.column.type = "numeric")

  
  # check b.weights
  check.input(b.weights, "b.weights", input.type = "character", 
              c.names = c.names, dat = dat, dat.column.type = "numeric")
  
  if (any(!grepl("^[[:alpha:]]", b.weights)))
    stop("Column names of bootstrap replicates must start with alphabetic ",
         "character")
  
  # check period
  removeCols <- c()
  periodNULL <- is.null(period)
  if (periodNULL) {
    period <- generateRandomName(20, colnames(dat))
    dat[, c(period) := 1]
    removeCols <- c(removeCols, period)
  }
  
  check.input(period, period, input.length = 1, input.type = "character",
              c.names = c.names)
  
  # check var
  if(is.null(var)){
    var <- generateRandomName(20, colnames(dat))
    dat[, c(var) := 1]
    removeCols <- c(removeCols, var)
  }
  
  check.input(var, "var", c.names = c.names, dat = dat, dat.column.type = c("numeric","logical"))
  
  # check national/realtive.share
  check.input(relative.share, "relative.share", input.length = 1, input.type = "logical")
  check.input(national, "national", input.length = 1, input.type = "logical")
  if(national == TRUE){
    warning("Parameter 'national' will be deprecated soon use paremter 'relative.share' instead!")
    relative.share <- national
  }
  
  # check fun and add.arg
  check.input(fun, "fun", input.type = "function")
  
  if (!is.null(add.arg)) {
    if (!is.list(add.arg) | !is.vector(add.arg))
      stop("add.arg needs to be a list or vector")
    
    if (length(names(add.arg)) == 0)
      stop("add.arg needs to be a named list or vector, names(add.arg) <- ?")
    
    if (any(!names(add.arg) %in% formalArgs(fun))) {
      notInFun <- !names(add.arg) %in% formalArgs(fun)
      stop(paste(names(add.arg)[notInFun], collapse = " "),
           " not argument(s) of supplied function.")
    }
    
    if (any(!unlist(add.arg) %in% c.names)) {
      notInData <- unlist(add.arg)
      notInData <- notInData[!notInData %in% c.names]
      stop(paste(notInData, collapse = " "), " not in column names of dat.")
    }
    
    add.arg <- as.list(add.arg)
  }
  
  fun_names <- names(formals(fun))
  eval_args <- c(list(var[1], weights), add.arg) # c(list(fun = fun, var = var, weights = weights), add.arg)
  names(eval_args)[1:2] <- fun_names[1:2]
  test.val <- dat[,do.call(fun, args = .SD), env = list(.SD = eval_args)]
  
  if (!is.numeric(test.val) & !is.integer(test.val))
    stop("Function in fun does not return integer or numeric value")
  
  if (length(test.val) > 1)
    stop("Function in fun does return more than one value. Only functions ",
         "which return a single value are allowed.")
  
  # check fun.adjust.var
  if (!is.null(fun.adjust.var) & !is.null(adjust.var)) {
    check.input(fun.adjust.var, "fun.adjust.var", input.type = "function")
    if(length(var)!=length(adjust.var)){
      stop("If adjust.var is not NULL, adjust.var and var must have the same length!")
    }
      
    check.input(adjust.var, "adjust.var", input.length = length(var), input.type = 
                  "character", c.names = c.names,
                dat = dat, dat.column.type = c("numeric","logical"))
    
    
    fun_names <- names(formals(fun.adjust.var))
    eval_args <- list(adjust.var[1], weights) # c(list(fun = fun, var = var, weights = weights), add.arg)
    names(eval_args)[1:2] <- fun_names[1:2]
    test.val <- dat[,do.call(fun.adjust.var, args = .SD), env = list(.SD = eval_args)]
    
    if (!is.numeric(test.val) & !is.integer(test.val))
      stop("Function in fun.adjust.var does not return integer ",
           "or numeric value")
  }else{
    if((is.null(fun.adjust.var) & !is.null(adjust.var))|
       (!is.null(fun.adjust.var) & is.null(adjust.var))){
      stop("adjust.var and fun.adjust.var must be a column in dat and a function OR both be NULL")
    }
  }

  # check group
  if (is.null(group))
    group <- list(NULL)
  
  if (!is.list(group))
    group <- as.list(group)
  
  if (!any(unlist(lapply(group, is.null))))
    group <- c(list(NULL), group)
  
  if (any(!unlist(group) %in% c.names))
    stop("Not all elements on group are column names in dat")
  
  # check period.mean
  if (!is.null(period.mean)) {
    check.input(period.mean, "period.mean", input.length = 1, input.type = "numeric")

    if (period.mean %% 2 == 0 & period.mean > 0) {
      warning("period.mean must be odd - average over periods",
              " will not be calculated")
      period.mean <- NULL
    } else {
      if (period.mean <= 1)
        period.mean <- NULL
    }
  }
  
  # check bias
  check.input(bias, "bias", input.length = 1, input.type = "logical")
   
  # check size.limit
  check.input(size.limit, "size.limit", input.length = 1, input.type = "numeric")
  
  # check cv.limit
  check.input(cv.limit, "cv.limit", input.length = 1, input.type = "numeric")
  
  # check p
  if (!is.null(p)) {
    
    check.input(p, "p", input.type = "numeric")
    
    if (any(!p %between% c(0, 1)))
      stop("Values in p must be between 0 and 1")
    
  }
  
  # check period.diff
  if (!is.null(period.diff)) {
    periods.dat <- dat[,unique(period), env = list(period = period)]
    period.diff <- strsplit(period.diff, "-")
    period.diff <- lapply(period.diff, trimws)
    
    rm.index <- rep(0, length(period.diff))
    for (i in seq_along(period.diff)) {
      if (any(!period.diff[[i]] %in% periods.dat)) {
        warning("Removing ", paste(period.diff[[i]], collapse = "-"),
                " from period.diff - period(s) not present in column ",
                period, "\n")
        rm.index[i] <- 1
      }
    }
    if (all(rm.index == 1)) {
      warning("No differences will be calculated\n")
      period.diff <- NULL
    } else {
      period.diff <- period.diff[rm.index == 0]
    }
  }
  
  
  ##########################################################
  
  ##########################################################
  # setup parameters
  # define columns in which NAs are present (will be discarded for
  #   the evaluation)
  
  
  col_cross <- unique(unlist(group))
  if (!is.null(col_cross)) {
    no.na <- unlist(dat[, lapply(.SD,
                                 function(z) {
                                   all(!is.na(z))
                                 }),
                        .SDcols = col_cross])
    no.na <- names(no.na)[!no.na]
    
    if (length(no.na) > 0) {
      print.no.na <- paste0(no.na, collapse = ", ")
      warning("Missing values found in column name(s) ", print.no.na,
              "\n Cells with missing values are discarded for the",
              "calculation!\n")
    }
    
  } else {
    no.na <- NULL
  }
  
  if (!is.null(p)) {
    p.names <- paste0("p", p)
  } else {
    p.names <- NULL
  }
  
  # calculate point estimates
  outx <- run.stError(
    dat = dat, period = period, var = var, weights = weights,
    b.weights = b.weights, fun = fun, relative.share = relative.share, 
    group = group, group.diff = group.diff,
    fun.adjust.var = fun.adjust.var, adjust.var = adjust.var,
    period.diff = period.diff, period.mean = period.mean, bias = bias,
    no.na = no.na, size.limit = size.limit, p = p, add.arg = add.arg)
  
  # remove columns
  if (length(removeCols) > 0) {
    dat[, c(removeCols) := NULL]
    
    removeCols2 <- removeCols[removeCols %in% colnames(outx)]
    outx[, c(removeCols2) := NULL]
    
    if(var %in% removeCols){
      outx[,est:="N"]
      var <- "N"
    }
  }
  
  
  outx[,estimate_type := fcase(est_type == "roll", "period average",
                               est_type == "diff", "period difference",
                               est_type == "diff_mean", "difference between period averages",
                               est_type == "diff_group", "group difference",
                               default = "direct")]

  outx.names <- colnames(outx)
  outx.names <- outx.names[!outx.names %in% c("val", "est_type", "stE", "mean",
                                              "size", p.names)]
  
  # get meta data like stE_high - size - increase in effektive sample size
  # flag stE if values are especially high
  outx[, stE_high := ((stE / val) * 100) > cv.limit]
  
  # create bool matrix for stE_high
  sd_bool <- subset(
    outx,
    select = unique(c("stE_high", outx.names[!outx.names %in% c("N", "n")]))
  )
  form <- as.formula(paste(
    paste(outx.names[!outx.names %in% c("N", "n", "est")], collapse = "+"),
    "est", sep = "~"))
  sd_bool <- dcast(sd_bool, form, value.var = "stE_high")
  
  # create matrix for increase of sample size
  if (nrow(outx[est_type == "roll"]) > 0) {
    # estimate (roughly) the effektive sample size per
    samp_eff <- outx[est_type == "roll"]
    samp_eff[,period := tstrsplit(period, "_", keep = 2), env = list(period = period)]
    
    if (bias) {
      samp_eff[, "mean" := NULL]
    }
    samp_eff[, c("size", "val", "est_type", "N", "n", "stE_high") := NULL]
    setnames(samp_eff, "stE", "stE_roll")
    same_names <- intersect(colnames(samp_eff), colnames(outx))
    
    samp_eff <- merge(samp_eff, outx[, mget(c("stE", "n", same_names))],
                      by = same_names)
    samp_eff[, n_inc := ((stE / stE_roll) ^ 2 - 1) * n]
    samp_eff[, c("stE_roll", "stE", "n") := NULL]
  } else {
    samp_eff <- NULL
  }
  
  # create Matrix for groups which have small sizes
  size_group <- unique(subset(
    outx[size == TRUE],
    select = c(outx.names[!outx.names %in% c("est", "N")])
  ))
  
  val.var <- c("val", "stE", p.names)
  if (bias) {
    val.var <- c(val.var, "mean")
  }
  
  form_names <- outx.names[!outx.names %in% c("est")]
  form <- as.formula(paste(
    paste(form_names, collapse = "+"), "est", sep = "~"))
  
  outx <- dcast(outx, form, value.var = val.var, fill = NA)
  # reorder output
  col.order <- as.vector(outer(paste0(val.var, "_"), var, FUN = "paste0"))
  outx.names <- colnames(outx)
  col.order <- c(outx.names[!outx.names %in% col.order], col.order)
  col.order <- col.order[col.order %in% outx.names]
  setcolorder(outx, col.order)
  
  param <- list(
    number.bweights = length(b.weights), period = period, var = var,
    fun = fun, fun.adjust.var = fun.adjust.var, adjust.var = adjust.var,
    group = group, group.diff = group.diff, 
    period.diff = period.diff, period.mean = period.mean,
    bias = bias, size.limit = size.limit, cv.limit = cv.limit, add.arg)
  
  output <- list(Estimates = outx, smallGroups = size_group, cvHigh = sd_bool,
                 stEDecrease = samp_eff, param = param)
  
  class(output) <- c("surveysd", class(output))
  
  return(output)
}

run.stError <- function(dat, period, var, weights, b.weights = paste0("w", 1:1000), fun, relative.share,
                        group, group.diff = NULL, period.diff = NULL, period.mean = NULL, 
                        fun.adjust.var = NULL, adjust.var = NULL,
                        bias = FALSE, no.na, size.limit = 20, p = NULL, add.arg){
  
  .SDx <- .SDw <- . <- .SD_add.args <- keep_row <- .SDy <- est_type <- 
    V1 <- N <- n <- difference_group_value <- help_direct_estimate <- 
    x <- size <- NULL
  
  # melt data set for easier data manipulation if var holds multiple values
  id.vars <- unique(c(period, unlist(group), unlist(add.arg), weights, b.weights))
  id.vars <- which(colnames(dat) %in% id.vars)
  m.vars <- paste(var,collapse="|")
  m.vars <- paste0("^(",m.vars,")$")
  m.vars <- list(m.vars) # used in patterns() for melting
  name_var <- paste0("name_var_",generateRandomName(nchar = 10, existingNames = id.vars))
  value_var <- paste0("value_var_",generateRandomName(nchar = 10, existingNames = id.vars))
  value_adjust.var <- NULL
  if(!is.null(adjust.var)){
    value_adjust.var <- paste0("value_adjust.var_",generateRandomName(nchar = 10, existingNames = id.vars))
    m.vars <- c(m.vars, list(paste0("^(",paste(adjust.var, collapse="|"),")$"))) # must be column index for melting
  }
  dat <- melt(dat,id.vars = id.vars, measure.vars = patterns(m.vars), 
              variable.name = c(name_var), value.name = c(value_var, value_adjust.var), variable.factor = FALSE)
  if(!is.null(adjust.var)){
    dat[, name_var := var[as.integer(name_var)], env = list(name_var = name_var)]
  }
  
  # apply fun.adjust.var
  b.value_var <- NULL
  if(!is.null(fun.adjust.var)){
    b.value_var <- paste(value_var,b.weights,sep=".")
    dat[,c(b.value_var) := mapply(fun.adjust.var, .SDx, .SDw, SIMPLIFY = FALSE), by=.(period,name_var),
        env = list(period = period,
                   name_var = name_var,
                   .SDx = as.list(value_adjust.var),
                   .SDw = as.list(b.weights))]
  }
  b.value_var <- c(value_var,b.value_var)
  
  # create point estimate for subnational result in relation to national level
  if (relative.share) {
    national_est <- paste("Nat",c(weights,b.weights), sep=".")
    dat[,c(national_est) := mapply(fun, .SDx, .SDw, MoreArgs = .SD_add.args, SIMPLIFY = FALSE), by=c(period, name_var), 
          env = list(
            .SDx = as.list(b.value_var),
            .SDw = as.list(c(weights,b.weights)),
            .SD_add.args = add.arg)]
  }
  
  # define parameter for quantile calculation
  if (!is.null(p)) {
    p.names <- paste0("p", p)
    np <- length(p.names)
  }
  
  # define additional parameters for mean over consecutive periods
  periods <- sort(unique(dat[[period]]))
  
  if (!is.null(period.mean)) {
    # formulate k consecutive periods
    if (length(periods) >= period.mean & period.mean %% 2 == 1) {
      periodsList <- unlist(lapply(
        periods[1:c(length(periods) - period.mean + 1)],
        function(z) {
          if (!((z + period.mean - 1) > max(periods))) {
            paste(z:c(z + period.mean - 1), collapse = "_")
          }
        }
      ))
    } else{
      periodsList <- NULL
      warning("Not enough periods present in data to calculate mean over ",
              period.mean, " periods.\n")
      period.mean <- NULL
    }
    
    # get periods for k period mean over period-differences
    # get for each difference a list of vectors that correspond to the
    # differences needed to use for the mean over differences
    if (!is.null(period.diff)) {
      period.diff.b <- TRUE
      period.diff.mean <- lapply(period.diff, function(z) {
        z <- as.numeric(z)
        steps <- (period.mean - 1) / 2
        z_upper <- (z[1] + steps):(z[1] - steps)
        z_lower <- (z[2] + steps):(z[2] - steps)
        diff.feasable <- all(c(z_lower, z_upper) %in% periods)
        if (diff.feasable) {
          lapply(1:period.mean, function(s) {
            c(z_upper[s], z_lower[s])
          })
        } else {
          warning("Cannot calculate differences between periods ", z[1],
                  " and ", z[2], " over ", period.mean, " periods.\n")
          NULL
        }
      })
      period.diff.mean[unlist(lapply(period.diff.mean, is.null))] <- NULL
    } else {
      period.diff.b <- FALSE
      period.diff.mean <- NULL
    }
  } else {
    if (is.null(period.diff)) {
      period.diff.b <- FALSE
    } else {
      period.diff.b <- TRUE
    }
    period.diff.mean <- NULL
    periodsList <- NULL
  }
  
  # variable name for estimate
  var_est <- generateRandomName(existingNames = c(unique(unlist(group)), "n" ,"N"))
  name_weights <- paste0("name_weights_", generateRandomName(nchar = 10, existingNames = id.vars))
  
  # iterate of groupings
  out <- lapply(group, function(z) {
    
    # use only unique values for grouping (duplicates are discarded)
    # if period in z also discard -> always group by period
    z <- unique(z[!z == period])
    z <- as.character(z)
    na.check <- z[z %in% no.na]
    if(length(na.check)>0){
      dat[, keep_row := rowSums(is.na(.SD))==0, .SDcols=c(na.check)]
    }else{
      dat[, keep_row := TRUE]
    }
    
    by.eval <- c(name_var, period, z)
    
    # ---------------------------------
    # calculate estimate
    var_value <- paste0("var",1:length(c(weights, b.weights)))
    if(relative.share){
      var.est <- dat[keep_row == TRUE, c(.(n=.N, N = sum(weights)),
                                         var=mapply(fun, .SDx, .SDw, MoreArgs = .SD_add.args, SIMPLIFY = FALSE),
                                         head(.SD,1)), 
                     by=c(by.eval), 
                     env = list(weights = weights,
                                .SDx = as.list(b.value_var),
                                .SDw = as.list(c(weights, b.weights)),
                                .SD_add.args = add.arg),
                     .SDcols = c(national_est)]
 
      var.est[,c(var_value) := mapply(`/`,.SDx,.SDy, SIMPLIFY = FALSE),
              env = list(.SDx = as.list(var_value),
                         .SDy = as.list(national_est))]
    }else{
      var.est <- dat[keep_row == TRUE, c(.(n=.N, N = sum(weights)),
                                         var=mapply(fun, .SDx, .SDw, MoreArgs = .SD_add.args, SIMPLIFY = FALSE)), 
                     by=c(by.eval), 
                     env = list(weights = weights,
                                .SDx = as.list(b.value_var),
                                .SDw = as.list(c(weights, b.weights)),
                                .SD_add.args = add.arg)]
    }
    
    var.est <- melt(var.est, id.vars = c(by.eval,"n","N"), measure.vars = var_value, 
                    value.name = var_est, variable.name = name_weights)
    var.est[,name_weights := factor(name_weights, labels = c(weights, b.weights)), env = list(name_weights = name_weights)]
    var.est[, est_type := "norm"]
    by.eval <- c(by.eval, name_weights)
    
    # add groups which are not in var.est
    # because they were not present in sample
    if (!is.null(period.mean) | period.diff.b) {
      # if group did not exist in period
      # add group with NA
      roll.miss <- unique(subset(dat[keep_row == TRUE], 
                                 select = c(name_var, period, z)))
      roll.miss <- lapply(
        roll.miss,
        function(l) {
          unique(na.omit(l))
        }
      )
      roll.miss[[name_weights]] <- c(weights, b.weights)
      roll.est <- data.table(expand.grid(roll.miss))
      
      if (nrow(roll.est) > nrow(unique(var.est, by=c(name_var, period, z, name_weights)))) {
        var.est <- merge(roll.est, var.est, by = c(name_var, period, z, name_weights),
                         all.x = TRUE)
        setkeyv(var.est, period)
        var.est[is.na(var_est), c("N","n") := .(0, 0), env = list(var_est = var_est)]
      }
    }
    
    # calculate average of estimate over 'period.mean'- periods
    if (!is.null(periodsList)) {
      roll.est <- var.est[, list(
        var_est = rollMeanC(var_est, period.mean, type = "c"),
        period = periodsList
      ), by = c(z, name_var, name_weights), 
      env = list(var_est = var_est,
                 period = period)]
      
      roll.est[, est_type := "roll"]
      roll.Nn <- var.est[name_weights == weights, list(
        N = rollMeanC(x = N, k = period.mean, type = "c"),
        n = rollMeanC(x = n, k = period.mean, type = "c"),
        period = periodsList),
        by = c(z, name_var), env = list(name_weights = name_weights,
                              period = period)]
      
      # merge results
      roll.est <- merge(roll.est, roll.Nn, by = c(z, period, name_var), all.x = TRUE)
      
    }
    
    # ----------------------------------
    # differences between selected groups within years and across and years
   
    # differences between periods
    if (period.diff.b) {
      
      # calcualte differences between periods
      diff.est <- lapply(period.diff,function(y){
        help_diff_group(var.est = var.est, by.eval = by.eval,
                        var_est = var_est, diff.group = y, diff.column = period)
      })
      diff.est <- rbindlist(diff.est)
      diff.est[, est_type := "diff"]
      
      # calcualte differences between periods and mean over consecutive
      #   differences
      if (!is.null(unlist(period.diff.mean))) {
        i.diff.mean <- (period.mean + 1) / 2
        # i.diff.mean <- 1 # nolint
        diff.mean.est <- lapply(period.diff.mean, function(d) {
          # calculate differences for all pairwise periods in d
          d.m.est <- lapply(d,function(y){
            help_diff_group(var.est = var.est, by.eval = by.eval, 
                            var_est = var_est, diff.group = y, diff.column = period)
          })
          # calcualte mean over consecutive differences
          d.m.est <- rbindlist(d.m.est)
          d.m.est <- d.m.est[,.(var_est = mean(var_est), n = mean(n), N=mean(N)), 
                             by = c(z, name_var, name_weights), env = list(var_est = var_est)] # 
          
          d.m.est[, c(period) := paste0(paste(d[[i.diff.mean]], collapse = "-"),
                                        "_mean")]
        })
        diff.mean.est <- rbindlist(diff.mean.est)
        diff.mean.est[, est_type := "diff_mean"]
      }
    }
    
    # difference between group within period
    if(group.diff == TRUE & length(z)>0){
      diff_group_col <- paste(z, collapse = "_")
      z_combinations <- unique(subset(dat[keep_row == TRUE], select = z))
      setorderv(z_combinations, z)
      var.est[,difference_group_value := do.call(paste,c(.SD, sep="_")), .SDcols=c(z)]
      z_combinations[,difference_group_value := do.call(paste,c(.SD, sep="_")), .SDcols=c(z)]
      diff.group.est <- list()
      for(i in 1:(nrow(z_combinations)-1)){
        for(j in (i+1):nrow(z_combinations)){
          # allow only differences between group if all but one values are the same
          # eg. male age (20;30] - female age(40;50] is not supported
          check_grouping <- z_combinations[i,.SD,.SDcols=c(z)] == z_combinations[j,.SD,.SDcols=c(z)] 
          if(sum(check_grouping==FALSE)!=1){
           next # skip this combination
          }
          ij_difference <- z_combinations[c(i,j),lapply(.SD, function(z){paste(unique(z), collapse = " - ")}), .SDcols=c(z)]
          
          y <- c(z_combinations[c(i,j),difference_group_value])
          by.eval_help <- by.eval[!by.eval %in%z]
          i.j.diff <- help_diff_group(var.est = var.est, by.eval = by.eval_help, 
                                      var_est = var_est, diff.group = y, 
                                      diff.column = "difference_group_value")
          i.j.diff[,c(z) := ij_difference]
          i.j.diff[,difference_group_value := NULL]
          diff.group.est <- c(diff.group.est, list(i.j.diff))
        }
      }
      diff.group.est <- rbindlist(diff.group.est)
      diff.group.est[, est_type := "diff_group"]
      var.est[,difference_group_value := NULL]
    }
    
    
    # add results to one data.table
    if (!is.null(period.mean)) {
      var.est <- rbind(var.est, roll.est, fill = TRUE, ignore.attr=TRUE)
    }
    if (period.diff.b) {
      var.est <- rbind(var.est, diff.est, fill = TRUE, ignore.attr=TRUE)
      
      if (!is.null(unlist(period.diff.mean))) {
        var.est <- rbind(var.est, diff.mean.est, fill = TRUE, ignore.attr=TRUE)
      }
    }
    if(group.diff == TRUE & length(z)>0){
      var.est <- rbind(var.est, diff.group.est, fill = TRUE, ignore.attr=TRUE)
    }
    var.est <- var.est[n>0] # ~ groups without any observations in them are discarded 
    
    # ----------------------------------
    # specify small groups
    var.est[, help_direct_estimate := name_weights == weights, env = list(name_weights = name_weights)]
    
    var.est.small <- var.est[help_direct_estimate == TRUE & N < size.limit]
    if (nrow(var.est.small) > 0) {
      small.group <- unique(var.est.small[, .SD, .SDcols=c(period, z)])
      cat(paste0("For grouping by ", paste(c(period, z), collapse = "~"),
                 ": \n"))
      if (nrow(var.est.small[N < size.limit]) > 10) {
        cat(paste0("Sample size lower ", size.limit, " for ",
                   nrow(var.est.small[N < size.limit]), " groups \n"))
      } else {
        cat(paste0("Sample size lower ", size.limit, " for groups \n"))
        print(small.group)
      }
    }
    
    by.eval2 <- c(period, z)

    if (!is.null(p)) {
      sd.est <- var.est[help_direct_estimate == FALSE, as.list(
        c(stE = sd(x), quantileNA(x, probs = p, p.names = p.names, np = np))),
        by = c(by.eval2), env = list(x = var_est)]
    } else {
      sd.est <- var.est[help_direct_estimate == FALSE, .(stE = sd(x)), by = c(by.eval2),
                        env = list(x = var_est)]
    }
    
    out.z <- merge(var.est[help_direct_estimate == TRUE], sd.est, by = c(by.eval2))
    
    if (bias) {
      bias.est <- var.est[help_direct_estimate == FALSE, .(mean = mean(x)), 
                          by = c(period, z), env = list(x = var_est)]
      out.z <- merge(out.z, bias.est, by = c(period, z))
    }
    
    # define size groups - groups with zero size do not fall under size-output
    # for groups with zero size the resutling estimatese will be NAs
    out.z[(!is.na(n)) & n > 0, size := n < size.limit]
    
    return(out.z)
    
  })
  
  out <- rbindlist(out, fill = TRUE, use.names = TRUE)
  setnames(out, c(var_est, name_var), c("val","est"))
  out[,c(name_weights,"help_direct_estimate"):=NULL]
  
  return(out) 
}



help_diff_group <- function(var.est, by.eval, var_est, diff.group, diff.column){
  
  . <- N <- n <- NULL
  
  by.eval <- by.eval[!by.eval %in% diff.column]
  
  var.est.diff <- var.est[diff.column %in% diff.group, env = list(diff.column = diff.column)]
  var.est.diff[,diff.column := factor(diff.column, levels = diff.group), env = list(diff.column = diff.column)]
  setorderv(var.est.diff,diff.column,order = c(-1)) # diff() ~ x_1-x_0
  var.est.diff <- var.est.diff[,.(N=mean(N), n=mean(n), var_est = diff(var_est)), 
                               by = c(by.eval), env = list(var_est = var_est)]
  
  var.est.diff[, c(diff.column) := paste(diff.group, collapse = "-")]
  var.est.diff <- subset(var.est.diff, select = c(diff.column, by.eval, "n", "N", var_est))
  return(var.est.diff)
  
}

#' @title Get sample selection (~deltas) from drawn bootstrap replicates 
#'
#' @description
#' Reconstruct sample selection, e.g. record was drawn or not drawn (delta = 0/1)
#' in each sampling stage from bootstrap replicates.
#' `get.selection()` needs the `cluster`, `strata` and `hid`/`pid` information (if not `NULL`)
#' to correctly reconstruct if a record was drawn in each sampling stage for each bootstrap replicate.
#' Is only needed if bootstrap replicates are drawn for a survey 
#' with existing bootstrap replicates from a previous period, 
#' see parameter `already.selected` in function [draw.bootstrap()].
#' 
#' @param dat either data.frame or data.table containing the survey data with
#'   rotating panel design. Should contain only survey data from a single time period.
#' @param b.rep character specifying the names of the columns in `dat`
#'   containing bootstrap replicates.
#' @param strata character vector specifying the name(s) of the column in `dat`
#'   by which the population was stratified.
#' @param cluster character vector specifying cluster in the data. 
#' @param hid character specifying the name of the column in `dat` containing
#'   the household id. If `NULL` (the default), the household structure is not
#'   regarded. `hid` and `pid` cannot both be `NULL`.
#' @param pid pid column in `dat` specifying the personal identifier. This
#'   identifier needs to be unique for each person throught the whole data set. 
#'   `hid` and `pid` cannot both be `NULL`.
#'   
#' @returns Returns a list of data.tables. 
#' The length of the list equals the number of sampling stages specified.
#' Each list entry contains a `data.table` with variables for sampling stage and/or
#' `hid`/`pid` as well as `length(attr(dat,"b.rep"))` columns each indicating if 
#' record/cluster was drawn in the respective sampling stage for the i-th boostrap replicate. 
#' @export
#'
#' @examples
#' 
#' library(surveysd)
#' library(data.table)
#' setDTthreads(1)
#' set.seed(1234)
#' eusilc <- demo.eusilc(n = 3, prettyNames = TRUE)
#' 
#' ## draw replicates with stratification
#' dat_boot <- draw.bootstrap(eusilc[year<2012], REP = 3, weights = "pWeight",
#'                            strata = "region", hid = "hid",
#'                            period = "year")
#' 
#' ## get selection matrix for year 2011 
#' dat_selection <- get.selection(dat_boot[year==2011])
#' print(dat_selection)
#' 
#' ## draw bootstrap replicates for year 2012
#' ## respecting already selected units for year 2011 ~ dat_selection
#' ## in order to mimic rotating panel design
#' dat_boot_2012 <- draw.bootstrap(eusilc[year==2012], REP = 3, weights = "pWeight",
#'                                 strata = "region", hid = "hid",
#'                                 period = "year", 
#'                                 already.selected = dat_selection)
#' 
#' 
#' 
get.selection <- function(dat, b.rep = attr(dat,"b.rep"),
                          strata = attr(dat,"strata"),
                          cluster = attr(dat,"cluster"),
                          hid = attr(dat,"hid"),
                          pid = attr(dat,"pid")){
  
  
  . <- ReplicateValues <- selection_col <- selection_prev <- selection_col_prev <-
    problem_col <- ReplicateNames <- c
  
  # -------------------------------
  # check inputs
  
  stages <- length(strata)
  if(is.null(strata)){
    stages <- 1
  }
  
  clusterNULL <- hidNULL <- pidNULL <- strataNULL <- FALSE
  if(is.null(hid)){
    hid <- generateRandomName(existingNames = colnames(dat))
    dat[,c(hid):= 1:.N]
    hidNULL <- TRUE
  }
  if(is.null(pid)){
    pid <- generateRandomName(existingNames = colnames(dat))
    dat[,c(pid):= 1:.N,by=c(hid)]
    pidNULL <- TRUE
  }
  if(is.null(cluster)){
    cluster <- generateRandomName(existingNames = colnames(dat))
    # if cluster is null default to hid
    # if hid is null -> 1:.N
    dat[,c(cluster):= hid, env = list(hid = hid)] 
    clusterNULL <- TRUE
  }
  if(is.null(strata)){
    strata <- generateRandomName(existingNames = colnames(dat))
    dat[,c(strata):= 1]
    strataNULL <- TRUE
  }

  if(all(hidNULL,pidNULL)){
    stop("hid and pid should not both be NULL on order to 
    link the selection variables to the original data!
         \nPlease specify either hid or pid")
  }
  
  
  # --------------------------------
  # melt data for getting deltas
  dat_melt <- melt(dat, id.vars=unique(c(hid,pid,strata,cluster)), measure.vars = b.rep,
                   variable.name = "ReplicateNames", value.name = "ReplicateValues")
  help_selection <- function(v, selection_prev=NULL){
    
    v_out <- rep(0,length(v))
    if(!is.null(selection_prev)){
      v_cutoff <- v[selection_prev==1]
      v_out[v>=v_cutoff] <- 1
      
    }else{
      v_out <- max(v)>1
    }
    
    return(v_out)
  }
  dat_selection <- list()
  selection_cols <- paste0("SelectionStage_",1:stages,"_",generateRandomName(existingNames = colnames(dat_melt), nchar = 10))
  problem_cols <- paste0("ProblemStage_",1:stages,"_",generateRandomName(existingNames = colnames(dat_melt), nchar = 10))
  
  for(i in 1:stages){
    
    # per stage define selected and not selected PSUs
    # get mean of unqiue values 
    # greater than 1 -> selected
    # lower than 1 -> not selected
    by_col <- c(strata[1:i],cluster[1:i],"ReplicateNames")
    selection_i <- selection_cols[i]
    problem_i <- problem_cols[i]
    if(i==1){
      dat_melt_i <- dat_melt[,.(ReplicateValues=unique(ReplicateValues)), by=c(by_col)]
      dat_melt_i[, c(selection_i) := ReplicateValues>1]
      
      dat_melt_i[,c(problem_i) := floor(.N/2)!=sum(selection_col), by=c(strata[i],"ReplicateNames"), 
                 env=list(selection_col = selection_i)]
    }else{
      dat_melt_i <- dat_melt[,.(ReplicateValues=unique(ReplicateValues)), by=c(by_col,selection_cols[1:(i-1)])]
      dat_melt_i[, c(selection_i) := 0]
      dat_melt_i[selection_prev>0 , c(selection_i) := ReplicateValues>mean(ReplicateValues), 
                 by=c(strata[1:(i-1)],cluster[1:(i-1)]), 
                 env = list(selection_prev = selection_cols[i-1])]
      
      dat_melt_i[selection_col_prev == 1,c(problem_i) := floor(.N/2)!=sum(selection_col), 
                 by=c(strata[i],"ReplicateNames"), 
                 env=list(selection_col = selection_i,
                          selection_col_prev = selection_cols[i-1])]
    }
    
    dat_problem <- dat_melt_i[problem_col == TRUE, env=list(problem_col = problem_i)]
    if(nrow(dat_problem)>0){
      # should not happen - faulty input
      # print error
      print_cols <- c(cluster[i][clusterNULL==FALSE],
                      strata[i][strataNULL==FALSE])
      rep_number <- dat_problem[,unique(ReplicateNames)]
      if(is.null(print_cols)){
        example_rep <- paste(rep_number[1:min(5,length(rep_number))],collapse=", ")
        if(length(rep_number)>5){
          example_rep <- paste0(example_rep,", ...")
        }
        stop("Uneven amount of unique replicate values for\n",
             uniqueN(rep_number)," replicate weights (",example_rep,")\n",
             "This function only accepts replicate weights drawn in the same fashion as 
             surveysd::draw.bootstrap(..., method = 'Preston', ...) or surveysd::rescaled.bootstrap(..., method = 'Preston', ...) !")
      }else{
        dat_problem <- unique(dat_problem[,.SD,.SDcols=c(print_cols,"ReplicateNames")])
        stop("Uneven amount of unique replicate values found in \n",
             paste(capture.output(print(dat_problem, row.names = FALSE)),collapse="\n"),
             "\nThis function only accepts replicate weights drawn in the same fashion as 
             surveysd::draw.bootstrap(..., method = 'Preston', ...) or surveysd::rescaled.bootstrap(..., method = 'Preston', ...) !")
      }
    }
    
    # merge dat_melt_i to dat_melt
    dat_melt[dat_melt_i,c(selection_i):=selection_col, on=c(by_col),
             env = list(selection_col = selection_i)]
    
    id_cols <- c()
    if(strataNULL==FALSE){
      id_cols <- c(id_cols, strata)
    }
    if(clusterNULL==FALSE){
      id_cols <- c(id_cols, cluster)
    }
    if(hidNULL==FALSE){
      id_cols <- c(id_cols, hid)
    }
    if(pidNULL==FALSE){
      id_cols <- c(id_cols, pid)
    }
    id_cols <- unique(id_cols)

    form_cast <- as.formula(paste0(paste(id_cols,collapse="+"),"~ReplicateNames"))
    dat_melt <- unique(dat_melt, by=c(id_cols, "ReplicateNames"))
    dat_selection_i <- dcast(dat_melt, formula = form_cast, value.var=selection_i)
    setnames(dat_selection_i, b.rep, paste0("delta_",i,"_",1:length(b.rep)))
    
    dat_selection <- c(dat_selection, list(dat_selection_i))
  }
  
  names(dat_selection) <- paste0("SamplingStage",1:stages)
  return(dat_selection)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @rdname computeFrac
#' @export
computeLinear <- function(curValue, target, x, w, boundLinear = 10) {
    .Call('_surveysd_computeLinear', PACKAGE = 'surveysd', curValue, target, x, w, boundLinear)
}

#' @rdname computeFrac
#' @export
computeLinearG1_old <- function(curValue, target, x, w, boundLinear = 10) {
    .Call('_surveysd_computeLinearG1_old', PACKAGE = 'surveysd', curValue, target, x, w, boundLinear)
}

#' @rdname computeFrac
#' @export
computeLinearG1 <- function(curValue, target, x, w, boundLinear = 10) {
    .Call('_surveysd_computeLinearG1', PACKAGE = 'surveysd', curValue, target, x, w, boundLinear)
}

#' Calculate mean by factors
#'
#' These functions calculate the arithmetic and geometric mean of the weight for each class. `geometric_mean` and
#' `arithmetic_mean` return a `numeric` vector of the same length as `w` which stores the averaged weight for each
#' observation. `geometric_mean_reference` returns the same value by reference, i.e. the input value `w` gets
#' overwritten by the updated weights. See examples.
#'
#' @md
#' @name cpp_mean
#' @param w An numeric vector. All entries should be positive.
#' @param classes A factor variable. Must have the same length as `w`.
#' @examples
#'
#' \dontrun{
#'
#' ## create random data
#' nobs <- 10
#' classLabels <- letters[1:3]
#' dat = data.frame(
#'   weight = exp(rnorm(nobs)),
#'   household = factor(sample(classLabels, nobs, replace = TRUE))
#' )
#' dat
#'
#' ## calculate weights with geometric_mean
#' geom_weight <- geometric_mean(dat$weight, dat$household)
#' cbind(dat, geom_weight)
#'
#' ## calculate weights with arithmetic_mean
#' arith_weight <- arithmetic_mean(dat$weight, dat$household)
#' cbind(dat, arith_weight)
#'
#' ## calculate weights "by reference"
#' geometric_mean_reference(dat$weight, dat$household)
#' dat
#' }
geometric_mean_reference <- function(w, classes) {
    invisible(.Call('_surveysd_geometric_mean_reference', PACKAGE = 'surveysd', w, classes))
}

geometric_mean <- function(w, classes) {
    .Call('_surveysd_geometric_mean', PACKAGE = 'surveysd', w, classes)
}

arithmetic_mean <- function(w, classes) {
    .Call('_surveysd_arithmetic_mean', PACKAGE = 'surveysd', w, classes)
}

rollMeanC <- function(x, k, type) {
    .Call('_surveysd_rollMeanC', PACKAGE = 'surveysd', x, k, type)
}

rollSumC <- function(x, k, type) {
    .Call('_surveysd_rollSumC', PACKAGE = 'surveysd', x, k, type)
}

#' @name PointEstimates
#' @title Weighted Point Estimates
#'
#' @description Predefined functions for weighted point estimates in package `surveysd`.
#'
#' @param x numeric vector
#' @param w weight vector
#'
#' @details Predefined functions are weighted ratio and weighted sum.
#'
#' @return
#' Each of the functions return a single numeric value
#' @examples
#' x <- 1:10
#' w <- 10:1
#' weightedRatio(x,w)
#' @export
weightedRatio <- function(x, w) {
    .Call('_surveysd_weightedRatio', PACKAGE = 'surveysd', x, w)
}

#' @rdname PointEstimates
#' @examples
#' x <- 1:10
#' w <- 10:1
#' weightedSum(x,w)
#' @export
weightedSum <- function(x, w) {
    .Call('_surveysd_weightedSum', PACKAGE = 'surveysd', x, w)
}

#' Perform one step of iterative proportional updating
#'
#' C++ routines to invoke a single iteration of the Iterative proportional updating (IPU) scheme. Targets and classes
#' are assumed to be one dimensional in the `ipf_step` functions. `combine_factors` aggregates several vectors of
#' type factor into a single one to allow multidimensional ipu-steps. See examples.
#'
#' `ipf_step` returns the adjusted weights. `ipf_step_ref` does the same, but updates `w` by reference rather than
#' returning. `ipf_step_f` returns a multiplicator: adjusted weights divided by unadjusted weights. `combine_factors` is
#' designed to make `ipf_step` work with contingency tables produced by [xtabs].
#'
#' @md
#' @name ipf_step
#' @param w a numeric vector of weights. All entries should be positive.
#' @param classes a factor variable. Must have the same length as `w`.
#' @param targets key figure to target with the ipu scheme. A numeric verctor of the same length as `levels(classes)`.
#' This can also be a `table` produced by `xtabs`. See examples.
#' @examples
#'
#' ############# one-dimensional ipu ##############
#'
#' ## create random data
#' nobs <- 10
#' classLabels <- letters[1:3]
#' dat = data.frame(
#'   weight = exp(rnorm(nobs)),
#'   household = factor(sample(classLabels, nobs, replace = TRUE))
#' )
#' dat
#'
#' ## create targets (same lenght as classLabels!)
#' targets <- 3:5
#'
#' ## calculate weights
#' new_weight <- ipf_step(dat$weight, dat$household, targets)
#' cbind(dat, new_weight)
#'
#' ## check solution
#' xtabs(new_weight ~ dat$household)
#'
#' ## calculate weights "by reference"
#' ipf_step_ref(dat$weight, dat$household, targets)
#' dat
#'
#' ############# multidimensional ipu ##############
#'
#' ## load data
#' factors <- c("time", "sex", "smoker", "day")
#' tips <- data.frame(sex=c("Female","Male","Male"), day=c("Sun","Mon","Tue"),
#' time=c("Dinner","Lunch","Lunch"), smoker=c("No","Yes","No"))
#' tips <- tips[factors]
#'
#' ## combine factors
#' con <- xtabs(~., tips)
#' cf <- combine_factors(tips, con)
#' cbind(tips, cf)[sample(nrow(tips), 10, replace = TRUE),]
#'
#' ## adjust weights
#' weight <- rnorm(nrow(tips)) + 5
#' adjusted_weight <- ipf_step(weight, cf, con)
#'
#' ## check outputs
#' con2 <- xtabs(adjusted_weight ~ ., data = tips)
#' sum((con - con2)^2)
#'
#' @rdname ipf_step
#' @export
ipf_step_ref <- function(w, classes, targets) {
    invisible(.Call('_surveysd_ipf_step_ref', PACKAGE = 'surveysd', w, classes, targets))
}

#' @rdname ipf_step
#' @export
ipf_step <- function(w, classes, targets) {
    .Call('_surveysd_ipf_step', PACKAGE = 'surveysd', w, classes, targets)
}

#' @rdname ipf_step
#' @export
ipf_step_f <- function(w, classes, targets) {
    .Call('_surveysd_ipf_step_f', PACKAGE = 'surveysd', w, classes, targets)
}

#' Plot surveysd-Objects
#'
#' Plot results of `calc.stError()`
#'
#' @param x object of class 'surveysd' output of function [calc.stError]
#' @param variable Name of the variable for which standard errors have been
#'   calcualated in `dat`
#' @param type can bei either `"summary"` or `"grouping"`, default value is
#'   `"summary"`. For `"summary"` a barplot is created giving an overview of the
#'   number of estimates having the flag `smallGroup`, `cvHigh`, both or none
#'   of them. For 'grouping' results for point estimate and standard error are
#'   plotted for pre defined groups.
#' @param groups If `type='grouping'` variables must be defined by which the
#'   data is grouped. Only 2 levels are supported as of right now. If only one
#'   group is defined the higher group will be the estimate over the whole
#'   period. Results are plotted for the first argument in `groups` as well as
#'   for the combination of `groups[1]` and `groups[2]`.
#' @param sd.type can bei either `'ribbon'` or `'dot'` and is only used if
#'   `type='grouping'`. Default is `"dot"`
#' For `sd.type='dot'` point estimates are plotted and flagged if the
#' corresponding standard error and/or the standard error using the mean over
#' k-periods exceeded the value `cv.limit` (see [calc.stError]).
#' For `sd.type='ribbon'` the point estimates including ribbons, defined by
#' point estimate +- estimated standard error are plotted.
#' The calculated standard errors using the mean over k periods are plotted
#' using less transparency. Results for the higher level (~`groups[1]`) are
#' coloured grey.
#' @param ... additional arguments supplied to plot.
#'
#' @examples
#'
#' library(surveysd)
#' 
#' set.seed(1234)
#' eusilc <- demo.eusilc(n = 3, prettyNames = TRUE)
#'
#' dat_boot <- draw.bootstrap(eusilc, REP = 3, hid = "hid", weights = "pWeight",
#'                            strata = "region", period = "year")
#'
#' # calibrate weight for bootstrap replicates
#' dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region")
#'
#' # estimate weightedRatio for povmd60 per period
#' group <- list("gender", "region", c("gender", "region"))
#' err.est <- calc.stError(dat_boot_calib, var = "povertyRisk",
#'                         fun = weightedRatio,
#'                         group = group , period.mean = NULL)
#'
#'
#' plot(err.est)
#'
#' # plot results for gender
#' # dotted line is the result on the national level
#' plot(err.est, type = "grouping", groups = "gender")
#'
#'
#' # plot results for rb090 in each db040
#' # with standard errors as ribbons
#' plot(err.est, type = "grouping", groups = c("gender", "region"), sd.type = "ribbon")
#'
#'
#' @export

plot.surveysd <- function(
  x, variable = x$param$var[1], type = c("summary", "grouping"), groups =
    NULL, sd.type = c("dot", "ribbon"), ...) {

  res_type <- GROUPING <- shape_bool <- shape_bool2 <- NULL


  #################
  # Input checking
  type <- type[1]
  sd.type <- sd.type[1]
  if (class(x)[1] != "surveysd") {
    stop("The data needs to be an object of class 'surveysd'!")
  }
  if (!type %in% c("summary", "grouping")) {
    stop("Parameter type can only take values 'summary' or 'grouping'!")
  }
  if (!variable %in% x$param$var) {
    stop("No results for ", variable, " present in the data!")
  }
  if (type == "grouping") {

    if (is.null(groups)) {
      stop("Paramter 'groups' cannot be NULL if type='grouping'!")
    }

    other_var <- unique(unlist(x$param$group))
    if (any(!groups %in% other_var)) {
      stop("Variables in 'groups' must contain variables from x$params$group!")
    }

    match_group <- lapply(
      x$param$group,
      function(z) {
        if (!is.null(z)) {
          identical(sort(z), sort(groups))
        }
      }
    )
    match_group <- any(unlist(match_group))
    if (length(groups) > 2) {
      stop("More than a maximum of 2 groups is not supported as of right now.")
    }
    if (!match_group) {
      if (length(groups) == 1) {
        stop("No results for ", groups, " present in the data")
      } else {
        stop("No results for the combination of ", groups[1], " and ",
             groups[2], " present in the data")
      }
    }
  }
  if (!sd.type %in% c("ribbon", "dot")) {
    stop("Parameter 'sd.type' can only take values 'ribbon' or 'dot'!")
  }
  # convert all factors for data tables in x
  # otherwise ther will be issues with merges
  x$Estimates <- convert_factors(x$Estimates)
  x$smallGroups <- convert_factors(x$smallGroups)
  x$cvHigh <- convert_factors(x$cvHigh)
  x$stEDecrease <- convert_factors(x$stEDecrease)
  #################
  # get variables from x and merge tables together
  period <- x$param$period

  poss.values <- c("Missing", "cvHigh", "SmallGroup+cvHigh", "SmallGroup", "OK")
  poss.color <- c("grey", "orangered3", "yellow2", "dodgerblue1", "green4")

  plot.x <- x$Estimates
  plot.x <- define_type(plot.x, x, variable = variable)
  values.plot <- plot.x[, unique(res_type)]
  color.plot <- poss.color[poss.values %in% values.plot]

  # define groups
  dt.eval("plot.x[grepl('^[[:digit:]]+$',", period,
          "),GROUPING:='Single Periods']")
  dt.eval("plot.x[grepl('-',", period, "),GROUPING:='Other']")
  dt.eval("plot.x[grepl('_',", period, "),GROUPING:='", x$param$period.mean,
          "-Period-Mean']")
  plot.x[, GROUPING := factor(GROUPING, levels = c("Single Periods", paste0(
    x$param$period.mean, "-Period-Mean"), "Other"))]

  #################
  # plot type 'summary'
  if (type == "summary") {
    title <- paste("Results of Groups per period for variable", variable)
    p1 <- ggplot(plot.x, aes(get(period), fill = res_type)) +
      geom_bar() + xlab("") + ylab("Count") + coord_flip() +
      facet_grid(GROUPING ~ ., scales = "free_y", space = "free_y") +
      theme(legend.title = element_blank()) +
      scale_fill_manual(breaks = values.plot,
                        values = color.plot) +
      ggtitle(title)
    plot(p1)
  } else if (type == "grouping") {
    #################
    # plot type 'grouping'
    if (length(groups) == 1) {
      groups <- c(period, groups)
    }

    if (length(other_var) > 0) {
      na_var <- other_var[!other_var %in% groups]
      if (length(na_var) > 0) {
        na_var <- paste(paste0("is.na(", na_var, ")"), collapse = "&")
        na_var <- paste0("&", na_var)
      }

      exp1 <- paste0("c(!is.na(", groups[1], ")&!is.na(", groups[2], ")",
                     na_var, ")")

      na_var <- other_var[!other_var %in% groups[1]]
      exp2_2 <- paste(paste0("is.na(", na_var, ")"), collapse = "&")
      exp2 <- paste0("c(!is.na(", groups[1], ")&", exp2_2, ")")
    } else {
      exp1 <- paste0("c(!is.na(", groups[1], ")&!is.na(", groups[2], "))")
      exp2 <- paste0("c(!is.na(", groups[1], ")&is.na(", groups[2], "))")
    }


    plot.group1 <- dt.eval("plot.x[", exp1, "]")
    plot.group2 <- dt.eval("plot.x[", exp2, "]")

    val_var <- paste0("val_", variable)
    ste_var <- paste0("stE_", variable)
    ste_bool <- variable


    # prepare plot.group1 for plotting
    plot.group1.period <- plot.group1[GROUPING == "Single Periods"]
    plot.group1.periodmean <- plot.group1[GROUPING == paste0(
      x$param$period.mean, "-Period-Mean")]

    # prepare plot.group2 for plotting
    plot.group2.period <- plot.group2[GROUPING == "Single Periods"]
    plot.group2.periodmean <- plot.group2[GROUPING == paste0(
      x$param$period.mean, "-Period-Mean")]

    # rename data columns for plot.group2
    ste_var2 <- paste0(ste_var, "2")
    val_var2 <- paste0(val_var, "2")
    ste_bool2 <- paste0(ste_bool, "2")

    # prepare output differently if period.mean has been used
    if (!is.null(x$param$period.mean)) {
      period.mean <- TRUE

      ste_var_mean <- paste0(ste_var, "_mean")
      ste_bool_mean <- paste0(ste_bool, "_mean")
      ste_var_mean2 <- paste0(ste_var_mean, "2")
      ste_bool_mean2 <- paste0(ste_bool_mean, "2")

      dt.eval("plot.group1.periodmean[,", period, ":=tstrsplit(", period,
              ",split='_',keep=", round((x$param$period.mean + 1) / 2), ")]")

      setnames(plot.group1.periodmean, c(ste_var, ste_bool),
               c(ste_var_mean, ste_bool_mean))

      plot.group1.period <- merge(
        plot.group1.period, dt.eval(
          "plot.group1.periodmean[,.(",
          paste(c(period, other_var, ste_var_mean, ste_bool_mean),
                collapse = ","),
          ")]"),
        all.x = TRUE)

      dt.eval("plot.group2.periodmean[,", period, ":=tstrsplit(", period,
              ",split='_',keep=", round((x$param$period.mean + 1) / 2), ")]")

      setnames(plot.group2.periodmean, c(ste_var, ste_bool),
               c(ste_var_mean, ste_bool_mean))
      plot.group2.period <- merge(
        plot.group2.period,
        dt.eval("plot.group2.periodmean[,.(",
                paste(c(period, other_var, ste_var_mean, ste_bool_mean),
                      collapse = ","), ")]"),
        all.x = TRUE)

      change.names <- c(ste_var, val_var, ste_bool, ste_var_mean, ste_bool_mean)
    } else {
      period.mean <- FALSE
      change.names <- c(ste_var, val_var, ste_bool)
    }

    setnames(plot.group2.period, change.names, paste0(change.names, "2"))
    select.group2 <- paste0(change.names, "2")

    # merge with plot.group
    if (period == groups[1]) {
      plot.group1.period <- merge(
        plot.group1.period,
        plot.group2.period[, mget(c(groups[1], select.group2))],
        all.x = TRUE, by = groups[1])
    } else {
      plot.group1.period <- merge(
        plot.group1.period,
        plot.group2.period[, mget(c(groups[1], period, select.group2))],
        all.x = TRUE, by = c(groups[1], period))
    }

    dt.eval("plot.group1.period[,", groups[1], ":=factor(", groups[1], ")]")
    dt.eval("plot.group1.period[,", groups[2], ":=factor(", groups[2], ")]")

    # plot results of subgroups and add results for mean over k periods
    p1 <- dt.eval("ggplot(plot.group1.period,aes(", period, ",", val_var,
                  ",group=", groups[2], "))+
                    geom_line(aes(colour=get(groups[2])))")

    if (sd.type == "ribbon") {
      p1 <- p1 + geom_ribbon(
        aes(ymin = get(val_var) - get(ste_var),
            ymax = get(val_var) + get(ste_var),
            fill = get(groups[2]), colour = get(groups[2])),
        linetype = 2, alpha = 0.1)


      # add results for group[1] -> higher level grouping
      p1 <- dt.eval("p1 + geom_line(aes(", period, ",", val_var2,
                    ",linetype='dotted'),color='black')")
      p1 <- p1 + geom_ribbon(
        aes(ymin = get(val_var2) - get(ste_var2),
            ymax = get(val_var2) + get(ste_var2)),
        fill = "grey", linetype = 2,  alpha = 0.1)

      if (period.mean) {
        # add results for k-mean-periods
        p1 <- p1 + geom_ribbon(
          aes(ymin = get(val_var) - get(ste_var_mean),
              ymax = get(val_var) + get(ste_var_mean), fill = get(groups[2]),
              colour = get(groups[2])),
          linetype = 2, alpha = 0.5)

        # add results for group[1] and k-period-mean
        p1 <- p1 + geom_ribbon(
          aes(ymin = get(val_var2) - get(ste_var_mean2),
              ymax = get(val_var2) + get(ste_var_mean2)),
          fill = "grey", linetype = 2, alpha = 0.5)
      }

    } else {
      plot.group1.period[, shape_bool := "Low st.Error"]
      plot.group1.period[get(ste_bool) == TRUE, shape_bool := "high st.Error"]
      plot.group1.period[, shape_bool2 := "Low st.Error"]
      plot.group1.period[get(ste_bool2) == TRUE, shape_bool2 := "high st.Error"]

      if (period.mean) {
        plot.group1.period[get(ste_bool_mean) == TRUE,
                           shape_bool := "high st.Error for mean"]
        plot.group1.period[get(ste_bool_mean2) == TRUE,
                           shape_bool2 := "high st.Error for mean"]
      }

      p1 <- dt.eval(
        "p1 + geom_point(data=plot.group1.period[get(ste_bool)==TRUE],
                         aes(", period, ",", val_var, ",shape=shape_bool))")

      # add results for group[1] -> higher level grouping
      p1 <- dt.eval("p1 + geom_line(aes(", period, ",", val_var2,
                    ",linetype='solid'))")
      p1 <- dt.eval(
        "p1 + geom_point(data=plot.group1.period[get(ste_bool2)==TRUE],
                         aes(", period, ",", val_var, ",shape=shape_bool2))")

      if (period.mean) {
        # add results for k-mean-periods
        p1 <- dt.eval("p1 + geom_point(data=plot.group1.period[", ste_bool_mean,
                      "==TRUE],
                      aes(", period, ",", val_var, ",shape=shape_bool))")

        # add results for group[1] and k-period-mean
        p1 <- dt.eval("p1 + geom_point(data=plot.group1.period[",
                      ste_bool_mean2, "==TRUE],
                      aes(", period, ",", val_var, ",shape=shape_bool2))")

      }
    }


    # define paramters for plot
    ylabel <- paste0("Point Estimate on variable ", x$param$var)
    p1 <- p1 + xlab(x$param$period) + ylab(ylabel) +
      theme(axis.text.x = element_text(angle = 45, hjust = 1))


    # split the plots into facets
    if (groups[1] == period) {
      p1 <- p1 + facet_wrap(~get(groups[2]))
      if (sd.type == "ribbon") {
        p1 <- p1 + guides(fill = 'none',
                          colour = 'none',
                          linetype = guide_legend(title = ""))
        p1 <- p1 + scale_linetype_manual(
          values = c("solid"), labels = paste0("Result for ", groups[1]))
      } else {
        shapes <- c(1, 16)
        names(shapes) <- c("high st.Error", "high st.Error for mean")
        p1 <- p1 + scale_shape_manual(values = shapes)
        p1 <- p1 + scale_colour_discrete(guide = 'none') +
        # p1 <- p1 + scale_colour_discrete(value="grey")
           theme(legend.title = element_blank())
        p1 <- p1 + scale_linetype_manual(values = c("dotted"), labels = paste0(
          "Result for ", groups[1]))
      }
    } else {
      p1 <- p1 + facet_wrap(~get(groups[1]))
      if (sd.type == "ribbon") {
        p1 <- p1 + guides(fill = guide_legend(title = groups[2]),
                          colour = guide_legend(title = groups[2]),
                          linetype = guide_legend(title = ""))
        p1 <- p1 + scale_linetype_manual(values = c("solid"), labels = paste0(
          "Result for ", groups[1]))
      } else {
        shapes <- c(1, 16)
        names(shapes) <- c("high st.Error", "high st.Error for mean")
        p1 <- p1 + scale_shape_manual(values = shapes) +
          guides(colour = guide_legend(title = groups[2]),
                 linetype = guide_legend(title = ""),
                 shape = guide_legend(title = ""))

        p1 <- p1 + scale_linetype_manual(values = c("dotted"), labels = paste0(
          "Result for ", groups[1]))
      }
    }

    plot(p1)
  }
}


define_type <- function(plot.x, x, variable = "HX080") {

  SMALLGROUP <- res_type <- NULL

  val_variable <- paste("val", variable, sep = "_")
  poss.values <- c("Missing", "cvHigh", "SmallGroup+cvHigh", "SmallGroup", "OK")

  # merge data
  merge.on <- copy(colnames(x$smallGroups))
  x$smallGroups[, SMALLGROUP := TRUE]
  plot.x <- x$smallGroups[plot.x,, on = c(merge.on)] # nolint
  merge.on <- unique(c(x$param$period, unlist(x$param$group)))
  plot.x <- x$cvHigh[plot.x,, on = c(merge.on)] # nolint

  # define type
  plot.x[!is.na(SMALLGROUP) & get(variable) == FALSE, res_type := "SmallGroup"]
  plot.x[!is.na(SMALLGROUP) & get(variable) == TRUE,
         res_type := "SmallGroup+cvHigh"]
  plot.x[is.na(get(val_variable)), res_type := "Missing"]
  plot.x[get(variable) == TRUE & is.na(res_type), res_type := "cvHigh"]
  plot.x[is.na(res_type), res_type := "OK"]
  plot.x[, res_type := factor(res_type, levels = poss.values)]

  return(plot.x)
}

convert_factors <- function(x) {
  x <- x[, lapply(
    .SD,
    function(z) {
      if (is.factor(z)) {
        as.character(z)
      } else {
        z
      }
    }
  )]
  return(x)
}

---
title: "Iterative Proportional Fitting"
output: rmarkdown::html_vignette
author:
  - Gregor de Cillia
vignette: >
  %\VignetteIndexEntry{Iterative Proportional Fitting}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: lib.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(data.table)
setDTthreads(1)
```

This vignette explains the usage of the `ipf()` function, which has been used for calibrating the labour force survey of Austria for several years.
It is based on the Iterative Proportional Fitting algorithm and gives some flexibility about the details of the implementation. See [@mekogu2016] or `vignette("methodology")` for more details.

## Setup

We will assume the output of `demo.eusilc()` is our population.
From this population, a sample without replacement is drawn.
The sample covers 10 percent of the population.
We assign a weight of one for all observations of the population and a weight of ten for all observations of the sample.

```{r setup}
library(surveysd)
population <- demo.eusilc(1, prettyNames = TRUE)
population[, pWeight := 1]
pop_sample <- population[sample(1:.N, floor(.N*0.10)), ]
pop_sample[, pWeight := 10]
```

## One constraint, one variable

We will start with an example where we want to adapt the weights of `pop_sample` such that the weighted number of males and females matches the ones of `population`.
We can see that this is currently not the case.

```{r}
(gender_distribution <- xtabs(pWeight ~ gender, population))
xtabs(pWeight ~ gender, pop_sample)
```

Due to random sampling (rather than stratified sampling), there are differences between the gender distributions.
We can pass `gender_distribution` as a parameter to `ipf()` to obtain modified weights.

```{r}
pop_sample_c <- ipf(pop_sample, conP = list(gender_distribution), w = "pWeight")
```

The resulting dataset, `pop_sample_c` is similar to `pop_sample` but has an additional column with the adjusted weights.

```{r}
dim(pop_sample)
dim(pop_sample_c)
setdiff(names(pop_sample_c), names(pop_sample))
```

We can now calculate the weighted number of males and females according to this new weight.
This will result in a match for the constraints.

```{r}
xtabs(calibWeight ~ gender, pop_sample_c)
xtabs(pWeight ~ gender, population)
```

In this simple case, `ipf` just performs a post stratification step.
This means, that all males and all females have the same weight.

```{r, fig.align="center", out.width="100%"}
xtabs(~ calibWeight + gender, pop_sample_c)
```

```{r, include = FALSE}
overrepresented_gender <- pop_sample_c[calibWeight < 10, ][1, gender]
```

All `r overrepresented_gender`s have been weighted down (`calibWeight < 10`) to compensate for the overrepresentation in the sample.

## One constraint, two variables

Let's now assume that we want to put constraints on the number of males and females for each age group. 
The numbers from the original population can be obtained with `xtabs()`.

```{r}
(con_ga <- xtabs(pWeight ~ gender + age, population))
xtabs(pWeight ~ gender + age, pop_sample)
```

Again, we can see that those constraints are not met.
Supplying the contingency table `con_ga` to `ipf()` will again resolve this.

```{r}
pop_sample_c2 <- ipf(pop_sample, conP = list(con_ga), w = "pWeight")
xtabs(pWeight ~ gender + age, population)
xtabs(calibWeight ~ gender + age, pop_sample_c2)
```

## Two constraints

Now we assume that we know the number of persons living in each nuts2 region from registry data.

```{r}
registry_table <- xtabs(pWeight ~ region, population)
```

However, those registry data does not provide any information about age or `gender`.
Therefore, the two contingency tables (`con_ga` and `registry_table`) have to be specified independently.
This can be done by supplying a list to `conP`

```{r}
pop_sample_c2 <- ipf(pop_sample, conP = list(con_ga, registry_table), w = "pWeight")
xtabs(pWeight ~ gender + age, population)
xtabs(calibWeight ~ gender + age, pop_sample_c2)
xtabs(pWeight ~ region, population)
xtabs(calibWeight ~ region, pop_sample_c2)
```

this time, the constraints are not matched perfectly.
That is, because we provided more than one constraint.
therefore, the `ipf()` algorithm had to work iteratively.

## Household Constraints

If the dataset has a household structure, household constraints can be passed
via the parameter `conH`. If this parameter is used, it is also necessary to
supply `hid`, which defines the column names that contains household ids.

```{r}
(conH1 <- xtabs(pWeight ~ hsize + region, data = population[!duplicated(hid)]))
pop_sample_hh <- ipf(pop_sample, hid = "hid", conH = list(conH1), w = "pWeight",
                     bound = 10)
xtabs(calibWeight ~ hsize + region, data = pop_sample_hh[!duplicated(hid)])
```

## Tolerances

If `conP` or `conH` contain several contingency tables or if `conP` and `conH`
are used at the same time, the ipf algorithm will operate iteratively. This
means that the calibrated dataset will satisfy the constraints only
approximately. The default tolerances of the approximation can be overwritten
using the parameters `conP` and `conH`.

Lowering the tolerances will improve the match between the constraints and
the contingency tables according to the calibrated weights. However, lower
tolerances will also make it so more iterations are necessary until a
convergence is met. If the constraints are too small, ipf will return with
a warning that indicates that a convergence could not be reached.

```{r}
ipf(pop_sample, conP = list(con_ga, registry_table), w = "pWeight",
    verbose = TRUE, epsP = 0.01)
ipf(pop_sample, conP = list(con_ga, registry_table), w = "pWeight",
    verbose = TRUE, epsP = 0.0001)
```

We see that changing the tolerances from `0.01` (one percent) to `0.0001`
increases the number of required iterations.

## References

---
title: "Introduction to surveysd"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    df_print: kable
vignette: >
  %\VignetteIndexEntry{surveysd}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)

library(data.table)
setDTthreads(1)
```

The goal of surveysd is to combine all necessary steps to use calibrated bootstrapping with
custom estimation functions. This vignette will cover the usage of the most important functions.
For insights in the theory used in this package, refer to `vignette("methodology")`.

### Load dummy data

A test data set based on `data(eusilc, package = "laeken")` can be created with `demo.eusilc()`

```{r}
library(surveysd)

set.seed(1234)
eusilc <- demo.eusilc(n = 2, prettyNames = TRUE)

eusilc[1:5, .(year, povertyRisk, gender, pWeight)]
``` 

### Draw bootstrap replicates

Use stratified resampling without replacement to generate 10 samples. Those samples are
consistent with respect to the reference periods.

```{r}
dat_boot <- draw.bootstrap(eusilc, REP = 10, hid = "hid", weights = "pWeight", 
                           strata = "region", period = "year")
```
   
### Calibrate bootstrap replicates

Calibrate each sample according to the distribution of `gender` (on a personal level) and `region` 
(on a household level). 

```{r}
dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region",
                          epsP = 1e-2, epsH = 2.5e-2, verbose = FALSE)
dat_boot_calib[1:5, .(year, povertyRisk, gender, pWeight, w1, w2, w3, w4)]
```

### Estimate with respect to a grouping variable

Estimate relative amount of persons at risk of poverty per period and `gender`.

```{r}
err.est <- calc.stError(dat_boot_calib, var = "povertyRisk", fun = weightedRatio, group = "gender")
err.est$Estimates
```

The output contains estimates (`val_povertyRisk`) as well as standard errors (`stE_povertyRisk`)
measured in percent. The rows with `gender = NA` denotes the aggregate over all genders for the corresponding year.

### Estimate with respect to several variables

Estimate relative amount of persons at risk of poverty per period for each `region`, 
`gender`, and combination of both.

```{r}
group <- list("gender", "region", c("gender", "region"))
err.est <- calc.stError(dat_boot_calib, var = "povertyRisk", fun = weightedRatio, group = group)
head(err.est$Estimates)
## skipping 54 more rows
```

---
title: "Introduction to surveysd"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    df_print: kable
vignette: >
  %\VignetteIndexEntry{surveysd}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)
```

The goal of surveysd is to combine all necessary steps to use calibrated bootstrapping with
custom estimation functions. This vignette will cover the usage of the most important functions.
For insights in the theory used in this package, refer to `vignette("methodology")`.

### Load dummy data

A test data set based on `data(eusilc, package = "laeken")` can be created with `demo.eusilc()`

```{r}
library(surveysd)

set.seed(1234)
eusilc <- demo.eusilc(n = 2, prettyNames = TRUE)

eusilc[1:5, .(year, povertyRisk, gender, pWeight)]
``` 

### Draw bootstrap replicates

Use stratified resampling without replacement to generate 10 samples. Those samples are
consistent with respect to the reference periods.

```{r}
dat_boot <- draw.bootstrap(eusilc, REP = 10, hid = "hid", weights = "pWeight", 
                           strata = "region", period = "year")
```
   
### Calibrate bootstrap replicates

Calibrate each sample according to the distribution of `gender` (on a personal level) and `region` 
(on a household level). 

```{r}
dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region",
                          epsP = 1e-2, epsH = 2.5e-2, verbose = FALSE)
dat_boot_calib[1:5, .(year, povertyRisk, gender, pWeight, w1, w2, w3, w4)]
```

### Estimate with respect to a grouping variable

Estimate relative amount of persons at risk of poverty per period and `gender`.

```{r}
err.est <- calc.stError(dat_boot_calib, var = "povertyRisk", fun = weightedRatio, group = "gender")
err.est$Estimates
```

The output contains estimates (`val_povertyRisk`) as well as standard errors (`stE_povertyRisk`)
measured in percent. The rows with `gender = NA` denotes the aggregate over all genders for the corresponding year.

### Estimate with respect to several variables

Estimate relative amount of persons at risk of poverty per period for each `region`, 
`gender`, and combination of both.

```{r}
group <- list("gender", "region", c("gender", "region"))
err.est <- calc.stError(dat_boot_calib, var = "povertyRisk", fun = weightedRatio, group = group)
head(err.est$Estimates)
## skipping 54 more rows
```

---
title: "Error estimation"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    df_print: kable
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{error estimation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)

library(data.table)
setDTthreads(1)
```

For the most part, this document will present the functionalities of the function 
`surveysd::calc.stError()` which generates point estimates and standard errors for
user-supplied estimation functions.

## Prerequisites

In order to use a dataset with `calc.stError()`, several weight columns have to be present. Each
weight column corresponds to a bootstrap sample. In the following examples, we will use the data 
from `demo.eusilc()` and attach the bootstrap weights using `draw.bootstrap()` and `recalib()`. Please 
refer to the documentation of those functions for more detail.


```{r}
library(surveysd)

set.seed(1234)
eusilc <- demo.eusilc(prettyNames = TRUE)
dat_boot <- draw.bootstrap(eusilc, REP = 10, hid = "hid", weights = "pWeight",
                           strata = "region", period = "year")
dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region",
                          epsP = 1e-2, epsH = 2.5e-2, verbose = FALSE)
dat_boot_calib[, onePerson := nrow(.SD) == 1, by = .(year, hid)]

## print part of the dataset
dat_boot_calib[1:5, .(year, povertyRisk, eqIncome, onePerson, pWeight, w1, w2, w3, w4, w5)]
```


## Estimator functions

The parameters `fun` and `var` in `calc.stError()` define the estimator to be used in the error 
analysis. There are two built-in estimator functions `weightedSum()` and `weightedRatio()` which can
be used as follows.


```{r}
povertyRate <- calc.stError(dat_boot_calib, var = "povertyRisk", fun = weightedRatio)
totalIncome <- calc.stError(dat_boot_calib, var = "eqIncome", fun = weightedSum)
```

Those functions calculate the ratio of persons at risk of poverty (in percent) and the total income.
By default, the results are calculated separately for each reference period.

```{r}
povertyRate$Estimates
totalIncome$Estimates
```

Columns that use the `val_` prefix denote the point estimate belonging to the "main weight" of the
dataset, which is `pWeight` in case of the dataset used here.

Columns with the `stE_` prefix denote standard errors calculated with bootstrap replicates. The
replicates result in using `w1`, `w2`, ..., `w10` instead of `pWeight` when applying the estimator.

`n` denotes the number of observations for the year and `N` denotes the total weight of those
persons.

### Custom estimators

In order to define a custom estimator function to be used in `fun`, the function needs to have
at least two arguments like the example below.

```{r}
## define custom estimator
myWeightedSum <- function(x, w) {
  sum(x*w)
}

## check if results are equal to the one using `surveysd::weightedSum()`
totalIncome2 <- calc.stError(dat_boot_calib, var = "eqIncome", fun = myWeightedSum)
all.equal(totalIncome$Estimates, totalIncome2$Estimates)
```

The parameters `x` and `w` can be assumed to be vectors with equal length with `w` being numeric
weight vector and `x` being the column defined in the `var` argument. It will be called once
for each period (in this case `year`) and for each weight column (in this case `pWeight`, `w1`, `w2`, ..., `w10`).

Custom estimators using additional parameters can also be supplied and parameter `add.arg` can be used
to set the additional arguments for the custom estimator.

```{r}
## use add.arg-argument
fun <- function(x, w, b) {
  sum(x*w*b)
}
add.arg = list(b="onePerson")

err.est <- calc.stError(dat_boot_calib, var = "povertyRisk", fun = fun,
                        period.mean = 0, add.arg=add.arg)
err.est$Estimates

# compare with direct computation
compare.value <- dat_boot_calib[,fun(povertyRisk,pWeight,b=onePerson),
                                 by=c("year")]
all((compare.value$V1-err.est$Estimates$val_povertyRisk)==0)
```

The above chunk computes the weighted poverty ratio for single person households.

### Adjust variable depending on bootstrap weights

In our example the variable `povertyRisk` is a boolean and is `TRUE` if the income is less
than 60% of the weighted median income. Thus it directly depends on the original weight vector
`pWeight`. To further reduce the estimated error one should calculate for each bootstrap replicate
weight $w$ the weighted median income $medIncome_{w}$ and then define $povertyRisk_w$ as

$$
povertyRisk_w = \cases{1 \quad\text{if Income}<0.6\cdot medIncome_{w}\\
                     0 \quad\text{else}}
$$

The estimator can then be applied to the new variable $povertyRisk_w$.
This can be realized using a custom estimator function.

```{r}
# custom estimator to first derive poverty threshold 
# and then estimate a weighted ratio
povmd <- function(x, w) {
 md <- laeken::weightedMedian(x, w)*0.6
 pmd60 <- x < md
 # weighted ratio is directly estimated inside the function
 return(sum(w[pmd60])/sum(w)*100)
}

err.est <- calc.stError(
  dat_boot_calib, var = "povertyRisk", fun = weightedRatio,
  fun.adjust.var = povmd, adjust.var = "eqIncome")
err.est$Estimates

```


The approach shown above is only valid if no grouping variables are supplied (parameter `group = NULL`).
If grouping variables are supplied one should use parameters `fun.adjust.var` and `adjust.var` such that
the $povertyRisk_w$ is first calculated for each `period` and then used for each grouping in `group`.


```{r}
# using fun.adjust.var and adjust.var to estimate povmd60 indicator
# for each period and bootstrap weight before applying the weightedRatio
povmd2 <- function(x, w) {
 md <- laeken::weightedMedian(x, w)*0.6
 pmd60 <- x < md
 return(as.integer(pmd60))
}

# set adjust.var="eqIncome" so the income vector is used to estimate
# the povmd60 indicator for each bootstrap weight
# and the resulting indicators are passed to function weightedRatio
group <- "gender"
err.est <- calc.stError(
  dat_boot_calib, var = "povertyRisk", fun = weightedRatio, group = "gender",
  fun.adjust.var = povmd2, adjust.var = "eqIncome")
err.est$Estimates
```


### Multiple estimators

In case an estimator should be applied to several columns of the dataset, `var` can be set to
a vector containing all necessary columns.

```{r}
multipleRates <- calc.stError(dat_boot_calib, var = c("povertyRisk", "onePerson"), fun = weightedRatio)
multipleRates$Estimates
```
Here we see the relative number of persons at risk of poverty and the relative number of one-person
households.

## Grouping

The `groups` argument can be used to calculate estimators for different subsets of the data. This
argument can take the grouping variable as a string that refers to a column name (usually a factor) 
in `dat`. If set, all estimators are not only split by the reference period but also by the
grouping variable. For simplicity, only one reference period of the above data is used.

```{r}
dat2 <- subset(dat_boot_calib, year == 2010)
for (att  in c("period", "weights", "b.rep"))
  attr(dat2, att) <- attr(dat_boot_calib, att)
```

To calculate the ratio of persons at risk of poverty for each federal state of Austria, 
`group = "region"` can be used.

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, group = "region")
povertyRates$Estimates
```

The last row with `region = NA` denotes the aggregate over all regions. Note that the
columns `N` and `n` now show the weighted and unweighted number of persons in each region.

### Several grouping variables

In case more than one grouping variable is used, there are several options of calling 
`calc.stError()` depending on whether combinations of grouping levels should be regarded or not.
We will consider the variables `gender` and `region` as our grouping variables and show three 
options on how `calc.stError()` can be called.

#### Option 1: All regions and all genders

Calculate the point estimate and standard error for each region and each gender. The number of 
rows in the output is therefore 

$$n_\text{periods}\cdot(n_\text{regions} + n_\text{genders} + 1) = 1\cdot(9 + 2 + 1) = 12.$$

The last row is again the estimate for the whole period.

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = c("gender", "region"))
povertyRates$Estimates

```

#### Option 2: All combinations of `region` and `gender`

Split the data by all combinations of the two grouping variables. This will result in a larger 
output-table of the size

$$n_\text{periods}\cdot(n_\text{regions} \cdot n_\text{genders} + 1) = 1\cdot(9\cdot2 + 1)= 19.$$

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = list(c("gender", "region")))
povertyRates$Estimates
```

#### Option 3: Cobination of Option 1 and Option 2

In this case, the estimates and standard errors are calculated for

* every gender,
* every region and
* every combination of region and gender.

The number of rows in the output is therefore 

$$n_\text{periods}\cdot(n_\text{regions} \cdot n_\text{genders} + n_\text{regions} + n_\text{genders} + 1) = 1\cdot(9\cdot2 + 9 + 2 + 1) = 30.$$

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = list("gender", "region", c("gender", "region")))
povertyRates$Estimates
```


## Group differences

If differences between groups need to be calculated, e.g difference of poverty rates between `gender = "male"` and `gender = "female"`, parameter `group.diff` can be utilised.
Setting `group.diff = TRUE` the differences and the standard error of these differences for all variables defined in `groups` will be calculated.

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = c("gender", "region"),
                             group.diff = TRUE)
povertyRates$Estimates
```

The resulting output table contains `r nrow(povertyRates$Estimates)` rows. `r sum(povertyRates$Estimates$estimate_type == "direct")` rows for all the direct estimators

$$n_\text{periods}\cdot(n_\text{regions} + n_\text{genders} + 1) = 1\cdot(9 + 2 + 1) = 12,$$

and another `r sum(povertyRates$Estimates$estimate_type == "group difference")` for all the differences within the variable `"gender"` and `"region"` seperately.
Variable `"gender"` has 2 unique values (`unique(dat2$gender)`) resulting in 1 difference, ~ `gender = "male"` - `gender = "female"` and variable `"region"` has
9 unique values (`unique(dat2$region)`) resulting in 

$$8 + 7 + 6 + 5 + 4 + 3 + 2 + 1  = \sum\limits_{1=1}^{9-1}i = 36$$

estimates. Thus the output contains 1 + 36 = 37 estimates with respect to group differences.

If a combintaion of grouping variables is used in `group` and `group.diff = TRUE` then differences between combinations will only be calculated
if one of the grouping variables differs.
For example the difference between the following groups would be calculated

- `gender = "female" & region = "Vienna"` - `gender = "male" & region = "Vienna"` 
- `gender = "female" & region = "Vienna"` - `gender = "female" & region = "Salzburg"`
- `gender = "male" & region = "Salzburg"` - `gender = "female" & region = "Salzburg"`

The difference between `gender = "female" & region = "Vienna"` and `gender = "male" & region = "Salzburg"` however would not be calculated.

Thus this leads to 

$$2\cdot(\sum\limits_{1=1}^{9-1}i) + 9\cdot1 = 81$$

results with respect to the differences.
The Output contains an additional column `estimate_type` and 

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = list(c("gender", "region")),
                             group.diff = TRUE)
povertyRates$Estimates[,.N,by=.(estimate_type)]
```

## Differences between survey periods

Differences of estimates between `period`s can be calculated using parameter `period.diff`.
`period.diff` expects a character vector (if not `NULL`) specifying for which `period`s the differences should be calcualed for.
The inputs should be specified in the form `"period2" - "period1"`.

```{r}
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             period.diff = c("2017 - 2016", "2016 - 2015", "2015 - 2014"))
povertyRates$Estimates
```

If additional grouping variables are supplied to `calc.stError()` die differences across `period`s are also carried out
for all variables in `group`.

```{r}
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             group = "gender",
                             period.diff = c("2017 - 2016", "2016 - 2015", "2015 - 2014"))
povertyRates$Estimates
```


## Averages across periods

With parameter `period.mean` averages across `period`s are calculated additional. The parameter accepts only odd integer values.
The resulting table will contain the direct estimates as well as rolling averages of length `period.mean`.

```{r}
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             period.mean = 3)
povertyRates$Estimates
```

if in addition the parameters `group` and/or `period.diff` are specified then differences and groupings of averages will be calculated.

```{r}
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             period.mean = 3, period.diff = "2016 - 2015",
                             group = "gender")
povertyRates$Estimates
```

---
title: "Methodology"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{methodology}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: lib.bib
---

```{r, echo = FALSE}
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(eval = FALSE)

library(data.table)
setDTthreads(1)
```

In the following we present the methodology in `surveysd` by applying the workflow described in
`vignette("surveysd")` to multiple consecutive years of EU-SILC data for one country. The 
methodology contains the following steps, in this order


- Draw $B$ bootstrap replicates from EU-SILC data for each year $y_t$, $t=1,\ldots,n_y$ separately. Since EU-SILC has a rotating panel design the bootstrap replicate of a household is carried forward through the years. That is, the bootstrap replicate of a household in the follow-up years is set equal to the bootstrap replicate of the same household when it first enters EU-SILC.
- Multiply each set of bootstrap replicates by the sampling weights to obtain uncalibrated bootstrap weights and calibrate each of the uncalibrated bootstrap weights using iterative proportional fitting.
- Estimate the point estimate of interest $\theta$, for each year and each calibrated bootstrap weight to obtain $\tilde{\theta}^{(i,y_t)}$, $t=1,\ldots,n_y$, $i=1,\ldots,B$. For fixed $y_t$ apply a filter with equal weights for each $i$ on $\tilde{\theta}^{(i,y^*)}$, $y^*\in \{y_{t-1},y_{t},y_{t+1}\}$ , to obtain $\tilde{\theta}^{(i,y_t)}$. Estimate the variance of $\theta$ using the distribution of $\tilde{\theta}^{(i,y_t)}$.


## Bootstrapping

Bootstrapping has long been around and used widely to estimate confidence intervals and standard errors of point estimates.[@efron1979}
Given a random sample $(X_1,\ldots,X_n)$ drawn from an unknown distribution $F$ the distribution of a point estimate $\theta(X_1,\ldots,X_n;F)$ can in many cases not be determined analytically. However when using bootstrapping one can simulate the distribution of $\theta$.

Let $s_{(.)}$ be a bootstrap sample, e.g. drawing $n$ observations with replacement from the sample $(X_1,\ldots,X_n)$, then one can estimate the standard deviation of $\theta$  using $B$ bootstrap samples through
$$sd(\theta) = \sqrt{\frac{1}{B-1}\sum\limits_{i=1}^B (\theta(s_i)-\overline{\theta})^2} \quad, $$

with $\overline{\theta}:=\frac{1}{B}\sum\limits_{i=1}^B\theta(s_i)$ as the sample mean over all bootstrap samples.

In context of sample surveys with sampling weights one can use bootstrapping to calculate so called bootstrap weights. These are computed via the bootstrap samples $s_{i}$, $i=1,\ldots,B$, where for each $s_{i}$ every unit of the original sample can appear $0$- to $m$-times. With $f_j^{i}$ as the frequency of occurrence of observation $j$ in bootstrap sample $s_i$ the uncalibrated bootstrap weights $\tilde{b}_{j}^{i}$ are defined as:

$$
  \tilde{b}_{j}^{i} = f_j^{i} w_j \quad,
$$

with $w_j$ as the calibrated sampling weight of the original sample.
Using iterative proportional fitting procedures one can recalibrate the bootstrap weights $\tilde{b}_{j}^{.}$, $j=1,\ldots,B$ to get the adapted or calibrated bootstrap weights $b_j^i$, $j=1,\ldots,B$.

### Rescaled Bootstrap

Since EU-SILC is a stratified sample without replacement drawn from a finite population the naive bootstrap procedure, as described above, does not take into account the heterogeneous inclusion probabilities of each sample unit. Thus it will not yield satisfactory results. Therefore we will use the so called rescaled bootstrap procedure introduced and investigated by [@raowu1988]. The bootstrap samples are selected without replacement and do incorporate the stratification as well as clustering on multiple stages (see [@chipprest2007],[@prest2009]).

For simplistic reasons we will only describe the rescaled bootstrap procedure for a two stage stratified sampling design. For more details on a general formulation please see [@prest2009].

### Sampling design

Consider the finite population $U$ which is divided into $H$ non-overlapping strata $\bigcup\limits_{h=1,\ldots,H} U_h = U$, of which each strata $h$ contains of $N_h$ clusters. For each strata $h$, $C_{hc}$, $c=1,\ldots,n_h$ clusters are drawn, containing $N_{hc}$ households. Furthermore in each cluster $C_{hc}$ of each strata $h$ simple random sampling is performed to select a set of households $Y_{hcj}$, $j=1,\ldots,n_{hc}$.

### Bootstrap procedure

In contrast to the naive bootstrap procedure where for a stage, containing $n$ sampling units, the bootstrap replicate is obtained by drawing $n$ sampling units with replacement, for the rescaled bootstrap procedure $n^*=\left\lfloor\frac{n}{2}\right\rfloor$ sampling units are drawn without replacement. Given a value $x$, $\lfloor x\rfloor$ denotes the largest integer smaller than $x$, whereas $\lceil x\rceil$ denotes the smallest integer lager then $x$. [@chipprest2007] have shown that the choice of either $\left\lfloor\frac{n}{2}\right\rfloor$ or $\left\lceil\frac{n}{2}\right\rceil$ is optimal for bootstrap samples without replacement, although $\left\lfloor\frac{n}{2}\right\rfloor$ has the desirable property that the resulting uncalibrated bootstrap weights will never be negative.

At the first stage the $i$-th bootstrap replicate, $f^{i,1}_{hc}$, for each cluster $C_{hc}$,$c=1,\ldots,n_h$, belonging to strata $h$, is defined by

$$
  f^{i,1}_{hc} = 1-\lambda_h+\lambda_h\frac{n_h}{n_h^*}\delta_{hc} \quad\quad \forall c \in \{1,\ldots,n_h\} 
$$
with
$$
  n_h^* = \left\lfloor\frac{n_h}{2}\right\rfloor
$$
$$
  \lambda_h = \sqrt{\frac{n_h^*(1-\frac{n_h}{N_h})}{n_h-n_h^*}} \quad ,
$$

where $\delta_{hc}=1$ if cluster $c$ is selected in the sub-sample of size $n_h^*$ and 0 otherwise.

The $i$-th bootstrap replicate at the second stage, $f^{i,2}_{hcj}$, for each household $Y_{hcj}$, $j=1,\ldots,n_{hc}$, belonging to cluster $c$ in strata $h$ is defined by

$$
  f^{i,2}_{hcj} = f^{i,1}_{hc} - \lambda_{hc}\sqrt{\frac{n_h}{n_h^*}}\delta_{hc}\left[\frac{n_{hc}}{n_{hc}^*}\delta_{hcj}-1\right] \quad\quad \forall c \in \{1,\ldots,n_h\}
$$
with
$$
  n_{hc}^* = \left\lfloor\frac{n_{hc}}{2}\right\rfloor 
$$
$$
  \lambda_{hc} = \sqrt{\frac{n_{hc}^*N_h(1-\frac{n_{hc}}{N_{hc}})}{n_{hc}-n_{hc}^*}} \quad ,
$$

where $\delta_{hcj}=1$ if household $j$ is selected in the sub sample of size $n_{hc}^*$ and 0 otherwise.

### Single PSUs

When dealing with multistage sampling designs the issue of single PSUs, e.g. a single response unit is present at a stage or in a strata, can occur. When applying bootstrapping procedures these single PSUs can lead to a variety of issues. For the methodology proposed in this work we combined single PSUs at each stage with the next smallest strata or cluster, before applying the bootstrap procedure.

### Taking bootstrap replicates forward

The bootstrap procedure above is applied on the EU-SILC data for each year $y_t$, $t=1,\ldots,n_y$ separately. Since EU-SILC is a yearly survey with rotating penal design the $i$-th bootstrap replicate at the second stage, $f^{i,2}_{hcj}$, for a household $Y_{hcj}$ is taken forward until the household $Y_{hcj}$ drops out of the sample. That is, for the household $Y_{hcj}$, which enters EU-SILC at year $y_1$ and drops out at year $y_{\tilde{t}}$, the bootstrap replicates for the years $y_2,\ldots,y_{\tilde{t}}$ are set to the bootstrap replicate of the year $y_1$.

### Split households
Due to the rotating penal design so called split households can occur. For a household participating in the EU-SILC survey it is possible that one or more residents move to a new so called split household, which is followed up on in the next wave. To take this dynamic into account we extended the procedure of taking forward the bootstrap replicate of a household for consecutive waves of EU-SILC by taking forward the bootstrap replicate to the split household. That means, that also any new individuals in the split household will inherit this bootstrap replicate.

Taking bootstrap replicates forward as well as considering split households ensures that bootstrap replicates are more comparable in structure with the actual design of EU-SILC.

### Uncalibrated bootstrap weights

Using the $i$-th bootstrap replicates at the second stage one can calculate the $i$-th uncalibrated bootstrap weights $b_{hcj}^{i}$ for each household $Y_{hcj}$ in cluster $c$ contained in strata $h$ by

$$
  \tilde{b}_{hcj}^{i} = f^{i,2}_{hcj} w_{hcj} \quad,
$$
where $w_{hcj}$ corresponds to the original household weight contained in the sample.

For ease of readability we will drop the subindices regarding strata $h$ and cluster $c$ for the following sections, meaning that the $j$-th household in cluster $c$ contained in strata $h$, $Y_{hcj}$, will now be denoted as the $j$-th household, $Y_{j}$, where $j$ is the position of the household in the data. In accordance to this the $i$-th uncalibrated bootstrap replicates for household $j$ are thus denoted as $\tilde{b}_j^{i}$ and the original household weight as $w_j$.


## Iterative proportional fitting (IPF)

The uncalibrated bootstrap weights $\tilde{b}_j^{i}$ computed through the rescaled bootstrap procedure yields population statistics that differ from the known population margins of specified sociodemographic variables for which the base weights $w_j$ have been calibrated. To adjust for this the bootstrap weights $\tilde{b}_{j}^{i}$ can be recalibrated using iterative proportional fitting as described in [@mekogu2016].

Let the original weight $w_{j}$ be calibrated for $n=n_P+n_H$ sociodemographic variables which are divided into the sets $\mathcal{P}:=\{p_{c}, c=1 \ldots,n_P\}$ and $\mathcal{H}:=\{h_{c}, c=1 \ldots,n_H\}$. $\mathcal{P}$ and $\mathcal{H}$ correspond to personal, for example gender or age, or household variables, like region or households size, respectively. Each variable in either $\mathcal{P}$ or $\mathcal{H}$ can take on $P_{c}$ or $H_{c}$ values with and $N^{p_c}_v$, $v=1,\ldots,P_c$, or $N^{h_c}_v$, $v=1,\ldots,H_c$, as the corresponding population margins. Starting with $k=0$ the iterative proportional fitting procedure is applied on each $\tilde{b}_j^{i}$, $i=1,\ldots, B$ separately. The weights are first updated for personal and afterwards updated for household variables. If constraints regarding the populations margins are not met $k$ is raised by 1 and the procedure starts from the beginning. For the following denote as starting weight $\tilde{b}_j^{[0]}:=\tilde{b}_j^{i}$ for fixed $i$.

### Adjustment and trimming for $\mathcal{P}$

The uncalibrated bootstrap weight $\tilde{b}_j^{[(n+1)k+c-1]}$ for the $j$-th observation is iteratively multiplied by a factor so that the projected distribution of the population matches the respective calibration specification $N_{p_c}$, $c=1, \ldots,n_P$.
For each $c \in \left\{1, \ldots,n_P\right\}$ the calibrated weights against $N^{p_c}_v$ are computed as
$$
  \tilde{b}_j^{[(n+1)k+c]} = {\tilde{b}_j}^{[(n+1)k+c-1]}\frac{N^{p_c}_v}{{\sum\limits_l} {\tilde{b}}_l^{[(n+1)k+c-1]}},
$$
where the summation in the denominator expands over all observations which have the same value as observation $j$ for the sociodemographic variable $p_c$.
If any weights $\tilde{b}_j^{[nk+c]}$ fall outside the range $\left[\frac{w_j}{4};4w_j\right]$ they will be recoded to the nearest of the two boundaries. The choice of the boundaries results from expert-based opinions and restricts the variance of which has a positive effect on the sampling error. This procedure represents a common form of weight trimming where very large or small weights are trimmed in order to reduce variance in exchange for a possible increase in bias ([@potter90],[@potter93]).

### Averaging weights within households

Since the sociodemographic variables $p_1,\ldots,p_{n_c}$ include person-specific variables, the weights $\tilde{b}_j^{[nk+n_p]}$ resulting from the iterative multiplication can be unequal for members of the same household. This can lead to inconsistencies between results projected with household and person weights. To avoid such inconsistencies each household member is assigned the mean of the household weights. That is for each person $j$ in household $a$ with $h_a$ household members, the weights are defined by
$$
  \tilde{b}_j^{[(n+1)k+n_p+1]} = \frac{{\sum\limits_{l\in a}} {\tilde{b}_l^{[(n+1)k+n_p]}}}{h_a}
$$
This can result in losing the population structure performed in the previous subsection.

###  Adjustment and trimming for $\mathcal{H}$

After adjustment for individual variables the weights $b_j^{[nk+n_p+1]}$ are updated for the set of household variables $\mathcal{H}$ according to a household convergence constraint parameter $\epsilon_h$. The parameters $\epsilon_h$ represent the allowed deviation from the population margins using the weights $b_j^{[nk+n_p+1]}$ compared to $N^{h_c}_v$, $c=1,\ldots,n_H$, $v=1,\ldots,H_c$.
The updated weights are computed as
$$
  b_j^{[(n+1)k+n_p+c+1]} =
  \begin{cases}
    b_j^{[(n+1)k+n_p+1]}\frac{N^{h_c}_v}{\sum\limits_{l} b_l^{[(n+1)k+n_p+1]}} \quad \text{if } \sum\limits_{l} b_j^{[(n+1)k+n_p+1]} \notin ((1-0.9\epsilon_h)N^{h_c}_v,(1+0.9\epsilon_h)N^{h_c}_v) \\
    b_j^{[(n+1)k+n_p+1]} \quad \text{otherwise}
  \end{cases}
$$
with the summation in the denominator ranging over all households $l$ which take on the same values for $h_c$ as observation $j$. As described in the previous subsection the new weight are recoded if they exceed the interval $[\frac{w_j}{4};4w_j]$ and set to the upper or lower bound, depending of $b_j^{[(n+1)k+n_p+c+1]}$ falls below or above the interval respectively. 

### Convergence
For each adjustment and trimming step the factor $\frac{N^{(.)}_v}{\sum\limits_{l} b_l^{[(n+1)k+j]}}$, $j\in \{1,\ldots,n+1\}\backslash \{n_p+1\}$, is checked against convergence constraints for households, $\epsilon_h$, or personal variables $\epsilon_p$, where $(.)$ corresponds to either a household or personal variable.
To be more precise for variables in $\mathcal{P}$ the constraints 

$$
\frac{N^{p_c}_v}{{\sum\limits_l} {\tilde{b}}_l^{[(n+1)k+j]}} \in ((1-\epsilon_p)N^{p_c}_v,(1+\epsilon_p)N^{p_c}_v)
$$
and for variables in $\mathcal{H}$ the constraints

$$
\frac{N^{h_c}_v}{{\sum\limits_l} {\tilde{b}}_l^{[(n+1)k+j]}} \in ((1-\epsilon_h)N^{h_c}_v,(1+\epsilon_h)N^{h_c}_v)
$$
are verified, where the sum in the denominator expands over all observations which have the same value for variables $h_c$ or $p_c$.
If these constraints hold true the algorithm reaches convergence, otherwise $k$ is raised by 1 and the procedure repeats itself.

The above described calibration procedure is applied on each year $y_t$ of EU-SILC separately, $t=1,\ldots n_y$, thus resulting in so called calibrated bootstrap sample weights $b_{j}^{(i,{y_t})}$, $i=1,\ldots,B$ for each year $y$ and each household $j$.

## Variance estimation

Applying the previously described algorithms to EU-SILC data for multiple consecutive years $y_t$, $t=1,\ldots n_y$, yields calibrated bootstrap sample weights $b_{j}^{(i,{y_t})}$ for each year $y_t$. Using the calibrated bootstrap sample weights it is straight forward to compute the standard error of a point estimate $\theta(\textbf{X}^{y_t},\textbf{w}^{y_t})$ for year $y_t$ with $\textbf{X}^{y_t}=(X_1^{y_t},\ldots,X_n^{y_t})$ as the vector of observations for the variable of interest in the survey and $\textbf{w}^{y_t}=(w_1^{y_t},\ldots,w_n^{y_t}$ as the corresponding weight vector, with

$$
  sd(\theta) = \sqrt{\frac{1}{B-1}\sum\limits_{i=1}^B (\theta^{(i,y_t)}-\overline{\theta^{(.,y_t)}})^2} 
$$
with
$$
  \overline{\theta^{(.,y_t)}} = \frac{1}{B}\sum\limits_{i=1}^B\theta^{(i,y_t)} \quad,
$$
where $\theta^{(i,y_t)}:=\theta(\textbf{X}^{y_t},\textbf{b}^{(i,{y_t})})$ is the estimate of $\theta$ in the year $y_t$ using the $i$-th vector of calibrated bootstrap weights.

As already mentioned the standard error estimation for indicators in EU-SILC yields high quality results for NUTS1 or country level. When estimation indicators on regional or other sub-aggregate levels one is confronted with point estimates yielding high variance. 

To overcome this issue we propose to estimate $\theta$ for 3, consecutive years using the calibrated bootstrap weights, thus calculating $\{\theta^{(i,y_{t-1})},\theta^{(i,y_t)},\theta^{(i,y_{t+1})}\}$, $i=1,\ldots,B$.
For fixed $i$ one can apply a filter with equal filter weights on the time series $\{\theta^{(i,y_{t-1})},\theta^{(i,y_t)},\theta^{(i,y_{t+1})}\}$ to create $\tilde{\theta}^{(i,y_t)}$

$$
  \tilde{\theta}^{(i,y_t)} = \frac{1}{3}\left[\theta^{(i,y_{t-1})}+\theta^{(i,y_t)}+\theta^{(i,y_{t+1})}\right] \quad .
$$


Doing this for all $i$, $i=1,\ldots,B$, yields $\tilde{\theta}^{(i,y_t)}$, $i=1,\ldots,B$. The standard error of $\theta$ can then be estimated with

$$
  sd(\theta) = \sqrt{\frac{1}{B-1}\sum\limits_{i=1}^B (\tilde{\theta}^{(i,y_t)}-\overline{\tilde{\theta}^{(.,y_t)}})^2}
$$
with
$$
  \overline{\tilde{\theta}^{(.,y_t)}}=\frac{1}{B}\sum\limits_{i=1}^B\tilde{\theta}^{(i,y_t)} \quad.
$$

Applying the filter over the time series of estimated $\theta^{(i,y_t)}$ leads to a reduction of variance for $\theta$ since the filter reduces the noise in $\{\theta^{(i,y_{t-1})},\theta^{(i,y_t)},\theta^{(i,y_{t+1})}\}$ and thus leading to a more narrow distribution for $\tilde{\theta}^{(i,y_t)}$.

It should also be noted that estimating indicators from a survey with rotating panel design is in general not straight forward because of the high correlation between consecutive years. However with our approach to use bootstrap weights, which are independent from each other, we can bypass the cumbersome calculation of various correlations, and apply them directly to estimate the standard error.
[@silcstudy] showed that using the proposed method on EU-SILC data for Austria the reduction in resulting standard errors corresponds in a theoretical increase in sample size by about 25$\%$. Furthermore this study compared this method to the use of small area estimation techniques and on average the use of bootstrap sample weights yielded more stable results.

## References

---
title: "Rao-Wu Bootstrap in surveysd"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Rao-Wu bootstrap}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: lib.bib
---

```{r, echo = FALSE}
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(eval = FALSE)
```

In the following we describe an extension of the methodology implemented in `surveysd`.
While the default rescaling method described in `vignette("Methodology")` follows the approach of Preston, this vignette introduces an alternative rescaling method based on the Rao-Wu bootstrap [@raowu1988].

### Mathematical Formulation

More formally, the replicate weights are constructed as follows:
$$
w^*_{hi} = \left(1 - \lambda_h + \lambda_h \cdot \frac{n_h}{m_h} \cdot r^*_{hi} \right) \cdot w_{hi}
$$
with
$$
 \lambda_h = \sqrt{\frac{m_h(1 - f_h)}{n_h - 1}}
$$
where:<br>
 - \( w_{hi} \) is the design weight for PSU \( i \) in stratum \( h \), \( w_h = N_h / n_h \) is the average design weight for the entire stratum \( h \),<br>
 - \( N_h \) is the total number of units in stratum \( h \),<br>
 - \( n_h \) is the number of PSUs in the original sample for stratum \( h \),<br>
 - \( r^*_{hi} \in \{0, 1, 2, \dots\} \) is a **resampling indicator** that represents how many times PSU \( i \) in stratum \( h \)  was drawn in the bootstrap ,<br>
 - \( m_h = n_h - 1 \) is the **number of units** to be drawn in each replicate for stratum \( h \),<br>
 - \( \lambda_h\) is the **scaling factor** used to adjust the weights during the bootstrap process,<br>
 - and \( f_h = n_h / N_h \) is the **sampling fraction** in stratum \( h \).<br>

### Use of Rao-Wu
The Rao-Wu bootstrap is particularly suited for complex survey designs with stratification and multi-stage selection. The design-based variance estimation benefits from a resampling method that accounts for the primary sampling stage and incorporates finite population corrections (FPC) directly in the replicate weights.
The method is appropriate when the sampling fraction in the first stage (i.e., the share of selected PSUs within strata) is relatively small, typically below 10%. 

Rao-Wu is **not suitable** when:<br>
 - The sampling fraction in the first stage is large and the first-stage sampling is without replacement.<br>
 - The survey does not include PSU-level identifiers, making a first-stage resampling infeasible since the method requires resampling entire PSUs to reflect first-stage sampling variability.<br>
 - The design is single-stage, where methods like the Preston bootstrap may be more appropriate.<br>

In those cases, it is recommended to fall back on alternative bootstrap methods such as the default Preston approach implemented in `surveysd`, which offers more flexibility for a wider range of designs without PSU information.

### Single PSUs
When dealing with multistage sampling designs, the issue of single PSUs, e.g. a single response unit at a stage or in a stratum, can arise. When applying resampling methods such as the Rao-Wu bootstrap, these single PSUs can introduce challenges in the resampling process. In the Rao-Wu method, we adjust for the presence of single PSUs by combining them with the next smallest stratum or cluster before applying the resampling procedure. This ensures that the resampling reflects the structure of the original design while maintaining appropriate variance estimates for the total and other statistics of interest.


### Example Implementation

#### Load Dataset
```{r}
library(surveysd)

set.seed(1234)
eusilc <- demo.eusilc(n = 2, prettyNames = TRUE)

eusilc[1:5, .(year, povertyRisk, gender, pWeight)]
``` 

#### Draw bootstrap replicates

For the bootstrap select 'method = "Rao-Wu"'. Otherwise the default "Preston" is used.
```{r, eval = FALSE}
dat_boot_rw <- draw.bootstrap(eusilc, 
                              method = "Rao-Wu",
                              REP = 10, 
                              hid = "hid", 
                              weights = "pWeight", 
                              strata = "region", 
                              period = "year")
```

#### Calibrate bootstrap replicates

Calibrate each sample according to the distribution of `gender` (on a personal level) and `region` 
(on a household level). 

```{r}
dat_boot_calib <- recalib(dat_boot_rw, 
                          conP.var = "gender", 
                          conH.var = "region",
                          epsP = 1e-2, 
                          epsH = 2.5e-2, 
                          verbose = FALSE)
dat_boot_calib[1:5, .(year, povertyRisk, gender, pWeight, w1, w2, w3, w4)]
```

---
title: "Error estimation"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    df_print: kable
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{error estimation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)
```

For the most part, this document will present the functionalities of the function 
`surveysd::calc.stError()` which generates point estimates and standard errors for
user-supplied estimation functions.

## Prerequisites

In order to use a dataset with `calc.stError()`, several weight columns have to be present. Each
weight column corresponds to a bootstrap sample. In the following examples, we will use the data 
from `demo.eusilc()` and attach the bootstrap weights using `draw.bootstrap()` and `recalib()`. Please 
refer to the documentation of those functions for more detail.

```{r}
library(surveysd)

set.seed(1234)
eusilc <- demo.eusilc(prettyNames = TRUE)
dat_boot <- draw.bootstrap(eusilc, REP = 10, hid = "hid", weights = "pWeight",
                           strata = "region", period = "year")
dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region",
                          epsP = 1e-2, epsH = 2.5e-2, verbose = FALSE)
dat_boot_calib[, onePerson := nrow(.SD) == 1, by = .(year, hid)]

## print part of the dataset
dat_boot_calib[1:5, .(year, povertyRisk, eqIncome, onePerson, pWeight, w1, w2, w3, w4, w5)]
```


## Estimator functions

The parameters `fun` and `var` in `calc.stError()` define the estimator to be used in the error 
analysis. There are two built-in estimator functions `weightedSum()` and `weightedRatio()` which can
be used as follows.


```{r}
povertyRate <- calc.stError(dat_boot_calib, var = "povertyRisk", fun = weightedRatio)
totalIncome <- calc.stError(dat_boot_calib, var = "eqIncome", fun = weightedSum)
```

Those functions calculate the ratio of persons at risk of poverty (in percent) and the total income.
By default, the results are calculated separately for each reference period.

```{r}
povertyRate$Estimates
totalIncome$Estimates
```

Columns that use the `val_` prefix denote the point estimate belonging to the "main weight" of the
dataset, which is `pWeight` in case of the dataset used here.

Columns with the `stE_` prefix denote standard errors calculated with bootstrap replicates. The
replicates result in using `w1`, `w2`, ..., `w10` instead of `pWeight` when applying the estimator.

`n` denotes the number of observations for the year and `N` denotes the total weight of those
persons.

### Custom estimators

In order to define a custom estimator function to be used in `fun`, the function needs to have
at least two arguments like the example below.

```{r}
## define custom estimator
myWeightedSum <- function(x, w) {
  sum(x*w)
}

## check if results are equal to the one using `surveysd::weightedSum()`
totalIncome2 <- calc.stError(dat_boot_calib, var = "eqIncome", fun = myWeightedSum)
all.equal(totalIncome$Estimates, totalIncome2$Estimates)
```

The parameters `x` and `w` can be assumed to be vectors with equal length with `w` being numeric
weight vector and `x` being the column defined in the `var` argument. It will be called once
for each period (in this case `year`) and for each weight column (in this case `pWeight`, `w1`, `w2`, ..., `w10`).

Custom estimators using additional parameters can also be supplied and parameter `add.arg` can be used
to set the additional arguments for the custom estimator.

```{r}
## use add.arg-argument
fun <- function(x, w, b) {
  sum(x*w*b)
}
add.arg = list(b="onePerson")

err.est <- calc.stError(dat_boot_calib, var = "povertyRisk", fun = fun,
                        period.mean = 0, add.arg=add.arg)
err.est$Estimates

# compare with direct computation
compare.value <- dat_boot_calib[,fun(povertyRisk,pWeight,b=onePerson),
                                 by=c("year")]
all((compare.value$V1-err.est$Estimates$val_povertyRisk)==0)
```

The above chunk computes the weighted poverty ratio for single person households.

### Adjust variable depending on bootstrap weights

In our example the variable `povertyRisk` is a boolean and is `TRUE` if the income is less
than 60% of the weighted median income. Thus it directly depends on the original weight vector
`pWeight`. To further reduce the estimated error one should calculate for each bootstrap replicate
weight $w$ the weighted median income $medIncome_{w}$ and then define $povertyRisk_w$ as

$$
povertyRisk_w = \cases{1 \quad\text{if Income}<0.6\cdot medIncome_{w}\\
                     0 \quad\text{else}}
$$

The estimator can then be applied to the new variable $povertyRisk_w$.
This can be realized using a custom estimator function.

```{r}
# custom estimator to first derive poverty threshold 
# and then estimate a weighted ratio
povmd <- function(x, w) {
 md <- laeken::weightedMedian(x, w)*0.6
 pmd60 <- x < md
 # weighted ratio is directly estimated inside the function
 return(sum(w[pmd60])/sum(w)*100)
}

err.est <- calc.stError(
  dat_boot_calib, var = "povertyRisk", fun = weightedRatio,
  fun.adjust.var = povmd, adjust.var = "eqIncome")
err.est$Estimates

```


The approach shown above is only valid if no grouping variables are supplied (parameter `group = NULL`).
If grouping variables are supplied one should use parameters `fun.adjust.var` and `adjust.var` such that
the $povertyRisk_w$ is first calculated for each `period` and then used for each grouping in `group`.


```{r}
# using fun.adjust.var and adjust.var to estimate povmd60 indicator
# for each period and bootstrap weight before applying the weightedRatio
povmd2 <- function(x, w) {
 md <- laeken::weightedMedian(x, w)*0.6
 pmd60 <- x < md
 return(as.integer(pmd60))
}

# set adjust.var="eqIncome" so the income vector is used to estimate
# the povmd60 indicator for each bootstrap weight
# and the resulting indicators are passed to function weightedRatio
group <- "gender"
err.est <- calc.stError(
  dat_boot_calib, var = "povertyRisk", fun = weightedRatio, group = "gender",
  fun.adjust.var = povmd2, adjust.var = "eqIncome")
err.est$Estimates
```


### Multiple estimators

In case an estimator should be applied to several columns of the dataset, `var` can be set to
a vector containing all necessary columns.

```{r}
multipleRates <- calc.stError(dat_boot_calib, var = c("povertyRisk", "onePerson"), fun = weightedRatio)
multipleRates$Estimates
```
Here we see the relative number of persons at risk of poverty and the relative number of one-person
households.

## Grouping

The `groups` argument can be used to calculate estimators for different subsets of the data. This
argument can take the grouping variable as a string that refers to a column name (usually a factor) 
in `dat`. If set, all estimators are not only split by the reference period but also by the
grouping variable. For simplicity, only one reference period of the above data is used.

```{r}
dat2 <- subset(dat_boot_calib, year == 2010)
for (att  in c("period", "weights", "b.rep"))
  attr(dat2, att) <- attr(dat_boot_calib, att)
```

To calculate the ratio of persons at risk of poverty for each federal state of Austria, 
`group = "region"` can be used.

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, group = "region")
povertyRates$Estimates
```

The last row with `region = NA` denotes the aggregate over all regions. Note that the
columns `N` and `n` now show the weighted and unweighted number of persons in each region.

### Several grouping variables

In case more than one grouping variable is used, there are several options of calling 
`calc.stError()` depending on whether combinations of grouping levels should be regarded or not.
We will consider the variables `gender` and `region` as our grouping variables and show three 
options on how `calc.stError()` can be called.

#### Option 1: All regions and all genders

Calculate the point estimate and standard error for each region and each gender. The number of 
rows in the output is therefore 

$$n_\text{periods}\cdot(n_\text{regions} + n_\text{genders} + 1) = 1\cdot(9 + 2 + 1) = 12.$$

The last row is again the estimate for the whole period.

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = c("gender", "region"))
povertyRates$Estimates

```

#### Option 2: All combinations of `region` and `gender`

Split the data by all combinations of the two grouping variables. This will result in a larger 
output-table of the size

$$n_\text{periods}\cdot(n_\text{regions} \cdot n_\text{genders} + 1) = 1\cdot(9\cdot2 + 1)= 19.$$

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = list(c("gender", "region")))
povertyRates$Estimates
```

#### Option 3: Cobination of Option 1 and Option 2

In this case, the estimates and standard errors are calculated for

* every gender,
* every region and
* every combination of region and gender.

The number of rows in the output is therefore 

$$n_\text{periods}\cdot(n_\text{regions} \cdot n_\text{genders} + n_\text{regions} + n_\text{genders} + 1) = 1\cdot(9\cdot2 + 9 + 2 + 1) = 30.$$

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = list("gender", "region", c("gender", "region")))
povertyRates$Estimates
```


## Group differences

If differences between groups need to be calculated, e.g difference of poverty rates between `gender = "male"` and `gender = "female"`, parameter `group.diff` can be utilised.
Setting `group.diff = TRUE` the differences and the standard error of these differences for all variables defined in `groups` will be calculated.

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = c("gender", "region"),
                             group.diff = TRUE)
povertyRates$Estimates
```

The resulting output table contains `r nrow(povertyRates$Estimates)` rows. `r sum(povertyRates$Estimates$estimate_type == "direct")` rows for all the direct estimators

$$n_\text{periods}\cdot(n_\text{regions} + n_\text{genders} + 1) = 1\cdot(9 + 2 + 1) = 12,$$

and another `r sum(povertyRates$Estimates$estimate_type == "group difference")` for all the differences within the variable `"gender"` and `"region"` seperately.
Variable `"gender"` has 2 unique values (`unique(dat2$gender)`) resulting in 1 difference, ~ `gender = "male"` - `gender = "female"` and variable `"region"` has
9 unique values (`unique(dat2$region)`) resulting in 

$$8 + 7 + 6 + 5 + 4 + 3 + 2 + 1  = \sum\limits_{1=1}^{9-1}i = 36$$

estimates. Thus the output contains 1 + 36 = 37 estimates with respect to group differences.

If a combintaion of grouping variables is used in `group` and `group.diff = TRUE` then differences between combinations will only be calculated
if one of the grouping variables differs.
For example the difference between the following groups would be calculated

- `gender = "female" & region = "Vienna"` - `gender = "male" & region = "Vienna"` 
- `gender = "female" & region = "Vienna"` - `gender = "female" & region = "Salzburg"`
- `gender = "male" & region = "Salzburg"` - `gender = "female" & region = "Salzburg"`

The difference between `gender = "female" & region = "Vienna"` and `gender = "male" & region = "Salzburg"` however would not be calculated.

Thus this leads to 

$$2\cdot(\sum\limits_{1=1}^{9-1}i) + 9\cdot1 = 81$$

results with respect to the differences.
The Output contains an additional column `estimate_type` and 

```{r}
povertyRates <- calc.stError(dat2, var = "povertyRisk", fun = weightedRatio, 
                             group = list(c("gender", "region")),
                             group.diff = TRUE)
povertyRates$Estimates[,.N,by=.(estimate_type)]
```

## Differences between survey periods

Differences of estimates between `period`s can be calculated using parameter `period.diff`.
`period.diff` expects a character vector (if not `NULL`) specifying for which `period`s the differences should be calcualed for.
The inputs should be specified in the form `"period2" - "period1"`.

```{r}
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             period.diff = c("2017 - 2016", "2016 - 2015", "2015 - 2014"))
povertyRates$Estimates
```

If additional grouping variables are supplied to `calc.stError()` die differences across `period`s are also carried out
for all variables in `group`.

```{r}
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             group = "gender",
                             period.diff = c("2017 - 2016", "2016 - 2015", "2015 - 2014"))
povertyRates$Estimates
```


## Averages across periods

With parameter `period.mean` averages across `period`s are calculated additional. The parameter accepts only odd integer values.
The resulting table will contain the direct estimates as well as rolling averages of length `period.mean`.

```{r}
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             period.mean = 3)
povertyRates$Estimates
```

if in addition the parameters `group` and/or `period.diff` are specified then differences and groupings of averages will be calculated.

```{r}
povertyRates <- calc.stError(dat_boot_calib[year>2013], var = "povertyRisk", fun = weightedRatio, 
                             period.mean = 3, period.diff = "2016 - 2015",
                             group = "gender")
povertyRates$Estimates
```

---
title: "Iterative Proportional Fitting"
output: rmarkdown::html_vignette
author:
  - Gregor de Cillia
vignette: >
  %\VignetteIndexEntry{Iterative Proportional Fitting}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: lib.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette explains the usage of the `ipf()` function, which has been used for calibrating the labour force survey of Austria for several years.
It is based on the Iterative Proportional Fitting algorithm and gives some flexibility about the details of the implementation. See [@mekogu2016] or `vignette("methodology")` for more details.

## Setup

We will assume the output of `demo.eusilc()` is our population.
From this population, a sample without replacement is drawn.
The sample covers 10 percent of the population.
We assign a weight of one for all observations of the population and a weight of ten for all observations of the sample.

```{r setup}
library(surveysd)
population <- demo.eusilc(1, prettyNames = TRUE)
population[, pWeight := 1]
pop_sample <- population[sample(1:.N, floor(.N*0.10)), ]
pop_sample[, pWeight := 10]
```

## One constraint, one variable

We will start with an example where we want to adapt the weights of `pop_sample` such that the weighted number of males and females matches the ones of `population`.
We can see that this is currently not the case.

```{r}
(gender_distribution <- xtabs(pWeight ~ gender, population))
xtabs(pWeight ~ gender, pop_sample)
```

Due to random sampling (rather than stratified sampling), there are differences between the gender distributions.
We can pass `gender_distribution` as a parameter to `ipf()` to obtain modified weights.

```{r}
pop_sample_c <- ipf(pop_sample, conP = list(gender_distribution), w = "pWeight")
```

The resulting dataset, `pop_sample_c` is similar to `pop_sample` but has an additional column with the adjusted weights.

```{r}
dim(pop_sample)
dim(pop_sample_c)
setdiff(names(pop_sample_c), names(pop_sample))
```

We can now calculate the weighted number of males and females according to this new weight.
This will result in a match for the constraints.

```{r}
xtabs(calibWeight ~ gender, pop_sample_c)
xtabs(pWeight ~ gender, population)
```

In this simple case, `ipf` just performs a post stratification step.
This means, that all males and all females have the same weight.

```{r, fig.align="center", out.width="100%"}
xtabs(~ calibWeight + gender, pop_sample_c)
```

```{r, include = FALSE}
overrepresented_gender <- pop_sample_c[calibWeight < 10, ][1, gender]
```

All `r overrepresented_gender`s have been weighted down (`calibWeight < 10`) to compensate for the overrepresentation in the sample.

## One constraint, two variables

Let's now assume that we want to put constraints on the number of males and females for each age group. 
The numbers from the original population can be obtained with `xtabs()`.

```{r}
(con_ga <- xtabs(pWeight ~ gender + age, population))
xtabs(pWeight ~ gender + age, pop_sample)
```

Again, we can see that those constraints are not met.
Supplying the contingency table `con_ga` to `ipf()` will again resolve this.

```{r}
pop_sample_c2 <- ipf(pop_sample, conP = list(con_ga), w = "pWeight")
xtabs(pWeight ~ gender + age, population)
xtabs(calibWeight ~ gender + age, pop_sample_c2)
```

## Two constraints

Now we assume that we know the number of persons living in each nuts2 region from registry data.

```{r}
registry_table <- xtabs(pWeight ~ region, population)
```

However, those registry data does not provide any information about age or `gender`.
Therefore, the two contingency tables (`con_ga` and `registry_table`) have to be specified independently.
This can be done by supplying a list to `conP`

```{r}
pop_sample_c2 <- ipf(pop_sample, conP = list(con_ga, registry_table), w = "pWeight")
xtabs(pWeight ~ gender + age, population)
xtabs(calibWeight ~ gender + age, pop_sample_c2)
xtabs(pWeight ~ region, population)
xtabs(calibWeight ~ region, pop_sample_c2)
```

this time, the constraints are not matched perfectly.
That is, because we provided more than one constraint.
therefore, the `ipf()` algorithm had to work iteratively.

## Household Constraints

If the dataset has a household structure, household constraints can be passed
via the parameter `conH`. If this parameter is used, it is also necessary to
supply `hid`, which defines the column names that contains household ids.

```{r}
(conH1 <- xtabs(pWeight ~ hsize + region, data = population[!duplicated(hid)]))
pop_sample_hh <- ipf(pop_sample, hid = "hid", conH = list(conH1), w = "pWeight",
                     bound = 10)
xtabs(calibWeight ~ hsize + region, data = pop_sample_hh[!duplicated(hid)])
```

## Tolerances

If `conP` or `conH` contain several contingency tables or if `conP` and `conH`
are used at the same time, the ipf algorithm will operate iteratively. This
means that the calibrated dataset will satisfy the constraints only
approximately. The default tolerances of the approximation can be overwritten
using the parameters `conP` and `conH`.

Lowering the tolerances will improve the match between the constraints and
the contingency tables according to the calibrated weights. However, lower
tolerances will also make it so more iterations are necessary until a
convergence is met. If the constraints are too small, ipf will return with
a warning that indicates that a convergence could not be reached.

```{r}
ipf(pop_sample, conP = list(con_ga, registry_table), w = "pWeight",
    verbose = TRUE, epsP = 0.01)
ipf(pop_sample, conP = list(con_ga, registry_table), w = "pWeight",
    verbose = TRUE, epsP = 0.0001)
```

We see that changing the tolerances from `0.01` (one percent) to `0.0001`
increases the number of required iterations.

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipf.r
\name{ipf}
\alias{ipf}
\title{Iterative Proportional Fitting}
\usage{
ipf(
  dat,
  hid = NULL,
  conP = NULL,
  conH = NULL,
  epsP = 1e-06,
  epsH = 0.01,
  verbose = FALSE,
  w = NULL,
  bound = 4,
  maxIter = 200,
  meanHH = TRUE,
  allPthenH = TRUE,
  returnNA = TRUE,
  looseH = FALSE,
  numericalWeighting = computeLinear,
  check_hh_vars = TRUE,
  conversion_messages = FALSE,
  nameCalibWeight = "calibWeight",
  minMaxTrim = NULL,
  print_every_n = 100
)
}
\arguments{
\item{dat}{a \code{data.table} containing household ids (optionally), base
weights (optionally), household and/or personal level variables (numerical
or categorical) that should be fitted.}

\item{hid}{name of the column containing the household-ids within \code{dat} or
NULL if such a variable does not exist.}

\item{conP}{list or (partly) named list defining the constraints on person
level.  The list elements are contingency tables in array representation
with dimnames corresponding to the names of the relevant calibration
variables in \code{dat}. If a numerical variable is to be calibrated, the
respective list element has to be named with the name of that numerical
variable. Otherwise the list element shoud NOT be named.}

\item{conH}{list or (partly) named list defining the constraints on
household level.  The list elements are contingency tables in array
representation with dimnames corresponding to the names of the relevant
calibration variables in \code{dat}. If a numerical variable is to be
calibrated, the respective list element has to be named with the name of
that numerical variable. Otherwise the list element shoud NOT be named.}

\item{epsP}{numeric value or list (of numeric values and/or arrays)
specifying the convergence limit(s) for \code{conP}. The list can contain
numeric values and/or arrays which must appear in the same order as the
corresponding constraints in \code{conP}. Also, an array must have the same
dimensions and dimnames as the corresponding constraint in \code{conP}.}

\item{epsH}{numeric value or list (of numeric values and/or arrays)
specifying the convergence limit(s) for \code{conH}. The list can contain
numeric values and/or arrays which must appear in the same order as the
corresponding constraints in \code{conH}. Also, an array must have the same
dimensions and dimnames as the corresponding constraint in \code{conH}.}

\item{verbose}{if TRUE, some progress information will be printed.}

\item{w}{name if the column containing the base weights within \code{dat} or NULL
if such a variable does not exist. In the latter case, every observation
in \code{dat} is assigned a starting weight of 1.}

\item{bound}{numeric value specifying the multiplier for determining the
weight trimming boundary if the change of the base weights should be
restricted, i.e. if the weights should stay between 1/\code{bound}*\code{w}
and \code{bound}*\code{w}.}

\item{maxIter}{numeric value specifying the maximum number of iterations
that should be performed.}

\item{meanHH}{if TRUE, every person in a household is assigned the mean of
the person weights corresponding to the household. If \code{"geometric"}, the
geometric mean is used rather than the arithmetic mean.}

\item{allPthenH}{if TRUE, all the person level calibration steps are
performed before the houshold level calibration steps (and \code{meanHH}, if
specified). If FALSE, the houshold level calibration steps (and \code{meanHH},
if specified) are performed after everey person level calibration step.
This can lead to better convergence properties in certain cases but also
means that the total number of calibration steps is increased.}

\item{returnNA}{if TRUE, the calibrated weight will be set to NA in case of
no convergence.}

\item{looseH}{if FALSE, the actual constraints \code{conH} are used for
calibrating all the hh weights. If TRUE, only the weights for which the
lower and upper thresholds defined by \code{conH} and \code{epsH} are exceeded are
calibrated. They are however not calibrated against the actual constraints
\code{conH} but against these lower and upper thresholds, i.e.
\code{conH}-\code{conH}*\code{epsH} and \code{conH}+\code{conH}*\code{epsH}.}

\item{numericalWeighting}{See \link{numericalWeighting}}

\item{check_hh_vars}{If \code{TRUE} check for non-unique values inside of a
household for variables in household constraints}

\item{conversion_messages}{show a message, if inputs need to be reformatted.
This can be useful for speed optimizations if ipf is called several times
with similar inputs (for example bootstrapping)}

\item{nameCalibWeight}{character defining the name of the variable for the
newly generated calibrated weight.}

\item{minMaxTrim}{numeric vector of length2, first element a minimum value
for weights to be trimmed to, second element a maximum value for weights to
be trimmed to.}

\item{print_every_n}{number of interation steps after which a summary table
is printed. The summary table shows all constraints which are not yet
reached according to \code{epsP} and \code{epsH}}
}
\value{
The function will return the input data \code{dat} with the calibrated
weights \code{calibWeight} as an additional column as well as attributes. If no
convergence has been reached in \code{maxIter} steps, and \code{returnNA} is \code{TRUE}
(the default), the column \code{calibWeights} will only consist of \code{NA}s. The
attributes of the table are attributes derived from the \code{data.table} class
as well as the following.
\tabular{ll}{
\code{converged} \tab Did the algorithm converge in \code{maxIter} steps? \cr
\code{iterations} \tab The number of iterations performed. \cr
\code{conP}, \code{conH}, \code{epsP}, \code{epsH} \tab See Arguments. \cr
\code{conP_adj}, \code{conH_adj} \tab Adjusted versions of \code{conP} and \code{conH} \cr
\code{formP}, \code{formH} \tab Formulas that were used to calculate \code{conP_adj} and
\code{conH_adj} based on the output table.
}
}
\description{
Adjust sampling weights to given totals based on household-level and/or
individual level constraints.
}
\details{
This function implements the weighting procedure described
here: \doi{10.17713/ajs.v45i3.120}.
Usage examples can be found in the corresponding vignette
(\code{vignette("ipf")}).

\code{conP} and \code{conH} are contingency tables, which can be created with \code{xtabs}.
The \code{dimnames} of those tables should match the names and levels of the
corresponding columns in \code{dat}.

\code{maxIter}, \code{epsP} and \code{epsH} are the stopping criteria. \code{epsP} and \code{epsH}
describe relative tolerances in the sense that
\deqn{1-epsP < \frac{w_{i+1}}{w_i} < 1+epsP}{1-epsP < w(i+1)/w(i) < 1+epsP}
will be used as convergence criterium. Here i is the iteration step and wi is
the weight of a specific person at step i.

The algorithm
performs best if all varables occuring in the constraints (\code{conP} and \code{conH})
as well as the household variable are coded as \code{factor}-columns in \code{dat}.
Otherwise, conversions will be necessary which can be monitored with the
\code{conversion_messages} argument. Setting \code{check_hh_vars} to \code{FALSE} can also
incease the performance of the scheme.
}
\examples{
\dontrun{

# load data
eusilc <- demo.eusilc(n = 1, prettyNames = TRUE)

# personal constraints
conP1 <- xtabs(pWeight ~ age, data = eusilc)
conP2 <- xtabs(pWeight ~ gender + region, data = eusilc)
conP3 <- xtabs(pWeight*eqIncome ~ gender, data = eusilc)

# household constraints
conH1 <- xtabs(pWeight ~ hsize + region, data = eusilc[!duplicated(hid)])

# simple usage ------------------------------------------

calibweights1 <- ipf(
  eusilc,
  conP = list(conP1, conP2, eqIncome = conP3),
  bound = NULL,
  verbose = TRUE
)

# compare personal weight with the calibweigth
calibweights1[, .(hid, pWeight, calibWeight)]

# advanced usage ----------------------------------------

# use an array of tolerances
epsH1 <- conH1
epsH1[1:4, ] <- 0.005
epsH1[5, ] <- 0.2

# create an initial weight for the calibration
eusilc[, regSamp := .N, by = region]
eusilc[, regPop := sum(pWeight), by = region]
eusilc[, baseWeight := regPop/regSamp]

calibweights2 <- ipf(
  eusilc,
  conP = list(conP1, conP2),
  conH = list(conH1),
  epsP = 1e-6,
  epsH = list(epsH1),
  bound = 4,
  w = "baseWeight",
  verbose = TRUE
)

# show an adjusted version of conP and the original
attr(calibweights2, "conP_adj")
attr(calibweights2, "conP")
}
}
\author{
Alexander Kowarik, Gregor de Cillia
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.ipf.R
\name{summary.ipf}
\alias{summary.ipf}
\title{Generate Summary Output for IPF Calibration}
\usage{
\method{summary}{ipf}(object, ...)
}
\arguments{
\item{object}{object of class ipf}

\item{...}{additional arguments}
}
\value{
a list of the following outputs
}
\description{
Generates a detailed summary of an Iterative Proportional Fitting (IPF) calibration, providing a complete tool for evaluating the calibration's success and the validity of the resulting weights.

The output is a list of data.tables for a comprehensive evaluation, including:

\strong{Calibration Results:}
\itemize{
\item \verb{calib_results_conP_*} and \verb{calib_results_conH_*}: Key diagnostic tables that compare calibrated margins to population targets and assess the goodness of fit via metrics like \code{maxFac}.
}

\strong{Data and Diagnostics:}
\itemize{
\item \verb{weighted data}: An excerpt of the final dataset with the calculated calibration weights.
\item \verb{distribution of the weights}: A statistical overview of the weight distribution (min, max, CV).
}

\strong{Detailed Margin Comparisons:}
\itemize{
\item \verb{conP_*}, \verb{conH_*}, \verb{*_adjusted}, \verb{*_original}, \verb{*_rel_diff_*}: Tables that compare original sample margins, calibrated margins, and population targets, along with their relative differences.
}
}
\examples{

\dontrun{
# load data
eusilc <- demo.eusilc(n = 1, prettyNames = TRUE)

# personal constraints
conP1 <- xtabs(pWeight ~ age, data = eusilc)
conP2 <- xtabs(pWeight ~ gender + region, data = eusilc)
conP3 <- xtabs(pWeight*eqIncome ~ gender, data = eusilc)

# household constraints
conH1 <- xtabs(pWeight ~ hsize + region, data = eusilc)

# simple usage ------------------------------------------

calibweights1 <- ipf(
 eusilc,
 conP = list(conP1, conP2, eqIncome = conP3),
 bound = NULL,
 verbose = TRUE
)
output <- summary(calibweights1)
# the output can easily be exported to an Excel file, e.g. with
# library(openxlsx)
# write.xlsx(output, "SummaryIPF.xlsx")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recalib.R
\name{recalib}
\alias{recalib}
\title{Calibrate weights}
\usage{
recalib(
  dat,
  hid = attr(dat, "hid"),
  weights = attr(dat, "weights"),
  b.rep = attr(dat, "b.rep"),
  period = attr(dat, "period"),
  conP.var = NULL,
  conH.var = NULL,
  conP = NULL,
  conH = NULL,
  epsP = 0.01,
  epsH = 0.02,
  ...
)
}
\arguments{
\item{dat}{either data.frame or data.table containing the sample survey for
various periods.}

\item{hid}{character specifying the name of the column in \code{dat} containing
the household ID.}

\item{weights}{character specifying the name of the column in \code{dat}
containing the sample weights.}

\item{b.rep}{character specifying the names of the columns in \code{dat}
containing bootstrap weights which should be recalibratet}

\item{period}{character specifying the name of the column in \code{dat} containing
the sample period.}

\item{conP.var}{character vector containig person-specific variables to which
weights should be calibrated or a list of such character vectors.
Contingency tables for the population are calculated per \code{period} using
\code{weights}. If a vector is supplied contingency tables will be calculated
for each vector entry. If a list is supplied contingency tables will be
calculated for each list entry. See examples for more details.}

\item{conH.var}{character vector containig household-specific variables to
which weights should be calibrated or a list of such character vectors.
Contingency tables for the population are calculated per \code{period} using
\code{weights}. If a vector is supplied contingency tables will be calculated
for each vector entry. If a list is supplied contingency tables will be
calculated for each list entry. See examples for more details.}

\item{conP}{list or (partly) named list defining the constraints on person
level.  The list elements are contingency tables in array representation
with dimnames corresponding to the names of the relevant calibration
variables in \code{dat}. If a numerical variable is to be calibrated, the
respective list element has to be named with the name of that numerical
variable. Otherwise the list element shoud NOT be named.}

\item{conH}{list or (partly) named list defining the constraints on
household level.  The list elements are contingency tables in array
representation with dimnames corresponding to the names of the relevant
calibration variables in \code{dat}. If a numerical variable is to be
calibrated, the respective list element has to be named with the name of
that numerical variable. Otherwise the list element shoud NOT be named.}

\item{epsP}{numeric value specifying the convergence limit for \code{conP.var}
or \code{conP}, see \code{\link[=ipf]{ipf()}}.}

\item{epsH}{numeric value specifying the convergence limit for \code{conH.var}
or \code{conH}, see \code{\link[=ipf]{ipf()}}.}

\item{...}{additional arguments passed on to function \code{\link[=ipf]{ipf()}} from this
package.}
}
\value{
Returns a data.table containing the survey data as well as the
calibrated weights for the bootstrap replicates. The original bootstrap
replicates are overwritten by the calibrated weights. If calibration of a
bootstrap replicate does not converge the bootsrap weight is not returned
and numeration of the returned bootstrap weights is reduced by one.
}
\description{
Calibrate weights for bootstrap replicates by using iterative proportional
updating to match population totals on various household and personal levels.
}
\details{
\code{recalib} takes survey data (\code{dat}) containing the bootstrap replicates
generated by \link{draw.bootstrap} and calibrates weights for each bootstrap
replication according to population totals for person- or household-specific
variables. \cr
\code{dat} must be household data where household members correspond to multiple
rows with the same household identifier. The data should at least containt
the following columns:
\itemize{
\item Column indicating the sample period;
\item Column indicating the household ID;
\item Column containing the household sample weights;
\item Columns which contain the bootstrap replicates (see output of
\link{draw.bootstrap});
\item Columns indicating person- or household-specific variables for which sample
weight should be adjusted.
}

For each period and each variable in \code{conP.var} and/or \code{conH.var} contingency
tables are estimated to get margin totals on personal- and/or
household-specific variables in the population.\cr
Afterwards the bootstrap replicates are multiplied with the original sample
weight and the resulting product ist then adjusted using \code{\link[=ipf]{ipf()}} to match the
previously calcualted contingency tables. In this process the columns of the
bootstrap replicates are overwritten by the calibrated weights.\cr
}
\examples{

library(surveysd)
library(data.table)
setDTthreads(1)
set.seed(1234)
eusilc <- demo.eusilc(n = 3, prettyNames = TRUE)

dat_boot <- draw.bootstrap(eusilc, REP = 1, hid = "hid",
                           weights = "pWeight",
                           strata = "region", period = "year")

# calibrate weight for bootstrap replicates
dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region",
                          verbose = TRUE)


# calibrate on other variables
dat_boot_calib <- recalib(dat_boot, conP.var = c("gender", "age"),
                          conH.var = c("region", "hsize"), verbose = TRUE)

# supply contingency tables directly
conP1 <- xtabs(pWeight ~ age + year, data = eusilc)
conP2 <- xtabs(pWeight ~ gender + year, data = eusilc)
conH1 <- xtabs(pWeight ~ region + year,
               data = eusilc[!duplicated(paste(hid,year))])
conH2 <- xtabs(pWeight ~ hsize + year,
               data = eusilc[!duplicated(paste(hid,year))])

conP <- list(conP1,conP2)
conH <- list(conH1,conH2)
dat_boot_calib <- recalib(dat_boot, conP.var = NULL,
                          conH.var = NULL, conP = conP,
                          conH = conH, verbose = TRUE)


# calibrate on gender x age
dat_boot_calib <- recalib(dat_boot, conP.var = list(c("gender", "age")),
                          conH.var = NULL, verbose = TRUE)

# identical
conP1 <- xtabs(pWeight ~ age + gender + year, data = eusilc)
conP <- list(conP1)
dat_boot_calib <- recalib(dat_boot, conP.var = NULL,
                          conH.var = NULL, conP = conP,
                          conH = NULL, verbose = TRUE)
                                                     
}
\seealso{
\code{\link[=ipf]{ipf()}} for more information on iterative
proportional fitting.
}
\author{
Johannes Gussenbauer, Alexander Kowarik, Statistics Austria
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateHHID.R
\name{generate.HHID}
\alias{generate.HHID}
\title{Generate new houshold ID for survey data with rotating panel design taking
into account split households}
\usage{
generate.HHID(dat, period = "RB010", pid = "RB030", hid = "DB030")
}
\arguments{
\item{dat}{data table of data frame containing the survey data}

\item{period}{column name of \code{dat} containing an indicator for the
rotations, e.g years, quarters, months, ect...}

\item{pid}{column name of \code{dat} containing the personal identifier. This
needs to be fixed for an indiviual throught the whole survey}

\item{hid}{column name of \code{dat} containing the household id. This needs
to for a household throught the whole survey}
}
\value{
the survey data \code{dat} as data.table object containing a new and
an old household ID. The new household ID which considers the split
households is now named \code{hid} and the original household ID has a
trailing "_orig".
}
\description{
Generating a new houshold ID for survey data using a houshold ID and a
personal ID.
For surveys with rotating panel design containing housholds, houshold members
can move from an existing household to a new one, that was not originally in
the sample. This leads to the creation of so called split households. Using a
peronal ID (that stays fixed over the whole survey), an indicator for
different time steps and a houshold ID, a new houshold ID is assigned to the
original and the split household.
}
\examples{
\dontrun{
library(surveysd)
library(laeken)
library(data.table)

eusilc <- surveysd:::demo.eusilc(n=4)

# create spit households
eusilc[,rb030split:=rb030]
year <- eusilc[,unique(year)]
year <- year[-1]
leaf_out <- c()
for(y in year) {
  split.person <- eusilc[year==(y-1)&!duplicated(db030)&!db030\%in\%leaf_out,
                         sample(rb030,20)]
  overwrite.person <- eusilc[year==(y)&!duplicated(db030)&!db030\%in\%leaf_out,
                             .(rb030=sample(rb030,20))]
  overwrite.person[,c("rb030split","year_curr"):=.(split.person,y)]

  eusilc[overwrite.person,
         rb030split:=i.rb030split,on=.(rb030,year>=year_curr)]
  leaf_out <- c(
    leaf_out,
    eusilc[rb030\%in\%c(overwrite.person$rb030,overwrite.person$rb030split),
    unique(db030)])
}

# pid which are in split households
eusilc[,.(uniqueN(db030)),by=list(rb030split)][V1>1]

eusilc.new <- generate.HHID(eusilc, period = "year", pid = "rb030split",
                            hid = "db030")

# no longer any split households in the data
eusilc.new[,.(uniqueN(db030)),by=list(rb030split)][V1>1]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/demo.eusilc.R
\name{demo.eusilc}
\alias{demo.eusilc}
\title{Generate multiple years of EU-SILC data}
\usage{
demo.eusilc(n = 8, prettyNames = FALSE)
}
\arguments{
\item{n}{Number of years to generate. Should be at least 1}

\item{prettyNames}{Create easy-to-read names for certain variables.
Recommended for demonstration purposes. Otherwise, use the original codes
documented in \link[laeken:eusilc]{laeken::eusilc}.}
}
\description{
Create a dummy dataset to be used for demonstrating the functionalities of
the \code{surveysd} package based on \link[laeken:eusilc]{laeken::eusilc}. Please refer to the
documentation page of the original data for details about the variables.
}
\details{
If \code{prettyNames} is \code{TRUE}, the following variables will be available in an
easy-to-read manner.
\itemize{
\item \code{hid} Household id. Consistent with respect to the reference period
(\code{year})
\item \code{hsize} Size of the household. derived from \code{hid} and \code{period}
\item \code{region} Federal state of austria where the household is located
\item \code{pid} Personal id. Consistent with respect to the reference period (\code{year})
\item \code{age} Age-class of the respondent
\item \code{gender} A persons gender (\code{"male"}, \code{"Female"})
\item \code{ecoStat} Ecnomic status
(\code{"part time"}, \code{"full time"}, \code{"unemployed"}, ...)
\item \code{citizenship} Citizenship (\code{"AT"}, \code{"EU"}, \code{"other"})
\item \code{pWeight} Personal sample weight inside the reference period
\item \code{year}. Simulated reference period
\item \code{povertyRisk}. Logical variable determining whether a respondent is at risk
of poverty
}
}
\examples{
demo.eusilc(n = 1, prettyNames = TRUE)[, c(1:8, 26, 28:30)]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rescaled.bootstrap.R
\name{rescaled.bootstrap}
\alias{rescaled.bootstrap}
\title{Draw bootstrap replicates}
\usage{
rescaled.bootstrap(
  dat,
  method = c("Preston", "Rao-Wu"),
  REP = 1000,
  strata = "DB050>1",
  cluster = "DB060>DB030",
  fpc = "N.cluster>N.households",
  single.PSU = c("merge", "mean"),
  return.value = c("data", "replicates"),
  run.input.checks = TRUE,
  already.selected = NULL,
  seed = NULL
)
}
\arguments{
\item{dat}{either data frame or data table containing the survey sample}

\item{method}{for bootstrap replicates, either \code{"Preston"} or \code{"Rao-Wu"}}

\item{REP}{integer indicating the number of bootstraps to be drawn}

\item{strata}{string specifying the column name in \code{dat} that is used for
stratification. For multistage sampling multiple column names can be
specified by \code{strata=c("strata1","strata2","strata3")} or
\code{strata=c("strata1>strata2>strata3")}. See Details for more
information.}

\item{cluster}{string specifying the column name in \code{dat} that is used for
clustering. For instance given a household sample the column containing
the household ID should be supplied.
For multistage sampling multiple column names can be specified
by \code{cluster=c("cluster1","cluster2","cluster3")} or
\code{cluster=c("cluster1>cluster2>cluster3")}.
See Details for more information.}

\item{fpc}{string specifying the column name in \code{dat} that contains the
number of PSUs at the first stage. For multistage sampling the number of
PSUs at each stage must be specified by \code{strata=c("fpc1","fpc2","fpc3")}
or \code{strata=c("fpc1>fpc2>fpc3")}.}

\item{single.PSU}{either "merge" or "mean" defining how single PSUs need to
be dealt with. For \code{single.PSU="merge"} single PSUs at each stage are
merged with the strata or cluster with the next least number of PSUs. If
multiple of those exist one will be select via random draw. For
\code{single.PSU="mean"} single PSUs will get the mean over all bootstrap
replicates at the stage which did not contain single PSUs.}

\item{return.value}{either "data", "replicates" and/or "selection"
specifying the return value of the function. For "data" the survey data is
returned as class \code{data.table}, for "replicates" only the bootstrap replicates
are returned as \code{data.table}. For "selection" list of data.tables with
length of \code{length(strata)} is returned containing 1:\code{REP} 0-1 columns
indicating if a PSU was selected for each sampling stage.}

\item{run.input.checks}{logical, if TRUE the input will be checked before applying
the bootstrap procedure}

\item{already.selected}{list of data.tables or \code{NULL} where each data.table contains
columns in \code{cluster}, \code{strata} and additionally 1:\code{REP} columns containing
0-1 values which indicate if a PSU was selected for each bootstrap replicate.
Each of the data.tables corresponds to one of the sampling stages. First entry
in the list corresponds to the first sampling stage and so on.}

\item{seed}{integer specifying the seed for the random number generator.}
}
\value{
returns the complete data set including the bootstrap replicates or
just the bootstrap replicates, depending on \code{return.value="data"} or
\code{return.value="replicates"} respectively.
}
\description{
Draw bootstrap replicates from survey data using either the rescaled
bootstrap for stratified multistage sampling, presented by J. Preston
(2009) or the Rao-Wu boostrap by J. N. K. Rao and C. F. J. Wu (1988)
}
\details{
For specifying multistage sampling designs the column names in
\code{strata},\code{cluster} and \code{fpc} need to be seperated by ">".\cr
For multistage sampling the strings are read from left to right meaning that
the first vector entry or column name before the first ">" is taken as the column for
stratification/clustering/number of PSUs at the first and the last vector entry
or column after
the last ">" is taken as the column for stratification/clustering/number of
PSUs at the last stage.
If for some stages the sample was not stratified or clustered one must
specify this by "1" or "I", e.g. \code{strata=c("strata1","I","strata3")} or
\code{strata=c("strata1>I>strata3")} if there was
no stratification at the second stage or
\code{cluster=c("cluster1","cluster2","I")} respectively
\code{cluster=c("cluster1>cluster2>I")}
if there were no clusters at the last stage.\cr
The number of PSUs at each stage is not calculated internally and must be
specified for any sampling design.
For single stage sampling using stratification this can usually be done by
adding over all sample weights of each PSU by each strata-code.\cr
Spaces in each of the strings will be removed, so if column names contain
spaces they should be renamed before calling this procedure!\cr
If \code{already.selected} is supplied the sampling of bootstrap replicates
considers if speficif PSUs have already been selected by a previous survey wave.
For a specific \code{strata} and \code{cluster} this could lead to more than \code{floor(n/2)}
records selected. In that case records will be de-selected such that \code{floor(n/2)} records,
with \code{n} as the total number of records, are selected for each
\code{strata} and \code{cluster}. This parameter ist mostly used by \link{draw.bootstrap} in
order to consider the rotation of the sampling units over time.
}
\examples{

library(surveysd)
library(data.table)
setDTthreads(1)
set.seed(1234)
eusilc <- demo.eusilc(n = 1,prettyNames = TRUE)

eusilc[,N.households := sum(pWeight[!duplicated(hid)]),
                                  by = .(region, year)]
eusilc.bootstrap <- rescaled.bootstrap(eusilc, REP = 10, strata = "region",
                                       cluster = "hid", fpc = "N.households")

eusilc[,new_strata := paste(region,hsize,sep="_")]
eusilc[,N.housholds := sum(pWeight[!duplicated(hid)]), by = new_strata]
eusilc.bootstrap <- rescaled.bootstrap(eusilc, REP = 10, strata = c("new_strata"),
                                       cluster = "hid", fpc = "N.households")


}
\references{
Preston, J. (2009). Rescaled bootstrap for stratified multistage
sampling. Survey Methodology. 35. 227-234.
Rao, J. N. K., and C. F. J. Wu. (1988). Resampling Inference with Complex Survey Data.
Journal of the American Statistical Association 83 (401): 231–41.
}
\author{
Johannes Gussenbauer, Eileen Vattheuer, Statistics Austria
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.selection.R
\name{get.selection}
\alias{get.selection}
\title{Get sample selection (~deltas) from drawn bootstrap replicates}
\usage{
get.selection(
  dat,
  b.rep = attr(dat, "b.rep"),
  strata = attr(dat, "strata"),
  cluster = attr(dat, "cluster"),
  hid = attr(dat, "hid"),
  pid = attr(dat, "pid")
)
}
\arguments{
\item{dat}{either data.frame or data.table containing the survey data with
rotating panel design. Should contain only survey data from a single time period.}

\item{b.rep}{character specifying the names of the columns in \code{dat}
containing bootstrap replicates.}

\item{strata}{character vector specifying the name(s) of the column in \code{dat}
by which the population was stratified.}

\item{cluster}{character vector specifying cluster in the data.}

\item{hid}{character specifying the name of the column in \code{dat} containing
the household id. If \code{NULL} (the default), the household structure is not
regarded. \code{hid} and \code{pid} cannot both be \code{NULL}.}

\item{pid}{pid column in \code{dat} specifying the personal identifier. This
identifier needs to be unique for each person throught the whole data set.
\code{hid} and \code{pid} cannot both be \code{NULL}.}
}
\value{
Returns a list of data.tables.
The length of the list equals the number of sampling stages specified.
Each list entry contains a \code{data.table} with variables for sampling stage and/or
\code{hid}/\code{pid} as well as \code{length(attr(dat,"b.rep"))} columns each indicating if
record/cluster was drawn in the respective sampling stage for the i-th boostrap replicate.
}
\description{
Reconstruct sample selection, e.g. record was drawn or not drawn (delta = 0/1)
in each sampling stage from bootstrap replicates.
\code{get.selection()} needs the \code{cluster}, \code{strata} and \code{hid}/\code{pid} information (if not \code{NULL})
to correctly reconstruct if a record was drawn in each sampling stage for each bootstrap replicate.
Is only needed if bootstrap replicates are drawn for a survey
with existing bootstrap replicates from a previous period,
see parameter \code{already.selected} in function \code{\link[=draw.bootstrap]{draw.bootstrap()}}.
}
\examples{

library(surveysd)
library(data.table)
setDTthreads(1)
set.seed(1234)
eusilc <- demo.eusilc(n = 3, prettyNames = TRUE)

## draw replicates with stratification
dat_boot <- draw.bootstrap(eusilc[year<2012], REP = 3, weights = "pWeight",
                           strata = "region", hid = "hid",
                           period = "year")

## get selection matrix for year 2011 
dat_selection <- get.selection(dat_boot[year==2011])
print(dat_selection)

## draw bootstrap replicates for year 2012
## respecting already selected units for year 2011 ~ dat_selection
## in order to mimic rotating panel design
dat_boot_2012 <- draw.bootstrap(eusilc[year==2012], REP = 3, weights = "pWeight",
                                strata = "region", hid = "hid",
                                period = "year", 
                                already.selected = dat_selection)



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{cpp_mean}
\alias{cpp_mean}
\alias{geometric_mean_reference}
\title{Calculate mean by factors}
\usage{
geometric_mean_reference(w, classes)
}
\arguments{
\item{w}{An numeric vector. All entries should be positive.}

\item{classes}{A factor variable. Must have the same length as \code{w}.}
}
\description{
These functions calculate the arithmetic and geometric mean of the weight for each class. \code{geometric_mean} and
\code{arithmetic_mean} return a \code{numeric} vector of the same length as \code{w} which stores the averaged weight for each
observation. \code{geometric_mean_reference} returns the same value by reference, i.e. the input value \code{w} gets
overwritten by the updated weights. See examples.
}
\examples{

\dontrun{

## create random data
nobs <- 10
classLabels <- letters[1:3]
dat = data.frame(
  weight = exp(rnorm(nobs)),
  household = factor(sample(classLabels, nobs, replace = TRUE))
)
dat

## calculate weights with geometric_mean
geom_weight <- geometric_mean(dat$weight, dat$household)
cbind(dat, geom_weight)

## calculate weights with arithmetic_mean
arith_weight <- arithmetic_mean(dat$weight, dat$household)
cbind(dat, arith_weight)

## calculate weights "by reference"
geometric_mean_reference(dat$weight, dat$household)
dat
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc.stError.R
\name{calc.stError}
\alias{calc.stError}
\title{Calcualte point estimates and their standard errors using bootstrap
weights.}
\usage{
calc.stError(
  dat,
  weights = attr(dat, "weights"),
  b.weights = attr(dat, "b.rep"),
  period = attr(dat, "period"),
  var = NULL,
  fun = weightedRatio,
  relative.share = FALSE,
  group = NULL,
  group.diff = FALSE,
  fun.adjust.var = NULL,
  adjust.var = NULL,
  period.diff = NULL,
  period.mean = NULL,
  bias = FALSE,
  size.limit = 20,
  cv.limit = 10,
  p = NULL,
  add.arg = NULL,
  national = FALSE
)
}
\arguments{
\item{dat}{either data.frame or data.table containing the survey data.
Surveys can be a panel survey or rotating panel survey, but does not need
to be. For rotating panel survey bootstrap weights can be created using
\link{draw.bootstrap} and \link{recalib}.}

\item{weights}{character specifying the name of the column in \code{dat}
containing the original sample weights. Used to calculate point estimates.}

\item{b.weights}{character vector specifying the names of the columns in
\code{dat} containing bootstrap weights. Used to calculate standard errors.}

\item{period}{character specifying the name of the column in \code{dat}
containing the sample periods.}

\item{var}{character vector containing variable names in \code{dat} on which \code{fun}
shall be applied for each sample period. If \code{var = NULL} the results will
reflect the sum of \code{weights}.}

\item{fun}{function which will be applied on \code{var} for each sample period.
Predefined functions are \link{weightedRatio}, \link{weightedSum}, but can also take
any other function which returns a double or integer and uses weights as
its second argument.}

\item{relative.share}{boolean, if \code{TRUE} point estimates resulting from \code{fun} will be
divided by the point estimate at population level per \code{period}.}

\item{group}{character vectors or list of character vectors containig
variables in \code{dat}. For each list entry \code{dat} will be split in subgroups
according to the containing variables as well as \code{period}. The
pointestimates are then estimated for each subgroup seperately. If
\code{group=NULL} the data will split into sample periods by default.}

\item{group.diff}{boolen, if \code{TRUE} differences and the standard error
between groups defined in \code{group} are calculated. See details for more explanations.}

\item{fun.adjust.var}{can be either \code{NULL} or a function. This argument can
be used to apply a function for each \code{period} and bootstrap weight to the
data. The resulting estimates will be passed down to \code{fun}. See details for
more explanations.}

\item{adjust.var}{can be either \code{NULL} or a character specifying the first
argument in \code{fun.adjust.var}.}

\item{period.diff}{character vectors, defining periods for which the
differences in the point estimate as well it's standard error is
calculated. Each entry must have the form of \code{"period1 - period2"}. Can be
NULL}

\item{period.mean}{odd integer, defining the range of periods over which the
sample mean of point estimates is additionally calcualted.}

\item{bias}{boolean, if \code{TRUE} the sample mean over the point estimates of
the bootstrap weights is returned.}

\item{size.limit}{integer defining a lower bound on the number of
observations on \code{dat} in each group defined by \code{period} and the entries in
\code{group}. Warnings are returned if the number of observations in a subgroup
falls below \code{size.limit}. In addition the concerned groups are available in
the function output.}

\item{cv.limit}{non-negativ value defining a upper bound for the standard
error in relation to the point estimate. If this relation exceed
\code{cv.limit}, for a point estimate, they are flagged and available in the
function output.}

\item{p}{numeric vector containing values between 0 and 1. Defines which
quantiles for the distribution of \code{var} are additionally estimated.}

\item{add.arg}{additional arguments which will be passed to fun. Can be
either a named list or vector. The names of the object correspond to the
function arguments and the values to column names in dat, see also
examples.}

\item{national}{DEPRECATED use \code{relative.share} instead! boolean, if TRUE point estimates resulting from fun will be
divided by the point estimate at the national level.}
}
\value{
Returns a list containing:
\itemize{
\item \code{Estimates}: data.table containing period differences and/or k period
averages for estimates of
\code{fun} applied to \code{var} as well as the corresponding standard errors, which
are calculated using the bootstrap weights. In addition the sample size,
\code{n}, and poplutaion size for each group is added to the output.
\item \code{smallGroups}: data.table containing groups for which the number of
observation falls below \code{size.limit}.
\item \code{cvHigh}: data.table containing a boolean variable which indicates for each
estimate if the estimated standard error exceeds \code{cv.limit}.
\item \code{stEDecrease}: data.table indicating for each estimate the theoretical
increase in sample size which is gained when averaging over k periods. Only
returned if \code{period.mean} is not \code{NULL}.
}
}
\description{
Calculate point estimates as well as standard errors of variables in surveys.
Standard errors are estimated using bootstrap weights (see \link{draw.bootstrap}
and \link{recalib}). In addition the standard error of an estimate can be
calcualted using the survey data for 3 or more consecutive periods, which
results in a reduction of the standard error.
}
\details{
\code{calc.stError} takes survey data (\code{dat}) and returns point estimates
as well as their standard Errors defined by \code{fun} and \code{var} for each sample
period in \code{dat}. \code{dat} must be household data where household members
correspond to multiple rows with the same household identifier. The data
should at least contain the following columns:
\itemize{
\item Column indicating the sample period;
\item Column indicating the household ID;
\item Column containing the household sample weights;
\item Columns which contain the bootstrap weights (see output of \link{recalib});
\item Columns listed in \code{var} as well as in \code{group}
}

For each variable in \code{var} as well as sample period the function \code{fun} is
applied using the original as well as the bootstrap sample weights.\cr
The point estimate is then selected as the result of \code{fun} when using the
original sample weights and it's standard error is estimated with the result
of \code{fun} using the bootstrap sample weights. \cr
\cr
\code{fun} can be any function which returns a double or integer and uses sample
weights as it's second argument. The predifined options are \code{weightedRatio}
and \code{weightedSum}.\cr
\cr
For the option \code{weightedRatio} a weighted ratio (in \\%) of \code{var} is
calculated for \code{var} equal to 1, e.g
\code{sum(weight[var==1])/sum(weight[!is.na(var)])*100}.\cr
Additionally using the option \code{national=TRUE} the weighted ratio (in \\%) is
divided by the weighted ratio at the national level for each \code{period}.
\cr
If \code{group} is not \code{NULL} but a vector of variables from \code{dat} then \code{fun} is
applied on each subset of \code{dat} defined by all combinations of values in
\code{group}.\cr
For instance if \code{group = "sex"} with "sex" having the values "Male" and
"Female" in \code{dat} the point estimate and standard error is calculated on the
subsets of \code{dat} with only "Male" or "Female" value for "sex". This is done
for each value of \code{period}. For variables in \code{group} which have \code{NA}s in
\code{dat} the rows containing the missings will be discarded. \cr
When \code{group} is a list of character vectors, subsets of \code{dat} and the
following estimation of the point estimate, including the estimate for the
standard error, are calculated for each list entry.\cr
\cr
If \code{group.diff = TRUE} difference between groups definded by \code{group} are calculated.
Differences are only calculated within each variables of \code{group},
e.g \code{group = c("gender", "region")} will calcualate estimates of each group and
also differences within \code{"gender"} and \code{"region"} seperately.
If grouping is done with multiple variables e.g \verb{group = list(c("gender","region")})
(~ grouping by \code{"gender"} x \code{"region"}) differences are calculated
only between groups where one of the grouping variables is different.
For instance the difference between \code{gender = "female" & region = "Vienna"} and
\code{gender = "male" & region = "Vienna"} OR \code{gender = "female" & region = "Vienna"} and
\code{gender = "female" & region = "Salzburg"} will be calculated.
The difference between \code{gender = "female" & region = "Vienna"} and
\code{gender = "male" & region = "Salzburg"} will not be calculated. The order of difference
is determined by order of value (alpha-numerical order) or
if grouping contains factor variables the factor levels determin the order.
\cr
The optional parameters \code{fun.adjust.var} and \code{adjust.var} can be used if the
values in \code{var} are dependent on the \code{weights}. As is for instance the case
for the poverty thershhold calculated from EU-SILC.
In such a case an additional function can be supplied using \code{fun.adjust.var}
as well as its first argument \code{adjust.var}, which needs to be part of the
data set \code{dat}. Then, before applying \code{fun} on variable \code{var}
for all \code{period} and groups, the function \code{fun.adjust.var} is applied to
\code{adjust.var} using each of the bootstrap weights seperately (NOTE: weight is
used as the second argument of \code{fun.adjust.var}).
Thus creating i=1,...,\code{length(b.weights)} additional variables.
For applying \code{fun} on \code{var} the estimates for the bootstrap replicate will
now use each of the corresponding new additional variables. So instead of
\deqn{fun(var,weights,...),fun(var,b.weights[1],...),
fun(var,b.weights[2],...),...}
the function \code{fun} will be applied in the way
\deqn{fun(var,weights,...),fun(var.1,b.weights[1],...),fun(var.2,
b.weights[2],...),...}

where \code{var.1}, \code{var.2}, \code{...} correspond to the estimates resulting from
\code{fun.adjust.var} and \code{adjust.var}.
NOTE: This procedure is especially usefull if the \code{var} is dependent on
\code{weights} and \code{fun} is applied on subgroups of the data set. Then it is not
possible to capture this procedure with \code{fun} and \code{var}, see examples for a
more hands on explanation.
\cr
When defining \code{period.diff} the difference of point estimates between periods
as well their standard errors are calculated.\cr
The entries in \code{period.diff} must have the form of \code{"period1 - period2"}
which means that the results of the point estimates for \code{period2} will be
substracted from the results of the point estimates for \code{period1}.\cr
\cr
Specifying \code{period.mean} leads to an improvement in standard error by
averaging the results for the point estimates, using the bootstrap weights,
over \code{period.mean} periods.
Setting, for instance, \code{period.mean = 3} the results in averaging these
results over each consecutive set of 3 periods.\cr
Estimating the standard error over these averages gives an improved estimate
of the standard error for the central period, which was used for
averaging.\cr
The averaging of the results is also applied in differences of point
estimates. For instance defining \code{period.diff = "2015-2009"} and
\code{period.mean = 3}
the differences in point estimates of 2015 and 2009, 2016 and 2010 as well as
2014 and 2008 are calcualated and finally the average over these 3
differences is calculated.
The periods set in \code{period.diff} are always used as the middle periods around
which the mean over \code{period.mean} years is build.
\cr
Setting \code{bias} to \code{TRUE} returns the calculation of a mean over the results
from the bootstrap replicates. In  the output the corresponding columns is
labeled \emph{_mean} at the end.\cr
\cr
If \code{fun} needs more arguments they can be supplied in \code{add.arg}. This can
either be a named list or vector.\cr
\cr
The parameter \code{size.limit} indicates a lower bound of the sample size for
subsets in \code{dat} created by \code{group}. If the sample size of a subset falls
below \code{size.limit} a warning will be displayed.\cr
In addition all subsets for which this is the case can be selected from the
output of \code{calc.stError} with \verb{$smallGroups}.\cr
With the parameter \code{cv.limit} one can set an upper bound on the coefficient
of variantion. Estimates which exceed this bound are flagged with \code{TRUE} and
are available in the function output with \verb{$cvHigh}.
\code{cv.limit} must be a positive integer and is treated internally as \\%, e.g.
for \code{cv.limit=1} the estimate will be flagged if the coefficient of
variantion exceeds 1\\%.\cr
\cr
When specifying \code{period.mean}, the decrease in standard error for choosing
this method is internally calcualted and a rough estimate for an implied
increase in sample size is available in the output with \verb{$stEDecrease}.
The rough estimate for the increase in sample size uses the fact that for a
sample of size \eqn{n} the sample estimate for the standard error of most
point estimates converges with a factor \eqn{1/\sqrt{n}} against the true
standard error \eqn{\sigma}.
}
\examples{
# Import data and calibrate

library(surveysd)
library(data.table)
setDTthreads(1)
set.seed(1234)
eusilc <- demo.eusilc(n = 4,prettyNames = TRUE)
dat_boot <- draw.bootstrap(eusilc, REP = 3, hid = "hid", weights = "pWeight",
                           strata = "region", period = "year")
dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region")

# estimate weightedRatio for povertyRisk per period

err.est <- calc.stError(dat_boot_calib, var = "povertyRisk",
                        fun = weightedRatio)
err.est$Estimates

# calculate weightedRatio for povertyRisk and fraction of one-person
# households per period

dat_boot_calib[, onePerson := .N == 1, by = .(year, hid)]
err.est <- calc.stError(dat_boot_calib, var = c("povertyRisk", "onePerson"),
                        fun = weightedRatio)
err.est$Estimates


# estimate weightedRatio for povertyRisk per period and gender and
# period x region x gender 

group <- list("gender", c("gender", "region"))
err.est <- calc.stError(dat_boot_calib, var = "povertyRisk",
                        fun = weightedRatio, group = group)
err.est$Estimates

# use average over 3 periods for standard error estimation
# and calculate estimate for difference of
# period 2011 and 2012 inclulding standard errors
period.diff <- c("2012-2011")
err.est <- calc.stError(
  dat_boot_calib, var = "povertyRisk", fun = weightedRatio,
  period.diff = period.diff,  # <- take difference of periods 2012 and 2011
  period.mean = 3)  # <- average over 3 periods
err.est$Estimates

# for more examples see https://statistikat.github.io/surveysd/articles/error_estimation.html

}
\seealso{
\link{draw.bootstrap} \cr
\link{recalib}
}
\author{
Johannes Gussenbauer, Alexander Kowarik, Statistics Austria
}
\keyword{manip}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{PointEstimates}
\alias{PointEstimates}
\alias{weightedRatio}
\alias{weightedSum}
\title{Weighted Point Estimates}
\usage{
weightedRatio(x, w)

weightedSum(x, w)
}
\arguments{
\item{x}{numeric vector}

\item{w}{weight vector}
}
\value{
Each of the functions return a single numeric value
}
\description{
Predefined functions for weighted point estimates in package \code{surveysd}.
}
\details{
Predefined functions are weighted ratio and weighted sum.
}
\examples{
x <- 1:10
w <- 10:1
weightedRatio(x,w)
x <- 1:10
w <- 10:1
weightedSum(x,w)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kishFactor.R
\name{kishFactor}
\alias{kishFactor}
\title{Kish Factor}
\usage{
kishFactor(w, na.rm = FALSE)
}
\arguments{
\item{w}{a numeric vector with weights}

\item{na.rm}{a logical value indicating whether NA values should be stripped before the computation proceeds.}
}
\value{
The function will return the the kish factor
}
\description{
Compute the design effect due to unequal weighting.
}
\details{
The factor is computed acording to 'Weighting for Unequal P_i', Leslie Kish, Journal of Official Statistics, Vol. 8. No. 2, 1992
\deqn{ deff = \sqrt n \sum_j w_j^2 / (\sum_j w_j)^2}
}
\examples{
kishFactor(rep(1,10))
kishFactor(rlnorm(10))
}
\author{
Alexander Kowarik
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_summary.ipf.R
\name{print.summary.ipf}
\alias{print.summary.ipf}
\title{Print method for IPF calibration summary}
\usage{
\method{print}{summary.ipf}(x, ...)
}
\arguments{
\item{x}{An object of class \code{summary.ipf}, as returned by \code{\link{summary.ipf}}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
The input object \code{x}, invisibly (for chaining).
}
\description{
Provides a concise summary of an IPF (Iterative Proportional Fitting) calibration summary object.
It extracts the calibration weight from \code{all_formulas}, computes the Kish factor for the weights,
and prints the first 10 rows of any \code{calib_results_} tables.
Useful for a quick overview of calibration results. Additional details can be explored with \code{str()} or \code{names()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R, R/computeFrac.R
\name{computeLinear}
\alias{computeLinear}
\alias{computeLinearG1_old}
\alias{computeLinearG1}
\alias{computeFrac}
\alias{numericalWeighting}
\title{Numerical weighting functions}
\usage{
computeLinear(curValue, target, x, w, boundLinear = 10)

computeLinearG1_old(curValue, target, x, w, boundLinear = 10)

computeLinearG1(curValue, target, x, w, boundLinear = 10)

computeFrac(curValue, target, x, w)
}
\arguments{
\item{curValue}{Current summed up value. Same as \code{sum(x*w)}}

\item{target}{Target value. An element of \code{conP} in \link{ipf}}

\item{x}{Vector of numeric values to be calibrated against}

\item{w}{Vector of weights}

\item{boundLinear}{The output \code{f} will satisfy
\verb{1/boundLinear <= f <= boundLinear}. See \code{bound} in \link{ipf}}
}
\value{
A weight multiplier \code{f}
}
\description{
Customize weight-updating within factor levels in case of numerical
calibration. The functions described here serve as inputs for \link{ipf}.
}
\details{
\code{computeFrac} provides the "standard" IPU updating scheme given as

\deqn{f = target/curValue}

which means that each weight inside the level will be multtiplied by the same
factor when doing the actual update step (\code{w := f*w}). \code{computeLinear} on the
other hand calculates \code{f} as

\ifelse{html}{
  \out{<div style="text-align: center;"> f<sub>i</sub> = a  &middot; x<sub>i</sub> + b </div>}
}{\deqn{f_i = ax_i+b}}

where \code{a} and \code{b} are chosen, so f satisfies the following two equations.

\ifelse{html}{
  \out{<div style="text-align: center;">&sum; f<sub>i</sub> w<sub>i</sub>
       x<sub>i</sub> = target</div>}
}{\deqn{\sum f_i * w_i * x_i = target}}
\ifelse{html}{
  \out{<div style="text-align: center;">&sum; f<sub>i</sub>
       w<sub>i</sub> = &sum; w<sub>i</sub></div>}
}{\deqn{\sum f_i * w_i = \sum w_i}}

\code{computeLinearG1} calculates \code{f} in the same way as \code{computeLinear}, but if
\code{f_i*w_i<1} \code{f_i} will be set to \code{1/w_i}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.surveysd}
\alias{plot.surveysd}
\title{Plot surveysd-Objects}
\usage{
\method{plot}{surveysd}(
  x,
  variable = x$param$var[1],
  type = c("summary", "grouping"),
  groups = NULL,
  sd.type = c("dot", "ribbon"),
  ...
)
}
\arguments{
\item{x}{object of class 'surveysd' output of function \link{calc.stError}}

\item{variable}{Name of the variable for which standard errors have been
calcualated in \code{dat}}

\item{type}{can bei either \code{"summary"} or \code{"grouping"}, default value is
\code{"summary"}. For \code{"summary"} a barplot is created giving an overview of the
number of estimates having the flag \code{smallGroup}, \code{cvHigh}, both or none
of them. For 'grouping' results for point estimate and standard error are
plotted for pre defined groups.}

\item{groups}{If \code{type='grouping'} variables must be defined by which the
data is grouped. Only 2 levels are supported as of right now. If only one
group is defined the higher group will be the estimate over the whole
period. Results are plotted for the first argument in \code{groups} as well as
for the combination of \code{groups[1]} and \code{groups[2]}.}

\item{sd.type}{can bei either \code{'ribbon'} or \code{'dot'} and is only used if
\code{type='grouping'}. Default is \code{"dot"}
For \code{sd.type='dot'} point estimates are plotted and flagged if the
corresponding standard error and/or the standard error using the mean over
k-periods exceeded the value \code{cv.limit} (see \link{calc.stError}).
For \code{sd.type='ribbon'} the point estimates including ribbons, defined by
point estimate +- estimated standard error are plotted.
The calculated standard errors using the mean over k periods are plotted
using less transparency. Results for the higher level (~\code{groups[1]}) are
coloured grey.}

\item{...}{additional arguments supplied to plot.}
}
\description{
Plot results of \code{calc.stError()}
}
\examples{

library(surveysd)

set.seed(1234)
eusilc <- demo.eusilc(n = 3, prettyNames = TRUE)

dat_boot <- draw.bootstrap(eusilc, REP = 3, hid = "hid", weights = "pWeight",
                           strata = "region", period = "year")

# calibrate weight for bootstrap replicates
dat_boot_calib <- recalib(dat_boot, conP.var = "gender", conH.var = "region")

# estimate weightedRatio for povmd60 per period
group <- list("gender", "region", c("gender", "region"))
err.est <- calc.stError(dat_boot_calib, var = "povertyRisk",
                        fun = weightedRatio,
                        group = group , period.mean = NULL)


plot(err.est)

# plot results for gender
# dotted line is the result on the national level
plot(err.est, type = "grouping", groups = "gender")


# plot results for rb090 in each db040
# with standard errors as ribbons
plot(err.est, type = "grouping", groups = c("gender", "region"), sd.type = "ribbon")


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R, R/ipf.r
\name{ipf_step}
\alias{ipf_step}
\alias{ipf_step_ref}
\alias{ipf_step_f}
\alias{combine_factors}
\title{Perform one step of iterative proportional updating}
\usage{
ipf_step_ref(w, classes, targets)

ipf_step(w, classes, targets)

ipf_step_f(w, classes, targets)

combine_factors(dat, targets)
}
\arguments{
\item{w}{a numeric vector of weights. All entries should be positive.}

\item{classes}{a factor variable. Must have the same length as \code{w}.}

\item{targets}{key figure to target with the ipu scheme. A numeric verctor of the same length as \code{levels(classes)}.
This can also be a \code{table} produced by \code{xtabs}. See examples.}

\item{dat}{a \code{data.frame} containing the factor variables to be combined.}
}
\description{
C++ routines to invoke a single iteration of the Iterative proportional updating (IPU) scheme. Targets and classes
are assumed to be one dimensional in the \code{ipf_step} functions. \code{combine_factors} aggregates several vectors of
type factor into a single one to allow multidimensional ipu-steps. See examples.
}
\details{
\code{ipf_step} returns the adjusted weights. \code{ipf_step_ref} does the same, but updates \code{w} by reference rather than
returning. \code{ipf_step_f} returns a multiplicator: adjusted weights divided by unadjusted weights. \code{combine_factors} is
designed to make \code{ipf_step} work with contingency tables produced by \link{xtabs}.
}
\examples{

############# one-dimensional ipu ##############

## create random data
nobs <- 10
classLabels <- letters[1:3]
dat = data.frame(
  weight = exp(rnorm(nobs)),
  household = factor(sample(classLabels, nobs, replace = TRUE))
)
dat

## create targets (same lenght as classLabels!)
targets <- 3:5

## calculate weights
new_weight <- ipf_step(dat$weight, dat$household, targets)
cbind(dat, new_weight)

## check solution
xtabs(new_weight ~ dat$household)

## calculate weights "by reference"
ipf_step_ref(dat$weight, dat$household, targets)
dat

############# multidimensional ipu ##############

## load data
factors <- c("time", "sex", "smoker", "day")
tips <- data.frame(sex=c("Female","Male","Male"), day=c("Sun","Mon","Tue"),
time=c("Dinner","Lunch","Lunch"), smoker=c("No","Yes","No"))
tips <- tips[factors]

## combine factors
con <- xtabs(~., tips)
cf <- combine_factors(tips, con)
cbind(tips, cf)[sample(nrow(tips), 10, replace = TRUE),]

## adjust weights
weight <- rnorm(nrow(tips)) + 5
adjusted_weight <- ipf_step(weight, cf, con)

## check outputs
con2 <- xtabs(adjusted_weight ~ ., data = tips)
sum((con - con2)^2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/draw.bootstrap.R
\name{draw.bootstrap}
\alias{draw.bootstrap}
\title{Draw bootstrap replicates}
\usage{
draw.bootstrap(
  dat,
  method = "Preston",
  REP = 1000,
  hid = NULL,
  weights,
  period = NULL,
  strata = NULL,
  cluster = NULL,
  totals = NULL,
  single.PSU = c("merge", "mean"),
  boot.names = NULL,
  split = FALSE,
  pid = NULL,
  seed = NULL,
  already.selected = NULL
)
}
\arguments{
\item{dat}{either data.frame or data.table containing the survey data with
rotating panel design.}

\item{method}{for bootstrap replicates, either \code{"Preston"} or \code{"Rao-Wu"}}

\item{REP}{integer indicating the number of bootstrap replicates.}

\item{hid}{character specifying the name of the column in \code{dat} containing
the household id. If \code{NULL} (the default), the household structure is not
regarded.}

\item{weights}{character specifying the name of the column in \code{dat}
containing the sample weights.}

\item{period}{character specifying the name of the column in \code{dat} containing
the sample periods. If \code{NULL} (the default), it is assumed that all
observations belong to the same period.}

\item{strata}{character vector specifying the name(s) of the column in \code{dat}
by which the population was stratified. If \code{strata} is a vector
stratification will be assumed as the combination of column names contained
in \code{strata}. Setting in addition \code{cluster} not NULL stratification will be
assumed on multiple stages, where each additional entry in \code{strata}
specifies the stratification variable for the next lower stage. see Details
for more information.}

\item{cluster}{character vector specifying cluster in the data. If not
already specified in \code{cluster} household ID is taken es the lowest level
cluster.}

\item{totals}{character specifying the name of the column in \code{dat} containing
the the totals per strata and/or cluster. Is ONLY optional if \code{cluster} is
\code{NULL} or equal \code{hid} and \code{strata} contains one columnname! Then the
households per strata will be calcualted using the \code{weights} argument. If
clusters and strata for multiple stages are specified \code{totals} needs to be
a vector of \code{length(strata)} specifying the column on \code{dat} that contain
the total number of PSUs at each stage. \code{totals} is interpreted from left
the right, meaning that the first argument corresponds to the number of
PSUs at the first and the last argument to the number of PSUs at the last
stage.}

\item{single.PSU}{either "merge" or "mean" defining how single PSUs need to
be dealt with. For \code{single.PSU="merge"} single PSUs at each stage are
merged with the strata or cluster with the next least number of PSUs. If
multiple of those exist one will be select via random draw. For
\code{single.PSU="mean"} single PSUs will get the mean over all bootstrap
replicates at the stage which did not contain single PSUs.}

\item{boot.names}{character indicating the leading string of the column names
for each bootstrap replica. If NULL defaults to "w".}

\item{split}{logical, if TRUE split households are considered using \code{pid},
for more information see Details.}

\item{pid}{column in \code{dat} specifying the personal identifier. This
identifier needs to be unique for each person throught the whole data set.
Can be \code{NULL}.}

\item{seed}{integer specifying the seed for the random number generator.}

\item{already.selected}{list of data.tables indicating if record was drawn
in previous \code{period}. \code{length(already.selected)} must be equal to
the number of sampling stages specified. See \code{\link[=get.selection]{get.selection()}} for an example.}
}
\value{
the survey data with the number of REP bootstrap replicates added as
columns.

Returns a data.table containing the original data as well as the
number of \code{REP} columns containing the bootstrap replicates for each
repetition.\cr
The columns of the bootstrap replicates are by default labeled "w\emph{Number}"
where \emph{Number} goes from 1 to \code{REP}. If the column names of the bootstrap
replicates should start with a different character or string the parameter
\code{boot.names} can be used.
}
\description{
Draw bootstrap replicates from survey data with rotating panel
design. Survey information, like ID, sample weights, strata and population
totals per strata, should be specified to ensure meaningfull survey
bootstraping.
}
\details{
\code{draw.bootstrap} takes \code{dat} and draws \code{REP} bootstrap replicates
from it.
\code{dat} must be household data where household members correspond to multiple
rows with the same household
identifier. For most practical applications, the following columns should be
available in the dataset
and passed via the corresponding parameters:
\itemize{
\item Column indicating the sample period (parameter \code{period}).
\item Column indicating the household ID (parameter \code{hid})
\item Column containing the household sample weights (parameter \code{weights});
\item Columns by which population was stratified during the sampling process
(parameter: \code{strata}).
}

As methods either the either the rescaled bootstrap for stratified
multistage sampling, presented by J. Preston (2009) (\code{method = "Preston"})
or the Rao-Wu boostrap by J. N. K. Rao and C. F. J. Wu (1988) (\code{method = "Rao-Wu"})
are supported.
\cr
For single stage sampling design a column the argument \code{totals} is optional,
meaning that a column of the number of PSUs at the first stage does not need
to be supplied. The number of PSUs is calculated and added to \code{dat} using
\code{strata} and \code{weights}. By setting \code{cluster} to NULL single stage sampling
design is always assumed and
if \code{strata} contains of multiple column names the combination of all those
column names will be used for stratification.
\cr
In the case of multi stage sampling design the argument \code{totals} needs to be
specified and needs to have the same number of arguments as \code{strata}.
\cr
If \code{cluster} is \code{NULL} or does not contain \code{hid} at the last stage, \code{hid}
will automatically be used as the final cluster. If, besides \code{hid},
clustering in additional stages is specified the number of column names in
\code{strata} and \code{cluster} (including \code{hid}) must be the same. If for any stage
there was no clustering or stratification one can set "1" or "I" for this
stage.
\cr
For example \verb{strata=c("REGION","I"),cluster=c("MUNICIPALITY","HID")} would
speficy a 2 stage sampling design where at the first stage the municipalities
where drawn stratified by regions
and at the 2nd stage housholds are drawn in each municipality without
stratification.
\cr
Bootstrap replicates are drawn for each survey period consecutively (\code{period}) using the
function \link{rescaled.bootstrap}.
Bootstrap replicates are drawn consistently in the way that in each \code{period} and
sampling stage always \eqn{\lfloor n/2 \rfloor} clusters are selected in each strata.
\cr
This ensures that the bootstrap replicates follow the same logic as the
sampled households, making the bootstrap replicates more comparable to the
actual sample units.
\cr
If \code{split} ist set to \code{TRUE} and \code{pid} is specified, the bootstrap replicates
are carried forward using the personal identifiers instead of the household
identifier.
This takes into account the issue of a houshold splitting up.
Any person in this new split household will get the same bootstrap replicate
as the person that has come from an other household in the survey.
People who enter already existing households will also get the same bootstrap
replicate as the other households members had in the previous periods.
\cr
\code{already.selected} can be specified in order to construct bootstrap replicates
considering that already drawn bootstrap replicates from a previous \code{period}
exist for the same survey. See \code{\link[=get.selection]{get.selection()}} for more explanations and examples.
}
\examples{

library(surveysd)
library(data.table)
setDTthreads(1)
set.seed(1234)
eusilc <- demo.eusilc(n = 2, prettyNames = TRUE)

## draw replicates without stratification or clustering
dat_boot <- draw.bootstrap(eusilc, REP = 1, weights = "pWeight",
                           period = "year")

## use stratification w.r.t. region and clustering w.r.t. households
dat_boot <- draw.bootstrap(
  eusilc, REP = 1, hid = "hid", weights = "pWeight",
  strata = "region", period = "year")

## stratification by multiple variables
dat_boot <- draw.bootstrap(
  eusilc, REP = 1, hid = "hid", weights = "pWeight",
  strata = c("region", "hsize"), period = "year")


# create split households
eusilc[, pidsplit := pid]
year <- eusilc[, unique(year)]
year <- year[-1]
leaf_out <- c()
for(y in year) {
split.person <- eusilc[
  year == (y-1) & !duplicated(hid) & !(hid \%in\% leaf_out),
  sample(pid, 20)
]
overwrite.person <- eusilc[
  (year == (y)) & !duplicated(hid) & !(hid \%in\% leaf_out),
  .(pid = sample(pid, 20))
]
overwrite.person[, c("pidsplit", "year_curr") := .(split.person, y)]

eusilc[overwrite.person, pidsplit := i.pidsplit,
       on = .(pid, year >= year_curr)]

# assign new PID to avoid duplicates in pidsplit
eusilc[overwrite.person, change_pid := TRUE, 
       on = .(pid = pidsplit, year >= year_curr)]
eusilc[change_pid == TRUE, pidsplit := as.integer(paste0(hid, 99))]
eusilc[, change_pid := NULL]

leaf_out <- c(leaf_out,
              eusilc[pid \%in\% c(overwrite.person$pid,
                                overwrite.person$pidsplit),
                     unique(hid)])
}

dat_boot <- draw.bootstrap(
  eusilc, REP = 1, hid = "hid", weights = "pWeight",
  strata = c("region", "hsize"), period = "year", split = TRUE,
  pid = "pidsplit")
# split households were considered e.g. household and
# split household were both selected or not selected
dat_boot[, data.table::uniqueN(w1), by = pidsplit][V1 > 1]


}
\references{
Preston, J. (2009). Rescaled bootstrap for stratified multistage
sampling. Survey Methodology. 35. 227-234.
Rao, J. N. K., and C. F. J. Wu. (1988). Resampling Inference with Complex Survey Data.
Journal of the American Statistical Association 83 (401): 231–41.
}
\seealso{
\code{\link[data.table:data.table]{data.table}} for more information on
data.table objects.
}
\author{
Johannes Gussenbauer, Alexander Kowarik, Statistics Austria
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.surveysd}
\alias{print.surveysd}
\title{Print function for surveysd objects}
\usage{
\method{print}{surveysd}(x, ...)
}
\arguments{
\item{x}{an object of class \code{'surveysd'}}

\item{...}{additonal parameters}
}
\description{
Prints the results of a call to \link{calc.stError}. Shows used variables and
function, number of point estiamtes
as well as properties of the results.
}
